             SUBROUTINE GET_CORFAC(ELESNO,CORFAC)

C=============================================================================
C  Purpose:  Retreives a reasonable value for electron energy correction
C            factors in CEM, PEM, or FEM...  uses CEMFIX, PEMFIX, FEMFIX
C            (CEMFIX is modified to be a little more robust).
C            Skips energy correction if relative change in raw energy
C            exceeds 15%
C
C  INPUTS:    ELESNO: ELES bank number
C  OUTPUTS:   CORFAC: electron energy correction factor ECOR = ERAW/corfac 
C
C  Author:    
C             Jeff Berryhill
C             University of Chicago
C             August 10, 1995
C  DEC 4 1995:  +.6% correction for central electrons in run 1b
C  DEC 4 1995:  new version of PEMFIX added
C  MAR 26 1996: new version of CEMFIX added, +.6% correction for central
C               electrons is rescinded
C=============================================================================
$$IMPLICIT NONE
$$INCLUDE 'C$INC:ANACTL.INC'
$$INCLUDE 'C$INC:ERLEVL.INC'
$$INCLUDE 'C$INC:JOBSTA.INC'
$$INCLUDE 'C$INC:BCS.INC'
$$INCLUDE 'YBOS$LIBRARY:BNKTYP.INC'
$$INCLUDE 'YBOS$LIBRARY:ERRCOD.INC'
$$INCLUDE 'C$INC:DSELES.INC'


C Local variables
C------------------------------------
       INTEGER ELESNO,STATUS,IDUMMY,IELES,DETELE
       REAL CORFAC
       REAL XLOCAL,ZSTRIP,ZLOCAL,DELZ
       REAL EDGECM(11)

       DATA EDGECM/4.22,24.16,48.32,72.48,96.64,120.80,
     &            144.97,169.12,193.28,217.45,245.96/
C External functions
C------------------------------------
       INTEGER BLOCAT
C------------------------------------

       CORFAC = 1.0
       
       STATUS = BLOCAT(IW,'ELES',ELESNO,IDUMMY,IELES)
       IF (STATUS.NE.YESUCC) THEN
         CALL ERLOGR('GET_CORFAC',ERWARN,0,0,'Where is the ELES bank?')
         RETURN
       ENDIF

       DETELE = IW(IELES+REGELE)
       IF (DETELE.EQ.2) THEN
         CALL FEMFIX(ELESNO,CORFAC)
         IF (CORFAC.LE.0.869565.OR.CORFAC.GE.1.17647) CORFAC = 1.0
         RETURN
       ENDIF

       IF (DETELE.EQ.1) THEN
         CALL PEMFIX(ELESNO,CORFAC)
         IF (CORFAC.LE.0.869565.OR.CORFAC.GE.1.17647) CORFAC = 1.0
         RETURN
       ENDIF

       IF (DETELE.EQ.0) THEN
         DELZ = RW(IELES + ZEXELE) - RW(IELES + ZRWELE)
         XLOCAL = RW(IELES+XWRELE)
         ZSTRIP = ABS(RW(IELES+ZSTELE))
         ZLOCAL=-999.
         DO IDUMMY=1,10
           IF (EDGECM(IDUMMY).LE.ZSTRIP.AND.
     &       EDGECM(IDUMMY+1).GT.ZSTRIP   ) THEN
             ZLOCAL=ZSTRIP-
     &        (EDGECM(IDUMMY)+(EDGECM(IDUMMY+1)-EDGECM(IDUMMY))*0.5)
           ENDIF ! strip in this tower?
         ENDDO ! loop over towers
                               
         IF (ABS(XLOCAL).GT.50.OR.ABS(ZLOCAL).GT.20) THEN
           CALL ERLOGR('GET_CORFAC',ERWARN,0,0,'Bad CES data')
           RETURN
         ENDIF
         CALL CEMFIX(ELESNO,CORFAC)
         IF (CORFAC.LE.0.869565.OR.CORFAC.GE.1.17647) CORFAC = 1.0
         RETURN
       ENDIF

       RETURN
       END



