C
      subroutine cprtrack(cesx,cesz,iwedge,ntrwd,ntrwd2,
     1 			ntrwd3,cesprx,tkcprx)
C
C     Description:
C       Return the number of tracks pointing at CPR hit position
C       in x-view and x-z view. This subroutine basically has the
C       same procedure done in PHOTON$SOURCE:CPR_NTUPLE.CDF.
C       CPR-track stuff were split from it for convenient use.
C     Input :
C             cesx:    CES X (Phocesx in phot92.inc)
C             cesz:    CES Z (Phocesz in phot92.inc)
C             iwedge:  Photon Wedge # (Phowed in phot92.inc)
C     Output:
C             ntrwd:   # of tracks pointing at CPR hit (wedge)
C             ntrwd2:  # of tracks pointing at CPR hit (wedge and z)
C             ntrwd3:  # of tracks pointing at CPR hit (x and z)
C	      cesprx:  ces local x extrapolated to cpr radius
C	      tkcprx:  local x of track with closest extrapolation to
C				ces hit
C
C     Author:
C             Takeshi Takano
C     Modified:
C	      Peter Wilson: added ntrwd3 with match in x.  Match window in 
C			x covers 2.5 cpr cells (5.6cm) + 3cm for position 
C			resolution.  Also put a maximum on the track z:
C	 		can't be more than 15cm beyond the nd of the 
C			chamber.
C
      implicit none

$$include 'photon$library:trakcm.inc'
$$include 'c$inc:jobsta.inc'

      real cesx,cesz
      integer iwedge,ntrwd,ntrwd2,ntrwd3
      real cesprx,cesprz
      integer ich
      integer k
      real parin(5)
      integer iwedtr
      real xcpr,zcpr,xdiff
      real tkcprx, tkcprz
      integer ierr

      integer oldrun, oldevt
      data oldrun/-1/
      data oldevt/-1/

      real zcrack
      data zcrack/121.6/                ! Cpr crack
      real xmax
      data xmax/8.6/       		! maximum track-cpr miss
      real zmax
      data zmax/250.0/			! check on end of chamber (235.26cm)
C
C     Start code
C     ==========
C

C--- CALL TRKDMP TO DUMP TRACKS TO COMMON BLOCKS FOR CPR TRACK CUT(QTRK)
      if (nrun.ne.oldrun .or. nev.ne.oldevt) then     	
         call trkdmp(2)
	 oldrun = nrun
	 oldevt = nev
      endif

C--- GET NTRWD2 FOR CPR TRACK CUT (FROM PHOTON$SOURCE:CPR_NTUPLE.CDF)

C (1) Get cpr hit position with ces hits and z vertex.
C     Only ich are used in this code.

      call cprchm(cesx,cesz,cesprx,cesprz,ich)

C (2) Get cpr hit position with track parameters. iwedtr, xcpr and zcpr
C     are used in this code.

      ntrwd  = 0
      ntrwd2 = 0
      ntrwd3 = 0
      xdiff = 99.
      tkcprx = 99.
      tkcprz = 9999.
      do k=1,n_track
         parin(1) = curvtr(k)
         parin(2) = imptr(k)
         parin(3) = phitr(k)
         parin(4) = ctantr(k)
         parin(5) = z0tr(k)

         call cprext(parin,iwedtr,xcpr,zcpr,ierr)
	 xcpr = -xcpr				! go to ces coordinates

         if(iwedge.eq.iwedtr) then
            ntrwd = ntrwd + 1
            if(abs(zcpr).lt.zcrack .and. ich.eq.0) then
	       ntrwd2 = ntrwd2 + 1
	       if (abs(xcpr-cesprx).lt.xmax) ntrwd3 = ntrwd3 + 1
	       if (abs(xcpr-cesprx).lt.xdiff) then
	       	  xdiff = xcpr-cesprx	
		  tkcprx = xcpr
		  tkcprz = zcpr
	       endif
            endif
            if(abs(zcpr).gt.zcrack .and. abs(zcpr).lt.zmax
     1				   .and. ich.eq.1) then
	       ntrwd2 = ntrwd2 + 1
	       if (abs(xcpr-cesprx).lt.xmax) ntrwd3 = ntrwd3 + 1
	       if (abs(xcpr-cesprx).lt.xdiff) then
	       	  xdiff = xcpr-cesprx	
		  tkcprx = xcpr
		  tkcprz = zcpr
	       endif
            endif
         endif
      enddo

      return
      end
C
C
C
