C======================================================================
	Subroutine Emtime(Bnknum,Time)
C======================================================================
C
C  Routine to give the hadron TDC time for an ELES bank if it had
C  sufficient EM shower leakage to fire the TDC  (about 100 MeV).
C  It returns the average time for all the towers in the ELES
C  cluster, although it is likely only the seed tower would fire
C  the TDC most of the time.
C
C  Input: Bnknum  (Integer*4)  ELES/ELEC Bank Number
C  Output: Time   (Real*4)     Time in nanoseconds after the collision
C			       so since the front of the hadron calorimeter
C			       is about 7 ft. from the interaction point
C			       this should be around 7 ns for a good 
C			       cluster.
C
C			       Time = -999.  if no tower has a TDC hit.
C  
C
$$IMPLICIT
C  Argument Declarations:
C  ======================
        INTEGER BNKNUM             
        REAL    Time
C
C===External Function Declarations:
C
        INTEGER BLOCAT, GTIETA
C  External Declarations:
C  ======================
$$INCLUDE 'C$INC:ERLEVL.INC'
$$INCLUDE 'C$INC:PIVALU.INC'
$$INCLUDE 'C$INC:BTMASK.INC'
$$INCLUDE 'YBOS$LIBRARY:ERRCOD.INC'
$$INCLUDE 'C$INC:DSELES.INC'
$$INCLUDE 'C$INC:DSTOWE.INC'
$$INCLUDE 'C$INC:BCS.INC'
C  Local Declarations:
C  ===================
        INTEGER STATUS, INDBNK, INTOWE, Inelec, MINETA, MAXETA, Ntime
        INTEGER IELOOP, IADDR,  INEXT,  ICODE,  IPHI,   IETA,   TOWTYP
        INTEGER LENBLK, JDESFT, IPHIS(49), IETAS(49), NNT, I, Incall
	Integer Ntow
	Real Timtow,Eme,Had

C  Executable Code:
C  ================
C
	Time = 0.
	Ntime = 0
C
C---Locate the bank in YBOS:
        STATUS = BLOCAT(IW,'ELEC',BNKNUM,INDBNK,Inelec)
        IF (STATUS.NE.YESUCC) THEN
          CALL ERLOGR('Emtime',ERWARN,0,STATUS,
     1   'Unable to locate YBOS bank ELEC')
          RETURN
        ENDIF

C---Loop over Elec
	Ntow = IW(Inelec)
	IADDR = 1
        DO 10 IELOOP = 1,Ntow
          ICODE  = IW(INelec + IADDR)
	  IPHI   = IAND(ICODE,MASK7)
          IETA   = IAND(ISHFT(ICODE,-7),MASK7)
          TOWTYP = TETTYP(IETA)
          LENBLK = TETBLK(TOWTYP)  ! block length for this tower type
	  EME = RW(Inelec+Iaddr+Teteme(Towtyp))
	  HAD = RW(Inelec+Iaddr+Tethae(Towtyp))
	  Timtow = 0
	  IF(Techat(Towtyp).ne.0) Timtow = Rw(Inelec+Iaddr+Techat(Towtyp))
	  IF(Tewhat(Towtyp).ne.0) Timtow = Rw(Inelec+Iaddr+Tewhat(Towtyp))
	    If(Timtow.ne.0.) then
		Ntime = Ntime + 1
		Time = Time + Timtow
	    Endif

C---New address
        IADDR = IADDR + LENBLK
10      CONTINUE                        ! end of loop over eta rings

	If(Ntime.ne.0) Time = Time/Float(Ntime)
	If(Ntime.eq.0) Time = -999.
C
C---Return to caller
C===================
        RETURN
        END
