C 11/8/96 Implement photon treaty cuts as standard.  Tight cuts are central
C photon treaty and loose cuts are photon treaty plug.
C
C 2/22/96 Plug photons added.  To mantain backward compatability, central
C photons always come first.  The definition of Ncencl stays the same
C and the total number of photon candidates is Ncencl+Nplugcl
C quantites filled for both central and plug are marked with a *.
C Plug-only quantites are tacked on the end.

	Integer Ncencl		! number of central candidates (=# of central ELES)
	Integer Nplugcl		! number of plug candidates (=# of plug ELES)
        integer npho            ! MIN(NCENCL+NPLUGCL,10)
	Integer Phodetect(10)   ! 0 central, 1 plug
	Integer Phowed(10)      ! TOWE seed tower phi
	Integer Phon3d(10)      ! number of 3-D tracks pointing to the cluster
	Integer Phobnk(10)   	!*Corresponding ELES bank number
	Integer Phojet(10)	! Corresponding CALS bank
	Real Phoet(10)		!*Uncorrected Et
	Real Phoe(10)		!*Uncorrected E 
	Real Phozv(10)		!* z of primary from ELES 
	Real Phoco4(10)		!*Et in cone of .4, not inc. the EM cluster
	Real Phosth(10)		!*sin(theta) for the cluster
	Real Phochi(10)		! CES (chi_strip+chi_wire)/2
	Real Phocesx(10)	! CES cluster position (x=r-phi, local coords)
	Real Phocesz(10)	! CES cluster position (CDF coords)
	Real Phochistr(10)	! CES chi_strip
	Real Phochiwir(10)	! CES chi_wire

C These cpr numbers are for clustering without CES seed position
	Real Phocpr3ph(10)	! CPR cluster charge (fC) for 3-channel group
	Real Phocpr3ps(10)	! CPR clus. x (local) extrap to CES (cm), 3-ch
	Real Phocpr5ph(10)	! CPR cluster charge (fC) for 5-channel group
	Real Phocpr5ps(10)	! CPR clus. x (local) extrap to CES (cm), 5-ch
	Real Phocpr16ph(10)	! CPR cluster charge (fC) for 16-channel group
	Real Phocpr16ps(10)	! CPR clus. x (local) extrap to CES (cm), 16-ch
C This is for clustering with CES seed position
	Real Phocpr5phseed(10)	! CPR cluster charge (fC) for 5-channel group

	Real Phodeta(10)	!*detector eta
	Real Phoeta(10)		!*eta
	Real Phocese(10)	!*CES E (from wires) (PES for plug when avail)
	Real Phopt(10)		! electron track Pt, from ELES
	Real Phopt2(10)		! 2nd track Pt
	Real Phophi(10)		!*phi
	Real Phodelx(10)	! x electron track-CES match from ELES
	Real Phodelz(10)	! z electron track-CES match from ELES
	Real Phoco7(10)		!*Et in cone of .7, not including the EM cluster

C variables added 26-OCT-1995
	Real Phohadem(10)	!*Had/EM, from ELES -
				!      3tower for central, 1 tower for plug
	Real Pholshr(10)	! Lshare, from ELES
	Real Phostr2(10)	! second CES strip cluster, z (CDF coords) 
	Real Phostre2(10)	! second CES strip cluster, E (none=0.)
	Real Phowir2(10)	! second CES wire cluster, x (local coords)
	Real Phowire2(10)	! second CES wire cluster, E (none=0.)
	Real Phosumpt4(10)	! Pt of 3-D tracks in a cone of .4
	Real Phosumpt7(10)	! Pt of 3-D tracks in a cone of .7
        Real Phoboxiso(10)      ! box isolation computed for 3x3 trigger
                                ! tower sum (excludes seed tower)
        Real PhoCESslide(10)	! CES sliding cut (see phoval code)
        Real PhoCESsig(10)	! (CES energy - CEM energy) /sigma

	Real Phocor(10)		!*Energy correction factor from C/PEMFIX
	Integer Phofid(10)	!*FIDELE result

	Real Phoceswht(10)	! photon weight CES method
	Real Phocprwht(10)	! photon weight CPR method
	Real Phocespg(10)	! photon probability, CES method
	Real Phocespb(10)	! bkgrd probability, CES method
	Real Phocprpg(10)	! photon probability, CPR method
	Real Phocprpb(10)	! bkgrd probability, CPR method
C 12-Sept-1996 CPR track information
	Integer Ntrwd(10)	! Number of Tracks pointing at cpr chamber
	Integer Ntrwd2(10)	! Number of Tracks pointing at cpr chamber
	Integer Ntrwd3(10)	! Number of Tracks pointing at cpr chamber 
	Real Cescprx(10)	! x position of CES cluster in cpr coordinates
	Real Tkcprx(10)		! x position of closest track to CES cluster

C PHOTON CUT WORD
	Integer Phowrd(10)	!*bit pattern of cuts

C bit definitions, bit on means it failed the cut
C bit  0 = Central photon
C bit  1 = Pt of "e"<5
C bit  2 = (Chi_strip+Chi_wire)/2 < 20
C bit  3 = Lshr < 0.2
C bit  4 = Had/EM < sliding cut
C bit  5 = N3D = 0
C bit  6 = Cone 0.4 calorimeter isolation <.15
C bit  7 = Sum Pt of tracks in cone 0.4 < 10.0
C bit  8 = Sum Pt of tracks in cone 0.4 <  2.0
C bit  9 = Second CES cluster < 1.0 GeV
C bit 10 = 3x3 Chi < 5.0
C bit 11 = CHI eta < 20.0 (passes cut if not fiducial to PES)
C bit 12 = CHI phi < 20.0          "               "
C bit 13 = VTX occ < 0.4
C bit 14 = Had/EM < sliding Plug cut
C bit 15 = all FIDELE cuts
C bit 16 = sliding second CES cluster cut (CEM only)
C bit 17 = 3x3 box energy sum < 4GeV
C bit 18 = 14<|Zstrip|<230
C bit 19 = |Xwire|<21.5
C bit 20 = Pt of "e"<1
C bit 21 = Pt of "e"<2
C bit 22 = (Chi_strip+Chi_wire)/2 < 4
C bit 23 = |(CES-CEM)/sigma| < 2
C bit 24 = Cone 0.4 calorimeter isolation <.10
C bit 25 = CPR track cut
C bit 26 = CPR pulse height < 500

	Logical PhoID(10)   	!*passed PHOTON TREATY ID cuts
	Logical Phoiso(10)  	!*passed PHOTON TREATY isolation cuts

	Logical PhoIDL(10)   	!*passed loose ID cuts (plug)
	Logical PhoisoL(10)  	!*passed loose isolation cuts (plug)

	Logical Phoisob(10)     !*passed photon BOX cut for ET<50 or
				! cone cut for ET>50

        INTEGER NUMPHOID        ! Number of Photons passing tight ID Cuts
        INTEGER NUMPHOISO       ! Number of Photons passing tight iso Cuts
        INTEGER NUMPHOIDL       ! Number of Photons passing Loose ID Cuts 
        INTEGER NUMPHOISOL      ! Number of Photons passing Loose Iso
        INTEGER NUMPHOISOB      ! Number of Photons passing BOX/CONE Iso
                                ! Cuts

        REAL STAT_PHO(10)       ! Photon Status



C 4-VECTOR ADDED 2/16/96
C THESE VALUES ARE CEMFIX CORRECTED ONLY.  IF LINKED TO 7_13 OR LATER, 
C IT INCLUDES THE .006 CORRECTION FOR 1B.
C AND CALUCLATED USING THE STRIP CLUSTER ETA AND PHI WITH THE 
C PRIMARY VERTEX.
	Real Pho_p4(4,10)	!* photon 4-vector


C Central
C FOR PHOID: N3D=0, CES AVERAGE CHI**2<20, LSHARE<.2,  AND HAD/EM<SLIDING CUT 
C FOR PHOISO: ET IN A CONE OF .4/ET < .15, 
C             PT OF TRACKS IN A CONE OF .4 < 2.0 GEV
C             NO SECOND CES STRIP CLUSTER WITH E>1GEV IN THE SAME WEDGE
C FOR PHOIDL: TRACKPT<5, CES AVERAGE CHI**2<20, LSHARE<.2,  
C		AND HAD/EM<SLIDING CUT 
C FOR PHOISOL: ET IN A CONE OF .4/ET < .15, 
C             PT OF TRACKS IN A CONE OF .4 < 10.0 GEV

C Plug
C FOR PHOID:   NOT IMPLEMENTED
C FOR PHOISO:  NOT IMPLEMENTED
C FOR PHOIDL: CHI(3X3)<5.0, PES CHI ETA<20 AND PHI<20 (WHEN AVAILABLE)
C            VTX OCC<40%, HAD/EM<.028+.00019*E
C FOR PHOISOL: ET IN A CONE OF .4/ET < .15, 



C PLUG-ONLY QUANTITIES
	Real Phovtx(10)		! VTX occupancy in % 
				!	(set to 1 for no expected hits)
	Real Pho3x3(10)		! 3x3 chi**2
	Real Phochie(10)	! PES eta chi**2 (-1.0 when not available)
	Real Phochip(10)	! PES phi chi**2 (-1.0 when not available)

C photon TDC time
        Real Photime(10)  
C corrected isolation variables
        Real Phovcor(10) ! multiple vertex correction (subtract from phoco4) 
        Real Pholcor(10) ! leakage correction 
C                        (subtract Pholcor*Phoet/Phocor from phoco4)

	Common/Phoinfoc/Ncencl,Phowed,Phon3d,Phobnk,Phoet,Phoe,Phozv,Phoco4,
     *		Phosth,Phochi,Phocesx,Phocesz,Phocpr3ph,Phocpr3ps,
     *		Phocpr5ph,Phocpr5ps,Phocpr16ph,Phocpr16ps,Phodeta,
     *		Phoeta,Phocese,Phopt,Phophi,Phodelx,Phodelz,Phojet,Phoco7,
     *		Phohadem,Pholshr,Phostr2,Phostre2,Phowir2,Phowire2,  
     *          Phosumpt4,Phosumpt7,PhoID,Phoiso,PhoIDL,PhoisoL,
     *          Pho_p4,Phowrd,
     *          Nplugcl,Phodetect,Phovtx,Pho3x3,Phochie,Phochip,
     *          Phocor,phoboxiso,PhoCESslide,
     *          NUMPHOID, NUMPHOISO, NUMPHOIDL, NUMPHOISOL, STAT_PHO, 
     *          NPHO,
     *          Phoceswht,Phocprwht,Phocpr5phseed,
     *          Phochistr,Phochiwir,PhoCESsig,
     *		Phocespg,Phocespb,Phocprpg,Phocprpb,phopt2,
     *		Ntrwd, Ntrwd2, Ntrwd3, Cescprx, Tkcprx,
     *		PhoIsob,Photime,Phovcor,Pholcor

