
C-------------------------------------------------------------------------------
	SUBROUTINE EXTRAP(PX,PY,PZ,Z0,IGOOD,XEX)
C-------------------------------------------------------------------------------
C
C	This routine crudely calculates the trajectory of a "track" to
C	the calorimeter.  If you want this to work, you should guarantee
C	that it is of high Pt, because straight lines are assumed.
C
C
C       Modified 1/23/98 PJW: Add IMPLICIT none, declare CTH, Add PIVALU.INC
C			so that TWOPI has a value!
C-------------------------------------------------------------------------------
C	Variable Declarations:
C-------------------------------------------------------------------------------
	IMPLICIT NONE
$$INCLUDE 'C$INC:PIVALU.INC'
	INTEGER IGOOD			! =0 if all went OK, -1 otherwise
	REAL PX,PY,PZ			! Momentum components of track
	REAL Z0				! Z of track at beampipe
	REAL XEX(3)			! XEX(1:3) = X,Y,Z of trk at calor.
	REAL PT				! Track Pt
	REAL CRV			! Track curvature
	REAL DVX			! Track impact parameter
	REAL PHI			! Track phi at the beam pipe
	REAL CTH			! Cos(Theta)
	REAL CHARGE			! Track charge
	   DATA CHARGE / 1.0 /		!  (assumed +1)
	REAL BFIELD			! Magnetic field of the solenoid
	   DATA BFIELD / 14.116 /	!  (in kG)

C-------------------------------------------------------------------------------
C	Must translate Px, Py, Pz, and z0 into standard 5 parameters:
C-------------------------------------------------------------------------------
	IGOOD = 0

	PT = SQRT(PX**2+PY**2)
	IF (PT .LT. 0.0001) THEN
	   IGOOD = -1
	   RETURN
	END IF
	CRV = CHARGE * 1.49898E-4 * BFIELD / PT	! Charge = +1 is assumed
	DVX = 0.0
	PHI = ATAN2(PY,PX)
	IF (PHI .LT. 0.0) PHI = PHI + TWOPI
	CTH = PZ/PT

C-------------------------------------------------------------------------------
C	Now extrapolate:
C-------------------------------------------------------------------------------
	CALL EMXRT(CRV,DVX,PHI,CTH,Z0,0,XEX)

	RETURN
	END
