
C------------------------------------------------------------------------------
       SUBROUTINE FALSE_MET(ETATRK,PHITRK,PTTRK,PTSEED,PTMIN,PHIRAD,
     &CONERAD,METPHI)
C------------------------------------------------------------------------------
C   This routine calculates the eta,phi,pt of a track cluster with a cone of
C   CONERAD which is centered on a seed track within PHIRAD in phi of the MET
C   vector, where we require any seed track to have Pt exceeding PTMIN
C
C    INPUT:  PTMIN   PT threshold for seed tracks
C            PHIRAD  phi range to look for seed track
C            CONERAD cone radius for track cluster
C            METPHI  phi of MET
C    OUTPUT: ETATRK     eta of track cluster momentum
C            PHITRK     phi of track cluster momentum
C            PTTRK      PT of track cluster momentum
C            PTSEED     PT of seed track of the cluster
C
C	1/22/98  PJW Fix bug that causes array bound violation if 
C		METTRK_PTR = 0
C
	IMPLICIT NONE
$$INCLUDE 'UC$ANA:TRAKCM.INC'
$$INCLUDE 'C$INC:PIVALU.INC'

       REAL ETATRK,PHITRK,PTTRK,PTMIN,PHIRAD,CONERAD,METPHI,HIPT
       REAL PXTOT,PYTOT,PZTOT,XEXTRKS(3),THETA,DPHI,PTSEED
       INTEGER METTRK_PTR,J,NUMTK(500),TKINDX(500,500),IGOOD2

               CALL UNPTRK(1)
               HIPT = PTMIN
               ETATRK = -999.0
               PHITRK = -999.0
               PTTRK = -999.0
               PTSEED = -999.0
               METTRK_PTR = 0
               DO J =  1,N_TRACK
                 DPHI = ABS(METPHI-PHITR(J))
                 IF (DPHI.GT.PI) DPHI = TWOPI-DPHI
                 IF (DPHI.LT.PHIRAD.AND.
     &               PTTR(J).GT.HIPT) THEN
                   HIPT = PTTR(J)
                   METTRK_PTR = J
                 ENDIF
               ENDDO
               IF (HIPT.GT.5.00) PTSEED = HIPT

               PXTOT = 0.0
               PYTOT = 0.0
               PZTOT = 0.0
               IF (METTRK_PTR .NE. 0) THEN
                 CALL MY_TRKISO(CONERAD,NUMTK,TKINDX)
                 DO J = 1,NUMTK(METTRK_PTR)
                   IF (ABS(Z0TR(METTRK_PTR) -
     &               Z0TR(TKINDX(METTRK_PTR,J))).LT.10.0) THEN
                     PXTOT = PXTOT + PXTR(TKINDX(METTRK_PTR,J))
                     PYTOT = PYTOT + PYTR(TKINDX(METTRK_PTR,J))
                     PZTOT = PZTOT + PZTR(TKINDX(METTRK_PTR,J))
                   ENDIF
                 ENDDO
               ENDIF

               PTTRK = SQRT(PXTOT**2+PYTOT**2)
               IF (PYTOT.NE.0.0.OR.PXTOT.NE.0.0) THEN
                 PHITRK = ATAN2(PYTOT,PXTOT)
                 IF (PHITRK.LT.0.0) PHITRK = PHITRK + TWOPI
                 IF (PHITRK.GT.TWOPI) PHITRK = PHITRK-TWOPI
               ENDIF
C   Only extrapolate if there is a seed!  PJW 1/22/98
	       IGOOD2 = -1		
	       IF (METTRK_PTR.GT.0) THEN
                 CALL EXTRAP(PXTOT,PYTOT,PZTOT,
     &              Z0TR(METTRK_PTR),IGOOD2,XEXTRKS)
	       ENDIF
               IF (IGOOD2.EQ.0) THEN
                 IF (SQRT(XEXTRKS(1)**2+XEXTRKS(2)**2).NE.0.0.OR.
     &               ABS(XEXTRKS(3)).NE.0.0) THEN
                   THETA = ATAN2(SQRT(XEXTRKS(1)**2+XEXTRKS(2)**2),
     &                           ABS(XEXTRKS(3)))
                   IF (XEXTRKS(3).LT.0.0) THETA = PI - THETA
                   IF (TAN(THETA/2.0).NE.0.0) THEN
                     ETATRK = (-1.0)*LOG(TAN(THETA/2.0))
                   ELSE
                     ETATRK = -999.0
                   ENDIF
                 ELSE
                   ETATRK = -999.0
                 ENDIF
               ELSE
                   ETATRK = -999.0
               ENDIF

               RETURN
               END
