
         integer function jets_trks(trksbank,jetsbank,cone_size,prim)

C  A function which determines if the track in the TRBANK with bank number
C  ITRKS is within the cone (with cone radius = cone_size) of the JETS bank
C  JETBANK. 
C  Input arguments
C  TRKSBANK     TRKS bank number
C  JETSBANK   JETS bank number
C  CONE_SIZE cone radius in eta-phi
C  PRIM(3)       primary vertex 
C
C  Output values
C  -1  something wrong (missing banks, unphysical values)
C   0  track intersects jet cone
C   1  track does not intersect jet cone
C   2  track has different z vertex from prim(3)

$$IMPLICIT NONE
$$INCLUDE 'YBOS$LIBRARY:ERRCOD.INC'    ! YBOS error codes
$$INCLUDE 'YBOS$LIBRARY:BNKTYP.INC'    ! bank type codes
$$INCLUDE 'UIPACK$LIBRARY:UINAMES.INC' ! UIPACK declarations
$$INCLUDE 'UIPACK$LIBRARY:UIERROR.INC'
$$INCLUDE 'UIPACK$LIBRARY:UIPARAM.INC'
$$INCLUDE 'C$INC:ERLEVL.INC'           ! error severity levels for error logger
$$INCLUDE 'C$INC:JOBSTA.INC'           ! job status parameters
$$INCLUDE 'C$INC:BCS.INC'              ! primary YBOS array
$$INCLUDE 'C$INC:PIVALU.INC'           ! easy as...
$$INCLUDE 'C$INC:DSJETS.INC'           ! JETS bank pointers
$$INCLUDE 'C$INC:TRKSCO.INC'           ! track parameter common block


         INTEGER TRKSBANK,JETSBANK
         CHARACTER*4 TRBANK
         REAL CONE_SIZE,PRIM(3)

         real xex(3),dxex(3),cosex(3),dcosex(3)
         integer region
         logical lconst
         INTEGER BLOCAT,INDJTBNK,IDJET,istat
         integer indtrbnk,idtrk,qtrkun,j
         real trkr,trketa,trkphi,etadiff,phidiff
$$INCLUDE 'C$INC:DSJETSSF.INC'
         trbank = 'TRKS'
         lconst = .false.
         jets_trks = 1
C        check that z0 of track and prim(3) are within 5 cm of each other
C        try getting QTRK bank
           istat = qtrkun(trksbank,0)
           if (istat.ne.0) then
           jets_trks = -1
           return
           endif
           if (abs(prim(3)-trksfp(3)).gt.5.0) then
             jets_trks = 2
             return
           endif

C        try getting JETS bank
         ISTAT = BLOCAT(IW,'JETS',jetsbank,INDJTBNK,IDJET)
         if (istat.ne.yesucc) then
           jets_trks = -1
           return
         endif

C        pick the most appropriate region for this jet
         if (abs(detajt(idjet)).lt.1.1) region = 0
         if (abs(detajt(idjet)).gt.1.1.and.abs(detajt(idjet)).lt.2.4)
     &     region = 1
         if (abs(detajt(idjet)).gt.2.4)
     &     region = 2

C        extrapolate track out to CEM PEM or FEM
       call trexrt(trbank,trksbank,xex,dxex,cosex,dcosex,region,lconst)
C        check that extrapolated position is not screwed up
         if (abs(xex(1)).gt.998.0) then
           jets_trks = -1
           return
         endif

C        calculate event eta and phi of track using line defined by PRIM and
C        XEX

C        translate to VXPRIM cooordinate system
         do j = 1,3
           xex(j) = xex(j) - prim(j)
         enddo
C        calculate eta
         trkr = 0.0
         trkr = sqrt(xex(1)*xex(1)+xex(2)*xex(2)+xex(3)*xex(3))
         IF (trkr.le.0.0001) THEN
           jets_trks = -1
           return
         endif
         IF (xex(3)/trkr.gt.1.0) THEN
           jets_trks = -1
           return
         endif
         if (TAN(0.5*ACOS(xex(3)/trkr)).le.0.0) then
           jets_trks = -1
           return
         endif
         trkETA = -1.0*LOG(TAN(0.5*ACOS(xex(3)/trkr)))
C        calculate phi
         IF (xex(2).eq.0.0.OR.xex(1).eq.0.0) THEN
           jets_trks = -1
           return
         endif
         trkPHI = PI + ATAN2(-xex(2),-xex(1))

C        check track/jet cone match
         etadiff = abs(eta_jt(idjet) - trketa)
         phidiff = abs(phi_jt(idjet) - trkphi)
         if (phidiff.gt.pi) phidiff = 2*pi - phidiff

         if (sqrt(etadiff**2 + phidiff**2).le.cone_size) then
           jets_trks = 0
           return
         endif

         return
         end


