	Subroutine JPFILL
C
C    This routine fills the common block JPBTAG with information on
C    tracks in jets, track and jet probabilities, etc.
C
C	        D. Gerdes, 4/22/93
C
C===Global declarations:
C
$$Implicit
C
$$Include 'c$inc:jobsta.inc'             ! job and status paramters
$$Include 'c$inc:erlevl.inc'             ! error severity levels
$$Include 'ybos$library:errcod.inc'      ! YESSUC, NOSUCC
$$Include 'c$inc:anactl.inc'             ! Lmonte
$$Include 'c$inc:bcs.inc'                ! primary YBOS array
$$Include 'c$inc:jetinfo.inc'  		 ! common block for jets
$$Include 'c$inc:svsbtc.inc'             ! common for SVX tracks
$$Include 'c$inc:dsjets.inc'             ! JETS bank definition
$$Include 'c$inc:pivalu.inc'             ! values of pi
$$Include 'c$inc:jpbtag.inc'             ! common block filled by this routine
C
C===Local declarations:
C
	Integer IJET, ITRK, ISVX         ! jet, track, SVXS loop indices
	Integer Ierr
	Integer Nhits,Nshr               ! # hits, # shared hits on a track
	Integer laymsk(jpmaxsvx)           ! bit map of layers hit (filled in
C                                                              SVSBTC_NEW)
	Integer ivtxt                    ! loop index for VTXT bank 
	Integer IND, INDDAT
	Integer Ntr                      ! counts # tracks in jet
	Integer Status                   ! YBOS return flag
	integer i
	Integer SVX_VSN                  ! version # of SVXS bank:
C                                           -1 = ??, 0 = Production output
C                                            1 = redone with DO_SVX_PAD
C                                            2 = redone with "6.15"/7.09
C
	Real    Cone                     ! cone size for jets
	Real    Dphi                     ! delta-phi(track-jet)
	Real    Dr_Track_Jet             ! eta-phi sepn bet. track, jet
	Real    Etajet                   ! jet eta in event coords.
	Real etaold, phiold              ! jet eta, phi before muon corrections
	Real    Etasep                   ! eta sepn bet. track, jet
	Real    Phidca                   ! phi(track) +/- pi/2 (for s.i.p.)
	Real    Pi_mass
	PArameter (Pi_mass=0.1396)
	Real    Phijet                   ! phi of jet
	Real    Phisep                   ! delta-phi(track-jet)
	Real    Pxtot,Pytot              ! components of total track momentum
	Real    Xjet(3)
	Logical six_1_5                  ! true if this is "6.15" or 7.09 data
	logical lvee(3)
	logical muon_found               ! true if jet contains a CMU-CMP muon
	integer ipair(3),ngoodh
	real xyzvtx(3)
	real vexerr(3,3)
	real zvert_fid
C
	common/svxhits/laymsk            ! shares this with SVSBTC_NEW
C
C===External functions referenced:
C
	Integer BLOCAT                   ! locate a YBOS bank
	Integer SVVRQK                   ! Return SVXS version # from QTRK
	Real    trkprb                   ! Calculate track probability
	Real    trkprb_615               ! Calculate track probability for 6.15
	real    Jetprb                   ! Calculate jet probability
	logical fidsvx                   ! SVX fiducial routine for jets
        logical vtxs_used                ! Is track used in primary vertex ?


C
C===Start of code:
C
C--- Initialize arrays. All track/jet probabilities initialized to 1.0
C
	DO IJET = 1, JPMAXJET
	   NSVXJT(IJET) = 0
           NGDPOS(IJET) = 0
           NGDNEG(IJET) = 0
	   JETPRBP(IJET) = 1.
	   JETPRBM(IJET) = 1.
	   JETPRBA(IJET) = 1.
	   fid(ijet) = .false.
	   DO ITRK = 1, MAX_TRK
	      SVXSJT(IJET,ITRK) = -1
	      GOOD_SVX(IJET,ITRK) = .false.
	      TRACKPROB(IJET,ITRK) = 1.0
	      NSHRHI(IJET,ITRK) = 0
	      NFNDHI(IJET,ITRK) = 0
	      GOODBTK(IJET,ITRK) = .FALSE.
	      PROMPTKS(IJET,ITRK) = .FALSE.
	      PVUSED(IJET,ITRK) = .false.    !  **** DG
	      DISPLKS(IJET,ITRK) = .FALSE.
	      PROMPTLM(IJET,ITRK) = .FALSE.
	      PRKPAIR(IJET,ITRK) = 0
	      DSKPAIR(IJET,ITRK) = 0
	      PRLMPAIR(IJET,ITRK) = 0
	   END DO
	END DO
C
	nsvxjet = 0            ! ***** DG
C
C---Fill the vertex position and error:
C
        XYZVTX(1)=VTXPOS(1)
        XYZVTX(2)=VTXPOS(2)
        XYZVTX(3)=VTXPOS(3)
C
        zvert_fid = vtxpos(3)
C
        VEXERR(1,1) = VTXERR(1)
        VEXERR(1,2) = VTXERR(2)
        VEXERR(1,3) = VTXERR(3)
        VEXERR(2,1) = VTXERR(2)
        VEXERR(2,2) = VTXERR(4)
        VEXERR(2,3) = VTXERR(5)
        VEXERR(3,1) = VTXERR(3)
	VEXERR(3,2) = VTXERR(5)
	VEXERR(3,3) = VTXERR(6)
C
C---See what version of tracking we are dealing with here...
C
	SVC = .FALSE.
	svx_vsn = svvrqk()
	if (svx_vsn .eq. 2) then
	   six_1_5 = .true.
C
C     With or without $SVC?
	   status = BLOCAT(IW,'$SVC',-1,IND,INDDAT)
	   IF (STATUS .EQ. YESUCC) THEN
              SVC = .TRUE.
	   END IF
C
	else
	   six_1_5 = .false.
	end if
C
C--- Start a loop over jets:
C
	CONE = 0.4        !  use this cone size to identify tracks with
C                            jets (regardless of the clustering conesize)
C
	DO IJET = 1,ICLUS
C
	   fid(IJET) = fidsvx(ijet,zvert_fid)        ! SVX fiducial flag
           PHIJET = ijet_phi(ijet)
	   ETAJET = ijet_eta(ijet)      ! event eta of jet
C          Get SVXS bank numbers for any tracks associated with jets
C          with |eta(detector)| < 2.5
C
	   IF (ABS(IJET_ETA(IJET)) .LE. 2.5 ) THEN ! get SVXS bank numbers of tracks
	      ntr = 0                       ! initially, no SVX tracks in this jet
	      DO ITRK = 1, NFC          ! start a loop over SVX tracks
	         phisep = abs(FINPAR(5,itrk) - phijet)
	         if (phisep .ge. pi) phisep = twopi - phisep
	         etasep = etasvx(itrk) - etajet
	         dr_track_jet = sqrt(phisep**2 + etasep**2)
	         if (dr_track_jet .le. cone
	1             .and.ntr.lt.40) then      ! track goes with jet
	            ntr = ntr + 1  ! increment counter
	            TRKSJT(IJET,ntr) = trklnk(itrk)   !trks bank number
	            SVXSJT(IJET,NTR) = ITRK   ! index into SVX common block
                    PVUSED(IJET,NTR) = VTXS_USED('SVXS',ITRK)
	         END IF                   
	      END DO         	          ! end loop over tracks
C
	      nsvxjt(ijet) = ntr          ! number of SVXS tracks in this jet
	      if (ntr .ge. 2) nsvxjet = nsvxjet + 1
C
C---For jets with SVX tracks, fill information about impact parameter, 
C   track quality, etc.
C
	     PXTOT = 0
	     PYTOT = 0
C
	     DO ISVX = 1, NSVXJT(IJET)      ! loop over SVX tracks in this jet
C
C
C----- Momentum of tracks in jet:
	        PTTRJT(IJET,ISVX) = 2.9973E-04*BMAGNT/
	1                           (2.*ABS(FINPAR(2,SVXSJT(IJET,ISVX))))
	        PXTOT = PXTOT + PTTRJT(IJET,ISVX)*
	1               COS(FINPAR(5,SVXSJT(IJET,ISVX)))
C
	        PYTOT = PYTOT + PTTRJT(IJET,ISVX)*
	1               SIN(FINPAR(5,SVXSJT(IJET,ISVX)))
C
C---- See if track passes BTKSEL quality cuts:
C
	     IF (PTTRJT(IJET,ISVX) .GE. PT_TRACK_MIN .AND. 
	2        ABS(FINPAR(4,SVXSJT(IJET,ISVX))) .LE. MAX_IMPACT
	3        .AND. NPARF(SVXSJT(IJET,ISVX)) .EQ. 5 .AND.
	4        ABS(FINPAR(3,SVXSJT(IJET,ISVX))) .LE. Z_ATT) THEN
	            CALL BTKSEL(SVXSJT(IJET,ISVX),XYZVTX,VEXERR,
	1                       GOODBTK(IJET,ISVX),NFNDHI(IJET,ISVX),
	2                       NSHRHI(IJET,ISVX),LVEE,IPAIR,ngoodh,2)
	            PROMPTKS(IJET,ISVX) = LVEE(1)     ! prompt K_s flag
	            DISPLKS(IJET,ISVX) = LVEE(2)      ! displaced K_s flag
	            PROMPTLM(IJET,ISVX) = LVEE(3)     ! prompt Lambda flag
	            PRKPAIR(IJET,ISVX) = IPAIR(1)     
	            DSKPAIR(IJET,ISVX) = IPAIR(2)
	            PRLMPAIR(IJET,ISVX) = IPAIR(3)
	            IF (GOODBTK(IJET,ISVX) .and.        ! veto prompt vee's
	1               .not.promptKs(ijet,isvx) .and.
	2               .not.promptLm(ijet,isvx)) Then
                       GOOD_SVX(IJET,ISVX) = .TRUE.
	            End if
	     END IF
C
	      NHITS = NFNDHI(IJET,ISVX)               ! # hits on this track
	      NSHR = NSHRHI(IJET,ISVX)                ! # shared hits
C
	        IF (FINCOR(13,SVXSJT(IJET,ISVX)) .EQ. 0. .OR. 
	1           FINPAR(2,SVXSJT(IJET,ISVX)) .EQ. 0.) THEN ! rare bad track
	            goto 100
	        END IF
C
C----- Reject 2-hit tracks with a hit in layer 0:
C
	             IF (NHITS .EQ. 2 .AND. 
	1                MOD(LAYMSK(SVXSJT(IJET,ISVX)),2) .EQ. 1) THEN
	                GOOD_SVX(IJET,ISVX) = .FALSE.
	             END IF
C
C
C
C----- Unsigned impact parameter significance:
C	     
	        D0SIGTR(IJET,ISVX) = ABS(FINPAR(4,SVXSJT(IJET,ISVX))/
	1                            SQRT(FINCOR(13,SVXSJT(IJET,ISVX))))
C
C----- Signed impact parameter significance (JETS axis):
C
	        PHIDCA = FINPAR(5,SVXSJT(IJET,ISVX)) +    !phi0 ofthis track
	1                SIGN(1.0,FINPAR(4,SVXSJT(IJET,ISVX)))*HALFPI
C 	        
	        DPHI = PHIDCA-PHIJET
	        SIGNIPTR(IJET,ISVX) = SIGN(1.0,COS(DPHI))*
	1                             ABS(FINPAR(4,SVXSJT(IJET,ISVX)))
	        SD0SIGTR(IJET,ISVX) = SIGNIPTR(IJET,ISVX)/
	1                             SQRT(FINCOR(13,SVXSJT(IJET,ISVX)))
C
C---Count number of "good" positive- and negative-i.p. tracks in the jet.
C
	        IF (GOOD_SVX(IJET,ISVX)) THEN
	           IF (SD0SIGTR(IJET,ISVX) .GE. 0) THEN
	              NGDPOS(IJET) = NGDPOS(IJET) + 1
	           ELSE
	              NGDNEG(IJET) = NGDNEG(IJET) + 1
	           END IF
	        END IF
C
C---Get track probability from resolution function:
C
	       IF (SIX_1_5 .and. good_svx(ijet,isvx).and.
	1          .not.Lmonte) THEN
	          TRACKPROB(IJET,ISVX) = 
	1             TRKPRB_615(NHITS,NSHR,SD0SIGTR(IJET,ISVX),SVC)
	       ELSE if (.not.six_1_5 .or. Lmonte) then
	           TRACKPROB(IJET,ISVX) =
	1             TRKPRB(NHITS,NSHR,SD0SIGTR(IJET,ISVX))
	       ELSE
	           TRACKPROB(IJET,ISVX) = 1.0
	       END IF
C
100	     CONTINUE
C
	      END DO         ! End loop over SVX tracks in this jet
C
C---Combine track probabilities into jet probability:
C
	      IF (NSVXJT(IJET) .GT. 0) THEN
	         JETPRBP(IJET) = JETPRB(IJET,1) ! use only + s.i.p. tracks
	         JETPRBM(IJET) = JETPRB(IJET,-1) ! use only - s.i.p. tracks
	         JETPRBA(IJET) = JETPRB(IJET,0) ! use all tracks
	      ELSE
	         JETPRBP(IJET) = 1.
	         JETPRBM(IJET) = 1.
	         JETPRBA(IJET) = 1.
	      END IF

	
C
	      PTTOT(IJET) = SQRT(PXTOT**2 + PYTOT**2)  ! SUM-PT
C
	   END IF           ! end "JET has |eta|<2.5" branch
C
101	CONTINUE
	END DO              ! end of loop over jets
C
C---Copy vertex error into JPBTAG common block
C
	SIGVTX = sqrt(vtxerr(1)**2+vtxerr(4)**2)
C
	RETURN              ! punt
	END
