C  DEC/CMS REPLACEMENT HISTORY, Element QDSETA.CDF
C  *2    14-JUL-1991 18:06:08 FLAVIA "Converted for UNIX"
C  *1    19-JUN-1991 19:24:37 FLAVIA "Creates SETA bank"
C  DEC/CMS REPLACEMENT HISTORY, Element QDSETA.CDF
	INTEGER FUNCTION QDSETA(RAD,ABS)
C
C  This routine makes the SETA bank as appropriate for jet physics.
C  SETA tries to correct tower theta and eta positions for the actual
C  event vertex.  To do so, it must make an assumption about the
C  longitudinal profile of the shower energy deposition in the tower.
C  This particular routine assumes the average shower position is located
C  at a fixed number of radiation (interaction) lengths in the calorimeter
C  for the EM (HAD) segment.  The radiation and interaction lengths are
C  calculated including all material in the detector, IE the interaction
C  thickness of the coil is subtracted when calculating the position of
C  CEM showers.  The information on tower positions is stored as
C  the Y and Z positions (side view) of a constant number of radiation
C  (interaction) lengths into the tower at the center of the tower.
C  All the special cases of the particular detector geometry are handled
C  'correctly', as per blueprint numbers.
C  Dave Brown, 4-28-88
C  Jun     1992 - Get vertex from VTVZ bank
C  Jul     1992 - Get vertex from EVTA or EVTX depending of the year
C

$$IMPLICIT
$$INCLUDE 'YBOS$LIBRARY:ERRCOD.INC'
$$INCLUDE 'YBOS$LIBRARY:BNKTYP.INC'
$$INCLUDE 'C$INC:DSTOWE.INC'
$$INCLUDE 'C$INC:TOWLOC.INC'
$$INCLUDE 'C$INC:BCS.INC'
$$INCLUDE 'C$INC:ERLEVL.INC'
$$INCLUDE 'C$INC:DSEVTA.INC'
$$INCLUDE 'C$INC:DSCMPB.INC'
$$INCLUDE 'C$INC:DSVTXS.INC'
$$INCLUDE 'C$INC:VTCAUX.INC'
$$INCLUDE 'C$INC:VXPVIN.INC'
$$INCLUDE 'C$INC:CALORIMETRY.INC'
C
C  Inputs- These are the nominal number of radiation,interaction lengths
C  at which to locate the shower.
C
	REAL RAD, ABS
C
C  Function calls
C
	INTEGER BTPACK, BMAKE, BLOCAT
        real jwb_zvertex
C
C  Local variables                 
C
        INTEGER VERIND,BLKIND
	INTEGER IETA,JETA
	INTEGER JDSETA, JDEVTA, JDDETS, IND, BNKTYP(3)
	REAL OLD_RAD, OLD_ABS
	REAL YEMPOS, YHAPOS, ZEMPOS, ZHAPOS, REMPOS, RHAPOS
	REAL ZVERT,THETA
	REAL YEM(0:41),ZEM(0:41),YHA(0:41),ZHA(0:41)
	REAL YCEM, ZPEM, ZFEM, YCHA, ZWHA, ZPHA, ZFHA
	REAL ACEMQ, APEM, AFEM
	REAL ACHAQ, AWHAQ, APHA, AFHA, ACOIL, AEND
	REAL RCEMQ, RPEM, RFEM, RCOIL, REND
	REAL DXDR_CEM, DXDR_PEM, DXDR_FEM
	REAL DXDA_CHA, DXDA_WHA, DXDA_PHA, DXDA_FHA
	REAL YTOW_7, ZTOW_10, ATOW_7
        INTEGER NVERT, KCLASS, STAT, INDDAT
        REAL ZVERTX,XVERTX,YVERTX
        LOGICAL LINIT


C
C  Front-face positions of the calorimeters
C
	DATA YCEM/175.26/, ZPEM/172.72/, ZFEM/623.2/
	DATA YCHA/214.63/, ZWHA/252.37/, ZPHA/226.0/, ZFHA/706.1/
C
C  Note- all the following numbers were calculated using blueprint
C  thickness of materials (iron, lead, scintilator, etc.) and Jim
C  Freeman's simulation database numbers for the radiation/absorption
C  lengths in those materials.
C
C
C  Absorption-length thickness of the calorimeters, assuming 90 degree
C  impact angle.                         
C
	DATA ACEMQ/.646/, APEM/.66/, AFEM/1.29/
	DATA ACHAQ/4.14/, AWHAQ/3.98/, APHA/4.9/, AFHA/6.42/
	DATA ACOIL/.14/, AEND/.021/
C                       
C  radiation-length thickness of the calorimeters, assuming 90 degree
C  impact angle
C
	DATA RCEMQ/17.97/, RPEM/16.94/, RFEM/28.5/
	DATA RCOIL/.83/, REND/.107/
C                                
C  cm/radiation length for the detectors.
C
	DATA DXDR_CEM/1.764/, DXDR_PEM/3.149/, DXDR_FEM/2.807/
C
C  cm/interaction length for the detectors
C
	DATA DXDA_CHA/31.5/, DXDA_WHA/27.9/, DXDA_PHA/29.8/, DXDA_FHA/35.64/
C
C  Odds and ends
C
	DATA YTOW_7/76.19/, ZTOW_10/192.4/
C
C  First time through, and anytime the parameters change,
C  compute the Y and Z tower positions, given the
C  nominal number of interaction lengths.
C
        DATA LINIT/.TRUE./
C
C       do some 1-time inits first
C       ==========================
        IF (LINIT) THEN
          LINIT = .FALSE.
          CALL LOCTOW
        ENDIF


	IF(ABS .NE. OLD_ABS .OR. RAD .NE. OLD_RAD)THEN
	  OLD_ABS = ABS
	  OLD_RAD = RAD
	  DO JETA=0,41
	    IETA = 43+JETA
C                            
C  EM stuff first
C
	    IF(JETA .LE. 9)THEN
	      THETA = TOWTAV(IETA)
	      YEM(JETA) = YCEM + DXDR_CEM*(RAD-RCOIL/SIN(THETA))
	      ZEM(JETA) = YEM(JETA)/TAN(THETA)
	    ELSE IF(JETA .EQ. 10)THEN
	      THETA = TOWTAV(IETA)
	      ZEM(JETA) = ZPEM + DXDR_PEM*COS(THETA)*(RAD-REND/COS(THETA))
C
C  This is a partial tower only, insure that the shower position never
C  extends out past what is 'physical'.
C
	      ZEM(JETA) = MIN(ZEM(JETA),ZTOW_10)
	      YEM(JETA) = ZEM(JETA)*TAN(THETA)
	    ELSE IF(JETA .LE. 23)THEN
	      THETA = TOWTAV(IETA)
	      ZEM(JETA) = ZPEM + DXDR_PEM*COS(THETA)*(RAD-REND/COS(THETA))
	      YEM(JETA) = ZEM(JETA)*TAN(THETA)
	    ELSE                     
	      THETA = TOWTAV(IETA)
	      ZEM(JETA) = ZFEM + DXDR_FEM*COS(THETA)*RAD
	      YEM(JETA) = ZEM(JETA)*TAN(THETA)
	    END IF
C
C  now HAD stuff
C
	    IF(JETA .LE. 6)THEN
	      THETA = TOWTAV(IETA)
	      YHA(JETA) = YCHA+DXDA_CHA*SIN(THETA)*
	1     (ABS-ACOIL/SIN(THETA)-ACEMQ)
	      ZHA(JETA) = YHA(JETA)/TAN(THETA)
	    ELSE IF(JETA .EQ. 7)THEN
C
C  Here, depending on ABS, the position may lay in the CHA or WHA.
C  To determine this, compute the radiation thickness of this CHA
C  tower, and compare the the input ABS.
C
	      THETA = TOWTAV(IETA)
	      ATOW_7 = ACOIL/SIN(THETA) + ACEMQ + YTOW_7/
	1     (DXDA_CHA*SIN(THETA))
	      IF(ABS .LT. ATOW_7)THEN
C
C  here we're in the CHA.
C
	        YHA(JETA) = YCHA+DXDA_CHA*SIN(THETA)*
	1       (ABS-ACOIL/SIN(THETA)-ACEMQ)
	        ZHA(JETA) = YHA(JETA)/TAN(THETA)
	      ELSE
C
C  WHA
C
	        ZHA(JETA) = ZWHA + (ABS-ATOW_7)*DXDA_WHA*COS(THETA)
	        YHA(JETA) = ZHA(JETA)*TAN(THETA)
	      END IF
	    ELSE IF(JETA .LE. 9)THEN
	      THETA = TOWTAV(IETA)
	      ZHA(JETA) = ZWHA+(ABS-ACOIL/SIN(THETA)-ACEMQ)*
	1     DXDA_WHA*COS(THETA)
	      YHA(JETA) = ZHA(JETA)*TAN(THETA)
	    ELSE IF(JETA .EQ. 10)THEN
	      THETA = TOWTAV(IETA)
	      ZHA(JETA) = ZWHA + (ABS-ACOIL/SIN(THETA)-.5*APEM/COS(THETA))*
	1     DXDA_WHA*COS(THETA)
	      YHA(JETA) = ZHA(JETA)*TAN(THETA)
	    ELSE IF(JETA .EQ. 11)THEN
	      THETA = TOWTAV(IETA)
	      ZHA(JETA) = ZWHA + (ABS-ACOIL/SIN(THETA)-APEM/COS(THETA))*
	1     DXDA_WHA*COS(THETA)
	      YHA(JETA) = ZHA(JETA)*TAN(THETA)
	    ELSE IF(JETA .LE. 23)THEN
	      THETA = TOWTAV(IETA)
	      ZHA(JETA) = ZPHA + (ABS-APEM/COS(THETA))*DXDA_PHA*COS(THETA)
	      YHA(JETA) = ZHA(JETA)*TAN(THETA)
	    ELSE
	      THETA = TOWTAV(IETA)
	      ZHA(JETA) = ZFHA + (ABS-AFEM/COS(THETA))*DXDA_FHA*COS(THETA)
	      YHA(JETA) = ZHA(JETA)*TAN(THETA)
	    END IF
	  END DO
	END IF
C
C  Next the stuff that occurs every event.
C
C
C  First, make the bank
C
	QDSETA = BTPACK(BNKTMX,0,2,BNKTYP(1))
	QDSETA = BTPACK(BNKTI4,0,1,BNKTYP(2))
	QDSETA = BTPACK(BNKTR4,0,4*TENETA,BNKTYP(3))
	QDSETA = BMAKE(IW,'SETA',1,4*TENETA+1,BNKTYP,IND,JDSETA)
	IF(QDSETA .NE. YESUCC)RETURN
	IW(JDSETA) = TENETA
C
           IF(DATA88) THEN
C
C---Locate the bank in YBOS:

              STAT = BLOCAT(IW,'EVTA',1,IND,INDDAT)
              IF (STAT.NE.YESUCC) THEN
                 CALL ERLOGR('ZETCOR',ERINFO,0,STAT,
     1           'Unable to locate Event attribute bank EVTA')
                 ZVERT = 0.0
              ELSE
                 ZVERT = RW(INDDAT + Z0EV)   ! event Z vertex
              ENDIF
C
           ELSEIF(DATA92) THEN

C JWB 3/26/98
             zvert = jwb_zvertex()
C JWB 3/26/98
           ENDIF
C
C  Loop over all the towers, and calculate eta and sin(theta)
C
	DO IETA=1,TENETA
	  IF(IETA .LE. 42)THEN
	    JETA = 42-IETA
	    ZEMPOS = -ZEM(JETA)-ZVERT
	    ZHAPOS = -ZHA(JETA)-ZVERT
	  ELSE
	    JETA = IETA-43
	    ZEMPOS = ZEM(JETA)-ZVERT
	    ZHAPOS = ZHA(JETA)-ZVERT
	  END IF 
	  YEMPOS = YEM(JETA)
	  YHAPOS = YHA(JETA)

	  REMPOS = SQRT(YEMPOS*YEMPOS + ZEMPOS*ZEMPOS)
	  RHAPOS = SQRT(YHAPOS*YHAPOS + ZHAPOS*ZHAPOS)

	  RW(JDSETA +        IETA) = YEMPOS/REMPOS
	  RW(JDSETA + TENETA+IETA) = YHAPOS/RHAPOS
	  RW(JDSETA+2*TENETA+IETA) = .5*LOG( (REMPOS+ZEMPOS)/(REMPOS-ZEMPOS))
	  RW(JDSETA+3*TENETA+IETA) = .5*LOG( (RHAPOS+ZHAPOS)/(RHAPOS-ZHAPOS))
	END DO

	RETURN
	END                               

***************************************************************************
        
