C  VAX/DEC CMS REPLACEMENT HISTORY, Element QRJTRK.CDF
C  *8    29-AUG-1993 09:16:30 BHATTI ""
C  *7    29-AUG-1993 08:58:51 BHATTI "REMOVE REFERENCE TO DSQTRK.INC"
C  *6    21-SEP-1992 14:38:35 JAM "get rid of some erlogr calls"
C  *5    24-JUL-1992 16:27:03 JAM "found bug in call to get # of tracks"
C  *4    29-APR-1992 08:05:43 JAM "Asseet's changes for the modified QTRK routines"
C  *3    13-APR-1992 12:13:24 JAM "get vertex from vtgzzt"
C  *2    14-JUL-1991 18:17:37 FLAVIA "Converted for UNIX"
C  *1    19-JUN-1991 19:32:11 FLAVIA "Make JTRK bank for computing jet tracking stuff"
C  VAX/DEC CMS REPLACEMENT HISTORY, Element QRJTRK.CDF
C ==============================================================================
      SUBROUTINE QRJTRK
C ==============================================================================
C
C       Description:
C	============
C       This routine creates and fills the JTRK bank.  This bank contains a
C	list of good CTC tracks and their eta-phi coordinates and momentum.
C	The JTRK bank is intended to be used in calculating the jet charge
C	fraction for the JETS banks.
C
C	Inputs:
C	=======
C	None
C
C	Outputs:
C	========
C	None
C
C       Preconditions necessary before call:
C	====================================
C       The QTRK and vertex banks must exist
C
C	Author:
C	=======
C	R. Markeloff			June 7, 1989
C
C ==============================================================================
C
$$IMPLICIT NONE
C
C	Global Declarations:
C	=====================
C
$$INCLUDE 'YBOS$LIBRARY:ERRCOD.INC'
$$INCLUDE 'YBOS$LIBRARY:BNKTYP.INC'
$$INCLUDE 'C$INC:ERLEVL.INC'
$$INCLUDE 'C$INC:BCS.INC'
$$INCLUDE 'C$INC:DSEVTX.INC'
$$INCLUDE 'C$INC:DSCMPB.INC'
$$INCLUDE 'C$INC:DSVTXS.INC'
CCC$$INCLUDE 'C$INC:DSQTRK.INC'
$$INCLUDE 'C$INC:PIVALU.INC'
$$INCLUDE 'C$INC:JOBSTA.INC'
C
C	Local Declarations:
C	====================
	Logical FLAG3D		! 3d track, secondary vertex flags
	Logical VTPC,SVX	! Flags to indicate detector
C
        Integer STATUS		! YBOS function value
        Integer QIND, QDAT	! QTRK index, data index
        Integer NTRKS		! Number of tracks in QTRK
        Integer HEADER(5)	! JTRK bank header
        Integer NDATA		! Number of data words in JTRK
	Integer JIND, JDAT	! JTRK index, data index
	Integer JEVIND, JEVDAT, blkind	! EVTA index, data index
	Integer NGOOD		! Number of good CTC tracks
	Integer I, J		! Loop indices
	Integer Q, HITLAY	! Track charge, superlayer bit pattern
        Integer NUMOUT	  	! Track number
	Integer IERR		! QTRKCT error return
	Integer BIT1		! First bit in selection code
	Integer SLBIT, NSL	! Superlayer hit bit, number of superlayers
	Integer NSLCUT		! Cut on number of superlayers
	Integer POINTER		! Pointer into JTRK bank
	Integer NVERT,KCLASS    ! Returned by VTGZZT
C
	Real    PX, PY, PZ	! Track momenta
	Real    ZVX, IMPCT	! z distance from vertex, impact parameter
	Real	PT, PTCUT	! Track Pt, Pt cut
	Real	ZVERT		! Vertex z position
	Real	XVERT,YVERT	! Vertex x,y position
	Real    ZCUT, IPCUT	! Cuts on z vertex distance, impact parmeter
	Real	PHI 		! Track phi
	Real	PXYZ(3)		! Momentum vector 
	Real	ETA, ETACUT	! Track eta, eta cut
C
C	External Declarations:
C	======================
        Integer BLOCAT, BMAKE, BTPACK	! Well known YBOS functions
	Integer JBYT			! CERNLIB bit extraction function
C
	Real	PRAPID 			! Calculates rapidity
        real    jwb_zvertex             ! gets z vertex
C
C	Initialize track quality cuts
C	-----------------------------
	DATA PTCUT, ZCUT, IPCUT, NSLCUT, ETACUT / 0.4, 5., 0.5, 2, 1.2 /
C
C ==============================================================================
C
C	Executable Code:
C	===============
C
C       Locate QTRK bank
C	----------------
        STATUS = BLOCAT(IW,'QTRK',-1,QIND,QDAT)
	IF (STATUS .NE. YESUCC) THEN
C	   CALL ERLOGR('QRJTRK',ERWARN,0,STATUS,'no QTRK bank')
	   RETURN
	ENDIF
C
C	Get number of tracks in QTRK bank
C	---------------------------------
	CALL QTRKCT(0,Q,PX,PY,PZ,ZVX,IMPCT,HITLAY,NTRKS,
     $	   FLAG3D,VTPC,SVX,IERR)
	IF (NTRKS .LE. 0) THEN
C	   CALL ERLOGR('QRJTRK',ERWARN,0,STATUS,'no tracks in QTRK')
	   RETURN
	ENDIF
C
C	Make the JTRK bank
C	------------------
	STATUS = BTPACK(BNKTMX,0,NTRKS+1,HEADER(1))
	STATUS = BTPACK(BNKTI4,0,1,HEADER(2))
	STATUS = BTPACK(BNKTEC,NTRKS,2,HEADER(3))
	STATUS = BTPACK(BNKTI4,0,1,HEADER(4))
	STATUS = BTPACK(BNKTR4,0,3,HEADER(5))
C
	NDATA = 1 + 4*NTRKS
	STATUS = BMAKE(IW,'JTRK',1,NDATA,HEADER,JIND,JDAT)
	IF (STATUS .NE. YESUCC) THEN
	   CALL ERLOGR('QRJTRK',ERWARN,0,STATUS,'BMAKE failure')
	   RETURN
	ENDIF

        zvert = jwb_zvertex()

C
C	Loop over tracks
C	----------------
	NGOOD = 0
	DO 100 I=1,NTRKS
C
C	   Call routine to unpack QTRK information
C	   ---------------------------------------
	   CALL QTRKCT(I,Q,PX,PY,PZ,ZVX,IMPCT,HITLAY,NUMOUT,
     $	   FLAG3D,VTPC,SVX,IERR)
C
	   IF (IERR .NE. 0) GOTO 100
C
	   PT = SQRT(PX**2 + PY**2)
C
C	   Cut on Pt, distance from vertex, and impact parameter
C	   -----------------------------------------------------
	   IF (PT .LT. PTCUT .OR. ABS(ZVX-ZVERT) .GT. ZCUT .OR.
     $	   ABS(IMPCT) .GT. IPCUT) GOTO 100
C
C	   Cut on number of superlayers
C	   ----------------------------
	   NSL = 0
	   DO 50 J=1,9
		SLBIT = JBYT(HITLAY,J,1)
		NSL = NSL + SLBIT
50	   CONTINUE
	   IF (NSL .LT. NSLCUT) GOTO 100
C
C	   Discard 2d tracks, secondaries, and non-CTC tracks
C	   --------------------------------------------------
	   IF (.NOT. FLAG3D) GOTO 100
C
C	   Calculate track phi
C	   -------------------
	   PHI = ATAN2(PY,PX)
	   IF (PHI .LT. 0.) PHI = TWOPI + PHI
C
C	   Calculate rapidity of track
C	   ---------------------------
	   PXYZ(1) = PX
	   PXYZ(2) = PY
	   PXYZ(3) = PZ
	   ETA = PRAPID(PXYZ)        		! Rapidity
C
C	   Do eta cut here
C	   ---------------
	   IF (ABS(ETA) .GT. ETACUT) GOTO 100
C
	   NGOOD = NGOOD + 1
	   POINTER = JDAT + 4*(NGOOD - 1) + 1
	   IW(POINTER) = I			! Track number
	   RW(POINTER + 1) = ETA
	   RW(POINTER + 2) = PHI
	   RW(POINTER + 3) = SQRT(PX**2 + PY**2 + PZ**2)
C
100	CONTINUE
C
C	First data word is number of good tracks
C	----------------------------------------
	IW(JDAT) = NGOOD
C
C	Remake the JTRK bank to set it to the correct length
C	----------------------------------------------------
	STATUS = BTPACK(BNKTMX,0,NGOOD+1,HEADER(1))
	STATUS = BTPACK(BNKTI4,0,1,HEADER(2))
	STATUS = BTPACK(BNKTEC,NGOOD,2,HEADER(3))
	STATUS = BTPACK(BNKTI4,0,1,HEADER(4))
	STATUS = BTPACK(BNKTR4,0,3,HEADER(5))
C
	NDATA = 1 + 4*NGOOD
	STATUS = BMAKE(IW,'JTRK',1,NDATA,HEADER,JIND,JDAT)
	IF (STATUS .NE. YESUCC) THEN
	   CALL ERLOGR('QRJTRK',ERWARN,0,STATUS,'BMAKE failure')
	   RETURN
	ENDIF
C
C	Return to caller
C	----------------
	RETURN
	END
