C======================================================================
C	File: MIOVAL.CDF
C======================================================================
C
C===Description:	This routine unpacks the CMIO banks and fills
C			a common block in MIOVAL.INC with the mion
C			variables which will be cut on.
C
C	Input variables: Logical BC=.TRUE. to get beam constrained momentum
C
C	Input conditions: CMIO bank present
C	Output variables: MIOINFO.INC common block filled
C
C===History:
C		7-NOV-1994      	Creation based on MUOVAL.CDF
c
c		8-Feb-1995	RR  	Fixed tracking isolation Req.	
C
C======================================================================

C===============================
	SUBROUTINE MIOVAL(BC)
C===============================

$$IMPLICIT
C
C===Global Declarations:
C
$$INCLUDE 'C$INC:BCS.INC'
$$INCLUDE 'C$INC:CMUO92.INC'
$$INCLUDE 'YBOS$LIBRARY:ERRCOD.INC'
$$INCLUDE 'c$inc:MIOINFO.INC'
$$INCLUDE 'c$inc:MUOINFO.INC'
$$include 'stntuple$source:my_bc.inc'
C
C===Ybos stuff
C
	INTEGER BLOCAT,BNUMB,BNEXT
C
C===Other externals
C
C
C===Local Variables
C------------------
	REAL VERTEXZ			!Z vertex with highest class
	REAL XVERT(5),YVERT(5),ZVERT(5)	!x,y,z of vertex
	REAL ZDIFF			!Difference between z of vertex & 
                                        !mion z0 (no beam constraint). not used
        REAL CV                         !Closest VTX vertex to mion z0
        REAL ZT                         !Z of track; beam
                                        !constrained when BC=.TRUE.; used for
                                        !track extrapolation
	REAL X(3),P(3)			!Vertex and 3-mom passed to FIDCMU
	REAL PTOT			!Mion Ptot from QTRK etc.
	INTEGER I			!Counter
	INTEGER IND,INDDAT		!CMIO bank and data indices
	INTEGER BANKNO			!CMIO bank #
	INTEGER NCMI			!Counter for # of CMIO banks found
	INTEGER NVERT,NVERT_GOOD	!Number of vertices
	INTEGER KCLASS(5),KCLASS_MAX	!Vertex class info
	INTEGER TRKS_NUM		!TRKS bank number for mion
	LOGICAL BC			!=.TRUE. for BC momentum
					!Otherwise they come from CMIO
	INTEGER STATUS			!YBOS return code
        INTEGER IERRBC
	INTEGER INDNXT			!Bank index to next SVXS bank
        LOGICAL FLAG3D                  !.TRUE. if track is 3-d
	REAL D0_OLD                     !Impact par. from QTRK (not used)
	REAL D0                         !Impact par. corrected for beam pos
        REAL QTP(3)                     !Track momentum from QTRK; beam
                                        !constrained when BC=.TRUE.
	INTEGER NASL                    !number of axial  superlayers from QTRK
	INTEGER NSSL                    !number of stereo superlayers from QTRK
        REAL SUMPT                      !SumPt of tracks inside cone of 0.25,
					!excluding mion track
        REAL HIGHPT                     !Highest Pt tracks inside cone of 0.25,
					!excluding mion track
        INTEGER FIDWRD                  !0=pass, 1=fail; as in CMIOFID.FOR

C
C=======================================================================
C	Executable code
C
C	Zero all the arrays
C	-------------------
	call vzero(bnk_cmio,MAXCMIO)
	call vzero(fid_cmio,MAXCMIO)
	call vzero(p4m_cmio,4*MAXCMIO)
	call vzero(pt_cmio,MAXCMIO)
	call vzero(d0_cmio,MAXCMIO)
	call vzero(z0_cmio,MAXCMIO)
	call vzero(em_cmio,MAXCMIO)
	call vzero(had_cmio,MAXCMIO)
	call vzero(isol_cmio,MAXCMIO)
	call vzero(bte_cmio,MAXCMIO)
	call vzero(muzv_cmio,MAXCMIO)
        call vzero(trks_cmio,MAXCMIO)
        call vzero(ch_z0_cmio,MAXCMIO)
        call vzero(ch_d0_cmio,MAXCMIO)
C
C Find the Z vertex(ices) of the event and decide which is best
C-------------------------------------------------------------- 
        vertexz = 999.
        call vtgzzt(5,nvert,kclass,zvert,xvert,yvert)
        kclass_max = -1
        do i=1,nvert
          if(kclass(i).gt.kclass_max) then
            nvert_good = i
            kclass_max = kclass(i)
          endif
        enddo
        if(nvert.gt.0) then
          vertexz = zvert(nvert_good)
        else
          vertexz = 999.
        endif
C
C	Get the CMIOs
C
	bankno=1
	ncmi=0
	do while(blocat(iw,'CMIO',bankno,ind,inddat).eq.yesucc)
	  ncmi = ncmi + 1
	  if(ncmi.le.maxcmio) then
	    bnk_cmio(ncmi)=bankno
C
C	Get stuff out of the bank
C       -------------------------
C
	    em_cmio(ncmi)=rw(inddat+moceme)
	    had_cmio(ncmi)=rw(inddat+mochae)
	    p4m_cmio(1,ncmi)=rw(inddat+mopx)
	    p4m_cmio(2,ncmi)=rw(inddat+mopy)
	    p4m_cmio(3,ncmi)=rw(inddat+mopz)
	    p4m_cmio(4,ncmi)=rw(inddat+moe4)
	    pt_cmio(ncmi)=rw(inddat+mopt)
	    d0_cmio(ncmi)=sqrt(rw(inddat+mox0)**2+rw(inddat+moy0)**2)
	    z0_cmio(ncmi)=rw(inddat+moz0)
	    bte_cmio(ncmi)=rw(inddat+mobden)
	    chrg_cmio(ncmi)=iw(inddat+mochrg)
C
C Get the corresponding TRKS banks info
C -------------------------------------
C
	    trks_num= ishft(iand(iw(inddat+motrks),'FF00'x),-8)
            trks_cmio(ncmi)=trks_num
	    call trackq(trks_num,bc,flag3d,d0_old,d0,nasl,nssl,
     &	    cv,zdiff,zt,qtp,ierrbc)
C
	    muzv_cmio(ncmi) = cv            ! closest vertex with class>=10
	    d0_cmio(ncmi) = d0              ! corrected impact parameter
	    nasl_cmio(ncmi)=nasl
	    nssl_cmio(ncmi)=nssl
C
            if(bc)then
	      pt_cmio(ncmi)=sqrt(qtp(1)**2+qtp(2)**2)
	      p4m_cmio(1,ncmi)=qtp(1)
	      p4m_cmio(2,ncmi)=qtp(2)
	      p4m_cmio(3,ncmi)=qtp(3)
	      ptot=sqrt(pt_cmio(ncmi)**2+p4m_cmio(3,ncmi)**2)
	      p4m_cmio(4,ncmi)=sqrt(ptot**2+0.01117)
c**************** add chi2  ******************
              ch_z0_cmio(ncmi) = z0chi2
              ch_d0_cmio(ncmi) = d0chi2
            endif
C
C Calculate calorimeter isolation (ratio ET/PT) of mion
C -----------------------------------------------------
C
	    if(pt_cmio(ncmi).gt.0.) then
	      isol_cmio(ncmi) = rw(inddat+moneen)/pt_cmio(ncmi)
	    else
	      isol_cmio(ncmi) = 999.
	    endif
C
C Track isolation and highest Pt track in cone of R=0.4
C-------------------------------------------------------
C
	    call tkiso(trks_num,-99,0.4,sumpt,highpt)	
            IF(pt_cmio(NCMI).GT.0.)THEN
              sumpt_cmio(ncmi)=sumpt/pt_cmio(NCMI)
            ELSE
              sumpt_cmio(ncmi)=999.
            ENDIF

            highpt_cmio(ncmi)=highpt
C
C Do the fiducial stuff
C ---------------------
C
	    call cmiofid(trks_num,bc,fidwrd)
            fid_cmio(ncmi)=fidwrd
C
	  endif				!If < 10 cmios found so far
	  bankno=bankno+1
	end do				!End of loop over CMIOs
	numcmio=ncmi
C
	Return
	End
