C======================================================================
C	File: MUOVAL.CDF
C======================================================================
C
C===Description:	This routine unpacks the CMUO banks and fills
C			a common block in MUOVAL.INC with the muon
C			variables which will be cut on.
C
C	Input variables: Logical BC=.TRUE. to get beam constrained momentum
C
C	Input conditions: CMUO bank present
C	Output variables: MUOVAL.INC common block filled
C
C===Author:
C		Tony Liss
C		University of Illinois
C
C		7-MAY-1993	Birth
C	       24-MAY-1993	Added fiducial stuff
C		4-JUN-1993	Get SVX or BC fit
C		7-JUN-1993	Add switch to trun off above
C	       13-JUL-1993	fix GETTRK call		-AJM
C	       28-JUL-1993	Get x0,y0,z0 always from CMUO
C	       29-JUL-1993	Explicitly set CSCALE(0)=0 (for GETTRK)
C	       24-AUG-1993	Don't assume SVXS#=TRKS# (Courtesy B. Harral)
C              29-JUN-1994      Always get Pt from b.c. fit; 
C                               get i.p. from non-b.c. fit and
C                               correct for beam offset and slope.(DG)
C	       11-JUL-1994	Make above change 100% backward compatible
C
C		1-NOV-1994      M. Contreras
C
C                               Use QTRKBM for beam constrained momentum
C                               and for muon Z (for track extrapolation) 
C                               when requested, ie when BC=.TRUE.
C                               Use non-beam constrained track parameters +
C                               beam position information to compute corrected
C                               impact parameter.
C                               Use non-beam constrained track Z to find
C                               distance to closest vertex of class >= 10.
C
C		8-Feb-95	Rob Roser
c				Fixed tracking isolation for cone of .4
c				and sum/pt of track in question!
C
C======================================================================

C===============================
	SUBROUTINE MUOVAL(BC)
C===============================

$$IMPLICIT
C
C===Global Declarations:
C
$$INCLUDE 'C$INC:BCS.INC'
$$INCLUDE 'C$INC:CMUO92.INC'
$$INCLUDE 'YBOS$LIBRARY:ERRCOD.INC'
$$INCLUDE 'c$inc:MUOINFO.INC'
$$INCLUDE 'c$inc:MIOINFO.INC'
$$include 'stntuple$source:my_bc.inc'
C
C===Ybos stuff
C
	INTEGER BLOCAT,BNUMB,BNEXT
C
C===Other externals
C
	INTEGER FIDCMU92
C
C===Local Variables
C------------------
	REAL VERTEXZ			!Z vertex with highest class
	REAL XVERT(5),YVERT(5),ZVERT(5)	!x,y,z of vertex
	REAL ZDIFF			!Difference between z of vertex & 
                                        !muon z0 (no beam constraint). not used
        REAL CV                         !Closest VTX vertex to muon z0
        REAL ZT                         !Z of track; beam
                                        !constrained when BC=.TRUE.; used for
                                        !track extrapolation
	REAL X(3),P(3)			!Vertex and 3-mom passed to FIDCMU
	REAL PTOT			!Muon Ptot from QTRK etc.
	INTEGER I			!Counter
	INTEGER IND,INDDAT		!CMUO bank and data indices
	INTEGER BANKNO			!CMUO bank #
	INTEGER NCMU			!Counter for # of CMUO banks found
	INTEGER NVERT,NVERT_GOOD	!Number of vertices
	INTEGER KCLASS(5),KCLASS_MAX	!Vertex class info
	INTEGER TRKS_NUM		!TRKS bank number for muon
	LOGICAL BC			!=.TRUE. for BC momentum
					!Otherwise they come from CMUO
        INTEGER IERRBC
	INTEGER STATUS			!YBOS return code
	INTEGER INDNXT			!Bank index to next SVXS bank
        LOGICAL FLAG3D                  !.TRUE. if track is 3-d
	REAL D0_OLD                     !Impact par. from QTRK (not used)
	REAL D0                         !Impact par. corrected for beam pos
        REAL QTP(3)                     !Track momentum from QTRK; beam
                                        !constrained when BC=.TRUE.
	INTEGER NASL                    !number of axial  superlayers from QTRK
	INTEGER NSSL                    !number of stereo superlayers from QTRK
        REAL SUMPT                      !SumPt of tracks inside cone of 0.25,
					!excluding muon track
        REAL HIGHPT                     !Highest Pt tracks inside cone of 0.25,
					!excluding muon track
C
C=======================================================================
C	Executable code
C
C	Zero all the arrays
C	-------------------
	call vzero(bnk_cmuo,MAXCMUO)
	call vzero(det_cmuo,MAXCMUO)
	call vzero(fid_cmuo,MAXCMUO)
	call vzero(p4m_cmuo,4*MAXCMUO)
	call vzero(pt_cmuo,MAXCMUO)
	call vzero(d0_cmuo,MAXCMUO)
	call vzero(z0_cmuo,MAXCMUO)
	call vzero(dx_cmu,MAXCMUO)
	call vzero(dx_cmp,MAXCMUO)
	call vzero(dx_cmx,MAXCMUO)
	call vzero(em_cmuo,MAXCMUO)
	call vzero(had_cmuo,MAXCMUO)
	call vzero(isol_cmuo,MAXCMUO)
	call vzero(bte_cmuo,MAXCMUO)
	call vzero(muzv_cmuo,MAXCMUO)
        call vzero(trks_cmuo,MAXCMUO)
        call vzero(ch_z0_cmuo,MAXCMUO)
        call vzero(ch_d0_cmuo,MAXCMUO)
C
C Find the Z vertex(ices) of the event and decide which is best
C-------------------------------------------------------------- 
        vertexz = 999.
        call vtgzzt(5,nvert,kclass,zvert,xvert,yvert)
        kclass_max = -1
        do i=1,nvert
          if(kclass(i).gt.kclass_max) then
            nvert_good = i
            kclass_max = kclass(i)
          endif
        enddo
        if(nvert.gt.0) then
          vertexz = zvert(nvert_good)
        else
          vertexz = 999.
        endif
C
C	Get the CMUOs
C
	bankno=1
	ncmu=0
	do while(blocat(iw,'CMUO',bankno,ind,inddat).eq.yesucc)
	  ncmu = ncmu + 1
	  if(ncmu.le.maxcmuo) then
	    bnk_cmuo(ncmu)=bankno
C
C	Get stuff out of the bank
C       -------------------------
C
	    em_cmuo(ncmu)=rw(inddat+moceme)
	    had_cmuo(ncmu)=rw(inddat+mochae)
	    dx_cmu(ncmu)=rw(inddat+morphm)
	    dx_cmp(ncmu)=rw(inddat+morphp)
	    dx_cmx(ncmu)=rw(inddat+morphx)
	    p4m_cmuo(1,ncmu)=rw(inddat+mopx)
	    p4m_cmuo(2,ncmu)=rw(inddat+mopy)
	    p4m_cmuo(3,ncmu)=rw(inddat+mopz)
	    p4m_cmuo(4,ncmu)=rw(inddat+moe4)
	    pt_cmuo(ncmu)=rw(inddat+mopt)
	    det_cmuo(ncmu)=iw(inddat+mostyp)
	    d0_cmuo(ncmu)=sqrt(rw(inddat+mox0)**2+rw(inddat+moy0)**2)
	    z0_cmuo(ncmu)=rw(inddat+moz0)
	    bte_cmuo(ncmu)=rw(inddat+mobden)
	    chrg_cmuo(ncmu)=iw(inddat+mochrg)
C
C Get the corresponding TRKS banks info
C -------------------------------------
C
	    trks_num= ishft(iand(iw(inddat+motrks),'FF00'x),-8)
            trks_cmuo(ncmu)=trks_num
	    call trackq(trks_num,bc,flag3d,d0_old,d0,nasl,nssl,
     &	    cv,zdiff,zt,qtp,ierrbc)
C
	    muzv_cmuo(ncmu) = cv            ! closest vertex with class>=10
	    d0_cmuo(ncmu) = d0              ! corrected impact parameter
	    nasl_cmuo(ncmu)=nasl
	    nssl_cmuo(ncmu)=nssl
C
            if(bc)then
	      pt_cmuo(ncmu)=sqrt(qtp(1)**2+qtp(2)**2)
	      p4m_cmuo(1,ncmu)=qtp(1)
	      p4m_cmuo(2,ncmu)=qtp(2)
	      p4m_cmuo(3,ncmu)=qtp(3)
	      ptot=sqrt(pt_cmuo(ncmu)**2+p4m_cmuo(3,ncmu)**2)
	      p4m_cmuo(4,ncmu)=sqrt(ptot**2+0.01117)
c**************** add chi2  ******************
              ch_z0_cmuo(ncmu) = z0chi2
              ch_d0_cmuo(ncmu) = d0chi2
            endif
C
C Calculate calorimeter isolation (ratio ET/PT) of muon
C -----------------------------------------------------
C
	    if(pt_cmuo(ncmu).gt.0.) then
	      isol_cmuo(ncmu) = rw(inddat+moneen)/pt_cmuo(ncmu)
	    else
	      isol_cmuo(ncmu) = 999.
	    endif
C
C Track isolation and highest Pt track in cone of R=0.4
C-------------------------------------------------------
C
	    call tkiso(trks_num,-99,0.4,sumpt,highpt)	

            IF(PT_CMUO(NCMU).GT.0.)THEN
              sumpt_cmuo(ncmu)=sumpt/pt_cmuo(NCMU)
            ELSE
              sumpt_cmuo(ncmu)=999.
            ENDIF

            highpt_cmuo(ncmu)=highpt
C
C Do the fiducial stuff
C ---------------------
C
	    p(1)=p4m_cmuo(1,ncmu)
	    p(2)=p4m_cmuo(2,ncmu)
	    p(3)=p4m_cmuo(3,ncmu)
C
C Get x0,y0 from CMUO bank
C        z0 (only) is after beam constraint if BC=.TRUE.
C-------------------------------------------------------
	    x(1)=rw(inddat+mox0)
	    x(2)=rw(inddat+moy0)
            if(bc)then
              x(3)=zt
            else
	      x(3)=z0_cmuo(ncmu)
	    endif
C
	    fid_cmuo(ncmu)=fidcmu92(p,x,float(chrg_cmuo(ncmu)))
C
	  endif				!If < 10 cmuos found so far
	  bankno=bankno+1
	end do				!End of loop over CMUOs
	numcmuo=ncmu
C
	Return
	End
