C     Version 6_2 (development, Jeff Berryhill 9/10/98):
C     - Add HAD tdc timing for photons (ptime) and electrons (etime)
C     - Add PJW's leakage (plcor) and multiple vertex (pvcor) corrections
C       for photon isolation
C     - Add trackpt-zvertex correspondence on a jet-by-jet basis
C       with jtvpt(20,njt) and jtnovpt(njt)
C     - npvert, vz,vq,etc. now reflects VTVZ banks of all classes
C     - Add ptnov: trackpt not associated with a class 12 vertex
C     - Add GENP block of variables:
C       ngenp         # of entries in block
C       ngid(ngenp)   particle id
C       ngn(ngenp)    genp pointer
C       ngnp(ngenp)   genp parent pointer
C       ngpid(ngenp)  parent particle id
C       gnp4(4,ngenp) four vector px-py-pz-m
C     - talk to variable vertexing added:
C       0 1st vtvz bank is zvertex
C       1 high sumpt vertex is zvertex
C       2 good lepton vertex is zvertex (else 1 is zvertex)
C       3 VXPRIM is zvertex
C
c     Version 4_4 (development, Peter Wilson 12/8/97):
c   - Fix MAJOR bug.  jt4eta and jt4phi were declared with length 5
c	BUT MAXJETS was set to 20 so we have been stepping on other arrays.
c	In the most recent case were stepping on CORFAC.  This has been
c	in the code since at least version 4_1!   
c	Solution:  made MAXJETS a parameter in stntuple.inc set to 20.
c		used this as the length for jt4eta, jt4phi.  However most of
c		the other jet variables are declared in JETINFO which is in 
c		C$INC and are explicitly declared with length 20.
c
c     Version 4_4 (development, Alexis Amadon 5/13/97)):
c
c   - Add guard energy (jtegrd, j1egrd) in list of variables.
c   - Add new corrected MET (metj1, corrected for everything, i.e. CEMFIX in 
c     photons and electrons, JTC96S(EY) for jets of cone size=0.7)
c   - Allow to exclude parts of the ntuple via talk_to, e.g. "no_pho"
c     excludes the subset of variables related to photons in stntuple.
c   - Reorder list of variables in ntuple, in particular revamp some
c     dilepton variables to "general" block.
c   
c     Create version 4_3 from development 3/21/97 Peter
c
c     Major fixes since 4_2 include getting the right CESSIG variable for
c     photons which had previously been the same as for electrons (in 
c     EXO_PHOVAL)
c    
c     Final Additions for Version 4_2 12/16/96 Carla and Peter
c
c   - Pick up MUOVAL, MIOVAL, QTRKBM from MY_MUOVAL, MY_MIOVAL, MY_QTRKBM 
c	respectively.  This is to get beam constraint CHI square available.
c
c     More revisions for Version 4_2 11/8/96-12/12/96 Peter Wilson
c
c   - Move MET calculations to after Cone 0.4 Jets but before big_cone 
c     jets so that jet corrected met is done with cone 0.4 jets.
c
c   - Make default for jets to NOT remove isolated photons and electrons.
c
c   - Correcting interference between TOPFND and STNTUPLE versions
c     of METJET and JETINFO.  Returned METJET to state of 1995.  
C     Rename METJET -> MY_METJET.
c     Add MY_JETINFO.INC to tell JETINFO whether to use 90 or 96
c     corrections.  Switch back to C$INC for JETINFO.
c
c   - Order photons according to photon treaty cuts.  New talk-to
c     for PHOISO_TYPE to choose between cone iso and mixed
c     cone/box.
c
c     STNTUPLE Version 4_2, 10/01/96 (Alexis Amadon):
c
c     Features in addition to version 4_1:
c
c   - When a TOPFND tight isolated lepton is found, pick its vertex as
c     primary vertex when seta bank is remade 
c     (see jetclu talk_to in uic file)
c   - allows the rewriting of the R=0.7 jet bank by assigning new PAR= 4
c
c     Features in addition to version 4_0:
c
c   - fixed routine METJET and variable METJT to take into account the 
c     JTC96S corrections in the corrected MET computation
c   - added the raw muon pt (murawpt(nmu)) in the list of ntuple variables
c   - added the negative tag background probability SCNPB(NJT) 
c     to the list of variables (for Method 2 fake background)
c   - added the TRIGBITS ntuple variable, in association with the TRIGGER 
c     talk_to variable which defines a set of triggers to be watched
c
c     STNTUPLE Version 4_2, 09/11/96 (Ray):
C     added 4 CES, CPR probability variables to photon subtuple
C     and bits 25 and 26 to PHOWRD.  Added three CPR track words.
C     Added institaneous luminosity and 2 tau backgroud probabilities
C
C=======================================================================
      SUBROUTINE FILL_STNTUPLE
C=======================================================================
c
C Description:
C
C Implicit none declaration:
      IMPLICIT NONE
C
C Production variables...
C
$$INCLUDE 'YBOS$LIBRARY:ERRCOD.INC'    ! YBOS error codes
$$INCLUDE 'YBOS$LIBRARY:BNKTYP.INC'    ! bank type codes
$$INCLUDE 'UIPACK$LIBRARY:UINAMES.INC' ! UIPACK declarations
$$INCLUDE 'UIPACK$LIBRARY:UIERROR.INC'
$$INCLUDE 'UIPACK$LIBRARY:UIPARAM.INC'
$$INCLUDE 'C$INC:ERLEVL.INC'           ! error severity levels for error logger
$$INCLUDE 'C$INC:JOBSTA.INC'           ! job status parameter
$$INCLUDE 'C$INC:BCS.INC'              ! primary YBOS array
$$INCLUDE 'C$INC:PIVALU.INC'           ! easy as...
C common blocks filled by TOPFND
$$INCLUDE 'C$INC:ELEINFO.INC'          ! ELES banks
$$INCLUDE 'C$INC:MUOINFO.INC'          ! CMUO information
$$INCLUDE 'C$INC:MIOINFO.INC'          ! CMIO information
$$INCLUDE 'C$INC:JETINFO.INC'  	       ! JETS information
$$INCLUDE 'STNTUPLE$SOURCE:MY_JETINFO.INC' ! More JETS information
$$INCLUDE 'C$INC:EVTINFO.INC'       ! cuts on leptons for dilep,lep+jets
$$INCLUDE 'C$INC:DILINFO.INC'       ! dilepton search topology cuts
$$INCLUDE 'C$INC:DILMORE.INC'       ! more dilepton info
$$include 'c$inc:cmuo92.inc'
C common block filled by SECVTX
$$INCLUDE 'C$INC:SECVTX.INC'
C common block filled by JPBTAG
$$INCLUDE 'C$INC:JPBTAG.INC'
C common block filled by BTGSMK
$$INCLUDE 'C$INC:BTGSCM.INC'
C common block filled by TAUFND
$$INCLUDE 'c$inc:TAUFND.INC'
C common block filled by METBTC
$$INCLUDE 'stNTUPLE$SOURCE:METSCM.INC'
C ELES bank pointers
$$INCLUDE 'C$INC:DSELES.INC'
C my own variables
$$INCLUDE 'stntuple$source:STNTUPLE.INC'
$$INCLUDE 'stntuple$source:stNTUPLE_var.INC'
$$INCLUDE 'stntuple$source:stntuple_talk.inc'
$$INCLUDE 'stntuple$source:exo_phoinfo.inc'
C VTVZ BANK OVERHEAD
$$INCLUDE 'C$INC:VTCAUX.INC'
$$INCLUDE 'C$INC:VXPVIN.INC'
$$include 'C$inc:tfzinfo.inc'
$$include 'C$inc:BADRUN.inc'
$$include 'stntuple$source:trigger_stuff.inc'
c    components to beam constrained chi2 from qtrkbm
$$include 'stntuple$source:my_bc.inc'
$$include 'c$inc:trksco.inc'
$$include 'c$inc:dsfitp.inc'
$$include 'c$inc:trfitc.inc'

        INTEGER ANPTRG                    ! A_C Filter Decision
        INTEGER I,J,K,IJ,JJ,KK,L,TEMP     ! Used as counters
C
C Setup for making NTUPLES.... 
C
        INTEGER ANGHAC                    !A_C histo stuff
        INTEGER ANGHBO                    !A_C histo stuff
        INTEGER ANGTH4                    !A_C histo stuff
C
        INTEGER NID, phoind(10),eleind(maxeles),muoind(15),tauind(10)  
        INTEGER BUFSIZ, br_code
        CHARACTER*32 TOPDIR
        CHARACTER*32 FILE_NAME        
        CHARACTER*1 OPNOPT


C LOCAL VARIABLES
        INTEGER BLOCAT,INDBNK,INDELE,istat
        integer bnknum,sindx,sdindx
        REAL VTXOCC1,VTXOCC2,VTOCCP,VTXOCC,genp_vtxocc(maxeles)
        real dimass,delta_r
        INTEGER VTXWORD,trkflag,ind,inddat
        integer aa_index,qtrkun,jetword,JETS_TRKS,JETMATCH
        integer minjet
        real*8 
     &   pspm(4),! array of jet var. required as input to AA correc. routine
               ! (1) phi, (2) tan(dip), (3) QDJSCO-corrected momentum, (4) mass
     &  pcor,   ! AA corrected jet momentum
     &  sigpcor,! associated error
     &  dummy
        integer phostub,stream
        REAL ETADIFF,PHIDIFF,CONE_SIZE,THETA
        integer dcorindex,mcorindex,nt_version,bnk_version
        logical blinfo
        real xseed(20,3),cvseed(20,6),qint(20),dz0cut
        REAL HELIM,DPHIMIN,DPHI_JETS,CORFAC_PHO,CORFAC(10)
	REAL LUMV(8)
        real z_jet(20),dz_jet(20),pt_out(20)

        integer nqtrk,nsvx,naxialsl,nstereosl,imsk,nreq(0:8),bcq
        DATA NREQ/5,3,5,3,5,3,5,3,5/   
        real bcp(3),bcpx,bcpy,bcpz,bcz0,bcd0,trptcut,minz
        real xcal(3),exeta,exphi,deltar,trackiso4
        integer qtrkbm
        real prapid,getphi
        real jwb_zvertex
        integer jwb_zclass

        integer indbtgs
        INTEGER CEM_ELE,CEM_LOOSE,CMU_MUON,TAG,TAG_WRD,TOP,TOP_WRD
        INTEGER HPTE_HPTE,HPTE_LCEM,HPTE_PEM,HPTE_CMU,HPTE_CMX,HPTE_CMIO
        INTEGER HPTM_HPTE,HPTM_LCEM,HPTM_PEM,HPTM_CMU,HPTM_CMX,HPTM_CMIO
        DATA CEM_ELE /'3FF'X/
        DATA CEM_LOOSE /'F33'X/
        DATA CMU_MUON /'DFE'X/
        DATA HPTE_HPTE/1/
        DATA HPTE_LCEM/2/
        DATA HPTE_PEM/3/
        DATA HPTE_CMU/4/
        DATA HPTE_CMX/5/
        DATA HPTE_CMIO/6/
        DATA HPTM_HPTE/7/
        DATA HPTM_LCEM/8/
        DATA HPTM_PEM/9/
        DATA HPTM_CMU/10/
        DATA HPTM_CMX/11/
        DATA HPTM_CMIO/12/
C
C+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
C
C   First we check to see if the histograming is open if so, then initialize
C       all the variables. Note the names of these variables are given by
C       STNTUPLE_BOOK.
C
        IF (ANGHAC().AND.ANGHBO()) THEN
C
C   First clear out all old values...
C
        dcorindex = 1		    ! point at relative corrections
        mcorindex = 2               ! point at full corrections
        IJ = 1
        CALL VZERO(STAT_ELES,MAXELES)
        CALL VZERO(STAT_CMUO,MAXCMUO)
        CALL VZERO(STAT_CMIO,MAXCMIO)
        CALL VZERO(STAT_TAUO,50)
        CALL VZERO(STAT_PHO,10)

        do k = 1,10
          corfac(k) = 1.0
        enddo
        cone_size = 0.4


C Now execute a series of subroutines which fill the variables in the
C common blocks for various physics objects.        

C calling TOPFND will fill the following common blocks:
C                 EVTINFO.INC
C                 ELEINFO.INC
C                 JETINFO.INC
C                 MUOINFO.INC
C                 MIOINFO.INC
C                 DILINFO.INC

         CALL TOPFND(.TRUE.,.TRUE.,.TRUE.)
         CALL JETINFO(2)
         CALL GET_VPT(IJET_VPT,npvert,IJET_NOVPT,ptnov)
         CALL TAUFND
         CALL METBTC
         CALL MY_ELEINFO
         CALL MY_MUOINFO
         CALL MY_MIOINFO

         call exo_pho
         call ntuple_trigger_stuff

c   Implement VTX occupancy for MC

        if (nrun.lt.40000) call egamma_genp_match(genp_vtxocc)

C FILL THE GENP BLOCK - FILLS  STNTUPLE_VAR.INC DIRECTLY
	IF (NRUN.LT.40000) CALL STN_GENP

C Start with filling the ntuple...


               version= 5000000 + vertexing*1000 +
     &                  jet_filter*100 +big_cone*10+jtc_version
               run = NRUN     ! Run number
               event = NEV    ! Event number
               IF (NRUN.LT.40000.and.nev.le.200000) THEN
                 ILUM = mcweight(nev)
                 IF (ilum.eq.0.0) RETURN
               ELSE IF (NRUN.LT.40000.and.nev.gt.200000) THEN
                 ILUM = 1.0
               ELSE
                 CALL LUMDUP(LUMV)
                 ILUM=LUMV(1) ! INSTANTANEOUS LUMINOSITY
               ENDIF

C
C Next we sort photons according to STAT_PHO descending order
C 
C Photon status points:
C -------------------------------------------------------
C   Status              Desciption
C   ------              ----------
C   1000.0 + Et         a central or plug ELES bank
C   2000.0 + Et         a plug ELES bank passing the
C                        loose photon ID cuts in PHOFND 
C   3000.0 + Et         a plug ELES bank passing the
C                        loose photon ID cuts and the loose photon
C                        isolation cuts in PHOFND
C   4000.0 + Et         a central ELES bank passing the Photon Treaty ID cuts
C                        in PHOFND
C   5000.0 + Et         a central ELES bank passing the Photon Treaty ID cuts
C                        and the Photon Treaty isolation cuts in PHOFND
C
C
        CALL SORTZV(STAT_PHO,PHOIND,NPHO,1,1,0)

C       get info for photons

        NPHOIDL = numphoidl
        nphoisol = NUMPHOISOL
        nphoid = NUMPHOID
        nphoiso = NUMPHOISO
   
        DO J = 1,npho
          JJ = PHOIND(J)
          pdetect(j) = PHODETECT(JJ)
          pzv(j) = PHOZV(JJ)
          pet(j) = PHOET(JJ)
          petc(j) = PHOET(JJ)/phocor(jj)
          call ucopy(pho_p4(1,jj),pp4(1,j),4)
          pco4(j) = PHOCO4(JJ)
          pcesx(j) = PHOCESX(JJ)
          pcesz(j) = PHOCESZ(JJ)
          pcpr5ph(j) = PHOCPR5PH(JJ)
          pcpr5ps(j) = PHOCPR5PS(JJ)
          pdteta(j) = PHODETA(JJ)
          peveta(j) = PHOETA(JJ)
          pcese(j) = PHOCESE(JJ)
          pphi(j) = PHOPHI(JJ)
          pco7(j) = PHOCO7(JJ)
          phadem(j) = PHOHADEM(JJ)
          plshr(j) = PHOLSHR(JJ)
          pstr2(j) = PHOSTR2(JJ)
          pstre2(j) = PHOSTRE2(JJ)
          pwir2(j) = PHOWIR2(JJ)
          pwire2(j) = PHOWIRE2(JJ)
          psumpt4(j) = PHOSUMPT4(JJ)
          pvtxocc(j) = PHOVTX(JJ)
          ppt(j) = phopt(jj)
          pn3d(j) = phon3d(jj)
          pchistr(j) = phochistr(jj)
          pchiwir(j) = phochiwir(jj)
          pchi(j) = phochi(jj)
          pchi3x3(j) = PHO3X3(JJ)
          pchieta(j) = PHOCHIE(JJ)
          pchiphi(j) = PHOCHIP(JJ)
          pstat(j) = STAT_PHO(JJ)
          pboxiso(j) = phoboxiso(jj)
          pcesflg(j) = iand(phowrd(jj),2**16).eq.0
          pe(j) = phoe(jj)
          pceswht(j) = phoceswht(jj)
          pcprwht(j) = phocprwht(jj)
          pcesslide(j) = phocesslide(jj)
          pwrd(j) = phowrd(jj)
          psth(j) = phosth(jj)
          pcessig(j) = phocessig(jj)
          pcespg(j) = phocespg(jj)
          pcespb(j) = phocespb(jj)
          pcprpg(j) = phocprpg(jj)
          pcprpb(j) = phocprpb(jj)
          ppt2(j) = phopt2(jj)
          pntrwd3(j) = ntrwd3(jj)
          pcescprx(j) = cescprx(jj)
          ptkcprx(j) = tkcprx(jj)
          ptime(j) = photime(jj)
          pvcor(j) = phovcor(jj)
          plcor(j) = pholcor(jj)
        ENDDO

C
C Next do Cosmic Stubs. Note: We remove stubs behind jets because there could
C   be real punch-through. We do NOT remove stubs behind the photon with 
C   largest status since this could be a cosmic.
C
        CALL COSMIC_STUBS(PHOIND(1),NCMUS,COSTUB,PHOSTUB)
C
C  EVTINFO variables...see C$TOP:TOPFND.CDF for definitions and cuts
C
	DILWRD = DIL_WRD

C  flag for this event being on the good run list

c       BR_CODE         a four bit word that indicates if the run is good
c                       or bad.  The bits in OK are defined as follows:
c
c                       Note:   OK is set to -1 if the run is not found.
c                               If this happens, please send mail to 
c                               FNALD::SACHA.  
c
c                       Bit 0:  Indicates the run is suitable for analyses
c                               that do not require the muon systems (the
c                               following detectors are in working order:
c                               CTC,all calorimeters,Trigger,BBC,DAQ,CES)
c                       Bit 1:  CMU is in working order
c                       Bit 2:  CMP is in working order
c                       Bit 3:  CMX is in working order

       STREAM = 2
       CALL BADRUN(NRUN,STREAM,BR_CODE)
       BRCODE = br_code
       IF (SVX_BIT .GT. 0) BRCODE = BRCODE + 16
       goodrun = good_run

C  flag for following proper lepton trigger path

        goodtrig = GOOD_TRIG
        trigword = TRWRD    !bit packed trigger information
                            ! (see C$TOP:GET_TRIG_INFO.CDF)
        njet10 = NJET_10    !# of jets > 10 GeV
        njet15 = NJET_15    !# of jets > 15 GeV
        njet20 = NJET_20    !# of jets > 20 GeV
        njet25 = NJET_25    !# of jets > 25 GeV
        nhpte  = NUM_HPTE    !# of high pt electrons
        nlcent = NUM_LOOSE_E !# of loose central electrons
        nlplug = NUM_PEM     !# of plug electrons
        nhptm  = NUM_HPTM    !# of high pt muons
        ncmio  = NUM_CMIO    !# of CMIO's
        ngtau  = NUMTIGHTTAU  !# of gold tau's
        nlepton = NUM_LOOSE_E + NUM_PEM +
     &                   NUM_HPTM + NUM_CMIO + NUMTIGHTTAU

C DILINFO variables...see C$TOP:DIL_TOPOLOGY.CDF for definitions and cuts

C       MET used for dilepton cuts (with jet corrections, and corrected
C       for hi-pt cmuo's,cmio's,and loose cmuo's with pt > 10 GeV)

        metdilep = MET_DIL  
        njtdilep = NJETMET_DIL    ! number of jets passing dilepton cuts
          
C       difference in phi between MET vector used for dilepton cuts and  

        DPHIMINJ = 360.0
        DO J = 1,NJETMET_DIL
           IF (DPHIJ_DIL(J).LT.DPHIMINJ) DPHIMINJ = DPHIJ_DIL(J)
        ENDDO
        dphiminl = MIN(DPHIL1_DIL,DPHIL2_DIL)
        dilmode = DIL_MODE    ! 1=ee,2=emu,3=mumu,4=etau,5=mutau

C 4-vectors,charge, and et of the leptons which pass dilepton cuts,along with a
C classification code.  For now I'm going to accomodate up to 3 leptons
C and up to 3 dilepton combinations.

C                           DILEPTON CLASSIFICATION CODE                     
C                    ----------------------------------------
c                                    HPTE_HPTE =1
c                                    HPTE_LCEM=2
c                                    HPTE_PEM=3
c                                    HPTE_CMU=4
c                                    HPTE_CMX=5
c                                    HPTE_CMIO=6
c                                    HPTM_HPTE=7
c                                    HPTM_LCEM=8
c                                    HPTM_PEM=9
c                                    HPTM_CMU=10
c                                    HPTM_CMX=11
c                                    HPTM_CMIO=12

        diltype1 = DIL_TYPE(1,1)  !code of 1st dilepton combo

C loop over ELES banks and compute their status

        nel = MIN(MAXELES,NUMELES)
        DO J = 1,nel          

C determine the status of the ELES bank STATUS= rank*1000+Et
C       ELES ranking
C       1  something there
C       2  central, with Et .gt. 10
C       3  plug,central,forward, passes loose R paper cuts
C       5  plug or central, passes loose dilepton cuts
C       6  central, passes tight R paper cuts
C       7  central, passes tight dilepton cuts except fiducial cut
C       8  central, passes tight dilepton cuts
C       9  central, passes tight dilepton cuts, isolated

C  if there's a bank, you get 1000 points....

             CALL GET_CORFAC(BNK_ELE(J),CORFAC(J))
             STAT_ELES(J) = 1000.0 + ET_ELE(J)/CORFAC(J)

C  central with Et .gt. 10 gets 2000....

       IF (DET_ELE(J).EQ.0.AND.ET_ELE(J)/CORFAC(J).GT.10.0)
     &         STAT_ELES(J) = 2000.0 + ET_ELE(J)/CORFAC(J)

C  satisfying loose R paper cuts gets 3000...           

             IF (DET_ELE(J) .EQ. 0) THEN
               IF (ET_ELE(J)/CORFAC(J).GT.20.0) THEN
                 IF (CON_ELE(J).EQ.0) THEN
                   IF (EP_ELE(J).LE.2.0) THEN
                     IF (HADEM_ELE(J).LE.0.1) THEN
C                      IF (ISOL_ELE(J) .LE. 0.1) THEN
                       IF (FID_ELE(J).EQ.0) THEN
                       STAT_ELES(J)=3000.0+ET_ELE(J)/CORFAC(J)
                       ENDIF
C                       ENDIF
                     ENDIF
                   ENDIF
                 ENDIF
               ENDIF
             ENDIF

C            Plug electrons...
             IF (DET_ELE(J) .EQ. 1) THEN
               IF (ET_ELE(J)/CORFAC(J) .GE. 15.0) THEN
C                 IF (ISOL_ELE(J) .LE. 0.1) THEN
                   IF (CHI3_ELE(J) .LE. 3.0) THEN
                     IF (HADEM_ELE(J) .LE. 0.1) THEN
                       IF (FID_ELE(J).EQ.0) THEN
                       IF (CON_ELE(J).EQ.0) THEN
                       STAT_ELES(J)=3000.0+ET_ELE(J)/CORFAC(J)
                       ENDIF
                       ENDIF
                     ENDIF
                   ENDIF
C                 ENDIF
               ENDIF
             ENDIF

C            Forward electrons...
             IF (DET_ELE(J) .EQ. 2) THEN
               IF (ET_ELE(J)/CORFAC(J) .GT. 10.0) THEN
                 IF (HADEM_ELE(J) .LT. 0.1) THEN
C                   IF (ISOL_ELE(J) .LT. 0.1) THEN
                   STAT_ELES(J) =3000.0+ET_ELE(J)/CORFAC(J)
C                   ENDIF
                 ENDIF
               ENDIF
             ENDIF

C  loose central and plug events get 5000....
             DO K = 1,MIN(MAXELES,NUM_PEM)
                IF (IND_PEM(K).EQ.J)
     &        STAT_ELES(J)=5000.0+ET_ELE(J)/CORFAC(J)
             ENDDO !NUM_PEM
             DO K = 1,MIN(MAXELES,NUM_LOOSE_E)
                IF (IND_LOOSE_E(K).EQ.J)
     &            STAT_ELES(J) =5000.0+ET_ELE(J)/CORFAC(J)
             ENDDO  !NUM_LOOSE_E
C   satisfying tight R paper cuts gets 6000....
          IF (DET_ELE(J) .EQ. 0) THEN
             HELIM=0.045+0.055*P4E_ELE(4,J)/CORFAC(J)/100.0
             IF (ET_ELE(J)/CORFAC(J) .GT. 20.0) THEN
               IF (EP_ELE(J) .LT. 2.0 .AND. EP_ELE(J) .GT. 0.5) THEN
                 IF (LSHR_ELE(J) .LE. 0.2) THEN
                   IF (CHIS_ELE(J) .LE. 10.0) THEN
                     IF (ABS(DELX_ELE(J)) .LE. 1.5) THEN
                       IF (ABS(DELZ_ELE(J)) .LE. 3.0) THEN
                         IF (ABS(ELEZV_ELE(J)) .LE. 60.0) THEN
                           IF (HADEM_ELE(J) .LE. HELIM) THEN
C                             IF (ISOL_ELE(J) .LE. 0.1) THEN
                               IF (FID_ELE(J).EQ.0) THEN
                                IF (CON_ELE(J).EQ.0) THEN
                       STAT_ELES(J)=6000.0+ET_ELE(J)/CORFAC(J)
                                ENDIF
                               ENDIF
C                             ENDIF
                           ENDIF
                         ENDIF
                       ENDIF
                     ENDIF
                   ENDIF
                 ENDIF
               ENDIF
             ENDIF
           ENDIF
                  
C   tight central nonfiducial get 7000....
           IF (DET_ELE(J).EQ.0) THEN
              IF (IAND(CEM_WRD(J),CEM_ELE).EQ.512)
     &           STAT_ELES(J)=7000.0+ET_ELE(J)/CORFAC(J)
           ENDIF  !DET_ELE
C    tight central fiducial get 8000....
           DO K = 1, MIN(MAXELES,NUM_HPTE)
              IF (IND_HPTE(K).EQ.J) THEN
                 STAT_ELES(J) = 8000.0 + ET_ELE(J)/CORFAC(J)
C     tight central fiducial isolated get 9000....
                 IF (IAND(CEM_ISO(IND_HPTE(K)),2).EQ.0)
     &           STAT_ELES(J) = 9000.0 + ET_ELE(J)/CORFAC(J)
              ENDIF !IND_HPTE
           ENDDO !NUM_HPTE
       ENDDO !Nel

C loop over CMUO banks and compute their status
C      1   something there
C      2   pt .gt. 10
C      3
C      4
C      5   passes tight dilepton cuts except fiducial cut
C      6   passes tight dilepton cuts
C      7   passes tight dilepton cuts plus isolation cut

        DO J = 1, MIN(15,NUMCMUO) 
C      if it's there the bank gets 1000 pts...
          STAT_CMUO(J) = 1000.0 + PT_CMUO(J)
          IF (PT_CMUO(J).GT.999.0) STAT_CMUO(J)=1999.0
C      pt .gt. 10 gets 2000....
          IF (PT_CMUO(J).GT.10.0)
     &      STAT_CMUO(J) = 2000.0 + PT_CMUO(J)
          IF (PT_CMUO(J).GT.999.0) STAT_CMUO(J)=2999.0
C      passes tight dilepton cuts except for fiducial cut gets 5000....
          IF (IAND(MUO_WRD(J),CMU_MUON).EQ.0)
     &      STAT_CMUO(J) = 5000.0 + PT_CMUO(J)
          IF (IAND(MUO_WRD(J),CMU_MUON).EQ.0.AND.PT_CMUO(J).GT.999.0)
     &      STAT_CMUO(J) = 5999.0
C      passes tight dilepton cuts gets 6000....
          DO K = 1, MIN(15,NUM_HPTM)
            IF (IND_HPTM(K).EQ.J) THEN
              STAT_CMUO(J) = 6000.0 + PT_CMUO(J)
              IF (PT_CMUO(J).GT.999.0) STAT_CMUO(J) = 6999.0
C      passes tight dilepton cuts + isolation cut gets 7000....
              IF (IAND(MUO_ISO(IND_HPTM(K)),2).EQ.0)
     &          STAT_CMUO(J) = 7000.0 + PT_CMUO(J)
              IF (IAND(MUO_ISO(IND_HPTM(K)),2).EQ.0
     &  .AND.PT_CMUO(J).GT.999.0) STAT_CMUO(J) = 7999.0
            ENDIF !IND_HPTM
          ENDDO !NUM_HPTM

        ENDDO !NUMCMUO

C loop over CMIO banks and compute their status
C      1 something there
C      2 pt .gt. 10
C      3 passes dilepton cuts
C      4

        DO J = 1, MIN(15,NUMCMIO)
C      if it's there the bank gets 1000 pts....
          STAT_CMIO(J) = 1000.0 + PT_CMIO(J)
          IF (PT_CMIO(J).GT.999.0) STAT_CMIO(J)=999.0
C      pt .gt. 10 gets 2000....
          IF (PT_CMIO(J).GT.10.0) 
     &      STAT_CMIO(J) = 2000.0 + PT_CMIO(J)
            IF (PT_CMIO(J).GT.999.0) STAT_CMIO(J)=2999.0
C      passes dilepton cuts gets 3000....
          DO K = 1, MIN(15,NUM_CMIO)
            IF (IND_CMIO(K).EQ.J)
     &        STAT_CMIO(J) = 3000.0 + PT_CMIO(J)
            IF (PT_CMIO(J).GT.999.0) STAT_CMIO(J)=3999.0
          ENDDO !IND_CMIO

        ENDDO !NUMCMIO

C loop over TAUO banks and compute their status....
C      1 something there
C      2 loose tau
C      3 tight tau  
C      4 loose tau and tight tau
       
        ntau = MIN(10,NUMTAU)
        DO J = 1,ntau
C      if it's there the bank gets 1000 pts....
          STAT_TAUO(J) = 1000.0 + TAU_ET(J)
C      loose tau gets 2000 pts....
          IF (loosetau(j).eq.1.0)
     &        STAT_TAUO(J) = 2000.0 + TAU_ET(J)
C      tight taus get 3000....
          IF (TIGHTTAU(J).EQ.1.0) 
     &      STAT_TAUO(J) = 3000.0 + TAU_ET(J)
C       'double tagged' taus get 4000
          if (loosetau(j).eq.1.0.and.TIGHTTAU(J).EQ.1.0)
     &      STAT_TAUO(J) = 4000.0 + TAU_ET(J)
        ENDDO !NUMTAU

C  get the three best electrons and fill the 4-vector, charge,
C  et, detector eta, phi, status

c   Sort electrons according to stat_eles descending order

        call sortzv(stat_eles,eleind,nel,1,1,0)

        DO J = 1,nel
           JJ = ELEIND(J)
           call vscale(P4E_ELE(1,JJ),1./corfac(jj),ep4(1,j),4)
           eetcor(j) = ET_ELE(JJ)/CORFAC(JJ)
           eet(j) = ET_ELE(JJ)
           echarge(j) = CHG_ELE(JJ)
           edteta(j) = DTETA_ELE(JJ)
           eeveta(j) = EVETA_ELE(JJ)
           ephi(j) = PHI_ELE(JJ)
           ept(j) = PTELE_ELE(JJ)
           estat(j) = STAT_ELES(JJ)
           ezv(j) = ELEZV_ELE(JJ)
           eep(j) = EP_ELE(JJ)
           ehadem(j) = HADEM_ELE(JJ)
           exces(j) = Xces_ELE(JJ)
           edelx(j) = DELX_ELE(JJ)
           ezces(j) = Zces_ELE(JJ)
           edelz(j) = DELZ_ELE(JJ)
           echis(j) = CHIS_ELE(JJ)
           echiw(j) = CHIW_ELE(JJ)
           eiso(j)  = ISOL_ELE(JJ)
           etiso(j) = SUMPT_ELE(JJ)
           eptrk(j) = PELE_ELE(JJ)
           elshr2(j) = lshr2_ele(jj)
           echi3(j) = CHI3_ELE(JJ)
           echid(j) = CHID_ELE(JJ)

           if (nrun.lt.40000) then
              evtxocc(j) = genp_vtxocc(jj)
           else
              VTXOCC = 0.0
              ISTAT = BLOCAT(IW,'ELES',BNK_ELE(JJ),INDBNK,INDELE)
              IF (ISTAT.NE.YESUCC) THEN
                 CALL ERLOGR ('JEFF',ERWARN,0,ISTAT,'CANNOT FIND ELES')
              ELSE
                 VTXWORD = IW(INDELE+VTPELE)  !Encoded word with VTX info
                 bnk_VERSION = IW(INDELE+VRDELE)  !Version # of bank structure
                 VTXOCC = VTOCCP(VTXWORD,bnk_VERSION)
              ENDIF
	      evtxocc(j) = vtxocc
	   endif
           IF (DET_ELE(JJ).EQ.0) eidwrd(j) = CEM_WRD(JJ)
           IF (DET_ELE(JJ).EQ.1) eidwrd(j) = PEM_WRD(JJ)
           IF (DET_ELE(JJ).EQ.0) eiswrd(j) = CEM_ISO(JJ)
           IF (DET_ELE(JJ).EQ.1) eiswrd(j) = PEM_ISO(JJ)
           econwrd(j) = CON_ELE(JJ)
           eztrk(j) = z0_ele(jj)   !  z of electron track at origin
           efid(j)  = fid_ele(jj)  !  bit-packed word telling how fiducial
C                                     the electron is; 0 = fiducial, other-
C                                     wise, the following bits are set:
C                             bit 0     CEM failed phi crack cut
C                                 1     CEM failed 90 crack cut
C                                 2     CEM failed tower 9 cut
C                                 3     CEM failed Zprime tower 9 cut
C                                 4     CEM Failed tower 7 chimney
C                                 5     PEM outer eta annuli
C                                 6     PEM inner eta annuli
C                                 7     PEM phi cut
C                                 8     PEM dead channel
C                                 9     FEM inner eta annuli
C                                10     FEM phi cut
           edet(j)  = det_ele(jj)  !  0 = central, 1 = plug, 2 = forward
           ebte(j)  = bte_ele(jj)  !  border tower energy
           enasl(j) = nasl(jj)     ! nb of axial superlayers hit by electron
           enssl(j) = nssl(jj)     ! nb of stereo superlayers
           etime(j) = time_ele(jj)
         ENDDO     
C
C This was wrong...see next comment ! DT 11/12/96
c    "Match" photons to electrons and fill photon VTX occupancy for MC
C
C         do j=1,Npho
C            jj = 1
C            do while (phobnk(phoind(j)).ne.bnk_ele(jj))
C               jj = jj + 1
C            end do
C            pelind(j) = eleind(jj)
C            if (nrun.lt.40000) pvtxocc(j) = genp_vtxocc(jj) 
C         end do 
C
C
C Here we loop over all the photons to match the electrons. A few notes are in
C   order: Certain variables are the ones which go into the NTUPLE (i.e, ordered
C   by best photon or electron, for example PELIND and PVTXOCC) other 
C   variables are ordered by the bank ordering (from which they were unpacked)
C   such as ELEIND, PHOBNK and BNK_ELE. Since all the other NTUPLE photon 
C   variables have been put into their correct order we have to fill these in
C   their correct order.
C
C   We call variables like PELIND as NTUPLE ordering
C   We call variables like PHOBNK as BANK ordering
C
         DO J=1,NPHO    ! Look at the J'th photon in NTUPLE ordering
C
C Next find the JJ electron in NTUPLE ordering. Do this by matching the the two
C   bank numbers. The bank numbers are storded in BANK ordering.
C
            JJ = 1
            DO WHILE (PHOBNK(PHOIND(J)).NE.BNK_ELE(ELEIND(JJ)))
               JJ = JJ + 1
            END DO
C
C Now that we have the JJ'th electron in NTUPLE ordering, set PELIND(J) to be
C   equal to that electron.
C

            PELIND(J) = JJ
C
C Do the same for PVTXOCC. Remember that PVTXOCC is in NTUPLE ordering and that
C   GENP_VTXOCC is in BANK ordering.
C
            IF (NRUN.LT.40000) PVTXOCC(J) = GENP_VTXOCC(ELEIND(JJ))
         END DO


C get the muons and fill the 4-vector,charge,et,detector eta, phi, status

        nmu = min(15,NUMCMUO+NUMCMIO)
        numcmuo = min(numcmuo,15)
        numcmio = min(numcmio,nmu-numcmuo)       

c  Sort muons, but take CMIOs into consideration: append CMIO list to
c  CMUO array first, then sort according to STAT_CMUO descending order:

        call ucopy(stat_cmio,stat_cmuo(numcmuo+1),numcmio)
        call sortzv(stat_cmuo,muoind,nmu,1,1,0)

        DO J = 1,nmu
           JJ = MUOIND(J) ! CMU bnk number

C   fill data for CMUOs
 
           IF (JJ.le.numcmuo) THEN
	      if (blocat(iw,'CMUO',jj,ind,inddat).eq.yesucc) then
	         murawpt(j) = rw(inddat+mopt)
              else
                 murawpt(j) = -999.
	      endif
              call ucopy(P4M_CMUO(1,JJ),mup4(1,j),4)
              mupt(j) = PT_CMUO(JJ)
              muq(j) = CHRG_CMUO(JJ)
              muhad(j) = HAD_CMUO(JJ)
              muem(j) = EM_CMUO(JJ)
              muchz0(j) = ch_z0_cmuo(jj)
              muchd0(j) = ch_d0_cmuo(jj)
                
C   do it yourself eta

              mueveta(j) = 999.0
              IF (P4M_CMUO(4,JJ).GT.0.0) THEN
                 IF (P4M_CMUO(3,JJ)/P4M_CMUO(4,JJ).LE.1.0) THEN
                    mueveta(j) = -1.0*LOG(TAN(0.5*ACOS(
     &                           P4M_CMUO(3,JJ)/P4M_CMUO(4,JJ))))
                 ELSE
                    WRITE (6,*)
     &              'This 4-vector sucks! Cannot calculate eta'
                 ENDIF
              ENDIF

C   do it yourself phi

              muPHI(j) = 360.0
              IF (P4M_CMUO(2,JJ).NE.0.0.OR.
     &            P4M_CMUO(1,JJ).NE.0.0) THEN
                 muPHI(j) = PI + ATAN2(-P4M_CMUO(2,JJ),-P4M_CMUO
     &                      (1,JJ))
              ELSE
                 WRITE (6,*)
     &           'This 4-vector sucks! Cannot calculate phi'
              ENDIF
              mustat(j) = STAT_CMUO(JJ)
              muzv(j) = MUZV_CMUO(JJ)
              mud0(j) = D0_CMUO(JJ)
              mudelz(j) = MUZV_CMUO(JJ) - Z0_CMUO(JJ)
              mudelx(j) = 0.0
              IF (DET_CMUO(JJ).EQ.1.OR.
     &            DET_CMUO(JJ).EQ.3) mudelx(j) = DX_CMU(JJ)
              IF (DET_CMUO(JJ).EQ.2) mudelx(j) = DX_CMP(JJ)
              IF (DET_CMUO(JJ).EQ.4) mudelx(j) = DX_CMX(JJ)
              muiso(j) = ISOL_CMUO(JJ)                
              mutiso(j) = SUMPT_CMUO(JJ)
              mucray(j) = COS_CMUO(JJ)
              muidwrd(j) = MUO_WRD(JJ)
              muiswrd(j) = MUO_ISO(JJ)
              muztrk(j) = z0_cmuo(jj)   ! z of muon track at origin
              mufid(j)  = fid_cmuo(jj)  ! word telling how fiducial mu is
C            = -1: Error in propagation
C            =  0: Muon in good CMU region
C            =  1: Muon falls in 90 degree crack
C            =  2: Muon fall into phi crack in central region
C            =  3: Muon in Chimney
C            =  4: Muon not in central region but still in good CTC region
C            =  5: Muon falls into phi crack in good, non-central CTC region
C            =  6: Muon detector eta too high for good CTC region
              mudet(j)  = det_cmuo(jj) ! Bit-packed word: bit 0 set=CMU,1=CMP,2=CMX
              mubte(j)  = bte_cmuo(jj)  ! border tower energy
              munasl(j) = nasl_cmuo(jj) ! nb of axial superlayers hit by mu
              munssl(j) = nssl_cmuo(jj) ! nb of stereo superlayers 
              mutime(j) = HTDC_CMUO(JJ) ! had tdc time of muon tower
              muvtxocc(j) = VTXOCC_CMUO(JJ) !VTX occupancy of muon track
              mudteta(j) = DTETA_CMUO(JJ) !detector eta of muon at CES
C    fill data for CMIOs
 
           ELSE  !MIOINDEX
              jj = jj - numcmuo
	      if (blocat(iw,'CMIO',jj,ind,inddat).eq.yesucc) then
                 murawpt(j) = rw(inddat+mopt)
              else
                 murawpt(j) = -999.
              endif
              call ucopy(P4M_CMIO(1,JJ),mup4(1,j),4)
              mupt(j) = PT_CMIO(JJ)
              muq(j) = CHRG_CMIO(JJ)
              muhad(j) = HAD_CMIO(JJ)
              muem(j) = EM_CMIO(JJ)
              muchz0(j) = ch_z0_cmio(jj)
              muchd0(j) = ch_d0_cmio(jj)
                                 
C    do it yourself EVENT eta
     
              muevETA(j) = 99.0
              IF (P4M_CMIO(4,JJ).GT.0.0) THEN
                 IF (P4M_CMIO(3,JJ)/P4M_CMIO(4,JJ).LE.1.0) THEN
                     muevETA(j) = -1.0*LOG(TAN(0.5*
     &               ACOS(P4M_CMIO(3,JJ)/P4M_CMIO(4,JJ))))
                 ELSE
                WRITE (6,*) 'This 4-vector sucks! Cannot calculate eta'
                 ENDIF
              ENDIF

C    do it yourself phi

              muPHI(j) = 360.0
              IF (P4M_CMIO(2,JJ).NE.0.0.OR.
     &            P4M_CMIO(1,JJ).NE.0.0) THEN
                 muPHI(j) = PI + ATAN2(-P4M_CMIO(2,JJ),-P4M_CMIO
     &                      (1,JJ))
              ELSE
                WRITE (6,*) 'This 4-vector sucks! Cannot calculate phi'
              ENDIF
              mustat(j) = STAT_CMIO(JJ)
              muzv(j) = MUZV_CMIO(JJ)
              mud0(j) = D0_CMIO(JJ)
              mudelz(j) = MUZV_CMIO(JJ) - Z0_CMIO(JJ)
              mudelx(j) = 0.0
              muiso(j) = ISOL_CMIO(JJ)                
              mutiso(j) = SUMPT_CMIO(JJ)
              mucray(j) = COS_CMIO(JJ)
              muidwrd(j) = MIO_WRD(JJ)
              muiswrd(j) = MIO_ISO(JJ)          
              muztrk(j) = z0_cmio(jj)   ! z of electron track at origin
              mufid(j)  = fid_cmio(jj)  ! word telling how fiducial mu is
C            = -1: Error in propagation
C            =  0: Muon in good CMU region
C            =  1: Muon falls in 90 degree crack
C            =  2: Muon fall into phi crack in central region
C            =  3: Muon in Chimney
C            =  4: Muon not in central region but still in good CTC region
C            =  5: Muon falls into phi crack in good, non-central CTC region
C            =  6: Muon detector eta too high for good CTC region
              mudet(j)  = 0 ! CMIOs don't leave hits in muon chambers by definition
              mubte(j)  = bte_cmio(jj)  ! border tower energy
              munasl(j) = nasl_cmio(jj) ! nb of axial superlayers hit by mu
              munssl(j) = nssl_cmio(jj) ! nb of stereo superlayers 
              mutime(j) = HTDC_CMIO(JJ) ! had tdc time of muon tower
              muvtxocc(j) = VTXOCC_CMIO(JJ) !VTX occupancy of muon track
              mudteta(j) = DTETA_CMIO(JJ) !detector eta of muon at CES
           ENDIF  !MIOINDEX
        ENDDO   !J

C  get the taus and fill the 4-vector,charge,et,detector eta,phi, status

        call sortzv(stat_tauo,tauind,ntau,1,1,0)

        DO J = 1,ntau
           JJ = TAUIND(J)
           tp4(1,j) = TAU_EX(JJ)
           tp4(2,j) = TAU_EY(JJ)
           tp4(3,j) = TAU_EZ(JJ)
           tp4(4,j) = TAU_E(JJ)
           tet(j) = TAU_ET(JJ)
           tq(j) = TAU_CHRG(JJ)
           tzv(j) = TAU_ZVERT(JJ)
           tdteta(j) = TAU_DETA(JJ)
           teveta(j) = TAU_ETA(JJ)                       
           tphi(j) = TAU_PHI(JJ)
           tpt(j) = TAU_PT(JJ)
           tptvis(j) = TAU_PTVIS(JJ)
           tstat(j) = STAT_TAUO(JJ)
           tdelr(j) = TAU_DELR(JJ)
           temf(j) = 0.0
           IF (TAU_ET(JJ).GT.0.0) temf(j) = TAU_ETEM(JJ)/TAU_ET(JJ)
           tpttr(1,j) = TAU_PTTR(jj,1)
           tpttr(2,j) = TAU_PTTR(jj,2)
           tpttr(3,j) = TAU_PTTR(jj,3)
           tpttr(4,j) = TAU_PTTR(jj,4)
           tmass(j) = TAU_MASS(JJ)
           tntr10(j) = TAU_NTR10(JJ)
           tnpi0(j) = TAU_NPI0(JJ)
           tmustub(j) = TAU_MUSTB(JJ)
        ENDDO
                                            
C   initialize jet counters

        JETNUM = 0
        DO J = 1, 20
           JETINDEX(J) = 20
        ENDDO  
C
C   Get vertex information from tracks in each Jet
        CALL JET_VTX(Z_JET,DZ_JET,PT_OUT)
*        PRINT *, (Z_JET(J),J=1,ICLUS)
C
C   index the good jet bank numbers
        NUMSKIP = 0
        JSKIP(1) = 20
        JSKIP(2) = 20
        DO J = 1, ICLUS
           GOODJET = .TRUE.
           IF (IJET_ET(J).LT.0.01) GOODJET = .FALSE. !Et cut

c   bit-packed input word jet_filter set in stntuple talk-to
c
c     bit 0 set => remove jets with |eta| > 2 from jet list
c         1        remove jets associated with isolated tight electrons
c         2         "      "        "      "   loose CEM
c         3         "      "        "      "   loose PEM
c         4         "      "        "      "   isolated tight photons
c         5         "      "        "      "   isolated loose photons
c         6         "      "        "     loosely isolated loose photons
c
c   by default, jet_filter is set to 18 (16+2), which causes to remove 
c   isolated tight electrons and photons only
c
c   11/20/96 PJW change default of jet_filter to 0
  
c     Option 1/ reject jets with |eta| > 2 

           IF (iand(jet_filter,1).ne.0.and.ABS(IJET_DETA(J)).GT.2.0) 
     +         GOODJET = .FALSE.  !eta cut

           IF (GOODJET) THEN     !tight isolated electron cut
              DO K = 1, NUM_HPTE
                 etadiff = abs(ijet_deta(j)-dteta_ele(IND_HPTE(K)))
                 phidiff = abs(ijet_phi(j)-phi_ele(IND_HPTE(K)))
                 if (phidiff.gt.pi) phidiff = 2*pi - phidiff
                 if (sqrt(phidiff**2+etadiff**2).lt.cone_size .and.
     +              IAND(CEM_ISO(IND_HPTE(K)),1).EQ.0) then
                    
c Optional stuff:

c     Option 2/ remove tight isolated electrons from the jet list

                    if (iand(jet_filter,2).ne.0) goodjet = .false.
                    NUMSKIP = NUMSKIP + 1
                    IF (NUMSKIP.LE.2) JSKIP(NUMSKIP) =
     &                 JTS_ELE(IND_HPTE(K))
                 ENDIF
              ENDDO
           ENDIF

           IF (GOODJET.AND.TWO_LEPTON) THEN

c     Option 3/ veto loose CEM

              IF (DIL_TYPE(1,1).EQ.HPTE_LCEM.OR. 
     &            DIL_TYPE(1,1).EQ.HPTM_LCEM) THEN
                 K = IND_LOOSE_E(DIL_TYPE(3,1))
                 NUMSKIP = NUMSKIP + 1
                 IF (NUMSKIP.LE.2) JSKIP(NUMSKIP) = JTS_ELE(K)
                 etadiff = abs(ijet_deta(j)-dteta_ele(K))
                 phidiff = abs(ijet_phi(j)-phi_ele(K))
                 if (phidiff.gt.pi) phidiff = 2*pi - phidiff
                 if (sqrt(phidiff**2+etadiff**2).lt.cone_size.and.
     +               iand(jet_filter,4).ne.0) 
     +               goodjet = .false.
              ENDIF

C     Option 4/ veto loose PEM

              IF (DIL_TYPE(1,1).EQ.HPTE_PEM.OR.
     &            DIL_TYPE(1,1).EQ.HPTM_PEM) THEN
                 K = IND_PEM(DIL_TYPE(3,1))
                 NUMSKIP = NUMSKIP + 1
                 IF (NUMSKIP.LE.2) JSKIP(NUMSKIP) = JTS_ELE(K)
                 etadiff = abs(ijet_deta(j)-dteta_ele(K))
                 phidiff = abs(ijet_phi(j)-phi_ele(K))
                 if (phidiff.gt.pi) phidiff = 2*pi - phidiff
                 if (sqrt(phidiff**2+etadiff**2).lt.cone_size.and.
     +               iand(jet_filter,8).ne.0) 
     +               goodjet = .false.
              ENDIF
           ENDIF

c     Option 5/ remove isolated tight photons

           if (goodjet.and.jet_filter.ge.16) then
              k = 1
              do while(k.le.npho.and.pstat(k).gt.5000.)
                 if (delta_r(ijet_deta(j),ijet_phi(j),pdteta(k),pphi(k))
     +               .lt.cone_size.and.iand(jet_filter,16).ne.0) 
     +               goodjet = .false.
                 k = k + 1
              end do

C
C     Option 6/ remove isolated loose photons
C Note: This is now redundant with option 5 because there are no loose 
C   central photons.   11/12/96 DT & PJW.
C   
C

              if (k.le.npho.and.jet_filter.ge.32) then
                 do while (k.le.npho.and.pstat(k).gt.5000.)
                    if (delta_r(ijet_deta(j),ijet_phi(j),
     +                  pdteta(k),pphi(k)).lt.cone_size.and.
     +                  iand(jet_filter,32).ne.0) goodjet = .false.
                    k = k + 1
                 end do

c     Option 7/ remove loosely isolated loose photons

                 if (k.le.npho.and.jet_filter.ge.64) then
                    do while (k.le.npho.and.pstat(k).gt.3000.)
                       if (delta_r(ijet_deta(j),ijet_phi(j),      
     +                     pdteta(k),pphi(k)).lt.cone_size.and.
     +                     iand(jet_filter,64).ne.0) goodjet = .false.
                       k = k + 1
                    end do 
                 endif
              endif
           endif 
                                            

C  End of jet selection cuts. Count it if it's good.

           IF (GOODJET) THEN
              JETNUM = JETNUM + 1
              JETINDEX(JETNUM) = J
           ENDIF
        ENDDO       !ICLUS

        njt = MIN(MAXJETS,JETNUM)
        CALL SORTZV(ijet_ET,JETINDEX,njt,1,1,1)

C          fill jet variables for the jets

        DO k = 1,njt
           jt4eta(k) = ijet_eta(jetindex(k))
           jt4phi(k) = ijet_phi(jetindex(k))
           JJ = JETINDEX(k)
           INDBTGS=0
           DO kk=1,BTNNJET
              IF(BTJETB(kk).EQ.IJETBNK(JJ)) INDBTGS=kk
           ENDDO
           IF (INDBTGS.EQ.0) THEN
              CALL ERLOGR('STNTUPLE',EREROR,0,0,
     1       'BTGS/JETINFO mismatch - tagging lost')
              INDBTGS=JJ ! just so that the following code doesn't crash
           ENDIF
           jtet(k) = IJET_ET(JJ)   ! jet Et
           jtm(k) = IJET_M(JJ)     ! jet mass
           call ucopy(IJET_P(1,JJ),jtp4(1,k),4) ! jet uncorrected 4-vector
           jteveta(k) = IJET_ETA(JJ)  ! jet event eta
           jtdteta(k) = IJET_DETA(JJ) ! jet detector eta
           jtphi(k) = IJET_PHI(JJ)  ! jet phi
           jtegrd(k) = ijet_egrd(jj) ! guard energy
           jtemf(k) = IJET_EMF(JJ)  ! jet electromagnetic fraction
           jtzvtx(k) = z_jet(jj)    ! jet vertex from tracks
           jtdzvtx(k) = dz_jet(jj)    ! jet vertex from tracks
           jt_ptout(k) = pt_out(jj)    ! sum_pt of tracks coming from other vtx 
           call ucopy(IJET_VPT(1,jj),jtvpt(1,k),20)
           jtnovpt(k) = ijet_novpt(jj)
C  do jet cluster/physics object correspondence (bit encoded)
C         0 =  no correspondence
C bit     0    MET
C         1    tau 3
C         2    tau 2
C         3    tau 1
C         4    muon 3
C         5    muon 2
C         6    muon 1
C         7    photon 2
C         8    photon 1
C         9    electron 3
C         10   electron 2
C         11   electron 1
         
           JTWORD(k) = 0
           if (ijet_et(jj).gt.0.01) then

C          electrons

             do j=1,min(3,nel)
                etadiff = abs(ijet_deta(jj)-dteta_ele(eleind(j)))
                phidiff = abs(ijet_phi(jj)-phi_ele(eleind(j)))
                if (phidiff.gt.pi) phidiff = 2*pi - phidiff
c               if (jts_ele(eleind(j)).eq.ijetbnk(jj))
                if (sqrt(phidiff**2+etadiff**2).lt.cone_size)
     &             jtword(k) = ibset(jtword(k),12-j)
             end do

C          photons

               do i = 1,min(2,npho)
                  etadiff = abs(ijet_deta(jj)-phodeta(phoind(i)))
                  phidiff = abs(ijet_phi(jj)-phophi(phoind(i)))
                  if (phidiff.gt.pi) phidiff = 2*pi - phidiff
                  if (sqrt(phidiff**2+etadiff**2).lt.cone_size)
     &                  jtword(k) = ibset(jtword(k),9-i)
               end do

C           muons

            do j=1,min(3,nmu)
               if (muoind(j).gt.numcmuo) then
                  trkflag = jets_trks(trks_cmio(muoind(j)-numcmuo),
     &                     ijetbnk(jj),cone_size,prim)
                  if (trkflag.eq.0) jtword(k) = ibset(jtword(k),7-j)
               else 
                  trkflag = jets_trks(trks_cmuo(muoind(j)),
     &                     ijetbnk(jj),cone_size,prim)
                  if (trkflag.eq.0) jtword(k) = ibset(jtword(k),7-j)
               endif
            end do

C     taus            

            do j = 1,min(3,ntau)
               if (tau_jetsbn(tauind(j)).eq.ijetbnk(jj))
     &            jtword(k) = ibset(jtword(k),4-j)
            end do

C      MET4 (within cone_size of phi)

             phidiff = abs(metphi(4) - ijet_phi(jj))
             if (phidiff.gt.pi) phidiff = 2*pi - phidiff
             if (phidiff.lt.cone_size) jtword(k) = ibset(jtword(k),0)
          endif
          jtcorfm(k) = JCORF(JJ,mcorindex)   ! jet energy correction factor
                                             ! for mass analysis
          jtcorfd(k) = JCORF(JJ,dcorindex)   ! jet energy correction factor

C        AA corrections....

c     Figure out what class index (aa_index) the jet belongs to :
c               1 : generic b-jet (default assumption)
c               2 : b-jet with SLT electron
c               3 : b-jet with SLT muon
c               4 : generic q-jet from W
               
C     tagger word for each jet
               
         TAG_WRD = 0
         aa_index = 1

C     1/ Find out now about SLT tag for each jet

         do j=1,5
            if (btnele(j).gt.0) then
               if (qtrkun(btnele(j),0).eq.0) then
                  etadiff = abs(trksp4(6) - IJET_ETA(JJ))
                  phidiff = abs(trksfp(5) - IJET_PHI(JJ))
                  if (phidiff.gt.pi) phidiff = 2*pi - phidiff

c    Jet tagged with electron!

                  if (sqrt(etadiff**2+phidiff**2).le.cone_size) THEN
                     tag_WRD = IBSET(TAG_WRD,1)
                     aa_index = 2
                  endif
               endif
            endif
            if (btnmuo(j).gt.0) then
               if (qtrkun(btnmuo(j),0).eq.0) then
                  etadiff = abs(trksp4(6) - IJET_ETA(JJ))
                  phidiff = abs(trksfp(5) - IJET_PHI(JJ))
                  if (phidiff.gt.pi) phidiff = 2*pi - phidiff
                                                                               
c    Jet tagged with muon!

                  if (sqrt(etadiff**2+phidiff**2).le.cone_size) THEN
                     tag_WRD = IBSET(TAG_WRD,2)
                     aa_index = 3
                  endif
               endif
            endif
         end do

C    Get AA correction factor

         pspm(1) = IJET_PHI(JJ)
         pspm(2) = (1. - exp(-2.*IJET_ETA(JJ))) / 2.
     +                   / exp(-IJET_ETA(JJ))
         pspm(3) = jcorf(jj,mcorindex) * sqrt(ijet_p(1,jj)**2 +
     +                   ijet_p(2,jj)**2 + ijet_p(3,jj)**2)
         pspm(4) = dummy
         call tmcorr_aa(aa_index,pspm,pcor,sigpcor,dummy)
         jtcorfa(k) = 1.0
         if (abs(pcor).lt.2000.0) jtcorfa(k) = pcor / pspm(3)
         jtntr(k) = IJET_NTR(JJ)  ! # of CTC tracks in jet
         jtntrp(k) = ngdpos(jj)   ! # of positive-impact-parameter SVX-tracks
         jtep(k) = IJET_EP(JJ)   !(jet energy)/(highest track p)
         jtnjtrk0(k) = NJTRK0(JJ) !good tracks in the jet from SECVTX

C tagger word for each jet

         IF (JVCALL) THEN
            TAG = JVTAG(indbtgs)   ! JETVTX tag flag
         ELSE 
            TAG = 0
         ENDIF
         IF (TAG.GT.0) TAG_WRD = IBSET(TAG_WRD,4)

         TAG = 0
         IF (JPCALL) THEN
            TAG = JPTAG(indbtgs)   ! JPBTAG tag flag
         ELSE
            TAG = 0
         ENDIF
         IF (TAG.GT.0) TAG_WRD = IBSET(TAG_WRD,3)
                
         TAG = 0
         IF (SCCALL) THEN
            TAG = SCTAG(indbtgs)   ! SECVTX TAG FLAG
         ELSE
            TAG = 0
         ENDIF
         IF (TAG.GT.0) TAG_WRD = IBSET(TAG_WRD,5)

C   test for SECVTX taggability

         IF (IJET_ET(JJ).GE.15.0 .and.
     &      ABS(IJET_DETA(JJ)).LE.2.0 .and.
     &      NJTRK0(JJ).GE.2) TAG_WRD = IBSET(TAG_WRD,0)
        
         jttag(k) = TAG_WRD
          
C background probabilities for each tagger
        
         IF (JVCALL) THEN
            jvbpb(k) = JVM1BG(indbtgs)   ! JETVTX bkg prob
         ELSE
            jvbpb(k) = -1
         ENDIF

         IF (JPCALL) THEN
            jpbpb(k) = JPM1BG(indbtgs)   ! JPBTAG bkg prob
         ELSE
            jpbpb(k) = -1
         ENDIF

         IF (SCCALL) THEN
            call scbg(indbtgs,scbpb(k),scnpb(k))  ! SECVTX bkg prob
         ELSE
            scbpb(k) = -1
            scnpb(k) = -1
         ENDIF

C  pos/neg JPBTAG probabilities

         IF (JPCALL) THEN
            jpppb(k) = JPPRBP(indbtgs)   ! JPBTAG + prob
         ELSE
            jpppb(k) = -1
         ENDIF

         IF (JPCALL) THEN
            jpnpb(k) = JPPRBM(indbtgs)   ! JPBTAG - prob
         ELSE
            jpnpb(k) = -1
         ENDIF

C TAU BACKGROUNDS
C	JTAU1(K)=
C	JTAU2(K)=

      ENDDO  ! k jet index
C******************************************************************************
C
C
C MISSING ET INFORMATION: Everything comes from EVTINFO.INC          
C      MET corrected for jets, muons, electrons from W's
C
C  Do the MET corrections BEFORE we find the big cone jets to make sure 
C  it is done with cone 0.4 jets.

c       call ucopy(met,miset,5)
c       call ucopy(metphi,misetphi,5)
       METX = MET(4)*COS(METPHI(4))
       METY = MET(4)*SIN(METPHI(4))
       MIN_ET = 10.0
       MAX_ETA = 2.0
       CALL MY_METJET(JSKIP(1),JSKIP(2),MIN_ET,MAX_ETA,METX,METY,
     &             METJETX,METJETY,METPHI1,NJ,PARNUM)
       metjt = SQRT(MAX(0.0,METJETX**2+METJETY**2))
       metjtphi = METPHI1
       
       call get_metc(htc,metc,metcphi,met(4),metphi(4))       
       call get_metcf(htcf,metcf,metcfphi,met(4),metphi(4))       

C  make a track cluster near MET and find eta,phi,pt

      CALL FALSE_MET(ETATRKS,PHITRKS,PTTRKS,SEEDPT,5.0,0.6,0.4,METPHI1)
 
C  SUMET, MET significance, Ht, out-of-time energy

      sumet = ETTOT
      sumetjet = 0.0            !sumet from jets
      DO J = 1,MIN(MAXJETS,JETNUM)
         sumetjet = sumetjet + IJET_ET(JETINDEX(J))
      ENDDO

      IF (ETTOT.GT.0.0) THEN
         metsig = SQRT(MAX(0.0,METX**2+METY**2))/SQRT(ETTOT)
      ELSE
         metsig = 0.0
      ENDIF
      etout = EOTIME

C******************************************************************************
C
C
C   Begin Big Jets
C
C cone of 0.7 jets, completely redo the Jets using bigger cone size

      if (big_cone.eq.1) cone_size=0.7
      if (big_cone.eq.2) cone_size=0.4
      if (big_cone.eq.3) cone_size=1.0
      CALL JETINFO(big_cone)

C     initialize jet counters

      JETNUM = 0
      DO J = 1, 20
         JETINDEX(J) = 20
      ENDDO  

C     index the good jet bank numbers

      DO J = 1, ICLUS
         GOODJET = .TRUE.
         IF (IJET_ET(J).LT.1.0) GOODJET = .FALSE. !Et cut

C  End of jet selection cuts. Count it if it's good.

         IF (GOODJET) THEN
            JETNUM = JETNUM + 1
            JETINDEX(JETNUM) = J
         ENDIF
      ENDDO       !ICLUS

      nj1 = min(MAXJETS,JETNUM)
      CALL SORTZV(ijet_ET,JETINDEX,nj1,1,1,1)
      DO K = 1,nj1
         JJ = JETINDEX(K)
         j1et(k) = IJET_ET(JJ)   ! jet Et
         j1m(k)  = IJET_M(JJ)    ! jet mass
         call ucopy(IJET_P(1,JJ),j1p4(1,k),4)  ! jet 4-vector
         j1eveta(k) = IJET_ETA(JJ)  ! jet event eta
         j1dteta(k) = IJET_DETA(JJ) ! jet detector eta
         j1phi(k)   = IJET_PHI(JJ)  ! jet phi
         j1egrd(k)  = ijet_egrd(jj) ! guard energy

C  do jet cluster/physics object correspondence (bit encoded)
C     0 =  no correspondence
C bit     0    MET
C         1    tau 3
C         2    tau 2
C         3    tau 1
C         4    muon 3
C         5    muon 2
C         6    muon 1
C         7    photon 2
C         8    photon 1
C         9    electron 3
C         10   electron 2
C         11   electron 1
       
         j1word(k) = 0
         if (ijet_et(jj).gt.0.01) then
         
C          electrons

            do j=1,min(3,nel)
               if (et_ele(eleind(j)).gt.0.0) then
                  etadiff = abs(ijet_deta(jj)-dteta_ele(eleind(j)))
                  phidiff = abs(ijet_phi(jj)-phi_ele(eleind(j)))
                  if (phidiff.gt.pi) phidiff = 2*pi - phidiff    
                  if (sqrt(etadiff**2+phidiff**2).lt.cone_size)
     &               j1word(k) = ibset(j1word(k),12-j)
               endif
            end do
              
C          photons

            do j = 1,nel
               do i=1,min(2,npho)
                  if (Phobnk(phoind(i)).eq.bnk_ele(j) .and.
     &               et_ele(j).gt.0.0) then
                     etadiff = abs(ijet_deta(jj)-dteta_ele(j))
                     phidiff = abs(ijet_phi(jj)-phi_ele(j))
                     if (phidiff.gt.pi) phidiff = 2*pi - phidiff
                     if (sqrt(etadiff**2+phidiff**2).lt.cone_size)
     &                  j1word(k) = ibset(j1word(k),9-i)
                  endif
               end do
            enddo

C           muons

            do j=1,min(3,nmu)
               if (muoind(j).gt.numcmuo) then
                  trkflag = jets_trks(trks_cmio(muoind(j)-numcmuo),
     &                      ijetbnk(jj),cone_size,prim)
                  if (trkflag.eq.0) j1word(k) = ibset(j1word(k),7-j)
               else
                  trkflag = jets_trks(trks_cmio(muoind(j)),ijetbnk(jj),
     &                   cone_size,prim)
                  if (trkflag.eq.0) j1word(k) = ibset(j1word(k),7-j)
               endif
            end do

C     taus 

            do j=1,min(3,ntau)
               if (tau_et(tauind(j)).gt.0.0) then
                  etadiff = abs(ijet_deta(jj)-tau_deta(tauind(j)))
                  phidiff = abs(ijet_phi(jj)-tau_phi(tauind(j)))
                  if (phidiff.gt.pi) phidiff = 2*pi - phidiff
                  if (sqrt(etadiff**2+phidiff**2).lt.cone_size)
     &               j1word(k) = ibset(j1word(k),4-j)
               endif
            end do        

C      MET4 (within cone_size of phi)

             phidiff = abs(metphi(4) - ijet_phi(jj))
             if (phidiff.gt.pi) phidiff = 2*pi - phidiff    
             if (phidiff.lt.cone_size) j1word(k) = ibset(j1word(k),0)
          endif   !j1word(k)

          J1JTS(K) = 0
          if (ijet_et(jj).gt.0.01) then

C          cone of 0.4 jets

             do j = 1,njt
                etadiff = abs(jt4eta(j)-ijet_eta(jj))
                phidiff = abs(jt4phi(j)-ijet_phi(jj))
                if (phidiff.gt.pi) phidiff = 2*pi - phidiff
                if (sqrt(etadiff**2 + phidiff**2).lt.cone_size)
     &             j1jts(k) = ibset(j1jts(k),j-1)
             end do
          endif

          j1corfm(k) = JCORF(JJ,mcorindex)   ! jet energy correction factor
                                             ! for mass analysis
          j1corfd(k) = JCORF(JJ,dcorindex)   ! jet energy correction factor
                                             ! for dilepton analysis
       ENDDO ! k jet1 index
       
      
               
C  tagging summary word: from BTGSCM.INC
C  0   E SLT
C  1   MU SLT
C  2   JETVTX TAG
C  3   2 JETVTX TAGS
C  4   JPBTAG TAG
C  5   2 JPBTAG TAGS
C  6   DPHI TAG
C  7   2 DPHI TAGS
C  8   SECVTX TAG
C  9   2 SECVTX TAGS
           
      btgwrd = BTGSUM
          
C TAGGING INFORMATION: from SECVTX.INC
C              primary vertex location
             
      call ucopy(prim,vxprim,3)
      
      TAGNUM = 0
      DO J = 1,4
         IF (ABS(TAGSEC(J)).EQ.1) THEN
            TAGNUM = TAGNUM + 1
            lxy(tagnum) = LXYSEC(J)
            lxyerror(tagnum) = LXYERR(J)
            ctau(tagnum) = CTAUSEC(J)
            tagchi(tagnum) = CHI2SEC(J)
         ENDIF
      ENDDO   !J
      ntag = tagnum

C VTVZ bank information: class, zvert, and sum(abs(pttracks)) of primaries

      vzev = jwb_zvertex()
      vclassev = jwb_zclass()

      bnknum = 1
      istat = blocat(iw,'VTVZ',bnknum,sindx,sdindx)
      if (istat.ne.yesucc) then
         CALL ERLOGR('STNTUPLE',EREROR,0,0,
     &               'unable to find VTVZ bank')
      else
         dz0cut = zmatch
         call jwb_vxpest(xseed,cvseed,npvert,blinfo,qint,dz0cut)
         do j = 1,npvert
               vclass(j) = IW(SDINDX+(J-1)*LVTVZ+5)
               vz(j) = RW(SDINDX+(J-1)*LVTVZ+6)
               vq(j) = qint(j)
         enddo
      endif
         
C    invariant mass of top two leptons for each species

      EEMASS = 0.0
      if (nel.ge.2) then
         EEMASS = EEMASS + (P4E_ELE(4,ELEIND(1))/ CORFAC(ELEIND(1))
     &          + P4E_ELE(4,ELEIND(2))/ CORFAC(ELEIND(2)))**2
         EEMASS = EEMASS - (P4E_ELE(3,ELEIND(1))/ CORFAC(ELEIND(1))
     &          + P4E_ELE(3,ELEIND(2))/ CORFAC(ELEIND(2)))**2
         EEMASS = EEMASS - (P4E_ELE(2,ELEIND(1))/ CORFAC(ELEIND(1))
     &          + P4E_ELE(2,ELEIND(2))/ CORFAC(ELEIND(2)))**2
         EEMASS = EEMASS - (P4E_ELE(1,ELEIND(1))/ CORFAC(ELEIND(1))
     &          + P4E_ELE(1,ELEIND(2))/ CORFAC(ELEIND(2)))**2
         eemass = SQRT(MAX(0.0,EEMASS))
      endif

      mumumass = 0.
      if (nmu.ge.2) then
         IF (MUOIND(1).le.numcmuo.AND.
     +              MUOIND(2).le.numcmuo) THEN
            i = MUOIND(1) 
            j = MUOIND(2)
            mumuMASS = dimass(P4M_CMUO(1,i),P4M_CMUO(1,j))
         else IF (MUOIND(1).le.numcmuo.AND.
     +         MUOIND(2).gt.numcmuo) THEN
            i = MUOIND(1) 
            j = MUOIND(2) - numcmuo
            mumuMASS = dimass(P4M_CMUO(1,i),P4M_CMIO(1,j))
         Else IF (MUOIND(1).gt.numcmuo.AND.
     +         MUOIND(2).le.numcmuo) THEN
            i = MUOIND(1) - numcmuo
            j = muoind(2) 
            mumuMASS = dimass(P4M_CMIO(1,i),P4M_CMUO(1,j))
         Else IF (MUOIND(1).gt.numcmuo.AND.
     +         MUOIND(2).gt.numcmuo) THEN
            i = MUOIND(1) - numcmuo
            j = muoind(2) - numcmuo
            mumumass = dimass(P4M_CMIO(1,i),P4M_CMIO(1,j))
         ENDIF
      endif

      TAUMASS = 0.0 
      if (ntau.ge.2) then
      TAUMASS = TAUMASS + (TAU_E(TAUIND(1))+
     &                              TAU_E(TAUIND(2)))**2
      TAUMASS = TAUMASS - (TAU_EZ(TAUIND(1))+
     &                              TAU_EZ(TAUIND(2)))**2
      TAUMASS = TAUMASS - (TAU_EY(TAUIND(1))+
     &                              TAU_EY(TAUIND(2)))**2
      TAUMASS = TAUMASS - (TAU_EX(TAUIND(1))+
     &                              TAU_EX(TAUIND(2)))**2
      tauMASS = SQRT(MAX(0.0,TAUMASS))
      endif

C              translate phoinfo indices into eleinfo indices
C              calculate diphoton invariant mass

      phomass = 0.
      if (npho.ge.2) 
     + phoMASS = dimass(pho_p4(1,phoind(1)),pho_p4(1,phoind(2)))

c  Do the METJ1 calculation. METJ1 is corrected for everything (EY jet corr.),
c  including electron and photon CEMFIX stuff. It comes from Dave's routine,
c  which was altered to deal with jets of 0.7 cone size (instead of 0.4)

      call cor_met(metj1,metj1phi)


C QTRK bank information
      trptcut = 0.0
      ntr = 0
      nqtrk = qtrkun(0,1)
      if (nqtrk.gt.0) then
      do j = 1,nqtrk
        istat = qtrkun(j,1)
        if (istat.eq.0) then
        imsk = iand(trkspc,tpmfc)
        if (imsk.eq.tp02fc) then
        if (trksp4(5).ge.trptcut) then
        if (ntr.lt.100) then
          ntr = ntr + 1
C these are the tracks we will fill 
C fill electron pointers
          do k = 1,nel
            jj = eleind(k)
            if (trksbn.eq.trks_ele(jj)) etrind(k) = ntr 
          enddo
C fill muon pointers
          do k = 1,nmu
            jj = muoind(k)
            if (jj.gt.numcmuo) jj = jj - numcmuo
            if (trksbn.eq.trks_cmuo(jj)) mutrind(k) = ntr 
          enddo
C fill tau pointers
          do k = 1,ntau
            jj = tauind(k)
            do kk = 1,20
            if (trksbn.eq.tau_bnktr(jj,kk)) ttrind(kk,k) = ntr 
            enddo
          enddo
C find vertex pointer
          minz = 999.0
          do k = 1,npvert
            if (abs(trksfp(z0fp)-vz(k)).lt.dz0cut) then
              if (abs(trksfp(z0fp)-vz(k)).lt.minz) then
                trvind(ntr) = k
                minz = abs(trksfp(z0fp)-vz(k))
              endif
            endif
          enddo
C fill number of svx layers hit
          nsvx = 0
          do k = 0,3
           if (svxlnk(k).ne.0) nsvx=nsvx+1
          enddo
          trnsvx(ntr) = nsvx
C fill number of axial superlayers hit
          naxialsl=0
          do k=0,8,2
            if (2*ctnhqt(k).gt.nreq(k)) naxialsl=naxialsl+1
          enddo
          trnasl(ntr) = naxialsl
C fill number of stereo superlayers hit
          nstereosl=0
          do k=1,7,2
            if (2*ctnhqt(k).gt.nreq(k)) nstereosl=nstereosl+1
          enddo
          trnssl(ntr) = nstereosl

          trcot(ntr) = trksfp(ctfp)
          trcurv(ntr) = trksfp(cufp)
          trz0(ntr) = trksfp(z0fp)
          trd0(ntr) = trksfp(d0fp)
          if (trksfp(cufp).eq.0.0) then
            trq(ntr)=0.0
          else if (trksfp(cufp).gt.0.0) then
            trq(ntr) = 1.0
          else
            trq(ntr) = -1.0
          endif
          trphi0(ntr) = trksfp(p0fp)
          do k = 1,4
          trp4(k,ntr) = trksp4(k)
          enddo
          trpt(ntr) = trksp4(5)
          treta(ntr) = trksp4(6)
          call emxrt(trksfp(cufp),trksfp(d0fp),trksfp(p0fp),
     &                   trksfp(ctfp),trksfp(z0fp),0,xcal)
          if (xcal(1).lt.999.0) then
          trexeta(ntr) = prapid(xcal)
          trexphi(ntr) = getphi(xcal)
          else
          trexeta(ntr) = 999.0
          trexphi(ntr) = 999.0
          endif
          istat = qtrkbm(100.0,bcq,bcpx,bcpy,bcpz,bcz0,bcd0)
          if (istat.eq.0) then
            minz = 999.0
            do k = 1,npvert
              if (abs(bcz0-vz(k)).lt.dz0cut) then
                if (abs(bcz0-vz(k)).lt.minz) then
                  trbcvind(ntr) = k
                  minz = abs(bcz0-vz(k))
                endif
              endif
            enddo
            trbcz0(ntr) = bcz0
            trbcd0(ntr) = bcd0
            trbcq(ntr) = real(bcq)
            trbcp4(1,ntr) = bcpx
            trbcp4(2,ntr) = bcpy
            trbcp4(3,ntr) = bcpz
            trbcp4(4,ntr) =sqrt(max(0.0,bcpx*bcpx+bcpy*bcpy+bcpz*bcpz)) 
            trbcpt(ntr) =sqrt(max(0.0,bcpx*bcpx+bcpy*bcpy))
            bcp(1) = bcpx
            bcp(2) = bcpy
            bcp(3) = bcpz
            trbceta(ntr) = prapid(bcp)
          else
            trbcvind(ntr) = 0
            trbcz0(ntr) = -999.0
            trbcd0(ntr) = -999.0
            trbcq(ntr) = -999.0
            do k = 1,4
            trbcp4(k,ntr) = -999.0
            enddo
            trbcpt(ntr) = -999.0
            trbceta(ntr) = -999.0
          endif !qtrkbm istat
        endif !ntr.le.100
        endif !trptcut
        endif !imsk for 3D tracks
        endif !qtrkun istat
      enddo   !loop over qtrk tracks
      endif   !nqtrk.gt.0

      do j = 1,ntr
        trackiso4 = 0.0
      do k = 1,nqtrk
        istat = qtrkun(k,1)
        if (istat.eq.0) then
        imsk = iand(trkspc,tpmfc)
        if (imsk.eq.tp02fc) then
         call emxrt(trksfp(cufp),trksfp(d0fp),trksfp(p0fp),
     &                   trksfp(ctfp),trksfp(z0fp),0,xcal)
         exeta = prapid(xcal)
         exphi = getphi(xcal)
         phidiff = abs(trexphi(j)-exphi)
         if (phidiff.gt.pi) phidiff = 2*pi - phidiff
         etadiff = abs(trexeta(j)-exeta)
         deltar = sqrt(phidiff*phidiff+etadiff*etadiff)
         if (deltar.lt.0.4) trackiso4 = trackiso4 + trksp4(5)
        endif !imsk
        endif !istat
      enddo
        if (trpt(j).ne.0.0) then
          triso4(j) = (trackiso4-trpt(j))/trpt(j)
        else
          triso4(j) = 999.0
        endif
      enddo

      CALL HFNT(1)  ! fills CWN

      ENDIF ! ANGBO STUFF...
C
      RETURN
      END


C=======================================================================
      SUBROUTINE BOOK_STNTUPLE
C=======================================================================
      IMPLICIT NONE
C
$$INCLUDE 'stntuple$source:exo_phoinfo.inc'
$$INCLUDE 'C$INC:EVTINFO.INC'         
$$INCLUDE 'stntuple$source:STNTUPLE.INC'
$$INCLUDE 'c$inc:eleinfo.inc'
$$INCLUDE 'c$inc:mioinfo.inc'
$$INCLUDE 'stntuple$source:stntuple_var.INC'
$$INCLUDE 'stntuple$source:stntuple_TALK.INC'
$$include 'C$inc:tfzinfo.inc'
$$include 'stntuple$source:trigger_stuff.inc'

        INTEGER ANGHAC                    !A_C HISTO STUFF
        INTEGER ANGHBO                    !A_C HISTO STUFF
        INTEGER ANGTH4                    !A_C HISTO STUFF
C
C       LOCAL VARIABLES
C
        INTEGER NID
        INTEGER BUFSIZ
        CHARACTER*32 TOPDIR
        CHARACTER*32 FILE_NAME
        CHARACTER*32 TITLE     !TITLE FOR NTUPLE
        CHARACTER*1 OPNOPT
        INTEGER STATUS
C 
        INTEGER I               ! Used to increment ntuples...
        integer ier
C
        STATUS=ANGTH4(1,TOPDIR,FILE_NAME,OPNOPT)
        BUFSIZ=40000
C  PJW 12/8/97 Make this a parameter in stntuple.inc MAXJETS = 20

C   Calling HBOOK to construct the ntuple with the array of variables
C    VAR matched with the array of ntuple field names CHAR, for 1 event

        NID  = 1
        TITLE = 'UC standard ntuple version 5'
        call hbnt(nid,title,' ')
        call hbset('BSIZE',4095,ier)   ! best if BSIZE > # of events
c                                      ! but BSIZE * # of ntuple var. < NWPAWC
c
c  General event info
c     
        call hbname(nid,'general',version,
     +'VERSION:i,RUN:i,EVENT:i,ILUM:R')
        call hbname(nid,'general',trigbits,'TRIGBITS:i')
        call hbname(nid,'general',goodrun,
     +'GOODRUN:l,BRCODE:i,GOODTRIG:l,TRIGWORD:i,NJET10,NJET15')
        call hbname(nid,'general',njet20,
     +'NJET20,NJET25,NHPTE,NLCENT,NLPLUG,NHPTM')
        call hbname(nid,'general',ncmio,
     +'NCMIO,NGTAU,NLEPTON,ETOUT')
c
c  Jets, cone of 0.4
c
       if (.not.no_jt) then
       call hbname(nid,'jets',njt,'NJT[0,20]:i')
       call hbname(nid,'jets',jtet,
     +'JTET(NJT):r,JTM(NJT):r,JTP4(4,NJT):r,JTEVETA(NJT):r,'//
     +'JTDTETA(NJT):r')
        call hbname(nid,'jets',jtphi,
     +'JTPHI(NJT):r,JTEMF(NJT):r,JTWORD(NJT):i,JTCORFM(NJT):r,'//
     +'JTCORFD(NJT):r')
        call hbname(nid,'jets',jtcorfa,'JTCORFA(NJT):r')
        call hbname(nid,'jets',jtntr,'JTNTR(NJT):i')
        call hbname(nid,'jets',jtntrp,'JTNTRP(NJT):i')
        call hbname(nid,'jets',jtep,'JTEP(NJT):r')
        call hbname(nid,'jets',jtnjtrk0,'JTNJTRK0(NJT):i')
        call hbname(nid,'jets',jttag,'JTTAG(NJT):i')
        call hbname(nid,'jets',jtegrd,'JTEGRD(NJT):r')
        call hbname(nid,'jets',jvbpb,'JVBPB(NJT):r')
        call hbname(nid,'jets',JPBPB,'JPBPB(NJT):r')
        call hbname(nid,'jets',SCBPB,'SCBPB(NJT):r')
        call hbname(nid,'jets',SCNPB,'SCNPB(NJT):r')
        call hbname(nid,'jets',JPPPB,'JPPPB(NJT):r')
        call hbname(nid,'jets',JPNPB,'JPNPB(NJT):r')
        call hbname(nid,'jets',JTAU1,'JTAU1(NJT):r')
        call hbname(nid,'jets',JTAU2,'JTAU2(NJT):r')
        call hbname(nid,'jets',JTZVTX,'JTZVTX(NJT):r')
        call hbname(nid,'jets',JTDZVTX,'JTDZVTX(NJT):r')
        call hbname(nid,'jets',JT_PTOUT,'JT_PTOUT(NJT):r')
        call hbname(nid,'jets',JTVPT,'JTVPT(20,NJT):r')
        call hbname(nid,'jets',JTNOVPT,'JTNOVPT(NJT):r')
       endif
c
c  Jets, cone of 1.0 or 0.7
c
       if (.not.no_j1) then
       call hbname(nid,'jt1s',nj1,'NJ1[0,20]:i')
       call hbname(nid,'jt1s',j1et,
     +'J1ET(NJ1):r,J1M(NJ1):r,J1P4(4,NJ1):r,J1EVETA(NJ1):r,'//
     +'J1DTETA(NJ1):r')
        call hbname(nid,'jt1s',j1phi,
     +'J1PHI(NJ1):r,J1WORD(NJ1):i,J1JTS(NJ1):i,J1CORFM(NJ1):r,'//
     +'J1CORFD(NJ1):r,j1egrd(nj1):r')
       endif
c
c     MET
c
      if (.not.no_met) then
      call hbname(nid,'met',met,'MET(5):r')
      call hbname(nid,'met',metphi,'METPHI(5):r')
      call hbname(nid,'met',metjt,'METJT:r,METJTPHI:r')
      call hbname(nid,'met',metj1,'METJ1:r,METJ1PHI:r')
      call hbname(nid,'met',metc,'metc:r,metcphi:r')
      call hbname(nid,'met',htc,'htc:r')
      call hbname(nid,'met',metcf,'metcf:r,metcfphi:r')
      call hbname(nid,'met',htcf,'htcf:r')
      call hbname(nid,'met',etatrks,'ETATRKS,PHITRKS,PTTRKS,SEEDPT')
      call hbname(nid,'met',sumet,'SUMET,SUMETJET,METSIG:r')
      endif
c
c  Electrons
c
      if (.not.no_el) then
       call hbname(nid,'electron',nel,'NEL[0,10]:i')
       call hbname(nid,'electron',ep4,
     +'EP4(4,NEL),EETCOR(NEL),EET(NEL),ECHARGE(NEL),EDTETA(NEL),'//
     +'EEVETA(NEL)')
       call hbname(nid,'electron',ephi,
     +'EPHI(NEL),EPT(NEL),ESTAT(NEL),EZV(NEL),EEP(NEL),EHADEM(NEL)')
        call hbname(nid,'electron',exces,
     +'EXCES(NEL),EDELX(NEL),EZCES(NEL),EDELZ(NEL),ECHIS(NEL),'//
     +'ECHIW(NEL)')
        call hbname(nid,'electron',eiso,
     +'EISO(NEL),ETISO(NEL),EIDWRD(NEL):i,EISWRD(NEL):i,'//
     +'ECONWRD(NEL):i')
        call hbname(nid,'electron',eztrk,
     +'EZTRK(NEL),EFID(NEL):i,EDET(NEL):i,EBTE(NEL),ENASL(NEL):i,'//
     +'ENSSL(NEL):i')
        call hbname(nid,'electron',eptrk,
     +'eptrk(nel),elshr2(nel),echi3(nel),echid(nel),evtxocc(nel),'//
     +'etime(nel),etrind(nel):i')
      endif
c
c  Muons
c
      if (.not.no_mu) then
       call hbname(nid,'muons',nmu,'NMU[0,15]:i')
       call hbname(nid,'muons',murawpt,'MURAWPT(NMU):r')
       call hbname(nid,'muons',mup4,
     +'MUP4(4,NMU):r,MUPT(NMU):r,MUQ(NMU):r,MUHAD(NMU):r,MUEM(NMU):r')
       call hbname(nid,'muons',mueveta,
     +'MUEVETA(NMU):r,MUPHI(NMU):r,MUSTAT(NMU):r,MUZV(NMU):r,'//
     +'MUD0(NMU):r')
       call hbname(nid,'muons',mudelz,
     +'MUDELZ(NMU):r,MUDELX(NMU):r,MUISO(NMU):r,MUTISO(NMU):r,'//
     +'MUCRAY(NMU):i')
       call hbname(nid,'muons',muidwrd,
     +'MUIDWRD(NMU):i,MUISWRD(NMU):i')
       call hbname(nid,'muons',muztrk,
     +'MUZTRK(NMU):r,MUFID(NMU):i,MUDET(NMU):i,MUBTE(NMU):r,'//
     +'MUNASL(NMU):i,MUNSSL(NMU):i,muchz0(nmu):r,muchd0(nmu):r,'//
     +'mutime(NMU):r,muvtxocc(NMU):r,mudteta(NMU):r,mutrind(nmu):i')
      endif
c
c  Taus
c
      if (.not.no_tau) then
       call hbname(nid,'taus',ntau,'NTAU[0,10]:i')
       call hbname(nid,'taus',tp4,
     +'TP4(4,NTAU):r,TET(NTAU):r,TQ(NTAU):i,TZV(NTAU):r,'//
     +'TDTETA(NTAU):r,TEVETA(NTAU):r')
        call hbname(nid,'taus',tphi,
     +'TPHI(NTAU):r,TPT(NTAU):r,TPTVIS(NTAU):r,TSTAT(NTAU):r,'//
     +'TDELR(NTAU):r,TEMF(NTAU):r,TMASS(NTAU):r')
       call hbname(nid,'taus',tpttr,
     +'tpttr(4,NTAU):r,tntr10(NTAU):i,tnpi0(NTAU):i,tmustub(NTAU):i')
       call hbname(nid,'taus',ttrind,'ttrind(20,ntau):i')
      endif
c
c  Dilepton analysis
c     
      if (.not.no_dil) then
        call hbname(nid,'dilepton',ndil,'NDIL')
        call hbname(nid,'dilepton',tzflag,'TZFLAG:l')
        call hbname(nid,'dilepton',dilwrd,'DILWRD:i,'//
     +'METDILEP:r,NJTDILEP,DPHIMINJ')
        call hbname(nid,'dilepton',dphiminl,
     +'DPHIMINL,DILMODE:i,DILTYPE1:i')
       call hbname(nid,'dilepton',eemass,
     +'EEMASS,MUMUMASS:r,TAUMASS,PHOMASS')
      endif      
c
c  Photons
c
      if (.not.no_pho) then
        call hbname(nid,'photons',npho,'NPHO[0,10]:i')
        call hbname(nid,'photons',ncencl,'NCENCL')
        call hbname(nid,'photons',nplugcl,'NPLUGCL')
        call hbname(nid,'photons',nphoidl,
     +'NPHOIDL,NPHOISOL,NPHOID,NPHOISO,NCMUS')
        call hbname(nid,'photons',costub,
     +'COSTUB:i,PDETECT(NPHO):i,PZV(NPHO),PET(NPHO),PETC(NPHO),'//
     +'PP4(4,NPHO)')
        call hbname(nid,'photons',pco4,
     +'PCO4(NPHO),PCESX(NPHO),PCESZ(NPHO),PCPR5PH(NPHO),'//
     +'PCPR5PS(NPHO)')
        call hbname(nid,'photons',pdteta,
     +'PDTETA(NPHO),PEVETA(NPHO),PCESE(NPHO),PPHI(NPHO),'//
     +'PCO7(NPHO),PHADEM(NPHO)')
        call hbname(nid,'photons',plshr,
     +'PLSHR(NPHO),PSTR2(NPHO),PSTRE2(NPHO),PWIR2(NPHO),'//
     +'PWIRE2(NPHO),PSUMPT4(NPHO)')
        call hbname(nid,'photons',pvtxocc,
     +'PVTXOCC(NPHO),PPT(NPHO),PN3D(NPHO):i,PCHI(NPHO),'//
     +'PCHI3X3(NPHO),PCHIETA(NPHO),PCHIPHI(NPHO),PSTAT(NPHO)')
        call hbname(nid,'photons',pboxiso,
     +'pboxiso(npho),pcesflg(npho):l,pe(npho),pceswht(npho),'//
     +'pcprwht(npho),pcesslide(npho),pwrd(npho):i,psth(npho)')
        call hbname(nid,'photons',pelind,
     +'pelind(npho):i,pchistr(npho),pchiwir(npho),pcessig(npho)')
        call hbname(nid,'photons',pcespg,
     +'pcespg(npho),pcespb(npho),pcprpg(npho),'//
     +'pcprpb(npho),ppt2(npho)')
        call hbname(nid,'photons',pntrwd3,
     +'pntrwd3(npho):i,pcescprx(npho),ptkcprx(npho),ptime(npho)')
        call hbname(nid,'photons',pvcor,
     +'pvcor(npho),plcor(npho)')
      endif
c
c  Tagging
c
      if (.not.no_tag) then
       call hbname(nid,'tagging',ntag,'NTAG[0,4]:i')
       call hbname(nid,'tagging',btgwrd,
     +'BTGWRD:i,LXY(NTAG):r,LXYERR(NTAG):r,CTAU(NTAG),TAGCHI(NTAG)')
      endif
c
c  Vertex
c
      if (.not.no_vtx) then
       call hbname(nid,'vertex',npvert,
     +'NPVERT[0,20]:i')
       call hbname(nid,'vertex',vxprim,
     +'VXPRIM(3),VCLASS(NPVERT):i,VZ(NPVERT),VQ(NPVERT),ptnov,'//
     +'vzev:r,vclassev:i')
      endif
C
C GENP VARIABLES
C
	IF(.NOT.NO_GENP) THEN
	  CALL HBNAME(NID,'GENP',NGENP,'NGENP[0,100]:I')
	  CALL HBNAME(NID,'GENP',NGID,'NGID(NGENP):I,'//
     1 	'NGN(NGENP):I,NGNP(NGENP):I,NGPID(NGENP):I,'//
     1 	'GNP4(4,NGENP):R')
	ENDIF
C
C track variables
C
         if(.not.no_trk) then
           call hbname(nid,'track',ntr,'ntr[0,100]:i')
           call hbname(nid,'track',trvind,'trvind(ntr):i')
           call hbname(nid,'track',trnsvx,'trnsvx(ntr):i')
           call hbname(nid,'track',trnasl,'trnasl(ntr):i')
           call hbname(nid,'track',trnssl,'trnssl(ntr):i')
           call hbname(nid,'track',trcot,'trcot(ntr):r')
           call hbname(nid,'track',trcurv,'trcurv(ntr):r')
           call hbname(nid,'track',trz0,'trz0(ntr):r')
           call hbname(nid,'track',trd0,'trd0(ntr):r')
           call hbname(nid,'track',trq,'trq(ntr):r')
           call hbname(nid,'track',trphi0,'trphi0(ntr):r')
           call hbname(nid,'track',trp4,'trp4(4,ntr):r')
           call hbname(nid,'track',trpt,'trpt(ntr):r')
           call hbname(nid,'track',treta,'treta(ntr):r')
           call hbname(nid,'track',trexeta,'trexeta(ntr):r')
           call hbname(nid,'track',trexphi,'trexphi(ntr):r')
           call hbname(nid,'track',trbcvind,'trbcvind(ntr):i')
           call hbname(nid,'track',trbcz0,'trbcz0(ntr):r')
           call hbname(nid,'track',trbcd0,'trbcd0(ntr):r')
           call hbname(nid,'track',trbcq,'trbcq(ntr):r')
           call hbname(nid,'track',trbcp4,'trbcp4(4,ntr):r')
           call hbname(nid,'track',trbcpt,'trbcpt(ntr):r')
           call hbname(nid,'track',trbceta,'trbceta(ntr):r')
           call hbname(nid,'track',triso4,'triso4(ntr):r')
         endif

      call hprnt(1)

c     CALL HBOOKN(NID,TITLE,NVAR1,TOPDIR,BUFSIZ,CHAR1)

      RETURN
      END

**********************************************************************
 
      real function mass(x4)   ! computes mass from 4-momentum vector
      implicit none
      real x4(4) ! 4-vector px,py,pz,E

      mass = sqrt(max(0.,x4(4)**2 - x4(1)**2 - x4(2)**2 - x4(3)**2))
      end

**********************************************************************

      real function dimass(x,y) ! computes mass of 2 objects combined
      implicit none
      real x(4),y(4),z(4),mass 

      call vadd(x,y,z,4)
      dimass = mass(z)
      end

**********************************************************************

      real function delta_r(eta1,phi1,eta2,phi2)
$$INCLUDE 'C$INC:PIVALU.INC'           ! easy as...
      etadiff = abs(eta1-eta2)
      phidiff = abs(phi1-phi2)
      if (phidiff.gt.pi) phidiff = 2*pi - phidiff
      delta_r = sqrt(phidiff**2+etadiff**2)
      end
      
**********************************************************************

      subroutine cor_met(metcor,metcorphi)
C
C Most the notes for this routine can be found in WWGG_ACCEPT.INC
C
      IMPLICIT NONE
      INTEGER I,J,arg
      INTEGER IERR
      real metcor,metcorphi
      INCLUDE 'stntuple$source:Stntuple_Var.Inc'   
      INCLUDE 'stntuple$source:WWGG_ACCEPT.INC'   
      data ET_THRESH/20./
      data ET_THRESH2/20./
      data LEP_THRESH/20./
      data IPRINT/0/
      data EE_THRESH/110./
C
C First we initialize all the variables for this particular event.
C
      CALL EVENT_INIT
C
C Here we check the overall event stuff...
C
      CALL CHECK_GOODRUNVERT
C
C If this isn't a goodrun or vertex or has ETOUT .NE. 0 then lets just get out.
C
c      IF (.NOT. GOODRUN_VERT) GOTO 999
C
C Next check for photons...
C
      CALL FIND_PHOTONS(ET_THRESH)
C
C Next check to see if there leptons in this events. This is done by calling 
C FIND_LEPTONS. The number of leptons as well as the type of leptons is kept in 
C the variable LEP_STAT.
C
      CALL FIND_LEPTONS
C
C Next we do the Jet stuff...
C 
C   Run FIND_JETS. This routine removes the photons and electrons from the jet
C   list. If events are tagged that are given a non-zero
C       TAG_STAT. TAG_STAT = (10**NGOODTAG)*I + TAG_STAT summed over all jets
C       (I) which are tagged (NGOODJET is summed after this is done). 
C    The value of NGOODJET is the number of jets in the jet list 
C       with Et(uncorr)>10, |eta_det|<2.0.
C
      CALL FIND_JETS

C Next calculate the different Corrected missing Et's. See DO_CORR_MET for 
C details of what the various CORR_MET's are...
C
      CALL DO_CORR_MET
      metcor = corr_met(3)
      metcorphi = corr_metphi(3)

  999 CONTINUE

      RETURN
      END
C
C*******************************************************************************
C
      SUBROUTINE FIND_JETS

      IMPLICIT NONE
      INCLUDE 'stntuple$source:Stntuple_Var.Inc'
      INCLUDE 'stntuple$source:WWGG_ACCEPT.INC'
      include 'stntuple$source:exo_phoinfo.inc'
C
      INTEGER PEM_ELE, PEM_ELE_MC
      INTEGER I,J,K,IJ
      REAL PI
      REAL ETADIFF,PHIDIFF, DELTAR_MIN,CONE_SIZE
      REAL DTETA_OBJ(40), PHI_OBJ(40)
      INTEGER ELE_OR_PHO(40)
      INTEGER JET_OBJ(40)

      PI = 3.14159
C
C In this routine we look to remove all Tight central electrons, Loose central
C electron, Plug electrons, Tight Central Photons and Plug photons 
C from the JET List...We only remove one jet per object. There may be two
C objects per jet, but not two jets per object. Note that sometimes because of
C a bug in JETCLU we don't find a jet...
C
C The returned information is kept in four variables: JET_ID, WHICH_JET and
C GOODJET, and GOODJET_ET. Each is indexed to the NJET ordering.
C
C JET_ID(I) Tells us the jet type. I is the JET number in the NJET list. 
C       = 0 No association
C       = 1 Tight Central Electron
C       = 2 Loose Central Electron
C       = 3 Plug Electron
C       = 4 Good central photon
C       = 5 Loose central photon (ET>10 but not above main threshold)
C       = 6 Plug photon
C
C Jets with a JET_ID()=0 have logical GOODJET() = .TRUE.
C Jets with a WHICH_JET() = 0 have logical GOODJET() = .TRUE.
C
C WHICH_JET(I) Tells us the which electron or photon. I is the JET number in
C the NJET list. The value of WHICH_JET(I) tells us the NEL number (if
C WHICH_JET(I) = 1 this is the first electron, 2 its the second electron
C assuming NEL>=2) or the NPHO number. For example if NEL=2 and NPHO=2, then if
C WHICH_JET(I) = 3 this is the first photon. 
C
C
C Loop over all electrons to find Central and Plug electrons
C
C LEPTON_STATUS:
C       0 = Nothing
C       1 = Tight Central Electron
C       2 = Loose Central Electron
C       3 = Plug Electron
C       4 = Tight Muon
C       5 = Loose Muon
C       6 = Loose CMIO
C       7 = Tight Tau
C       8 = Cheating Mu
C
C First electrons...
C
       NEL_N_NPHO = 0
       DO I = 1, NEL
         NEL_N_NPHO = NEL_N_NPHO + 1
         IF (LEPTON_STATUS(I) .LE. 3 .AND. LEPTON_STATUS(I) .GT. 0) THEN
           DTETA_OBJ(NEL_N_NPHO) = EDTETA(I)
           PHI_OBJ(NEL_N_NPHO) = EPHI(I)
           ELE_OR_PHO(NEL_N_NPHO) = 1
         ELSE
          DTETA_OBJ(NEL_N_NPHO) = -999.
          PHI_OBJ(NEL_N_NPHO) = -999.
           ELE_OR_PHO(NEL_N_NPHO) = -1
         ENDIF
       ENDDO
C
C Next identify the photons...
C
C PHO_ID() = 0 Nothing
C PHO_ID() = 1 Good central photon
C PHO_ID() = 2 Loose central photon (ET>10 but not above main threshold)
C PHO_ID() = 3 Plug photon
C
        DO I = 1, NPHO
         NEL_N_NPHO = NEL_N_NPHO + 1
         IF (PHO_ID(I) .GT. 0) THEN
          DTETA_OBJ(NEL_N_NPHO) = PDTETA(I)
          PHI_OBJ(NEL_N_NPHO) = PPHI(I)
           ELE_OR_PHO(NEL_N_NPHO) = 2
         ELSE
          DTETA_OBJ(NEL_N_NPHO) = -999.
          PHI_OBJ(NEL_N_NPHO) = -999.
           ELE_OR_PHO(NEL_N_NPHO) = -1
         ENDIF
        ENDDO

        IF (NEL_N_NPHO .NE. NEL+NPHO) WRITE (6,*) 'FUCKED UP NEL_N_NPHO'
C
C Here we do the Delta R matching...We only remove one jet per object. 
C There may be two objects per jet, but not two jets per object. 
C
        CONE_SIZE = 0.7
  123   CONTINUE
        DO I = 1, NEL_N_NPHO
          IF (DTETA_OBJ(I) .GT. -800) THEN
            DELTAR_MIN = CONE_SIZE
            JET_OBJ(I) = -999.
            DO J = 1, NJ1
              ETADIFF = ABS(J1DTETA(J)-DTETA_OBJ(I))
              PHIDIFF = ABS(J1PHI(J)  -PHI_OBJ(I))
              IF (PHIDIFF.GT.PI) PHIDIFF = 2*PI - PHIDIFF
              IF (SQRT(PHIDIFF**2+ETADIFF**2).LT.DELTAR_MIN) THEN
                DELTAR_MIN = SQRT(PHIDIFF**2+ETADIFF**2)
                JET_OBJ(I) = J
              ENDIF
            ENDDO ! Loop over all jets
C
C Check to see if we found the jet for this object...If we didn't that could
C mean that either the photon or electron was position mismeasured. For now,
C simply expand the cone size by 0.05 till it finds something...
C
            IF (JET_OBJ(I) .LE. 0 .OR. JET_OBJ(I) .GT. 20) THEN
c              WRITE (6,555) CONE_SIZE, RUN, EVENT, 
c     &            ELE_OR_PHO(I), DTETA_OBJ(I), PHI_OBJ(I)
              CONE_SIZE = CONE_SIZE + 0.05
              IF (ABS(DTETA_OBJ(I) + 1.36427) .LT. 0.0001) THEN
                WRITE (6,*) 'Assuming this is a JETCLU problem...'
                CONE_SIZE = 1.5
              ENDIF
              IF (CONE_SIZE .LT. 1.0) THEN
                GOTO 123
              ENDIF
            ELSE
              GOODJET(JET_OBJ(I)) = .FALSE. 
              WHICH_JET(JET_OBJ(I)) = I
              IF (I .LE. NEL) THEN
                JET_ID(JET_OBJ(I)) = LEPTON_STATUS(I)
              ELSE
                JET_ID(JET_OBJ(I)) = PHO_ID(I-NEL) + 3
              ENDIF
            ENDIF
          ENDIF ! Good object with this NEL_N_NPHO number??    
        ENDDO ! Loop over all objects to be removed...
C

       DO I = 1, NJ1
         IF (.NOT. GOODJET(I)) THEN
           NJNOCOR = NJNOCOR + 1
         ENDIF
       ENDDO
C
       DO I = 1, NJ1      
        IF (J1ET(I) .GT. 10..AND.ABS(J1DTETA(I)).LT.2.0.AND.
     &      GOODJET(I)) THEN
          NGOODJET = NGOODJET + 1
          GOODJET_ET(NGOODJET) = J1ET(I)*J1CORFM(I)
        ENDIF ! Good Jet passing Et and eta cuts??  
       ENDDO
       
 555  FORMAT(1X,'Didnt match this object: CONE_SIZE = ', 
     & F4.2, 2X, I5, 2X, I6, 2X, I1, 2X, F8.4,2X, F8.4)

      RETURN
      END
C
C******************************************************************************
C

      SUBROUTINE FIND_LEPTONS

      IMPLICIT NONE
      INCLUDE 'stntuple$source:STNTUPLE_VAR.INC'   
      INCLUDE 'stntuple$source:WWGG_ACCEPT.INC'   

      INTEGER I,J
      INTEGER NLEP_TOT,LEP_NUM
      REAL LEPTON_STATUS_ET(30)
      REAL EVENT_STATUS
      INTEGER NGOODMU
C
C LEPTON_STATUS():
C       0 = Nothing
C       1 = Tight Central Electron
C       2 = Loose Central Electron
C       3 = Plug Electron
C       4 = Tight Muon
C       5 = Loose Muon
C       6 = Loose CMIO
C       7 = Tight Tau
C       8 = Cheating Mu
C
C Event Status:
C
C       11 = Tight Central Electron/Tight Central Electron
C       11 = TCE/TCE
C       12 = TCE/LCE
C       13 = TCE/PE
C       14 = TCE/TCM
C       15 = TCE/LM
C       16 = TCE/LCMIO
C       17 = TCE/TT
C       41 = Tight Central Muon/Tight Central Electron (Same as 14)
C       41 = TCM/TCE
C       42 = TCM/LCE
C       43 = TCM/PE
C       44 = TCM/TCM
C       45 = TCM/LM
C       46 = TCM/LCMIO
C       47 = TCM/TT
C
c Local Variables
c ---------------
      INTEGER CEM_ELE,CMU_MUON,CEM_LOOSE,PEM_ELE,MIO_MUON,MIO_MUON_MET
      INTEGER CONV_CUT,DIL_ISO,SGL_ISO
      INTEGER PEM_ELE_MC

      CEM_LOOSE = 3891
      CONV_CUT = 3
      LEP_STAT = 0.0
      NLEP_TOT = NEL + NMU + NTAU
      DO I = 1, NLEP_TOT
        LEPTON_STATUS(I) = 0
        LEPTON_STATUS_ET(I) = 0.
      ENDDO
      LEP_NUM = 0
C
C Loop over electrons
C
      DO I = 1, NEL 
        LEP_NUM = LEP_NUM + 1
        IF (EDET(I) .EQ. 0 .AND. EETCOR(I) .GT. LEP_THRESH.AND.
     &      IAND(EIDWRD(I),CEM_LOOSE).EQ.0.AND.
     &      IAND(ECONWRD(I),CONV_CUT).EQ.0) THEN
          LEPTON_STATUS(LEP_NUM) = 2
          LEPTON_STATUS_ET(LEP_NUM) = EETCOR(I)
        ENDIF
      if (efid(I).eq.0.and.edet(I).eq.0
     +    .and.eiso(I).lt.0.1.and.(eep(I).lt.4.or.ept(I).gt.50.).and.
     +    econwrd(I).eq.0.and.elshr2(I).lt.0.2.and.echis(I).lt.10.
     +    .and.abs(edelx(I)).lt.1.5.and.abs(edelz(I)).lt.3..and.
     +    ehadem(I).le.0.055+0.00045*ep4(4,I)
     +    .AND.EETCOR(I).GT.LEP_THRESH) then 
c        IF (ESTAT(I) .GT. 9000 .AND. EETCOR(I) .GT. LEP_THRESH) THEN
          LEPTON_STATUS(LEP_NUM) = 1
          LEPTON_STATUS_ET(LEP_NUM) = EETCOR(I)
          GOOD_E = .TRUE.
          GOLD_LEPTON = .TRUE.
        ENDIF
C
C Plug electrons...
C
C        PEM_ELE = 255
        PEM_ELE = 183
        PEM_ELE_MC = 135
C
C The VTX occupancy is funny. In MC the VTX occ may or may not be filled. If,
C at GENP, we were able to associate a photon we give it a zero, an electron a
C 1, otherwise give it a -1.  We allow only electrons to pass so a given object
C CANNOT be called a photon AND and electron.
C
        IF (EDET(I) .EQ. 1 .AND. EETCOR(I) .GT. LEP_THRESH
     &    .AND. IAND(EISWRD(I),1) .EQ. 0
     &    .AND. EVTXOCC(I) .GT. 0.5) THEN
C
C Since the plug stuff isn't done correctly in QFL allow looser cuts. 
C
         IF ( (RUN .LT. 10 .AND. IAND(EIDWRD(I),PEM_ELE_MC) .EQ. 0) .OR.
     &    (IAND(EIDWRD(I),PEM_ELE) .EQ. 0)) THEN 
          GOOD_PLUG_E = .TRUE.
          LEPTON_STATUS(LEP_NUM) = 3
          LEPTON_STATUS_ET(LEP_NUM) = EETCOR(I)
         ENDIF ! Passes Plug ID??
        ENDIF ! Plug EM Cluster??
C
      ENDDO
C
C
C Loop over muons
C
c      DO I = 1, NMU          
c        LEP_NUM = LEP_NUM + 1           
c        IF (MUDET(I) .NE. 5 .AND. MUSTAT(I).GT.6000..AND. 
c     &    MUPT(I) .GT. LEP_THRESH ) THEN
c          LEPTON_STATUS(LEP_NUM) = 5
c          LEPTON_STATUS_ET(LEP_NUM) = MUPT(I)
c        ENDIF
c        IF (MUDET(I) .EQ. 5 .AND. MUSTAT(I).GT.3000..AND.
c     &    MUPT(I) .GT. LEP_THRESH.and.ABS(MUDELX(I)) .lt. 0.001 ) THEN
c          LEPTON_STATUS(LEP_NUM) = 6
c          LEPTON_STATUS_ET(LEP_NUM) = MUPT(I)
c        ENDIF
c        IF (MUSTAT(I) .GT. 7000.AND. MUPT(I) .GT. LEP_THRESH ) THEN
c          LEPTON_STATUS(LEP_NUM) = 4
c          LEPTON_STATUS_ET(LEP_NUM) = MUPT(I)
c          GOOD_MU = .TRUE.
c          GOLD_LEPTON = .TRUE.
c        ENDIF  
c      ENDDO
c
C Loop over taus
C
c      DO I = 1, NTAU
c        LEP_NUM = LEP_NUM + 1
c        IF (TSTAT(I) .GT. 4000 .AND. TET(I) .GT. LEP_THRESH) THEN
c          LEPTON_STATUS(LEP_NUM) = 7
c          LEPTON_STATUS_ET(LEP_NUM) = TET(I)
c        ENDIF  
c      ENDDO
c      IF (LEP_NUM .NE. NLEP_TOT) WRITE (6,*) 'DIDNT FIND ALL LEPTONS'
C
C****************************************************************************
C
C Now figure out what type of event this was....
C
      EVENT_STATUS = 0.
C
C Here we count the number of good leptons in this event!! Call this NGOODLEP
C
      NGOODLEP = 0
      NGOODMU = 0
      DO I = 1, 10
        LEP_ET(I) = -999.
        LEP_ID(I) = 0
        WHICH_LEP(I) = 0
      ENDDO

      DO I = 1, NLEP_TOT
        IF (LEPTON_STATUS(I) .GT. 0) THEN
          IF (LEPTON_STATUS(I) .EQ. 4 .OR. LEPTON_STATUS(I) .EQ. 5 .OR. 
     &      LEPTON_STATUS(I) .EQ. 6) NGOODMU = NGOODMU + 1
          EVENT_STATUS = EVENT_STATUS + 
     &       (10**NGOODLEP)*float(LEPTON_STATUS(I))
          NGOODLEP = NGOODLEP + 1
          WHICH_LEP(NGOODLEP) = I
          LEP_ID(NGOODLEP) = LEPTON_STATUS(I)
          LEP_ET(NGOODLEP) = LEPTON_STATUS_ET(I)
        ENDIF ! Is this a good lepton
      ENDDO  ! Loop over leptons...


      LEP_STAT = EVENT_STATUS
      IF (GOOD_MU .AND. NGOODMU .GE. 2) GOOD_MM = .TRUE. 
      IF (GOLD_LEPTON .AND. LEP_STAT .GT. 10) GOOD_LL = .TRUE.
C
C Here we treat electrons separately for Z->ee and DY finding...
C
       DO I = 1, NEL
      if (efid(i).eq.0.and.edet(i).eq.0
     +    .and.eiso(i).lt.0.1.and.(eep(i).lt.4.or.ept(i).gt.50.).and.
     +    econwrd(i).eq.0.and.elshr2(i).lt.0.2.and.echis(i).lt.10.
     +    .and.abs(edelx(i)).lt.1.5.and.abs(edelz(i)).lt.3..and.
     +    ehadem(i).le.0.055+0.00045*ep4(4,i)
     +    .AND.EETCOR(I).GT.LEP_THRESH) then 
c        IF (ESTAT(I) .GT. 9000.0 .AND.EETCOR(I) .GT. LEP_THRESH ) 
          NGOODCE = NGOODCE + 1
       endif
       ENDDO ! Loop over all ELES banks
C
      RETURN
      END

C*******************************************************************************

      subroutine DO_CORR_MET

      IMPLICIT NONE
      INCLUDE 'stntuple$source:STNTUPLE_VAR.INC'
      INCLUDE 'stntuple$source:WWGG_ACCEPT.INC'
      include 'c$inc:evtinfo.inc'

      INTEGER I,J,K
      INTEGER ITEMP
      REAL THRESHOLD, ETA_CUT
C
      REAL EX(10), EY(10)
      REAL DELEX(10,10), DELEY(10,10)
      REAL CORR_FACT(10)                ! Underlying event correction
C
C In this routine we recalculate the Met for jet corrections and muons...Also
C calculate H_T (HT_EVENT)
C
C The different Corrected missing Et's.
C
C       CORR_MET(1) = Met(1) Corrected for muons. Should be equal to Met(4)
C       CORR_MET(2) = Met(1) Corrected for muons with a looser HAD cut. 
C       CORR_MET(3) = CORR_MET(1) Corrected for Jets (96S EY) and photons.
C       Equivalent to underlying event correction = 1.0
C       CORR_MET(4) = CORR_MET(1) Corrected for Jets (96S NN) and photons.
C       Equivalent to underlying event correction = 1.0
C       CORR_MET(5) = CORR_MET(1) Corrected for Jets (96S NN), photons and an
C       overall underlying event correction = 1.5
C       CORR_MET(6) = CORR_MET(1) Corrected for Jets (96S NN) and an
C       overall underlying event correction = 1.5
C
C Note that CORR_MET(4) is the default...
C
C Corrections for Jets (Not photon or electron)
C       DELEX(1,I) = JET Corrections for EY 
C       DELEX(2,I) = JET Corrections for NN 
C Corrections for photons and electrons 
C       DELEX(3,I) = Remove Jet, add corrected electron or photon
C       DELEX(4,I) = Remove Jet, add jet
C  Where I is the overall underlying event correction 1=1.0, 2=1.5
C
C Start by initializing the CORR_MET and CORR_METPHI arrays...
C Next fill all the EX and EY arrays. Note that EX and EY are the net ET
C vector, it is the opposite direction of the MET.
C
      DO I = 1, 10
        CORR_MET(I)    = 0.0
        CORR_METPHI(I) = 0.0
        EX(I) = -MET(4)*COS(METPHI(4)) 
        EY(I) = -MET(4)*SIN(METPHI(4)) 
        DO K = 1, 10
          DELEX(I,K) = 0.0
          DELEY(I,K) = 0.0
        ENDDO
        CORR_FACT(I) = 1.0
      ENDDO
      CORR_FACT(2) = 1.5
c      HT_EVENT = 0.0
C
C Next fill the delta X and delta Y arrays for jets. If the jet is really a
C photon or electron remove the jet and add the corrected object energy.
C
C Up to three loops are used.
C       K: Underlying event correction 1=1.0, 2=1.5
C       I: Loop over jets
C       J: Loop over electrons or photons...
C
      THRESHOLD = 8.0
      ETA_CUT = 99999.
      DO K = 1, 2
       DO I = 1, NJ1
        IF (GOODJET(I)) THEN
         IF (J1ET(I) .GT. THRESHOLD.AND.
     &        ABS(J1DTETA(I)).LT.ETA_CUT) THEN
C EY Corrections
            DELEX(1,K) = DELEX(1,K) + J1P4(1,I)*(J1CORFM(I) 
     &             - CORR_FACT(K))
            DELEY(1,K) = DELEY(1,K) + J1P4(2,I)*(J1CORFM(I) 
     &             - CORR_FACT(K))
C NN Corrections
            DELEX(2,K) = DELEX(2,K) + J1P4(1,I)*(J1CORFD(I) 
     &             - CORR_FACT(K))
            DELEY(2,K) = DELEY(2,K) + J1P4(2,I)*(J1CORFD(I) 
     &             - CORR_FACT(K))

c            IF (K .EQ. 1 .and. ABS(J1DTETA(I)).LT.2.0) 
c     &            HT_EVENT = HT_EVENT + (J1ET(I)*J1CORFM(I))
         ENDIF ! Passes Et threshold, eta requirement
        ELSE
C
C NEL_N_NPHO is the number of leptons and photons in the event...is
C the number of the JET associated with the J1h JET.
C
          IF (JET_ID(I) .EQ. 0) WRITE (6,*) 
     &      'What the ***. This is a good jet'    !,idnevt
          IF (WHICH_JET(I) .EQ. 0) WRITE (6,*) 
     &      'What the ***. This is a good jet'    !,idnevt
          IF (JET_ID(I) .LE. 3) THEN
C
C Electrons
            ITEMP = WHICH_JET(I)
               DELEX(3,K) = DELEX(3,K) - CORR_FACT(K)*J1P4(1,I) 
     &             + EP4(1,ITEMP)
               DELEY(3,K) = DELEY(3,K) - CORR_FACT(K)*J1P4(2,I) 
     &             + EP4(2,ITEMP)
               DELEX(4,K) = DELEX(4,K) - (CORR_FACT(K)-1.0)*J1P4(1,I) 
               DELEY(4,K) = DELEY(4,K) - (CORR_FACT(K)-1.0)*J1P4(2,I) 
               DELEX(5,K) = DELEX(5,K) - 
     &          ((EET(ITEMP)/EETCOR(ITEMP))-1.0)*EP4(1,ITEMP)
               DELEY(5,K) = DELEY(5,K) - 
     &          ((EET(1)/EETCOR(1))-1.0)*EP4(2,ITEMP)

c               IF (K .EQ. 1) HT_EVENT = HT_EVENT + EETCOR(ITEMP)
             ELSE
C
C Photons...
C
               ITEMP = WHICH_JET(I) - NEL
               DELEX(3,K) = DELEX(3,K) -CORR_FACT(K)*J1P4(1,I) 
     &             + PP4(1,ITEMP)
               DELEY(3,K) = DELEY(3,K) -CORR_FACT(K)*J1P4(2,I) 
     &             + PP4(2,ITEMP)

               DELEX(4,K) = DELEX(4,K) -(CORR_FACT(K)-1.0)*J1P4(1,I) 
               DELEY(4,K) = DELEY(4,K) -(CORR_FACT(K)-1.0)*J1P4(2,I) 

               DELEX(5,K) = DELEX(5,K) - 
     &          ((PET(ITEMP)/PETC(ITEMP))-1.0)*PP4(1,ITEMP)
               DELEY(5,K) = DELEY(5,K) - 
     &          ((PET(ITEMP)/PETC(ITEMP))-1.0)*PP4(2,ITEMP)

Cqqq
C            write(6,*) ep4(1,ITEMP)*(PET(ITEMP)/PETC(ITEMP)), 
C     &          ep4(2,ITEMP)*(PET(ITEMP)/PETC(ITEMP)), 
C     &     PETC(ITEMP)/PET(ITEMP), delex(5,1), deley(5,1)

c               IF (K .EQ. 1) HT_EVENT = HT_EVENT + PETC(ITEMP)
               
          ENDIF ! Is this associated e or gamma??
        ENDIF ! Goodjet??
       ENDDO ! Loop over all jets...
      ENDDO ! Loop over correction factors
 
      EX(3) = CORR_FACT(1)*EX(1) + DELEX(1,1) + DELEX(3,1)
      EY(3) = CORR_FACT(1)*EY(1) + DELEY(1,1) + DELEY(3,1)

      EX(4) = CORR_FACT(1)*EX(1) + DELEX(2,1) + DELEX(3,1)
      EY(4) = CORR_FACT(1)*EY(1) + DELEY(2,1) + DELEY(3,1)

      EX(5) = CORR_FACT(2)*EX(1) + DELEX(2,2) + DELEX(3,2)
      EY(5) = CORR_FACT(2)*EY(1) + DELEY(2,2) + DELEY(3,2)

      EX(6) = CORR_FACT(2)*EX(1) + DELEX(2,2) + DELEX(4,2)
      EY(6) = CORR_FACT(2)*EY(1) + DELEY(2,2) + DELEY(4,2)

      EX(7) = CORR_FACT(1)*EX(1) + DELEX(2,1) + DELEX(5,1)
      EY(7) = CORR_FACT(1)*EY(1) + DELEY(2,1) + DELEY(5,1)
C
C Next calculated the phi...
C
      DO I = 1, 10
        CORR_MET(I) = SQRT(EX(I)*EX(I) + EY(I)*EY(I))
        CORR_METPHI(I) = ATAN2(-EY(I),-EX(I))
        IF (CORR_METPHI(I) .LT. 0) CORR_METPHI(I) = 
     &      CORR_METPHI(I) + 2.0*(3.14159)
      ENDDO
Cqqq
C      write (6,*) corr_met(7), ex(7), ey(7)
C      write (6,*) ex(1), ey(2)
C      write (6,*) delex(2,1), deley(2,1)
C      write (6,*) delex(5,1), deley(5,1)
    
c      HT_EVENT = HT_EVENT + CORR_MET(7)
C
C Finally add in the muons...
C
c      DO I = 1, 10
c        IF (LEP_ID(I) .EQ. 3 .OR. LEP_ID(I) .EQ. 4 .OR. LEP_ID(I) .EQ.
c     &    5) HT_EVENT = HT_EVENT + LEP_ET(I)
c      ENDDO
C
C Clear out the unused MET's just in case...
C
      CORR_MET(2) = -999.
      CORR_METPHI(2) = -999.
      DO I = 8, 10
        CORR_MET(I) = -999.
        CORR_METPHI(I) = -999.
      ENDDO

      RETURN
      END

C
C************************************************************************
C

      SUBROUTINE FIND_PHOTONS(XDUMMY2)

      IMPLICIT NONE
      INCLUDE 'stntuple$source:STNTUPLE_VAR.INC'   
      INCLUDE 'stntuple$source:WWGG_ACCEPT.INC'   
      include 'stntuple$source:exo_phoinfo.inc'

      REAL XDUMMY2
      REAL ERATIO(10),SLIDE
      INTEGER CUT_NUM
      INTEGER I,J
      INTEGER NHPHO,NLPHO,NRPHO,NHPASSLOW
      integer nuse

C
C Next we look at the photon candidates we have. Start by looking at the ID 
C Variables...Then the Isolation variables...Make sure we are only looking at 
C       the Central ELES banks with Et>25. 
C
C PHO_ID() = 0 Nothing
C PHO_ID() = 1 Good central photon
C PHO_ID() = 2 Loose central photon (ET>10 but not above main threshold)
C PHO_ID() = 3 Plug photon
C
C We also have a NGOODPHO counter. Also a PHO_ET(I) where I is referenced to
C the list of photons which pass the good cuts.  WHICH_PHO tells which photon.
C
C We note that if XDUMMY2 equals 99, then we are doing a diphoton analysis.
C There is a trigger which requires both photons to pass the boxiso cut and
C have |CESX|<17.5.
C This trigger has a 10 GeV threshold, however when both photons are above 16
C there is another trigger which doesn't require isolation or CESX. 
C To be safe if both
C are above 18 we remove the boxiso requirement and put on a caliso, trackiso
C and had/em cut.
C
C Bit number: This number is used in PHO_STAT, HPHO_STAT and
C        LPHO_STAT (lower pho_stat).
C
C 1  2 Detect
C 2  4    ET
C 3  8    PT
C 4  16   Chiavg
C 5  32   CESFLG
C 6  64   CESZ
C 7  128  CESX
C 8  256  CESSIG
C 9  512  Had/EM
C 10 1024 Cal Iso
C 11 2048 Track iso
C 12 4096 Box iso
C

      DO J = 1, NPHO
          ERATIO(J) = PCO4(J)/PETC(J)
          PHO_STAT(J) = 0
          LPHO_STAT(J) = 0
          HPHO_STAT(J) = 0
          CUT_NUM = 1
          IF (PDETECT(J) .NE. 0) THEN
            PHO_STAT(J) = PHO_STAT(J) + 2**CUT_NUM
          ENDIF
          CUT_NUM = CUT_NUM + 1
          IF (PETC(J) .LT. ET_THRESH) THEN
            PHO_STAT(J) = PHO_STAT(J) + 2**CUT_NUM
          ENDIF
          CUT_NUM = CUT_NUM + 1
          IF (PPT(J) .GT. 1. .OR. PN3D(J) .GT. 1) THEN
            PHO_STAT(J) = PHO_STAT(J) + 2**CUT_NUM
          ENDIF
          CUT_NUM = CUT_NUM + 1
          IF (ABS(PCHI(J)) .GT. 10.) THEN
            PHO_STAT(J) = PHO_STAT(J) + 2**CUT_NUM
          ENDIF
C          CUT_NUM = CUT_NUM + 1
C          IF (PCHIWIR(J) .GT. 15.) THEN
C            PHO_STAT(J) = PHO_STAT(J) + 2**CUT_NUM
C          ENDIF
C          CUT_NUM = CUT_NUM + 1
C          IF (PCHISTR(J) .GT. 10.) THEN
C            PHO_STAT(J) = PHO_STAT(J) + 2**CUT_NUM
C          ENDIF
C          CUT_NUM = CUT_NUM + 1
C          IF (PLSHR(J) .GT. 0.2) THEN
C            PHO_STAT(J) = PHO_STAT(J) + 2**CUT_NUM
C          ENDIF
          CUT_NUM = CUT_NUM + 1
C This is the Doug Benjamin sliding cut.
          IF (.NOT.PCESFLG(J)) THEN
            PHO_STAT(J) = PHO_STAT(J) + 2**CUT_NUM
          ENDIF
          CUT_NUM = CUT_NUM + 1
          IF (ABS(PCESZ(J)).LT.14..OR.ABS(PCESZ(J)).GT.217.) THEN
C          IF (ABS(PCESZ(J)).LT.14..OR.ABS(PCESZ(J)).GT.230.) THEN
            PHO_STAT(J) = PHO_STAT(J) + 2**CUT_NUM
          ENDIF
          CUT_NUM = CUT_NUM + 1
          IF (ABS(PCESX(J)).GT.21.0) THEN
            PHO_STAT(J) = PHO_STAT(J) + 2**CUT_NUM
          ENDIF
          CUT_NUM = CUT_NUM + 1
          IF (ABS(PCESSIG(J)).GT.2.0) THEN
            PHO_STAT(J) = PHO_STAT(J) + 2**CUT_NUM
          ENDIF
          IF (PHO_STAT(J) .EQ. 0) THEN
            NCENTPHO25ID = NCENTPHO25ID + 1
          ENDIF
C
C The next cuts are a little more complicated
C
C Next do the ISO Cuts...Here we deal with the special case of using Bob's cuts
C when XDUMMY2=99, and 12<Photon Et<25.
C
C
          LPHO_STAT(J) = PHO_STAT(J)
          HPHO_STAT(J) = PHO_STAT(J)

          CUT_NUM = CUT_NUM + 1
          SLIDE = 0.055+0.00045*PE(J)
          IF (PHADEM(J) .GT. SLIDE) THEN 
            IF (XDUMMY2 .NE. 99. .OR. PETC(J) .GT. ET_THRESH2) 
     &          PHO_STAT(J) = PHO_STAT(J) + 2**CUT_NUM
            HPHO_STAT(J) = HPHO_STAT(J) + 2**CUT_NUM
          ENDIF

          CUT_NUM = CUT_NUM + 1
          IF (ERATIO(J) .GT. 0.10) THEN
            IF (XDUMMY2 .NE. 99. .OR. PETC(J) .GT. ET_THRESH2) 
     &          PHO_STAT(J) = PHO_STAT(J) + 2**CUT_NUM
            HPHO_STAT(J) = HPHO_STAT(J) + 2**CUT_NUM
          ENDIF

          CUT_NUM = CUT_NUM + 1
Cqqq
          IF (PSUMPT4(J).GT. 5.00) THEN
            IF (XDUMMY2 .NE. 99. .OR. PETC(J) .GT. ET_THRESH2) 
     &          PHO_STAT(J) = PHO_STAT(J) + 2**CUT_NUM
            HPHO_STAT(J) = HPHO_STAT(J) + 2**CUT_NUM
          ENDIF

          CUT_NUM = CUT_NUM + 1
          IF (PBOXISO(J) .GT. 4.0) THEN
            IF (XDUMMY2 .EQ. 99. .AND. PETC(J) .LT. ET_THRESH2) 
     &        PHO_STAT(J) = PHO_STAT(J)  + 2**CUT_NUM
            LPHO_STAT(J) = LPHO_STAT(J) + 2**CUT_NUM
          ENDIF
C
C Here mark it as failing only if we haven't before....
C
          IF (ABS(PCESX(J)).GT.17.5 .AND. ABS(PCESX(J)).LT.21.0) THEN
            LPHO_STAT(J) = LPHO_STAT(J) + 128
          ENDIF

          IF (XDUMMY2 .EQ. 99 .AND. PETC(J) .LT. ET_THRESH2) THEN
              HPHO_STAT(J) = HPHO_STAT(J) + 4
          ENDIF
Cqqq
C          write (6,*) pho_stat(j), LPHO_STAT(J), HPHO_STAT(J)
      ENDDO ! Loop over all Photons
C
C Now that we have given all the photon candidates a status. We sort and
C catagorize them. NGOODPHO counts the number of good photon candidates, note 
C that just because NGOODPHO >= 2 does not mean that this is a good diphoton
C event. For example a good central photon and a good plug photon would give
C NGOODPHO = 2, but this is NOT a good diphoton as GOOD_DIPHO requires to good
C central photons. If we are doing the diphoton analysis (XDUMMY=99) then we
C have the added complication that the photon definitions change if passing the
C trigger threshold requires having one good leg below 18 GeV. 
C
C
C First things first. If we are doing the diphoton analysis...
C   mark that this is a diphoton event. 
C
C The logicals USEHIGH, USELOW and USEREG tell us which analysis we are doing.
C If this is the diphoton analysis (XDUMMY2 = 99) we need either HIGH or LOW.
C Use HIGH if the event has two photons above the high threshold and passing
C the high ET cuts. Use low if both photons are above the low threshold and at
C least one is below the high threshold. USEREG if it fails both or we arn't
C doing the diphoton analysis.
C
      NHPHO = 0
      NLPHO = 0
      NRPHO = 0
      NHPASSLOW = 0
      USEHIGH = .FALSE.
      USELOW = .FALSE.
      USEREG = .FALSE.
      DO J = 1, NPHO
          IF (PHO_STAT(J) .EQ. 0)  NRPHO = NRPHO + 1
          IF (LPHO_STAT(J) .EQ. 0) THEN
            IF (PETC(J) .LT. 18.0) THEN
              NLPHO = NLPHO + 1
            ELSE
              NHPASSLOW = NHPASSLOW + 1
            ENDIF
          ENDIF
          IF (HPHO_STAT(J) .EQ. 0) NHPHO = NHPHO + 1
      ENDDO
      IF (XDUMMY2 .EQ. 99) THEN
        IF (NLPHO .GT. 1) THEN
          GOOD_DIPHO = .TRUE.
          USELOW = .TRUE.
        ENDIF
        IF (NLPHO .GT. 0 .AND. (NLPHO+NHPASSLOW) .GT. 1) THEN
          GOOD_DIPHO = .TRUE.
          USELOW = .TRUE.
        ENDIF
        IF (NHPHO .GT. 1) THEN
          GOOD_DIPHO = .TRUE.
          USEHIGH = .TRUE.
          USELOW = .FALSE.
        ENDIF
        IF (.NOT. GOOD_DIPHO) USEREG = .TRUE.
      ELSE
        IF (NRPHO .GT. 1) GOOD_DIPHO = .TRUE.
        USEREG = .TRUE.
      ENDIF
      NUSE = 0
      IF (USEHIGH) NUSE = NUSE + 1
      IF (USELOW)  NUSE = NUSE + 1
      IF (USEREG) NUSE = NUSE + 1
      IF (NUSE .GT. 1) WRITE (6,*) 'USEHIGH + USELOW + USEREG > 1',
     &  USEHIGH, USELOW, USEREG !,IDNEVT
      IF (XDUMMY2 .NE. 99 .AND. USEHIGH) WRITE (6,*) 'FUCKED UP USEHIGH'
      IF (XDUMMY2 .NE. 99 .AND. USELOW) WRITE (6,*) 'FUCKED UP USELOW'
      IF (XDUMMY2 .NE. 99 .AND. .NOT. USEREG) 
     &          WRITE (6,*) 'FUCKED UP USEREG'
C
C Now that we know which case we are looking at: Three cases:
C 
C       1=Use the regular cuts: (USEREG = .TRUE. )  Use PHO_STAT
C       2=Use the low cuts:     (USELOW = .TRUE. )  Use LPHO_STAT
C       3=Use the high cuts:    (USEHIGH = .TRUE. ) Use HPHO_STAT
C
      DO J= 1, NPHO
          IF (
     &      (USEREG  .AND.  PHO_STAT(J) .EQ. 0) .OR. 
     &      (USELOW  .AND. LPHO_STAT(J) .EQ. 0) .OR.
     &      (USEHIGH .AND. HPHO_STAT(J) .EQ. 0)) THEN
            GOOD_PHO = .TRUE.
            PHO_ID(J) = 1
            NGOODPHO = NGOODPHO + 1
            NCENTPHO25IDISO = NCENTPHO25IDISO + 1
            PHO_ET(NGOODPHO) = PETC(J)
            WHICH_PHO(NGOODPHO) = J
C
C Next do the fakes weighting stuff...
C If the ET>35 use CPR, otherwise use CES.
C
            IF (PETC(J) .GT. 35.0) THEN
              PHO_WHT(NGOODPHO) = PCPRWHT(J)
            ELSE
              PHO_WHT(NGOODPHO) = PCESWHT(J)
            ENDIF
          ENDIF ! Good tight central photon??
          
      ENDDO

C
C Check for loose photons...There must be a good central photon already. Loose
C photons may be 
C  2) good central photons which pass all but the ET cut 
C       but are required to be above 10 GeV
C  3) Plug. Note that this doesn't work for MC since the plug variables are
C  screwed up.
C
      DO J = 1, NPHO
C
C Here pho_stat=4 implies that it passes all the cuts except the ET_Cut
C
        IF (PHO_STAT(J) .EQ. 4 .AND. PETC(J) .GT. 10. 
     &      .AND. PHO_ID(J) .EQ. 0) THEN
          NGOODPHO = NGOODPHO + 1
          PHO_ID(J) = 2
          PHO_ET(NGOODPHO) = PETC(J)
          WHICH_PHO(NGOODPHO) = J
        ENDIF
        IF (PSTAT(J) .GT. 3010 .AND. PDETECT(J) .EQ. 1) THEN
          NGOODPHO = NGOODPHO + 1
          PHO_ID(J) = 3
          PHO_ET(NGOODPHO) = PETC(J)
          WHICH_PHO(NGOODPHO) = J
        ENDIF
C
C If this is MC then the PSTAT can be wrong. Check again...
C
        IF (RUN .LT. 10) THEN
              SLIDE=0.028+0.00019*PET(J)
              IF (PETC(J).GT.10.0.AND.PDETECT(J) .EQ. 1 .AND. 
     &         PCHI3X3(J).LE.5.0 .AND.
     &         PCHIETA(J).LE.20.0 .AND.
     &         PCHIPHI(J).LE.20.0 .AND.
     &         PVTXOCC(J).LE.0.4 .AND.
     &         PHADEM(J).LT.SLIDE .AND.
     &         ERATIO(J) .LT. 0.15) THEN
                NGOODPHO = NGOODPHO + 1
                PHO_ID(J) = 3
                PHO_ET(NGOODPHO) = PETC(J)
                WHICH_PHO(NGOODPHO) = J
              ENDIF
        ENDIF ! MC ??

      ENDDO ! Loop over all photons...
C
C Now that we've got info about the photons set some photon flags.
C
      IF (GOOD_DIPHO) EVENT_TOT(9) = EVENT_TOT(9) + 1
      IF (NCENTPHO25IDISO .GT. 1) GOOD_LOOSEG = .TRUE.
      IF (NGOODPHO .GT. 1) GOOD_LOOSEG = .TRUE.
      IF (NGOODPHO .GT. 2 .AND. GOOD_DIPHO) GOOD_TRIPHO = .TRUE.
C
      RETURN
      END

C
C************************************************************************
C

      SUBROUTINE EVENT_INIT

      IMPLICIT NONE
      INCLUDE 'stntuple$source:STNTUPLE_VAR.INC'   
      INCLUDE 'stntuple$source:WWGG_ACCEPT.INC'   
      INTEGER I
C
C First initialize global event variables
C
      GOODRUN_VERT = .FALSE.
C
C Next initialize photon variables
C
      NCENTPHO        = 0
      NCENTPHO25      = 0
      NCENTPHO25ID    = 0
      NCENTPHO25IDISO = 0

      NGOODPHO = 0
      GOOD_PHO = .FALSE.
      GOOD_DIPHO = .FALSE.
      GOOD_GG3JET = .FALSE.
      GOOD_TRIPHO = .FALSE.
      GOOD_LOOSEG = .FALSE.
      DO I = 1, 10
        PHO_ET(I) = 0.0
        PHO_WHT(I) = 0.0
        WHICH_PHO(I) = 0
        PHO_ID(I) = 0
      ENDDO
C
C Next initialize electron variables
C
      GOOD_ZEE_CC = .FALSE.
      GOOD_DY_CC  = .FALSE.
      GOOD_PLUG_E = .FALSE.
      NGOODCE = 0
C
C Next initialize generic lepton and dilepton variables...
C
      GOOD_E =  .FALSE.
      GOOD_MU = .FALSE.
      GOLD_LEPTON = .FALSE.
      GOOD_LL = .FALSE.
      GOOD_MM = .FALSE.
      DO I = 1, NEL+NMU+NTAU
        LEPTON_STATUS(I) = 0
      ENDDO
C
C Next generic lepton-photon
C
      GOOD_LG = .FALSE.
      GOOD_LGM = .FALSE.
      DELTA_RPHI(1) = -999.
      DELTA_RPHI(2) = -999.
      DELTA_RPHI(3) = -999.
      LL_MASS = -999.
      LG_MASS = -999.
C
C Next initialize the jet variables...
C

      NGOODJET = 0
      TAG_STAT = 0
      NGOODTAG = 0
      NEL_N_NPHO = 0
      NJNOCOR = 0
      DO I = 1, 20
        JET_ID(I) = 0
        WHICH_JET(I) = 0
        GOODJET(I) = .TRUE.
        GOODJET_ET(I) = -999.
        IF (I .GT. NJ1) GOODJET(I) = .FALSE.
      ENDDO
C
      RETURN
      END

C
C************************************************************************
C

      SUBROUTINE CHECK_GOODRUNVERT

      IMPLICIT NONE
      INCLUDE 'stntuple$source:STNTUPLE_VAR.INC'   
      INCLUDE 'stntuple$source:WWGG_ACCEPT.INC'   
      include 'stntuple$source:exo_phoinfo.inc'
      INTEGER I

      IF (GOODRUN) THEN
        IF (NPVERT .GT. 0. .AND. VCLASS(1).GE.11) THEN
          IF (RUN .LT. 10 .OR. ETOUT .EQ. 0) THEN
          IF (ABS(VZ(1)).LT.60.) THEN
            GOODRUN_VERT = .TRUE.
C
C Note that NPHO is not Equal to NEL. NPHO does not count Forward ELES banks
C
            IF (NEL .GE. 2) THEN
              DO I = 1, NPHO
                IF (PDETECT(I) .EQ. 0) NCENTPHO = NCENTPHO + 1
              ENDDO
              IF (NCENTPHO .GE. 2) THEN
                DO I = 1, NPHO
                  IF (PDETECT(I) .EQ. 0.AND.PETC(I) .GE. ET_THRESH) 
     &              NCENTPHO25 = NCENTPHO25 + 1
                ENDDO
              ENDIF ! Two Central ELES Banks
            ENDIF ! Two ELES banks??
          ENDIF ! |Z|< 60 CM
          ENDIF ! ETOUT
        ENDIF ! Good Vertex
      ENDIF ! GoodRun ??

      RETURN
      END
C

