C
C 	This is a series of routines for module BTGSMK.  This module will
C make the combined Btag bank, BTGS.  You must run VXPRIM and the taggers 
C upstream of this module, it only makes the bank and prints tagging results, 
C if requested.  
C 	To get soft lepton tags (from DATA_TAGGER) you need to puff with:
C Talk PREPAD disable cal_exp true trk_exp true show exit
C
C 	There is a talk-to to tell it to make the bank or to unpack it.  
C The talk-to also allows a choice of how much dumping to do.
C
C 09/29/94  Ray Culbertson
C 11/08/94  Ray Culbertson - fix bug in JETVTX covariance matrix
C 12/01/94  Ray Culbertson - fix so that print works in all cases
C 12/05/94  Ray Culbertson - fix bug that didn't allow mutiple talk-to's
C 01/19/95  Ray Culbertson - create version 1, fix SLT's.  Previously an SLT
C  had to be attached to a jet, now it can be anywhere in the event.
C  If the version 1 bank is unpacked, the SLT's will be ignored (none found),
C  since they were incorrect anyway.  Also now require 2 good positive I.P. 
C  tracks as well as positive probability<.01 for a JPBTAG tag.
C 02/21/95  Ray Culbertson - Bug fix.  If there were SLT's in the bank, 
C  words were dropped off the end of the bank when it was written out.
C  This can be seen as 4 tracks in a SECVTX tag and and only 3 listed in the
C  dump.  Info can't be recovered, bank needs to be remade.
C 03/03/95  Ray Culbertson - add talk-to option to turn off SLT call
C 03/31/95  Ray Culbertson - Add a flag summarizing tags in the event.
C 05/19/95  Ray Culbertson - Add Monte Carlo SLT call
C 08/01/95  Ray Culbertson - Fix bug where an SLT could be carried from one 
C		event to the next - Monte Carlo only
C
C ***************************************************************************

	SUBROUTINE BTGSIN
C
C Init routine for module BTGSMK.  Set default mode and dump flags - make
C bank, don't dump.
C
C 09/29/94  Ray Culbertson
C

$$IMPLICIT
$$INCLUDE 'C$INC:BTGSMK.INC'

	BTNMODE=1 ! FILL COMMON, MAKE NEW BANK
	BTNDUMP=0 ! NO PRINTS
	BTNSLT=.TRUE.  ! CALL SLT CODE

	RETURN
	END

C ***************************************************************************

	SUBROUTINE BTGSEV
C
C Event routine for module BTGSMK.  Make the combined Btag bank, BTGS or
C just unpack it.  Also print results as requested.
C
C 09/29/94  Ray Culbertson
C

$$IMPLICIT
$$INCLUDE 'C$INC:JOBSTA.INC'
$$INCLUDE 'C$INC:BTGSCM.INC'
$$INCLUDE 'C$INC:BTGSMK.INC'

	INTEGER I,ITAGJ,ITAGE


	BTGSDF=.FALSE.  ! NOT DF

	IF(BTNMODE.EQ.1) THEN
	  CALL BTGCTC
	  CALL BTGCTB
	ELSEIF(BTNMODE.EQ.2) THEN
	  CALL BTGBTC
	ENDIF

	ITAGE=0
	ITAGJ=0
	IF(BTNELE(1).GT.0) ITAGJ=IBSET(ITAGJ,0)
	IF(BTNMUO(1).GT.0) ITAGJ=IBSET(ITAGJ,1)
	IF(ITAGJ.GT.0) THEN
	  ITAGE=1
	  IF(BTNTAG.LT.100) THEN
	    BTNTAG=BTNTAG+1
	    BTNRU(BTNTAG)=NRUN
	    BTNEV(BTNTAG)=NEV
	    BTNJT(BTNTAG)=0
	    BTNFL(BTNTAG)=ITAGJ
	  ENDIF
	ENDIF

	DO I=1,BTNNJET
	  ITAGJ=0
	  IF(JVTAG(I).EQ.1) ITAGJ=ITAGJ+4
	  IF(JPTAG(I).EQ.1) ITAGJ=ITAGJ+8
	  IF(DPTAG(I).EQ.1) ITAGJ=ITAGJ+16
	  IF(SCTAG(I).EQ.1) ITAGJ=ITAGJ+32
	  IF(ITAGJ.GT.0) THEN
	    ITAGE=1
	    IF(BTNTAG.LT.100) THEN
	      BTNTAG=BTNTAG+1
	      BTNRU(BTNTAG)=NRUN
	      BTNEV(BTNTAG)=NEV
	      BTNJT(BTNTAG)=BTJETB(I)
	      BTNFL(BTNTAG)=ITAGJ
	    ENDIF
	  ENDIF
	ENDDO
	BTNEVT=BTNEVT+ITAGE

	IF(BTNDUMP.LE.0) RETURN

	IF(BTNDUMP.EQ.2) THEN
	  CALL BTGSPR(6,-1)
	ELSEIF(BTNDUMP.EQ.4) THEN
	  CALL BTGSPR(6, 0)
	ELSE
	  IF(ITAGE.EQ.1) THEN
	    IF(BTNDUMP.EQ.1) THEN
	      CALL BTGSPR(6,-1)
	    ELSE
	      CALL BTGSPR(6, 0)
	    ENDIF
	  ENDIF
	ENDIF

	RETURN
	END

C ***************************************************************************

	SUBROUTINE BTGSTK
C
C Talk-to routine for module BTGSMK.  Set flags for mode: either make the
C the bank or unpack it.  Also set the print flag.
C
C Talk BTGSMK mode 1 show exit  ! make the bank (default)
C Talk BTGSMK mode 2 show exit  ! unpack the bank
C Talk BTGSMK dump 0 show exit  ! no dump (default)
C Talk BTGSMK dump 1 show exit  ! Short dump of events with tags
C Talk BTGSMK dump 2 show exit  ! Short dump of all events
C Talk BTGSMK dump 3 show exit  ! Long dump of events with tags
C Talk BTGSMK dump 4 show exit  ! Long dump of all events
C
C
C 09/29/94  Ray Culbertson
C

$$IMPLICIT
$$INCLUDE 'UIPACK$LIBRARY:UINAMES.INC'
$$INCLUDE 'UIPACK$LIBRARY:UIERROR.INC'
$$INCLUDE 'S_I$LIBRARY:NOARGINC.INC'
$$INCLUDE 'C$INC:BTGSMK.INC'

	INTEGER GROUP,STATUS
	CHARACTER*20 VERB
	DATA GROUP /0/

	IF(GROUP.EQ.0) THEN
	  STATUS=UIASGP(GROUP)          ! get uipack verb group assigned
	  STATUS=UIDFGP(GROUP)          ! define verb group
	  STATUS=UIDFVB('EXIT')         ! here are the known verbs
	  STATUS=UIDFVB('MODE')
	  STATUS=UIDFVB('SHOW')
	  STATUS=UIDFVB('DUMP')
	  STATUS=UIDFVB('SLT')
	  STATUS=UILKGP(GROUP)          ! lock the group
	ENDIF

	STATUS=UIUSGP(GROUP,N$A)

	VERB='SHOW'
	DO WHILE(STATUS.NE.UIABRT)
	  IF(VERB(1:4) .EQ. 'EXIT') RETURN
	  IF(VERB(1:4) .EQ. 'SHOW') THEN
	    PRINT*
	    PRINT 100,BTNMODE
	    PRINT 200,BTNDUMP
	    PRINT 300,BTNSLT
	    PRINT*
	  ELSEIF(VERB(1:4) .EQ. 'MODE') THEN
	    PRINT*
	    PRINT 100,BTNMODE
	    STATUS=UIGTIN('BTGSMK mode? ',BTNMODE,1,2)
	  ELSEIF(VERB(1:4) .EQ. 'DUMP') THEN
	    PRINT*
	    PRINT 200,BTNDUMP
	    STATUS=UIGTIN('BTGSMK print level? ',BTNDUMP,0,4)
	  ELSEIF(VERB(1:4) .EQ. 'SLT') THEN
	    PRINT*
	    PRINT 300,BTNSLT
	    STATUS=UIGTLO('Do SLTs? ',BTNSLT)
	  ENDIF
	  STATUS = UIGTVB('BTGSMK>> ',VERB)

	ENDDO

100	FORMAT(1X,'BTGSMK mode is ',I1,', options are:'/,
	1	  '       1 Fill the common and make the bank'/,
	1	  '       2 Unpack the bank into the common')
200	FORMAT(1X,'BTGSMK print level is ',I1,', options are:'/,
	1	  '       0 No dump'/,
	1	  '       1 Short dump of events with tags'/,
	1	  '       2 Short dump of all events'/,
	1	  '       3 Long dump of events with tags'/,
	1	  '       4 Long dump of all events')
300	FORMAT(1X,'BTGSMK do SLT =',L2)

	RETURN
	END

C ***************************************************************************

	SUBROUTINE BTGSFI
C
C Finish routine for module BTGSMK.  If no dumps are requested, no action
C is taken.  Otherwise list the first hundred tags.
C
C 09/29/94  Ray Culbertson
C

$$IMPLICIT
$$INCLUDE 'C$INC:BTGSMK.INC'

	INTEGER I,J
	CHARACTER*8 CHECK(6)

	IF(BTNDUMP.LE.0) RETURN

	PRINT*
	PRINT*, '*************************** BTGS SUMMARY *******',
	1	'*********************'
	PRINT*
	PRINT 100,BTNTAG,BTNEVT
100	FORMAT(16X,'Found ',I4,' tags in ',I4,' events.')
	IF(BTNTAG.GT.0) THEN
	  PRINT*
	  PRINT*,'    Run    Event    JETS     Ele     Muo    ',
	1	'Jetvtx  JPbtag  D-phi   Secvtx'
	  DO I=1,BTNTAG
	    DO J=1,6
	      CHECK(J)=' '
	      IF(BTEST(BTNFL(I),J-1)) CHECK(J)='       v'
	    ENDDO
	    IF(BTNJT(I).GT.0) THEN
	      PRINT 200,BTNRU(I),BTNEV(I),BTNJT(I),CHECK
200	FORMAT(1X,2I8,I7,6A8)
	    ELSE
	      PRINT 300,BTNRU(I),BTNEV(I),CHECK
300	FORMAT(1X,2I8,7X,6A8)
	    ENDIF

	  ENDDO
	ENDIF
	PRINT*
	PRINT*, '*************************** BTGS SUMMARY *******',
	1	'*********************'
	PRINT*


	RETURN
	END


C *************************************************************************

	SUBROUTINE BTGCTC
C
C Fill in BTGSCM.INC from the tagger's commons or banks.  DATA_TAGGER is called
C to find potential soft lepton tags.  A-C is polled to see if each tagger 
C was called, if not, that tagger is ignored.  Note that this means if BTDPHI
C banks are present but it wasn't used in this path, it will be ignored.
C
C 09/29/94  Ray Culbertson
C 01/19/95  Ray Culbertson - create version 1, fix SLT's.
C

$$IMPLICIT

$$INCLUDE 'C$INC:JOBSTA.INC'
$$INCLUDE 'C$INC:BCS.INC'
$$INCLUDE 'A_C$LIBRARY:ANERROR.INC'
$$INCLUDE 'A_C$LIBRARY:ANCOMP_EXP_ID.INC'
$$INCLUDE 'C$INC:ANACTL.INC'
$$INCLUDE 'YBOS$LIBRARY:ERRCOD.INC'
$$INCLUDE 'C$INC:ERLEVL.INC'
$$INCLUDE 'C$INC:PIVALU.INC'
$$INCLUDE 'C$INC:BTMASK.INC'

$$INCLUDE 'C$INC:DSJETS.INC'
$$INCLUDE 'C$INC:JETINFO.INC'

$$INCLUDE 'C$INC:JETVTX.INC'

$$INCLUDE 'C$INC:JPBTAG.INC'

$$INCLUDE 'C$INC:DSBTAG.INC'
$$INCLUDE 'C$INC:DSCMPB.INC'
$$INCLUDE 'C$INC:DSFITP.INC'
$$INCLUDE 'C$INC:DSSVXS.INC'
$$INCLUDE 'C$INC:TRKSCO.INC'
$$INCLUDE 'C$INC:TRFITC.INC'

$$INCLUDE 'C$INC:SECVTX.INC'
$$INCLUDE 'C$INC:BTGSCM.INC'
$$INCLUDE 'C$INC:BTGSMK.INC'

	INTEGER BLOCAT,ISTAT,BNEXT,BDATA,BNUMB
	INTEGER STATUS,ANGOPS,PSNUMB
	CHARACTER*25 PSNAME
	INTEGER I,J,K,IJI
	INTEGER NDP,NDPB,INDP,INDPD,JETSN,INNEXT
	INTEGER INDJ,INDJD,IPTSC
	REAL DIFFMIN,DIFF,PHIDIFF
	REAL JPM1NEG

	INTEGER N,IND,INDD,BLKINS,NELE,NMUO
	REAL PHITR(200),ETATR(200)
	REAL COTT
	INTEGER NTR,BANKTR(200)

	REAL PT,PP(4,200),PVTX(4),PAR(5)
	INTEGER CHRG(200)
	REAL X,Y,ARG,E

	REAL PPE,PN,PNE
	REAL SIGNIF

C DATA_TAGGER
	INTEGER QTRK_SKIP(2),QTRK_ELE(5),CMUO_MUO(5),QTRK_MUO(5)
	REAL FAKE_ELE2,FAKE_ELE4,FAKE_MUO2,FAKE_MUO4,ETA
	INTEGER TYPE_MUO(5)
	REAL FAKE_MUX2,FAKE_MUX4
	INTEGER IERR,QTRKUN
C STANDALONE_MC_TAGGER
	INTEGER LEP_TYPE,LEP_BANK,NTAGS
	REAL PXTAG(20),PYTAG(20),PZTAG(20),EFF_TAG(20),EFF_TRK(20)
	INTEGER TAG_TYPE(20),HOEM_CUT(20)
	LOGICAL RUN94
	LOGICAL DY_REMOVAL(20),DILEP_REMOVAL(20)
	INTEGER NQTRK,IQTRK
	REAL PSLT


$$INCLUDE 'C$INC:DSJETSSF.INC'


	IF(.NOT.BTGSDF) THEN
C IF THIS IS NOT DF, CHECK WHICH TAGGERS WERE CALLED

	  STATUS=ANGOPS('JETVTX',PSNAME,PSNUMB)
	  IF(STATUS.EQ.ANSUCC) THEN  ! JETVTX WAS CALLED
	    JVCALL=.TRUE.
	  ELSE
	    JVCALL=.FALSE.
	  ENDIF

	  STATUS=ANGOPS('JPBTAG',PSNAME,PSNUMB)
	  IF(STATUS.EQ.ANSUCC) THEN
	    JPCALL=.TRUE.
	  ELSE
	    JPCALL=.FALSE.
	  ENDIF

	  STATUS=ANGOPS('BTDPHI',PSNAME,PSNUMB)
	  IF(STATUS.EQ.ANSUCC) THEN
	    DPCALL=.TRUE.
	  ELSE
	    DPCALL=.FALSE.
	  ENDIF

	  STATUS=ANGOPS('SECVTX',PSNAME,PSNUMB)
	  IF(STATUS.EQ.ANSUCC) THEN
	    SCCALL=.TRUE.
	  ELSE
	    SCCALL=.FALSE.
	  ENDIF
	ELSE
	  JVCALL=.TRUE.
	  JPCALL=.TRUE.
	  DPCALL=.TRUE.
	  SCCALL=.TRUE.
	ENDIF

C GET THE LIST OF SOFT LEPTON TAGS

	DO I=1,5
	  QTRK_ELE(I)=0
	  QTRK_MUO(I)=0
	  BTNELE(I)=0
	  BTNMUO(I)=0
	ENDDO
	IF(BTNSLT) THEN
	  IF(LMONTE) THEN

	    LEP_TYPE=3  ! DO NOT EXCLUDE PRIMARY LEPTONS
	    LEP_BANK=0
	    IF(EXPTYP.EQ.RUN1A) THEN
	      RUN94=.FALSE.
	    ELSE
	      RUN94=.TRUE.
	    ENDIF
	    CALL STANDALONE_MC_TAGGER(LEP_TYPE,LEP_BANK,RUN94,
	1	NTAGS,PXTAG,
	1	PYTAG,PZTAG,TAG_TYPE,EFF_TAG,EFF_TRK,
	1	DY_REMOVAL,DILEP_REMOVAL,HOEM_CUT,IERR)
	    NELE=0
	    NMUO=0
	    NQTRK=QTRKUN(0,0)
            IF (NTAGS.GE.1) THEN
	     DO I=1,NTAGS
C FIND ASSOCIATED QTRK
	      PSLT=PI+ATAN2(-PYTAG(I),-PXTAG(I))
	      DIFFMIN=999.
	      IQTRK=0
	      DO J=1,NQTRK
	        STATUS=QTRKUN(J,0)
	        PHIDIFF=ABS( TRKSFP(5)-PSLT )
	        IF(PHIDIFF.GE.PI) PHIDIFF=TWOPI-PHIDIFF
	        IF(PHIDIFF.LT.DIFFMIN) THEN
	          IQTRK=J
	          DIFFMIN=PHIDIFF
	        ENDIF
	      ENDDO
	      IF(TAG_TYPE(I).EQ.1) THEN
	          IF(HOEM_CUT(I).EQ.1.OR.HOEM_CUT(I).EQ.3) THEN
	            NELE=NELE+1
	            BTNELE(NELE)=IQTRK
	          ENDIF
	      ELSE
	          NMUO=NMUO+1
	          BTNMUO(NMUO)=IQTRK
	      ENDIF
	     ENDDO
            ENDIF

	  ELSE  !  DATA SLT'S

	    QTRK_SKIP(1)=0
	    QTRK_SKIP(2)=0
	    CALL DATA_TAGGER(QTRK_SKIP,.TRUE.,0,
	1	QTRK_ELE,CMUO_MUO,QTRK_MUO,TYPE_MUO,
	1	FAKE_ELE2,FAKE_MUO2,FAKE_MUX2,
	1	FAKE_ELE4,FAKE_MUO4,FAKE_MUX4,IERR)

C STORE QTRK NUMBERS OF SLT'S

	    DO J=1,5
	      BTNELE(J)=QTRK_ELE(J)
	      BTNMUO(J)=QTRK_MUO(J)
	    ENDDO

	  ENDIF  ! FOR MC OR DATA

	  IF(IERR.NE.0) CALL ERLOGR('BTGCTC',EREROR,0,0,
	1	'Error finding soft lepton tags')

	ENDIF  ! DO SLT'S


C SVXS TRACK LIST 

	NTR=0
	N=-1
	ISTAT=BLOCAT(IW,'SVXS',N,IND,INDD)
	DO WHILE(IND.GT.0)  ! LOOP OVER SVXS BANKS
	  ISTAT=BNUMB(IW,IND,N)    ! GET THE BANK NUMBER

	  IF(NTR.LT.200) THEN
	    NTR=NTR+1
	    BLKINS=INDD+DHSSVS+VHSSVS
	    PHITR(NTR)=RW(BLKINS+P0SV)
	    COTT=RW(BLKINS+CTSV)
	    ETATR(NTR)=LOG(SQRT(1.+COTT**2)+COTT)
	    BANKTR(NTR)=N
	    IF(N.LT.200) THEN
	      PAR(1)=RW(BLKINS+CTSV)
	      PAR(2)=RW(BLKINS+CUSV)
	      PAR(3)=RW(BLKINS+Z0SV)
	      PAR(4)=RW(BLKINS+D0SV)
	      PAR(5)=RW(BLKINS+P0SV)
	      CALL CALPAR(PAR,PT,PP(1,N))
	      PP(4,N)=SQRT(PP(1,N)**2+PP(2,N)**2+PP(3,N)**2+
	1	.13957**2)
	      CHRG(N)=SIGN(1.,RW(BLKINS+CUSV))
	    ENDIF
	  ENDIF

	  ISTAT=BNEXT(IW,IND,INNEXT) ! NEXT SVXS BANK
	  IND=INNEXT
	  ISTAT=BDATA(IW,IND,INDD)  ! GET ITS DATA INDEX
	ENDDO  ! LOOP OVER SVXS BANKS


C COPY OVER PRIMARY VERTEX
	CALL GETXYZ(.FALSE.,BTNPR,BTNPRCV,BTNPRFL)

C FILL IN JETINFO COMMON, CLUSTER SIZE=.4
	CALL JETINFO(2)
	BTNNJET=ICLUS

C BIG LOOP OVER JETINFO JETS
	DO IJI=1,ICLUS

C COPY OVER JET VARIABLES
	  BTNPJ(1,IJI)=IJET_P(1,IJI)
	  BTNPJ(2,IJI)=IJET_P(2,IJI)
	  BTNPJ(3,IJI)=IJET_P(3,IJI)
	  BTNET(IJI)=IJET_ET(IJI)
	  BTNDETA(IJI)=IJET_DETA(IJI)
	  BTNETA(IJI)=IJET_ETA(IJI)
	  BTNPHI(IJI)=IJET_PHI(IJI)
	  BTJETB(IJI)=IJETBNK(IJI)

C MAKE A LIST OF THE SVXS TRACKS IN THIS JET
	  BTNNTRK(IJI)=0
	  DO I=1,NTR
	    PHIDIFF=ABS( PHITR(I) - IJET_PHI(IJI) )
	    IF(PHIDIFF.GE.PI) PHIDIFF=TWOPI-PHIDIFF
	    DIFF=SQRT( (ETATR(I)-IJET_ETA(IJI))**2+
	1	PHIDIFF**2)
	    IF(DIFF.LT. .4) THEN
	      BTNNTRK(IJI)=BTNNTRK(IJI)+1
	      BTNTKNO(BTNNTRK(IJI),IJI)=BANKTR(I)
	    ENDIF
	  ENDDO


	  IF(JVCALL) THEN  ! JETVTX WAS CALLED

C COPY OVER JETVTX VARIABLES
C BACKGROUND PROB
	    CALL JVPRED(VTX_NUMTRK(IJI),IJET_ET(IJI),IJET_DETA(IJI),
	1	JVM1BG(IJI),PPE,PN,PNE)

	    JVTAG(IJI)=0			! DEFAULT NOT TAGGED
	    JVNTRK(IJI)=VTX_NUMTRK(IJI)	! # NUMBER OF TRACKS IN JET
	    IF(JVNTRK(IJI).LT.0) JVNTRK(IJI)=0
	    JVNSEC(IJI)=VTX_TRK(IJI)	! # OF TRACK IN SECONDARY
	    IF(JVNSEC(IJI).LT.0) JVNSEC(IJI)=0
	    IF(VTX_TRK(IJI).GT.0) THEN  ! THERE IS A SECONDARY
	      JVSEC(1,IJI)=VTX_SECVTX(IJI,1)
	      JVSEC(2,IJI)=VTX_SECVTX(IJI,2)
	      JVSEC(3,IJI)=VTX_SECVTX(IJI,3)
C	      DO I=1,6
C	        JVSECCV(I,IJI)=VTX_SECERR(IJI,I)
C	      ENDDO
	      JVSECCV(1,IJI)=VTX_SECERR(IJI,1)
	      JVSECCV(2,IJI)=0.
	      JVSECCV(3,IJI)=0.
	      JVSECCV(4,IJI)=VTX_SECERR(IJI,2)
	      JVSECCV(5,IJI)=0.
	      JVSECCV(6,IJI)=VTX_SECERR(IJI,3)
	      JVCHI2(IJI)=VTX_CHI(IJI)
	      JV2DLEN(IJI)=VTX_D2L(IJI)
	      JV2DLENE(IJI)=VTX_D2E(IJI)
	      JVPVTX(1,IJI)=VTX_P(IJI,1)
	      JVPVTX(2,IJI)=VTX_P(IJI,2)
	      JVPVTX(3,IJI)=VTX_P(IJI,3)
	      JVPVTX(4,IJI)=0.
	      IF(VTX_MASS(IJI).GT.0.) JVPVTX(4,IJI)=VTX_MASS(IJI)
	      JVCVTX(IJI)=VTX_CSUM(IJI)
	      IF(VTX_TAG(IJI).GT.0) THEN
	        DO I=1,VTX_TRK(IJI)
	          JVSCNO(I,IJI)=VTX_SVXBNK(IJI,I)
	        ENDDO

C DEFAULT JETVTX CUTS ARE NOT WHAT IS NEEDED TO DECIDE A TRUE TAG - 
C ADD VERTEX SEPARATION GT 3 SIGMA
	        IF(VTX_D2E(IJI).GT.0.) THEN
	          SIGNIF=VTX_D2L(IJI)/VTX_D2E(IJI)
	          IF(SIGNIF.GT.3.) THEN
	            JVTAG(IJI)=1
	          ELSEIF(SIGNIF.LT.-3.) THEN
	            JVTAG(IJI)=-1
	          ENDIF
	        ENDIF

	      ENDIF ! TAG FLAG
	    ENDIF ! SECONDARY FOUND
	  ELSE  ! JETVTX NOT CALLED

	    JVTAG(IJI)=0
	    JVNTRK(IJI)=0
	    JVNSEC(IJI)=0
	    JVM1BG(IJI)=-1.

	  ENDIF


C CHECK IF JPBTAG WAS CALLED

	  IF(JPCALL) THEN  ! JPBTAG WAS CALLED

C REQUIRE AT LEAST 2 POSITIVE I.P. TRACKS WITH COMBINED PROBABILITY<.01
C TO MAKE A TAG
	    IF(JETPRBP(IJI).LT. .01 .AND. NGDPOS(IJI).GE.2) THEN
	      JPTAG(IJI)=1
	    ELSE
	      JPTAG(IJI)=0
	    ENDIF
	    JPPRBP(IJI)=JETPRBP(IJI)
	    JPPRBM(IJI)=JETPRBM(IJI)

C THE PARAMETERIZED METHOD 1 BACKGROUND PROBABILITY
C SAVE POSITIVE PROB IN JPM1BG
	    CALL JPMETH1(IJET_ET(IJI),NGDPOS(IJI),NGDNEG(IJI),
	1	IJET_DETA(IJI),JPM1BG(IJI),JPM1NEG)

	    PVTX(1)=0.
	    PVTX(2)=0.
	    PVTX(3)=0.
	    PVTX(4)=0.
	    JPCVTX(IJI)=0
	    J=0  ! THE NUMBER OF GOOD TRACKS
	    DO I=1,NSVXJT(IJI)
	      IF(GOOD_SVX(IJI,I)) THEN
	        J=J+1
	        JPTKNO(J,IJI)=SVXSJT(IJI,I)
	        JPIPSIG(J,IJI)=SD0SIGTR(IJI,I)
	        JPTKPRB(J,IJI)=TRACKPROB(IJI,I)
C ADD UP P AND CHARGE FOR LOW PROB TRACKS
	        IF(SD0SIGTR(IJI,I).GE.0. .AND. 
	1	TRACKPROB(IJI,I).LT. .01) THEN
	          PVTX(1)=PVTX(1)+PP(1,SVXSJT(IJI,I))
	          PVTX(2)=PVTX(2)+PP(2,SVXSJT(IJI,I))
	          PVTX(3)=PVTX(3)+PP(3,SVXSJT(IJI,I))
	          PVTX(4)=PVTX(4)+PP(4,SVXSJT(IJI,I))
	          JPCVTX(IJI)=JPCVTX(IJI)+CHRG(SVXSJT(IJI,I))
	        ENDIF
	      ENDIF
	    ENDDO
	    JPNTRK(IJI)=J
	    JPPVTX(1,IJI)=PVTX(1)
	    JPPVTX(2,IJI)=PVTX(2)
	    JPPVTX(3,IJI)=PVTX(3)
	    JPPVTX(4,IJI)=0.
	    ARG=PVTX(4)**2-PVTX(1)**2-PVTX(2)**2-PVTX(3)**2
	    IF(ARG.GT.0) JPPVTX(4,IJI)=SQRT(ARG)

	  ELSE  ! JPBTAG NOT CALLED

	    JPTAG(IJI)=0
	    JPNTRK(IJI)=0
	    JPPRBP(IJI)=0.
	    JPPRBM(IJI)=0.
	    JPM1BG(IJI)=-1.

	  ENDIF ! JPBTAG WAS CALLED


C D-PHI INFORMATION FOR THIS JET

	  IF(DPCALL) THEN  ! BTDPHI WAS CALLED

C LOOK FOR D-PHI BANKS AND ASSOCIATE THEIR CONE<.7 JETS WITH JETINFO 
C (CONE<.4) JETS.  THIS .4 CONE JET IS TAGGED BY DPHI IF THERE IS A 
C DPHI JET WITHIN .4.  IF MORE THAN ONE, TAKE CLOSEST.
C THIS MEANS THE SAME DPHI B COULD POSSIBLY TAG MORE THAN ONE JET.

	    NDP=-1
	    ISTAT=BLOCAT(IW,'BTAG',NDP,INDP,INDPD)

	    DIFFMIN=.4
	    NDPB=-1

	    DO WHILE(INDP.GT.0)  ! LOOP OVER D-PHI BANKS
	      ISTAT=BNUMB(IW,INDP,NDP)    ! GET THE BANK NUMBER
C THE JET BANK NUMBER ASSOCIATED WITH THIS CLUSTER
	      JETSN=IW(INDPD+JNUMBT)
	      IF(JETSN.GT.0) THEN
	         ISTAT=BLOCAT(IW,'JETS',JETSN,INDJ,INDJD)
	        IF(ISTAT.EQ.YESUCC) THEN
	          PHIDIFF=PHI_JT(INDJD)-IJET_PHI(IJI)
	          IF(PHIDIFF.GE.PI) PHIDIFF=TWOPI-PHIDIFF
	          DIFF=SQRT( (ETA_JT(INDJD)-IJET_ETA(IJI))**2+
	1	PHIDIFF**2)
	          IF(DIFF.LT.DIFFMIN) THEN
	            DIFFMIN=DIFF
	            NDPB=NDP
	          ENDIF
	        ENDIF  ! FOUND JET BANK
	      ENDIF  !  D-PHI TAG ASSOCIATED WITH A JET

	      ISTAT=BNEXT(IW,INDP,INNEXT) ! NEXT BTAG BANK
	      INDP=INNEXT
	      ISTAT=BDATA(IW,INDP,INDPD)  ! GET ITS DATA INDEX
	    ENDDO  ! LOOP OVER D-PHI BANKS

	    DPTAG(IJI)=0
	    DPNSEC(IJI)=0
	    DPM1BG(IJI)=-1.
	    IF(NDPB.GT.0) THEN   !  FOUND A D-PHI TAG
	      ISTAT=BLOCAT(IW,'BTAG',NDPB,INDP,INDPD)
	      DPTAG(IJI)=1
	      DPNSEC(IJI)=IW(INDPD+NUTRBT)
	      DPSEC(1,IJI)=RW(INDPD+XVTRBT)
	      DPSEC(2,IJI)=RW(INDPD+YVTRBT)
	      DPSEC(3,IJI)=RW(INDPD+ZVTRBT)
	      DPSECCV(1,IJI)=RW(INDPD+COV1BT)
	      DPSECCV(2,IJI)=RW(INDPD+COV2BT)
	      DPSECCV(3,IJI)=RW(INDPD+COV3BT)
	      DPSECCV(4,IJI)=RW(INDPD+COV4BT)
	      DPSECCV(5,IJI)=RW(INDPD+COV5BT)
	      DPSECCV(6,IJI)=RW(INDPD+COV6BT)
	      DPCHI2(IJI)=RW(INDPD+CHI2BT)

	      DPPVTX(1,IJI)=RW(INDPD+PXTRBT)
	      DPPVTX(2,IJI)=RW(INDPD+PYTRBT)
	      DPPVTX(3,IJI)=RW(INDPD+PZTRBT)
	      E = RW(INDPD+ENTRBT)
	      ARG=E**2 - DPPVTX(1,IJI)**2 - 
	1	DPPVTX(2,IJI)**2 - DPPVTX(3,IJI)**2
	      IF(ARG.GT.0.) THEN
	        DPPVTX(4,IJI)=SQRT(ARG)
	      ELSE
	        DPPVTX(4,IJI)=0.
	      ENDIF

C 2D LENGTH AND ERROR
	      X=DPSEC(1,IJI)-BTNPR(1)
	      Y=DPSEC(2,IJI)-BTNPR(2)
	      DP2DLEN(IJI)=SQRT(X**2+Y**2)
	      ARG=DPSECCV(1,IJI)*DPSEC(1,IJI)**2 + 
	1	DPSECCV(4,IJI)*DPSEC(2,IJI)**2 +
	1	2*DPSECCV(2,IJI)*DPSEC(1,IJI)*DPSEC(2,IJI) +
	1	BTNPRCV(1)*BTNPR(1)**2 + BTNPRCV(4)*BTNPR(2)**2 + 
	1	2*BTNPRCV(2)*BTNPR(1)*BTNPR(2)
	      IF(ARG.GT.0.AND.DP2DLEN(IJI).GT.0.) THEN
	        DP2DLENE(IJI)=SQRT(ARG)/DP2DLEN(IJI)
	      ELSE
	        DP2DLENE(IJI)=0.
	      ENDIF

	      DPCVTX(IJI)=0
	      DO I=1,DPNSEC(IJI)
	        DPTKNO(I,IJI)=IW( INDPD+TRNUBT+NWBTBT*(I-1) )
	        DPCVTX(IJI)=DPCVTX(IJI)+CHRG(DPTKNO(I,IJI))   
	      ENDDO
	    ENDIF

	  ELSE  ! BTDPHI NOT CALLED

	    DPTAG(IJI)=0
	    DPNSEC(IJI)=0
	    DPM1BG(IJI)=-1.

	  ENDIF ! BTDPHI CALLED OR NOT

C SECVTX INFORMATION FOR THIS JET

	  IF(SCCALL) THEN ! SECVTX WAS CALLED

	    SCTAG(IJI)=TAGSEC(IJI)
	    SCNTRK(IJI)=NJTRK1(IJI)
	    SCNSEC(IJI)=NTTRK(IJI)
	    SC2DLEN(IJI)=LXYSEC(IJI)
	    SC2DLENE(IJI)=LXYERR(IJI)
	    SCCVTX(IJI)=CHRGSEC(IJI)
	    PVTX(1)=0.
	    PVTX(2)=0.
	    PVTX(3)=0.
	    DO I=1,SCNTRK(IJI)
	      SCTKNO(I,IJI)=JTRK1(I,IJI)
C FLAG: 1 IF THIS TRACK IS IN VERTEX
	      SCTKFL(I,IJI)=0
	      DO J=1,SCNSEC(IJI)
	        IF(TTRK(J,IJI).EQ.JTRK1(I,IJI)) THEN
	          SCTKFL(I,IJI)=1
	          PVTX(1)=PVTX(1)+PP(1,TTRK(J,IJI))
	          PVTX(2)=PVTX(2)+PP(2,TTRK(J,IJI))
	          PVTX(3)=PVTX(3)+PP(3,TTRK(J,IJI))
	        ENDIF
	      ENDDO
	    ENDDO
	    SCPVTX(1,IJI)=PVTX(1)
	    SCPVTX(2,IJI)=PVTX(2)
	    SCPVTX(3,IJI)=PVTX(3)
	    SCPVTX(4,IJI)=0.
	    IF(MASSEC(IJI).GT.0) SCPVTX(4,IJI)=MASSEC(IJI)
	    SCSEC(1,IJI)=SVTXSEC(1,IJI)
	    SCSEC(2,IJI)=SVTXSEC(2,IJI)
	    SCSEC(3,IJI)=SVTXSEC(3,IJI)
	    DO I=1,6
	      SCSECCV(I,IJI)=SVTXERR(I,IJI)
	    ENDDO
	    SCCHI2(IJI)=CHI2SEC(IJI)
	  ELSE  ! SECVTX NOT CALLED

	    SCTAG(IJI)=0
	    SCM1BG(IJI)=-1
	    SCNTRK(IJI)=0
	    SCNSEC(IJI)=0

	  ENDIF ! SECVTX CALLED OR NOT

	ENDDO  ! IJI BIG LOOP OVER JETINFO JET LIST

C SET EVENT FLAG
	BTGSUM=0
	IF(BTNELE(1).GT.0) BTGSUM=IBSET(BTGSUM,0)
	IF(BTNMUO(1).GT.0) BTGSUM=IBSET(BTGSUM,1)
	DO IJI=1,BTNNJET
	  IF(JVTAG(IJI).NE.0) THEN
	    IF(BTEST(BTGSUM,2)) BTGSUM=IBSET(BTGSUM,3)
	    BTGSUM=IBSET(BTGSUM,2)
	  ENDIF
	  IF(JPTAG(IJI).NE.0) THEN
	    IF(BTEST(BTGSUM,4)) BTGSUM=IBSET(BTGSUM,5)
	    BTGSUM=IBSET(BTGSUM,4)
	  ENDIF
	  IF(DPTAG(IJI).NE.0) THEN
	    IF(BTEST(BTGSUM,6)) BTGSUM=IBSET(BTGSUM,7)
	    BTGSUM=IBSET(BTGSUM,6)
	  ENDIF
	  IF(SCTAG(IJI).NE.0) THEN
	    IF(BTEST(BTGSUM,8)) BTGSUM=IBSET(BTGSUM,9)
	    BTGSUM=IBSET(BTGSUM,8)
	  ENDIF
	ENDDO


	RETURN
	END


C *************************************************************************

	SUBROUTINE BTGSPR(LUN,MODE)
C
C Dump the BTGS bank.
C
C LUN= THE LOGICAL UNIT NUMBER TO WRITE TO
C MODE=-1 : PRINT ONLY JET INFORMATION AND TAGGING FLAGS
C MODE=0  : DUMP ALL INFORMATION
C MODE=1  : DUMP ALL INFORMATION FOR JETVTX ONLY
C MODE=2  : DUMP ALL INFORMATION FOR JPBTAG ONLY
C MODE=3  : DUMP ALL INFORMATION FOR DPHI ONLY
C MODE=4  : DUMP ALL INFORMATION FOR SECVTX ONLY
C
C 09/29/94  Ray Culbertson
C

$$IMPLICIT

$$INCLUDE 'C$INC:BCS.INC'
$$INCLUDE 'YBOS$LIBRARY:ERRCOD.INC'

$$INCLUDE 'C$INC:JOBSTA.INC'
$$INCLUDE 'C$INC:BTGSCM.INC'

$$INCLUDE 'C$INC:PIVALU.INC'
$$INCLUDE 'C$INC:BTMASK.INC'

	INTEGER LUN,MODE
	INTEGER I,J,K,IJ,NELE,NMUO

	INTEGER ISTAT,BLOCAT,N,NB,IND,INDD,INNEXT
	REAL PAR(5),PAR0(5),COV(15),COV0(15)
	REAL SIGNIF,SIGMAD,PT,ETA

	INTEGER MODE2,BNKIND,VERN
	REAL RLOW,RHIGH
	INTEGER SVXHIT,CTCHIT,VTXHIT,TOWNR,PATTERN,EXPAT
	INTEGER NEXT,FTCODE,FTHIST,FTHIS2,NDOF,IQTRK
	REAL CHI2,CHICTC,CHIVTX
	REAL CTAU
	INTEGER NTEMP,TEMPL(100)
	CHARACTER*65 LINE
	CHARACTER*4 PATSTR


	CALL TREVIN(ISTAT)

	WRITE(LUN,*)
	WRITE(LUN,*) '  *******************************************',
	1	'*********************************'
	WRITE(LUN,5) NRUN,NEV
5	FORMAT(1X,'  *********** BTGS DUMP *******   RUN',I7,
	1	'  EVENT',I7,' *******************')
	WRITE(LUN,*) '  *******************************************',
	1	'*********************************'

	WRITE(LUN,*)
	DO I=1,5
	  IF(BTNELE(I).GT.0) WRITE(LUN,210) BTNELE(I)
210	FORMAT(18X, 'Soft electron tag, QTRK number:',I4)
	  IF(BTNMUO(I).GT.0) WRITE(LUN,220) BTNMUO(I)
220	FORMAT(18X, 'Soft muon tag, QTRK number:',I4)
	ENDDO


	DO IJ=1,BTNNJET

	WRITE(LUN,*)
	WRITE(LUN,10) BTJETB(IJ)
10	FORMAT(1X,'                    ********** JETS',I3,
	1	' *********')

	WRITE(LUN,100) BTNET(IJ),BTNDETA(IJ),INT(RADDEG*BTNPHI(IJ)),
	1	BTNNTRK(IJ),BTNPR(1),BTNPR(2),BTNPR(3)
100	FORMAT(1X,'Et:',F7.2,'  Deta:',F7.3,'  phi:',I4,
	1	'  NSVXS:',I3,'  primary:',2F8.4,F8.2)


	LINE=' '
	N=11
200	FORMAT(A7,I3)
	IF(JVCALL) THEN
	  WRITE(LINE(N:N+9),200) 'JETVTX:',JVTAG(IJ)
	  N=N+15
	ENDIF
	IF(JPCALL) THEN
	  WRITE(LINE(N:N+9),200) 'JPBTAG:',JPTAG(IJ)
	  N=N+15
	ENDIF
	IF(DPCALL) THEN
	  WRITE(LINE(N:N+9),200) 'BTDPHI:',DPTAG(IJ)
	  N=N+15
	ENDIF
	IF(SCCALL) THEN
	  WRITE(LINE(N:N+9),200) 'SECVTX:',SCTAG(IJ)
	  N=N+15
	ENDIF
	WRITE(LUN,*) LINE

C	WRITE(LUN,200) JVTAG(IJ),JPTAG(IJ),
C	1	DPTAG(IJ),SCTAG(IJ)
C200	FORMAT(11X, 'JETVTX:',I3,'    JPBTAG:',I3,
C	1	'    D-PHI:',I3,'    SECVTX:',I3)


C DUMP INFO ON TRACKS NEAR THIS JET

	IF(MODE.GE.0) THEN

	  WRITE(LUN,*)
	  IF(BTNNTRK(IJ).LE.0) THEN
	    WRITE(LUN,*) 'No SVX tracks in this jet.'
	  ELSE
	    WRITE(LUN,*) ' SVXS QTRK     Pt    eta    phi   ',
	1	' D(mu)   D/sigma   chi^2/Dof  L3210'
	    DO I=1,BTNNTRK(IJ)
	      ISTAT=BLOCAT(IW,'SVXS',BTNTKNO(I,IJ),IND,INDD)
	      IF(ISTAT.NE.YESUCC) THEN
	        PRINT*,'BTGSPR: Could not locate SVXS track #',
	1	BTNTKNO(I,IJ)
	      ELSE
	        CALL GTSVXS(IND,MODE2,PAR0,COV0,VERN,
     1                    SVXHIT,CTCHIT,VTXHIT,TOWNR,
     1                    PATTERN,RLOW,RHIGH,EXPAT,NEXT,FTCODE,
     2                    FTHIST,FTHIS2,NDOF,CHI2,CHICTC,CHIVTX)

	        IQTRK=IAND(ISHFT(NEXT,-16),MASK16)

	        IF(COV0(13).GT. 1.E-20) THEN  ! IF FIT WAS OK

C FIND PARAMETERS WITH RESPECT TO THE PRIMARY
	          CALL TRCHOR(PAR0,COV0,BTNPR,PAR,COV)

	          SIGMAD=COV(13)+SIN(PAR(5))**2*BTNPRCV(1)+
	1	COS(PAR(5))**2*BTNPRCV(4)-
	1	2.*COS(PAR(5))*SIN(PAR(5))*BTNPRCV(2)
	          IF(SIGMAD.GT.0.) THEN
	            SIGMAD=SQRT(SIGMAD)
	            SIGNIF=PAR(4)/SIGMAD
	          ELSE
	            SIGNIF=-1.
	          ENDIF
	        ELSE ! FIT FAILED
	          CHI2=-1.
	          SIGNIF=-1.
	        ENDIF ! FIT FAILED

	        PT=0.
	        IF(PAR(2).NE.0.) PT=1.5E-4*BMAGNT/PAR(2)
	        ETA=LOG(SQRT(1.+PAR(1)**2)+PAR(1))

	        PATSTR='0000'
	        DO J=1,4
	          IF(BTEST(PATTERN,4-J)) PATSTR(J:J)='1'
	        ENDDO

	        WRITE(LUN,250) BTNTKNO(I,IJ),IQTRK,PT,ETA,
	1	INT(RADDEG*PAR(5)),10000.*PAR(4),SIGNIF,
	1	CHI2/SVXHIT,PATSTR
250	FORMAT(1X,I4,I5,F9.2,F7.2, I7  ,F9.0,F9.2,F10.2,5X,A4)
	      ENDIF ! TRACK FOUND
	    ENDDO  ! LOOP OVER SVXS BANKS
	  ENDIF ! NO TRACKS

	ENDIF ! MODE.GE.0

	IF((MODE.EQ.0 .OR. MODE.EQ.1) .AND. JVCALL) THEN
	  WRITE(LUN,*)
	  WRITE(LUN,*) '                               JETVTX:'

	  WRITE(LUN,300) JVNTRK(IJ),JVNSEC(IJ),JVM1BG(IJ)
300	FORMAT(1X,'Accepted',I3,' tracks,',I3,' left in secondary,',
	1	' background prob:',F9.5)
	  IF(JVNSEC(IJ).GT.1) THEN
	     WRITE(LUN,320) (JVSCNO(I,IJ),I=1,JVNSEC(IJ))
320	FORMAT(1X,'Secondary vertex track list:',20I3)
	     SIGNIF=0.
	     IF(JV2DLENE(IJ).GT.0) SIGNIF=JV2DLEN(IJ)/JV2DLENE(IJ)
	     PT=SQRT(JVPVTX(1,IJ)**2+JVPVTX(2,IJ)**2)
	     CTAU=JV2DLEN(IJ)*JVPVTX(4,IJ)/PT
	     WRITE(LUN,*) '     x        y      Chi^2     ',
	1	'Lxy/sigma   mass  charge     Pt     ctau_t'
	     WRITE(LUN,310) JVSEC(1,IJ),JVSEC(2,IJ),
	1	JVCHI2(IJ),SIGNIF,JVPVTX(4,IJ),JVCVTX(IJ),
	1	PT,CTAU
310	FORMAT(1X,F8.4,F9.4,F11.2,F11.2,F8.2,I6,F11.3,F8.3)

	  ENDIF ! TAG FOUND
	ENDIF ! MODE OK TO PRINT JETVTX


	IF((MODE.EQ.0 .OR. MODE.EQ.2) .AND. JPCALL) THEN
	  WRITE(LUN,*)
	  WRITE(LUN,*) '                               JPBTAG:'

	  WRITE(LUN,400) JPNTRK(IJ),JPPRBP(IJ),JPPRBM(IJ),JPM1BG(IJ)
400	FORMAT(1X,'Accepted',I3,' tracks.  +Prob:',F9.6,
	1	'  -Prob:',F9.6,', background prob:',F8.5)
	  IF(JPNTRK(IJ).GT.0) WRITE(LUN,*) 
	1	'        Track     I.P./sigma      Prob:'
	  NTEMP=0
	  DO I=1,JPNTRK(IJ)
	     WRITE(LUN,*) JPTKNO(I,IJ),JPIPSIG(I,IJ),
	1	JPTKPRB(I,IJ)
	    IF(JPIPSIG(I,IJ).GE.0. .AND. JPTKPRB(I,IJ).LT. .01) 
	1	NTEMP=NTEMP+1
	  ENDDO
	  IF(NTEMP.GT.0) THEN
	     PT=SQRT(JPPVTX(1,IJ)**2+JPPVTX(2,IJ)**2)
	     WRITE(LUN,410) NTEMP
410	FORMAT(1X,I4,' low probabilty positive tracks:',
	1	'        mass  charge     Pt')
	     WRITE(LUN,420) JPPVTX(4,IJ),JPCVTX(IJ),PT
420	FORMAT(1X,39X,F8.2,I6,F11.3)
	  ENDIF
	ENDIF ! MODE OK TO PRINT JPBTAG


	IF( (MODE.EQ.0 .OR. MODE.EQ.3) .AND. DPCALL) THEN

	  WRITE(LUN,*)
	  WRITE(LUN,*) '                               D-PHI:'

	  IF(DPTAG(IJ).EQ.0) THEN
	     WRITE(LUN,*) 'D-phi found no secondary.'
	  ELSE

	    WRITE(LUN,500) DPNSEC(IJ)
500	FORMAT(1X,I3,' tracks in secondary.')
	    WRITE(LUN,520) (DPTKNO(I,IJ),I=1,DPNSEC(IJ))
520	FORMAT(1X,'S.V. track list:',20I3)

	    SIGNIF=0.
	    IF(DP2DLENE(IJ).GT.0) SIGNIF=DP2DLEN(IJ)/DP2DLENE(IJ)
	    PT=SQRT(DPPVTX(1,IJ)**2+DPPVTX(2,IJ)**2)
	    CTAU=DP2DLEN(IJ)*DPPVTX(4,IJ)/PT
	    WRITE(LUN,*) '     x        y      Chi^2     ',
	1	'Lxy/sigma   mass  charge     Pt     ctau_t'
	    WRITE(LUN,510) DPSEC(1,IJ),DPSEC(2,IJ),
	1	DPCHI2(IJ),SIGNIF,DPPVTX(4,IJ),DPCVTX(IJ),
	1	PT,CTAU
510	FORMAT(1X,F8.4,F9.4,F11.2,F11.2,F8.2,I6,F11.3,F8.3)

	  ENDIF
	ENDIF ! MODE OK TO PRINT D-PHI


	IF( (MODE.EQ.0 .OR. MODE.EQ.4) .AND. SCCALL) THEN

	  WRITE(LUN,*)
	  WRITE(LUN,*) '                               SECVTX:'

	  WRITE(LUN,600) SCNTRK(IJ),SCNSEC(IJ)
600	FORMAT(1X,'Accepted',I3,'  tracks,',I3,' in secondary.')

	  IF(SCNTRK(IJ).GT.0) THEN
	     WRITE(LUN,620) (SCTKNO(I,IJ),I=1,SCNTRK(IJ))
620	FORMAT(1X,'Accepted tracks :',20I3)
	  ENDIF

	  IF(SCNSEC(IJ).GT.0) THEN
	    NTEMP=0
	    DO I=1,SCNTRK(IJ)
	      IF( BTEST(SCTKFL(I,IJ),0) ) THEN
	        NTEMP=NTEMP+1
	        TEMPL(NTEMP)=SCTKNO(I,IJ)
	      ENDIF
	    ENDDO

	    WRITE(LUN,621) (TEMPL(I),I=1,NTEMP)
621	FORMAT(1X,'Secondary vertex tracks:',20I3)

	    SIGNIF=0.
	    IF(SC2DLENE(IJ).GT.0) SIGNIF=SC2DLEN(IJ)/SC2DLENE(IJ)

	    PT=SQRT(SCPVTX(1,IJ)**2+SCPVTX(2,IJ)**2)
	    CTAU=SC2DLEN(IJ)*SCPVTX(4,IJ)/PT
	    WRITE(LUN,*) '     x        y      Chi^2     ',
	1	'Lxy/sigma   mass  charge     Pt     ctau_t'
	    WRITE(LUN,610) SCSEC(1,IJ),SCSEC(2,IJ),
	1	SCCHI2(IJ),SIGNIF,SCPVTX(4,IJ),SCCVTX(IJ),
	1	PT,CTAU
610	FORMAT(1X,F8.4,F9.4,F11.2,F11.2,F8.2,I6,F11.3,F8.3)

	  ENDIF
	ENDIF ! MODE OK TO PRINT SECVTX

	WRITE(LUN,*)

	ENDDO ! IJ LOOP OVER JETS

	RETURN
	END


c *************************************************************************

	SUBROUTINE BTGCTB

C
C Pack BTGSCM.INC into BTGS bank.
C
C 09/29/94  Ray Culbertson
C
C 01/19/95  Ray Culbertson - create version 1, fix SLT's.  Previously an SLT
C had to be attached to a jet, now it can be anywhere in the event.
C If the version 1 bank is unpacked, the SLT's will be ignored (none found),
C since they were incorrect anyway.
C
$$IMPLICIT

$$INCLUDE 'C$INC:BCS.INC'
$$INCLUDE 'YBOS$LIBRARY:ERRCOD.INC'
$$INCLUDE 'C$INC:ERLEVL.INC'

$$INCLUDE 'C$INC:BTGSCM.INC'
$$INCLUDE 'C$INC:DSBTGS.INC'


	CHARACTER*30 TYBBT
	CHARACTER*24 TYBJV
	CHARACTER*37 TYBJP
	CHARACTER*24 TYBDP
	CHARACTER*30 TYBSC
	CHARACTER*150 TYBTGS

	INTEGER I,J,IJ
	INTEGER NWDBT,NWDJV,NWDJP,NWDDP,NWDSC
	INTEGER ISTAT,BMAKE,BDROP
	INTEGER WIND,TIND,NWORD,NBANK,IND,INDD,INDEX,INDEX0
	INTEGER NELE,NMUO


C DROP ANY BTGS BANKS WHICH WERE IN THE EVENT
	ISTAT=BDROP(IW,'BTGS')

C COUNT SLT'S
	NELE=0
	NMUO=0
	DO I=1,5
	  IF(BTNELE(I).GT.0) NELE=NELE+1
	  IF(BTNMUO(I).GT.0) NMUO=NMUO+1
	ENDDO


C ONE BANK FOR EACH JET
	DO IJ=1,BTNNJET

C JET VARIABLES
	  WRITE(TYBBT,100) NBTIJT,' I4 ',NBTRJT,' R4 ',
	1	BTNNTRK(IJ)+NELE+NMUO,'(',NBTITR,' I4 ',')'
100	FORMAT(I4,A4,I4,A4,I4,A1,I4,A4,A1)
	  NWDBT=NBTIJT+NBTRJT+
	1	(BTNNTRK(IJ)+NELE+NMUO)*NBTITR  ! LENGTH OF THE JET HEADER

	  IF(JVCALL) THEN
C JETVTX VARIABLES:
	    WRITE(TYBJV,200) NJVIJT,' I4 ',NJVRJT,' R4 ',
	1	JVNSEC(IJ),' I4 '
200	FORMAT(I4,A4,I4,A4,I4,A4)
	    NWDJV=NJVIJT+NJVRJT+JVNSEC(IJ)*NJVITR ! LENGTH OF JETVTX SPACE
	  ELSE
	    NWDJV=0
	    TYBJV=' '
	  ENDIF

C JPBTAG VARIABLES:
	  IF(JPCALL) THEN
	    WRITE(TYBJP,300) NJPIJT,' I4 ',NJPRJT,' R4 ',
	1	JPNTRK(IJ),'(',NJPITR,' I4 ',NJPRTR,' R4)'
300	FORMAT(I4,A4,I4,A4, I4,A1, I4,A4,I4,A4 )
	    NWDJP=NJPIJT+NJPRJT+JPNTRK(IJ)*(NJPITR+NJPRTR) ! LENGTH JPBTAG SPACE
	  ELSE
	    NWDJP=0
	    TYBJP=' '
	  ENDIF

C D-PHI VARIABLES:
	  IF(DPCALL) THEN
	    WRITE(TYBDP,200) NDPIJT,' I4 ',NDPRJT,' R4 ',
	1	DPNSEC(IJ),' I4 '
	    NWDDP=NDPIJT+NDPRJT+DPNSEC(IJ)*NDPITR ! LENGTH OF D-PHI SPACE
	  ELSE
	    NWDDP=0
	    TYBDP=' '
	  ENDIF

C SECVTX VARIABLES:
	  IF(SCCALL) THEN
	    WRITE(TYBSC,100) NSCIJT,' I4 ',NSCRJT,' R4 ',
	1	SCNTRK(IJ),'(',NSCITR,' I4 ',')'
	    NWDSC=NSCIJT+NSCRJT+SCNTRK(IJ)*NSCITR ! LENGTH OF SECVTX SPACE
	  ELSE
	    NWDSC=0
	    TYBSC=' '
	  ENDIF

C CREATE THE BANK

	  TYBTGS=TYBBT//TYBJV//TYBJP//TYBDP//TYBSC
C	PRINT*,TYBBT
C	PRINT*,TYBJV
C	PRINT*,TYBJP
C	PRINT*,TYBDP
C	PRINT*,TYBSC
C	PRINT*
C	PRINT*,TYBTGS

	  CALL BTYBLD(IW,TYBTGS,WIND,TIND,NWORD)
	  CALL BOPTN(IW,0)

	  NBANK=BTJETB(IJ)  ! BANK NUMBER SAME AS JETS NUMBER

	  ISTAT=BMAKE(IW,'BTGS',NBANK,NWORD,IW(TIND),IND,INDD)
	  IF(ISTAT.NE.YESUCC) THEN
	    CALL ERLOGR('BTGCTB',EREROR,0,0,'Error creating BTGS')
	    RETURN
	  ENDIF
	  CALL WDROP(IW,WIND,1)

C FILL THE BANK

	  INDEX0=INDD-1
	  INDEX=INDEX0

C JET VARIABLES
	  IW(INDEX+1)=1    !  VERSION NUMBER
	  IW(INDEX+2)=BTNPRFL
	  IW(INDEX+3)=BTNNTRK(IJ)
	  IW(INDEX+4)=NELE
	  IW(INDEX+5)=NMUO

	  IW(INDEX+6)=NWDBT
	  IW(INDEX+7)=NWDJV
	  IW(INDEX+8)=NWDJP
	  IW(INDEX+9)=NWDDP
	  IW(INDEX+10)=NWDSC
	  IW(INDEX+11)=0
	  INDEX=INDEX+NBTIJT

	  RW(INDEX+1)= BTNPR(1)
	  RW(INDEX+2)= BTNPR(2)
	  RW(INDEX+3)=BTNPR(3)
	  DO I=1,6
	    RW(INDEX+3+I)=BTNPRCV(I)
	  ENDDO
	  RW(INDEX+10)=BTNPJ(1,IJ)
	  RW(INDEX+11)=BTNPJ(2,IJ)
	  RW(INDEX+12)=BTNPJ(3,IJ)
	  RW(INDEX+13)=BTNET(IJ)
	  RW(INDEX+14)=BTNDETA(IJ)
	  RW(INDEX+15)=BTNETA(IJ)
	  RW(INDEX+16)=BTNPHI(IJ)
	  INDEX=INDEX+NBTRJT

	  DO I=1,BTNNTRK(IJ)
	    IW(INDEX+I)=BTNTKNO(I,IJ)
	  ENDDO
	  INDEX=INDEX+BTNNTRK(IJ)*NBTITR
	  DO I=1,NELE
	    IW(INDEX+I)=BTNELE(I)
	  ENDDO
	  INDEX=INDEX+NELE*NBTITR
	  DO I=1,NMUO
	    IW(INDEX+I)=BTNMUO(I)
	  ENDDO
	  INDEX=INDEX+NMUO*NBTITR

	  INDEX0=INDEX0+NWDBT
	  INDEX=INDEX0

C JETVTX VARIABLES
	  IF(JVCALL) THEN
	    IW(INDEX+1)=JVTAG(IJ)
	    IW(INDEX+2)=JVNTRK(IJ)
	    IW(INDEX+3)=JVNSEC(IJ)
	    IW(INDEX+4)=JVCVTX(IJ)
	    INDEX=INDEX+NJVIJT

	    RW(INDEX+1)=JVM1BG(IJ)
	    RW(INDEX+2)=JVSEC(1,IJ)
	    RW(INDEX+3)=JVSEC(2,IJ)
	    RW(INDEX+4)=JVSEC(3,IJ)
	    DO I=1,6
	      RW(INDEX+4+I)=JVSECCV(I,IJ)
	    ENDDO
	    RW(INDEX+11)=JVCHI2(IJ)
	    RW(INDEX+12)=JV2DLEN(IJ)
	    RW(INDEX+13)=JV2DLENE(IJ)
	    RW(INDEX+14)=JVPVTX(1,IJ)
	    RW(INDEX+15)=JVPVTX(2,IJ)
	    RW(INDEX+16)=JVPVTX(3,IJ)
	    RW(INDEX+17)=JVPVTX(4,IJ)
	    INDEX=INDEX+NJVRJT

	    DO I=1,JVNSEC(IJ)
	      IW(INDEX+I)=JVSCNO(I,IJ)
	    ENDDO
	    INDEX=INDEX+JVNSEC(IJ)*NJVITR

	    INDEX0=INDEX0+NWDJV
	    INDEX=INDEX0
	  ENDIF

C JPBTAG VARIABLES
	  IF(JPCALL) THEN
	    IW(INDEX+1)=JPTAG(IJ)
	    IW(INDEX+2)=JPNTRK(IJ)
	    IW(INDEX+3)=JPCVTX(IJ)
	    INDEX=INDEX+NJPIJT

	    RW(INDEX+1)=JPM1BG(IJ)
	    RW(INDEX+2)=JPPRBP(IJ)
	    RW(INDEX+3)=JPPRBM(IJ)
	    RW(INDEX+4)=JPPVTX(1,IJ)
	    RW(INDEX+5)=JPPVTX(2,IJ)
	    RW(INDEX+6)=JPPVTX(3,IJ)
	    RW(INDEX+7)=JPPVTX(4,IJ)
	    INDEX=INDEX+NJPRJT

	    J=0
	    DO I=1,JPNTRK(IJ)
	      IW(INDEX+1+J)=JPTKNO(I,IJ)
	      RW(INDEX+2+J)=JPIPSIG(I,IJ)
	      RW(INDEX+3+J)=JPTKPRB(I,IJ)
	      J=J+3
	    ENDDO
	    INDEX=INDEX+JPNTRK(IJ)*(NJPITR+NJPRTR)

	    INDEX0=INDEX0+NWDJP
	    INDEX=INDEX0
	  ENDIF

C D-PHI VARIABLES
	  IF(DPCALL) THEN
	    IW(INDEX+1)=DPTAG(IJ)
	    IW(INDEX+2)=DPNSEC(IJ)
	    IW(INDEX+3)=DPCVTX(IJ)
	    INDEX=INDEX+NDPIJT

	    RW(INDEX+1)=DPM1BG(IJ)
	    RW(INDEX+2)=DPSEC(1,IJ)
	    RW(INDEX+3)=DPSEC(2,IJ)
	    RW(INDEX+4)=DPSEC(3,IJ)
	    DO I=1,6
	      RW(INDEX+4+I)=DPSECCV(I,IJ)
	    ENDDO
	    RW(INDEX+11)=DPCHI2(IJ)
	    RW(INDEX+12)=DP2DLEN(IJ)
	    RW(INDEX+13)=DP2DLENE(IJ)
	    RW(INDEX+14)=DPPVTX(1,IJ)
	    RW(INDEX+15)=DPPVTX(2,IJ)
	    RW(INDEX+16)=DPPVTX(3,IJ)
	    RW(INDEX+17)=DPPVTX(4,IJ)
	    INDEX=INDEX+NDPRJT

	    DO I=1,DPNSEC(IJ)
	      IW(INDEX+I)=DPTKNO(I,IJ)
	    ENDDO
	    INDEX=INDEX+DPNSEC(IJ)*NDPITR

	    INDEX0=INDEX0+NWDDP
	    INDEX=INDEX0
	  ENDIF

C SECVTX VARIABLES
	  IF(SCCALL) THEN
	    IW(INDEX+1)=SCTAG(IJ)
	    IW(INDEX+2)=SCNTRK(IJ)
	    IW(INDEX+3)=SCNSEC(IJ)
	    IW(INDEX+4)=SCCVTX(IJ)
	    INDEX=INDEX+NSCIJT

	    RW(INDEX+1)=SCM1BG(IJ)
	    RW(INDEX+2)=SC2DLEN(IJ)
	    RW(INDEX+3)=SC2DLENE(IJ)
	    RW(INDEX+4)=SCSEC(1,IJ)
	    RW(INDEX+5)=SCSEC(2,IJ)
	    RW(INDEX+6)=SCSEC(3,IJ)
	    DO I=1,6
	      RW(INDEX+6+I)=SCSECCV(I,IJ)
	    ENDDO
	    RW(INDEX+13)=SCPVTX(1,IJ)
	    RW(INDEX+14)=SCPVTX(2,IJ)
	    RW(INDEX+15)=SCPVTX(3,IJ)
	    RW(INDEX+16)=SCPVTX(4,IJ)
	    RW(INDEX+17)=SCCHI2(IJ)
	    INDEX=INDEX+NSCRJT

	    J=0
	    DO I=1,SCNTRK(IJ)
	      IW(INDEX+J+1)=SCTKNO(I,IJ)
	      IW(INDEX+J+2)=SCTKFL(I,IJ)
	      J=J+NSCITR
	    ENDDO
	    INDEX=INDEX+SCNTRK(IJ)*NSCITR

	    INDEX0=INDEX0+NWDSC
	    INDEX=INDEX0
	  ENDIF

	ENDDO  !  LOOP OVER JETS


	RETURN
	END

c *************************************************************************

	SUBROUTINE BTGBTC

C
C Unpack the BTGS bank into BTGSCM.INC.
C
C 09/29/94  Ray Culbertson
C
C 01/19/95  Ray Culbertson - update to reflect version 1 changes - see BTGCTB
C

$$IMPLICIT

$$INCLUDE 'C$INC:BCS.INC'
$$INCLUDE 'YBOS$LIBRARY:ERRCOD.INC'

$$INCLUDE 'C$INC:BTGSCM.INC'
$$INCLUDE 'C$INC:DSBTGS.INC'

	INTEGER BLOCAT,ISTAT,BNEXT,BDATA,BNUMB
	INTEGER IND,INDD,N,INDEX,INDEX0,INNEXT
	INTEGER I,J,IJ,IVER
	INTEGER NWDBT,NWDJV,NWDJP,NWDDP,NWDSC
	INTEGER NELE,NMUO


	DO I=1,5
	  BTNELE(I)=0
	  BTNMUO(I)=0
	ENDDO


	IJ=0

	N=-1
	ISTAT=BLOCAT(IW,'BTGS',N,IND,INDD)
	DO WHILE(IND.GT.0)  ! LOOP OVER BTGS BANKS
	  ISTAT=BNUMB(IW,IND,N)    ! GET THE BANK NUMBER


	  INDEX0=INDD-1
	  INDEX=INDEX0
	  IJ=IJ+1
C JET VARIABLES

	  BTJETB(IJ)=N   ! JETS NUMBER IS BANK NUMBER

	  IVER=IW(INDEX+1)	! VERSION NUMBER
	  BTNPRFL=IW(INDEX+2)
	  BTNNTRK(IJ)=IW(INDEX+3)

C IGNORE THE SLT'S IN VERSION 0.  THEY WERE REQUIRED TO BE ATTACHED TO JETS
C WHICH IS WRONG.
	  IF(IVER.GE.1) THEN
	    NELE=IW(INDEX+4)
	    NMUO=IW(INDEX+5)
	  ELSE
	    NELE=0
	    NMUO=0
	  ENDIF

C THE NUMBER OF WORDS IN EACH SUBBLOCK
	  NWDBT=IW(INDEX+6)
C CHECK IF EACH TAGGER IS PRESENT
	  NWDJV=IW(INDEX+7)
	  IF(NWDJV.GT.0) THEN
	    JVCALL=.TRUE.
	  ELSE
	    JVCALL=.FALSE.
	  ENDIF
	  NWDJP=IW(INDEX+8)
	  IF(NWDJP.GT.0) THEN
	    JPCALL=.TRUE.
	  ELSE
	    JPCALL=.FALSE.
	  ENDIF
	  NWDDP=IW(INDEX+9)
	  IF(NWDDP.GT.0) THEN
	    DPCALL=.TRUE.
	  ELSE
	    DPCALL=.FALSE.
	  ENDIF
	  NWDSC=IW(INDEX+10)
	  IF(NWDSC.GT.0) THEN
	    SCCALL=.TRUE.
	  ELSE
	    SCCALL=.FALSE.
	  ENDIF
C SPARE	  IW(INDEX+11)=0
	  INDEX=INDEX+NBTIJT

	  BTNPR(1)=RW(INDEX+1)
	  BTNPR(2)=RW(INDEX+2)
	  BTNPR(3)=RW(INDEX+3)
	  DO I=1,6
	    BTNPRCV(I)=RW(INDEX+3+I)
	  ENDDO
	  BTNPJ(1,IJ)=RW(INDEX+10)
	  BTNPJ(2,IJ)=RW(INDEX+11)
	  BTNPJ(3,IJ)=RW(INDEX+12)
	  BTNET(IJ)=RW(INDEX+13)
	  BTNDETA(IJ)=RW(INDEX+14)
	  BTNETA(IJ)=RW(INDEX+15)
	  BTNPHI(IJ)=RW(INDEX+16)
	  INDEX=INDEX+NBTRJT

	  DO I=1,BTNNTRK(IJ)
	    BTNTKNO(I,IJ)=IW(INDEX+I)
	  ENDDO
	  INDEX=INDEX+BTNNTRK(IJ)*NBTITR
	  DO I=1,NELE
	    BTNELE(I)=IW(INDEX+I)
	  ENDDO
	  INDEX=INDEX+NELE*NBTITR
	  DO I=1,NMUO
	    BTNMUO(I)=IW(INDEX+I)
	  ENDDO
	  INDEX=INDEX+NMUO*NBTITR

	  INDEX0=INDEX0+NWDBT
	  INDEX=INDEX0

C JETVTX VARIABLES
	  IF(JVCALL) THEN
	    JVTAG(IJ)=IW(INDEX+1)
	    JVNTRK(IJ)=IW(INDEX+2)
	    JVNSEC(IJ)=IW(INDEX+3)
	    JVCVTX(IJ)=IW(INDEX+4)
	    INDEX=INDEX+NJVIJT

	    JVM1BG(IJ)=RW(INDEX+1)
	    JVSEC(1,IJ)=RW(INDEX+2)
	    JVSEC(2,IJ)=RW(INDEX+3)
	    JVSEC(3,IJ)=RW(INDEX+4)
	    DO I=1,6
	      JVSECCV(I,IJ)=RW(INDEX+4+I)
	    ENDDO
	    JVCHI2(IJ)=RW(INDEX+11)
	    JV2DLEN(IJ)=RW(INDEX+12)
	    JV2DLENE(IJ)=RW(INDEX+13)
	    JVPVTX(1,IJ)=RW(INDEX+14)
	    JVPVTX(2,IJ)=RW(INDEX+15)
	    JVPVTX(3,IJ)=RW(INDEX+16)
	    JVPVTX(4,IJ)=RW(INDEX+17)
	    INDEX=INDEX+NJVRJT

	    DO I=1,JVNSEC(IJ)
	      JVSCNO(I,IJ)=IW(INDEX+I)
	    ENDDO
	    INDEX=INDEX+JVNSEC(IJ)*NJVITR

	    INDEX0=INDEX0+NWDJV
	    INDEX=INDEX0
	  ENDIF

C JPBTAG VARIABLES
	  IF(JPCALL) THEN
	    JPTAG(IJ)=IW(INDEX+1)
	    JPNTRK(IJ)=IW(INDEX+2)
	    JPCVTX(IJ)=IW(INDEX+3)
	    INDEX=INDEX+NJPIJT

	    JPM1BG(IJ)=RW(INDEX+1)
	    JPPRBP(IJ)=RW(INDEX+2)
	    JPPRBM(IJ)=RW(INDEX+3)
	    JPPVTX(1,IJ)=RW(INDEX+4)
	    JPPVTX(2,IJ)=RW(INDEX+5)
	    JPPVTX(3,IJ)=RW(INDEX+6)
	    JPPVTX(4,IJ)=RW(INDEX+7)
	    INDEX=INDEX+NJPRJT

	    J=0
	    DO I=1,JPNTRK(IJ)
	      JPTKNO(I,IJ)=IW(INDEX+1+J)
	      JPIPSIG(I,IJ)=RW(INDEX+2+J)
	      JPTKPRB(I,IJ)=RW(INDEX+3+J)
	      J=J+3
	    ENDDO
	    INDEX=INDEX+JPNTRK(IJ)*(NJPITR+NJPRTR)

	    INDEX0=INDEX0+NWDJP
	    INDEX=INDEX0
	  ENDIF

C D-PHI VARIABLES
	  IF(DPCALL) THEN
	    DPTAG(IJ)=IW(INDEX+1)
	    DPNSEC(IJ)=IW(INDEX+2)
	    DPCVTX(IJ)=IW(INDEX+3)
	    INDEX=INDEX+NDPIJT

	    DPM1BG(IJ)=RW(INDEX+1)
	    DPSEC(1,IJ)=RW(INDEX+2)
	    DPSEC(2,IJ)=RW(INDEX+3)
	    DPSEC(3,IJ)=RW(INDEX+4)
	    DO I=1,6
	      DPSECCV(I,IJ)=RW(INDEX+4+I)
	    ENDDO
	    DPCHI2(IJ)=RW(INDEX+11)
	    DP2DLEN(IJ)=RW(INDEX+12)
	    DP2DLENE(IJ)=RW(INDEX+13)
	    DPPVTX(1,IJ)=RW(INDEX+14)
	    DPPVTX(2,IJ)=RW(INDEX+15)
	    DPPVTX(3,IJ)=RW(INDEX+16)
	    DPPVTX(4,IJ)=RW(INDEX+17)
	    INDEX=INDEX+NDPRJT

	    DO I=1,DPNSEC(IJ)
	      DPTKNO(I,IJ)=IW(INDEX+I)
	    ENDDO
	    INDEX=INDEX+DPNSEC(IJ)*NDPITR

	    INDEX0=INDEX0+NWDDP
	    INDEX=INDEX0
	  ENDIF

C SECVTX VARIABLES
	  IF(SCCALL) THEN
	    SCTAG(IJ)=IW(INDEX+1)
	    SCNTRK(IJ)=IW(INDEX+2)
	    SCNSEC(IJ)=IW(INDEX+3)
	    SCCVTX(IJ)=IW(INDEX+4)
	    INDEX=INDEX+NSCIJT

	    SCM1BG(IJ)=RW(INDEX+1)
	    SC2DLEN(IJ)=RW(INDEX+2)
	    SC2DLENE(IJ)=RW(INDEX+3)
	    SCSEC(1,IJ)=RW(INDEX+4)
	    SCSEC(2,IJ)=RW(INDEX+5)
	    SCSEC(3,IJ)=RW(INDEX+6)
	    DO I=1,6
	      SCSECCV(I,IJ)=RW(INDEX+6+I)
	    ENDDO
	    SCPVTX(1,IJ)=RW(INDEX+13)
	    SCPVTX(2,IJ)=RW(INDEX+14)
	    SCPVTX(3,IJ)=RW(INDEX+15)
	    SCPVTX(4,IJ)=RW(INDEX+16)
	    SCCHI2(IJ)=RW(INDEX+17)
	    INDEX=INDEX+NSCRJT

	    J=0
	    DO I=1,SCNTRK(IJ)
	      SCTKNO(I,IJ)=IW(INDEX+J+1)
	      SCTKFL(I,IJ)=IW(INDEX+J+2)
	      J=J+NSCITR
	    ENDDO
	    INDEX=INDEX+SCNTRK(IJ)*NSCITR

	    INDEX0=INDEX0+NWDSC
	    INDEX=INDEX0
	  ENDIF

	  ISTAT=BNEXT(IW,IND,INNEXT) ! NEXT BTGS BANK
	  IND=INNEXT
	  ISTAT=BDATA(IW,IND,INDD)  ! GET ITS DATA INDEX
	ENDDO  ! LOOP OVER BTGS BANKS

	BTNNJET=IJ

C SET EVENT FLAG
	BTGSUM=0
	IF(BTNELE(1).GT.0) BTGSUM=IBSET(BTGSUM,0)
	IF(BTNMUO(1).GT.0) BTGSUM=IBSET(BTGSUM,1)
	DO IJ=1,BTNNJET
	  IF(JVTAG(IJ).NE.0) THEN
	    IF(BTEST(BTGSUM,2)) BTGSUM=IBSET(BTGSUM,3)
	    BTGSUM=IBSET(BTGSUM,2)
	  ENDIF
	  IF(JPTAG(IJ).NE.0) THEN
	    IF(BTEST(BTGSUM,4)) BTGSUM=IBSET(BTGSUM,5)
	    BTGSUM=IBSET(BTGSUM,4)
	  ENDIF
	  IF(DPTAG(IJ).NE.0) THEN
	    IF(BTEST(BTGSUM,6)) BTGSUM=IBSET(BTGSUM,7)
	    BTGSUM=IBSET(BTGSUM,6)
	  ENDIF
	  IF(SCTAG(IJ).NE.0) THEN
	    IF(BTEST(BTGSUM,8)) BTGSUM=IBSET(BTGSUM,9)
	    BTGSUM=IBSET(BTGSUM,8)
	  ENDIF
	ENDDO

	RETURN
	END

