c ===========================================================================
	subroutine cosmic_cmuo(i,j,answer)
c
c
c	Routine flags events which have a cosmic ray going through them
c
c	 inputs:
c		i		cmuo muon bank number
c		j       	cmuo muon bank number
c	outputs:
c		answer 		true -- cosmic ray identified
c
c
c	Written by:
c		Andrew Martin
c		Rob Roser
c
c ===========================================================================
c ===========================================================================
c
$$implicit
C
$$include 'c$inc:jobsta.inc'
$$include 'c$inc:bcs.inc'
$$include 'ybos$library:errcod.inc'
$$include 'c$inc:cmuo92.inc'
$$include 'c$inc:muoinfo.inc'
$$include 'c$inc:pivalu.inc'
c
	integer i,j,ind,inddat,status,blocat
	logical answer
c
	real eta1,eta2,phi1,phi2,seta,dphi,tdc1,tdc2
	real seta_cut/0.1/
	real dphi_cut/178.5/
	real seta_cut2/0.25/
	real dphi_cut2/177.0/
	real tdc_max_1a/18./,tdc_min_1a/-14./
	real tdc_max_1b/22./,tdc_min_1b/-14./
	real tdc_diff_1a/14./,tdc_diff_1b/14./
	real tdc_max,tdc_min,tdc_diff
        integer muo_muon
        DATA MUO_MUON         /'1C'X/
c
	dphi=0.
	seta=10.
	answer=.false.

c check if both "muons" are legit -- otherwise, bail in a big way
c -----------------------------------------------------------------------
	   if(bnk_cmuo(i).eq.0)goto 9999
	   if(bnk_cmuo(j).eq.0)goto 9999
           IF(IAND(MUO_WRD(I),MUO_MUON).NE.0)goto 9999
           IF(IAND(MUO_WRD(J),MUO_MUON).NE.0)goto 9999

c guard against divide by 0...
c ----------------------------
	if(p4m_cmuo(1,i).eq.0.)goto 9999
	if(p4m_cmuo(1,j).eq.0.)goto 9999
	if((p4m_cmuo(4,i)-p4m_cmuo(3,i)).eq.0.)goto 9999
	if((p4m_cmuo(4,j)-p4m_cmuo(3,j)).eq.0.)goto 9999
c
c get dphi
c ---------
	phi1 = (atan2(-p4m_cmuo(2,i),-p4m_cmuo(1,i))+pi)*raddeg
	phi2 = (atan2(-p4m_cmuo(2,j),-p4m_cmuo(1,j))+pi)*raddeg
	dphi = abs(phi1-phi2)
	if(dphi.gt.180.) dphi=360-dphi
c
c get sum eta
c ------------
	eta1 = 0.5*alog((p4m_cmuo(4,i)+p4m_cmuo(3,i))/
     >		(p4m_cmuo(4,i)-p4m_cmuo(3,i)))
	eta2 = 0.5*alog((p4m_cmuo(4,j)+p4m_cmuo(3,j))/
     >		(p4m_cmuo(4,j)-p4m_cmuo(3,j)))
	seta = abs(eta1+eta2)
c
c pure back-to-back cut
c ---------------------
	if((dphi.gt.dphi_cut).and.(seta.lt.seta_cut)) then
	  answer=.true.
	  goto 9999
	endif
c
c get had TDC values
c ------------------
	status= blocat(iw,'CMUO',bnk_cmuo(i),ind,inddat)
	if(status.eq.yesucc) then
	  tdc1 = rw(inddat+mochat)
	  if(tdc1.eq.-1000.) goto 9999
	  if((tdc1.le.0.00001).and.(tdc1.ge.0.)) goto 9999
	else
	  call erlogr('COSMIC_CMUO',bnk_cmuo(i),status,
     >		'CMUO bank lost')
	  goto 9999
	endif
	status= blocat(iw,'CMUO',bnk_cmuo(j),ind,inddat)
	if(status.eq.yesucc) then
	  tdc2 = rw(inddat+mochat)
	  if(tdc2.eq.-1000.) goto 9999
	  if((tdc2.le.0.00001).and.(tdc2.ge.0.)) goto 9999
	else
	  call erlogr('COSMIC_CMUO',bnk_cmuo(i),status,
     >		'CMUO bank lost')
	  goto 9999
	endif
c
c had tdc cut
c ------------
	if(nrun.gt.50000) then
	  tdc_max=tdc_max_1b
	  tdc_min=tdc_min_1b
	  tdc_diff=tdc_diff_1b
	else
	  tdc_max=tdc_max_1a
	  tdc_min=tdc_min_1a
	  tdc_diff=tdc_diff_1a
	endif
	if((dphi.gt.dphi_cut2).and.(seta.lt.seta_cut2)) then
	  if((tdc1.gt.tdc_max).or.(tdc1.lt.tdc_min).or.
     >		(tdc2.gt.tdc_max).or.(tdc2.lt.tdc_min).or.
     >		(abs(tdc1-tdc2).gt.tdc_diff)) then
	    answer = .true.
	    goto 9999
	  endif
	endif
9999	return
	end


c===========================================================================
c===========================================================================
c
c
c
	subroutine cosmic_cmio(i,j,answer)
c
c
c
c	Routine flags events which have a cosmic ray going through them
c
c	 inputs:
c		i		cmuo muon bank number
c		j       	cmio muon bank number
c	outputs:
c		answer 		true -- cosmic ray identified
c
c
c	Written by:
c		Andrew Martin
c		Rob Roser
c
c ===========================================================================
c ===========================================================================
c
c
$$implicit
C
$$include 'c$inc:jobsta.inc'
$$include 'c$inc:bcs.inc'
$$include 'ybos$library:errcod.inc'
$$include 'c$inc:cmuo92.inc'
$$include 'c$inc:muoinfo.inc'
$$include 'c$inc:mioinfo.inc'
$$include 'c$inc:pivalu.inc'
c
	integer i,j,ind,inddat,status,blocat
	logical answer
c
	real eta1,eta2,phi1,phi2,seta,dphi,tdc1,tdc2
	real seta_cut/0.1/
	real dphi_cut/178.5/
	real seta_cut2/0.25/
	real dphi_cut2/177.0/
	real tdc_max_1a/18./,tdc_min_1a/-14./
	real tdc_max_1b/22./,tdc_min_1b/-14./
	real tdc_diff_1a/14./,tdc_diff_1b/14./
	real tdc_max,tdc_min,tdc_diff
        integer mio_muon,muo_muon
        DATA MIO_MUON         /'1C'X/
        DATA MUO_MUON         /'1C'X/
c
c get dphi
c --------
	dphi=0.
	seta=10.
	answer=.false.

c check if both "muons" are legit -- otherwise, bail in a big way
c ----------------------------------------------------------------
	   if(bnk_cmuo(i).eq.0)goto 9999
	   if(bnk_cmio(j).eq.0)goto 9999
           IF(IAND(MUO_WRD(I),MUO_MUON).NE.0)goto 9999
           IF(IAND(MIO_WRD(j),MIO_MUON).NE.0)goto 9999

c guard against divide by 0...
c ----------------------------
	if(p4m_cmuo(1,i).eq.0.)goto 9999
	if(p4m_cmio(1,j).eq.0.)goto 9999
	if((p4m_cmuo(4,i)-p4m_cmuo(3,i)).eq.0.)goto 9999
	if((p4m_cmio(4,j)-p4m_cmio(3,j)).eq.0.)goto 9999


	phi1 = (atan2(-p4m_cmuo(2,i),-p4m_cmuo(1,i))+pi)*raddeg
	phi2 = (atan2(-p4m_cmio(2,j),-p4m_cmio(1,j))+pi)*raddeg
	dphi = abs(phi1-phi2)
	if(dphi.gt.180.) dphi=360-dphi
c
c get sum eta
c ------------
	eta1 = 0.5*alog((p4m_cmuo(4,i)+p4m_cmuo(3,i))/
     >		(p4m_cmuo(4,i)-p4m_cmuo(3,i)))
	eta2 = 0.5*alog((p4m_cmio(4,j)+p4m_cmio(3,j))/
     >		(p4m_cmio(4,j)-p4m_cmio(3,j)))
	seta = abs(eta1+eta2)
c
c pure back-to-back cut
c ---------------------
	if((dphi.gt.dphi_cut).and.(seta.lt.seta_cut)) then
	  answer=.true.
	  goto 9999
	endif
c
c get had TDC values
c -------------------
	status= blocat(iw,'CMUO',bnk_cmuo(i),ind,inddat)
	if(status.eq.yesucc) then
	  tdc1 = rw(inddat+mochat)
	  if(tdc1.eq.-1000.) goto 9999
	  if((tdc1.le.0.00001).and.(tdc1.ge.0.)) goto 9999
	else
	  call erlogr('COSMIC_CMIO',bnk_cmuo(i),status,
     >		'CMUO bank lost')
	  goto 9999
	endif
	status= blocat(iw,'CMIO',bnk_cmio(j),ind,inddat)
	if(status.eq.yesucc) then
	  tdc2 = rw(inddat+mochat)
	  if(tdc2.eq.-1000.) goto 9999
	  if((tdc2.le.0.00001).and.(tdc2.ge.0.)) goto 9999
	else
	  call erlogr('COSMIC_CMIO',bnk_cmio(j),status,
     >		'CMIO bank lost')
	  goto 9999
	endif
c
c had tdc cut
c -----------
	if(nrun.gt.50000) then
	  tdc_max=tdc_max_1b
	  tdc_min=tdc_min_1b
	  tdc_diff=tdc_diff_1b
	else
	  tdc_max=tdc_max_1a
	  tdc_min=tdc_min_1a
	  tdc_diff=tdc_diff_1a
	endif
	if((dphi.gt.dphi_cut2).and.(seta.lt.seta_cut2)) then
	  if((tdc1.gt.tdc_max).or.(tdc1.lt.tdc_min).or.
     >		(tdc2.gt.tdc_max).or.(tdc2.lt.tdc_min).or.
     >		(abs(tdc1-tdc2).gt.tdc_diff)) then
	    answer = .true.
	    goto 9999
	  endif
	endif
9999	return
	end

c===========================================================================
c===========================================================================

	subroutine cosmic_2cmio(i,j,answer)
c
c
c
c	Routine flags events which have a cosmic ray going through them
c
c	 inputs:
c		i		cmio muon bank number
c		j       	cmio muon bank number
c	outputs:
c		answer 		true -- cosmic ray identified
c
c
c	Written by:
c		Andrew Martin
c		Rob Roser
c
c ===========================================================================
c ===========================================================================
c
c
$$implicit
C
$$include 'c$inc:jobsta.inc'
$$include 'c$inc:bcs.inc'
$$include 'ybos$library:errcod.inc'
$$include 'c$inc:cmuo92.inc'
$$include 'c$inc:mioinfo.inc'
$$include 'c$inc:pivalu.inc'
c
	integer i,j,ind,inddat,status,blocat
	logical answer
c
	real eta1,eta2,phi1,phi2,seta,dphi,tdc1,tdc2
	real seta_cut/0.1/
	real dphi_cut/178.5/
	real seta_cut2/0.25/
	real dphi_cut2/177.0/
	real tdc_max_1a/18./,tdc_min_1a/-14./
	real tdc_max_1b/22./,tdc_min_1b/-14./
	real tdc_diff_1a/14./,tdc_diff_1b/14./
	real tdc_max,tdc_min,tdc_diff
        integer mio_muon
        DATA MIO_MUON         /'1C'X/
c
c get dphi
c ---------
	dphi=0.
	seta=10.
	answer=.false.

c check if both "muons" are legit -- otherwise, bail in a big way
c ----------------------------------------------------------------
	   if(bnk_cmio(i).eq.0)goto 9999
	   if(bnk_cmio(j).eq.0)goto 9999
           IF(IAND(MIO_WRD(I),MIO_MUON).NE.0)goto 9999
           IF(IAND(MIO_WRD(j),MIO_MUON).NE.0)goto 9999

c guard against divide by 0...
c ---------------------------
	if(p4m_cmio(1,i).eq.0.)goto 9999
	if(p4m_cmio(1,j).eq.0.)goto 9999
	if((p4m_cmio(4,i)-p4m_cmio(3,i)).eq.0.)goto 9999
	if((p4m_cmio(4,j)-p4m_cmio(3,j)).eq.0.)goto 9999


	phi1 = (atan2(-p4m_cmio(2,i),-p4m_cmio(1,i))+pi)*raddeg
	phi2 = (atan2(-p4m_cmio(2,j),-p4m_cmio(1,j))+pi)*raddeg
	dphi = abs(phi1-phi2)
	if(dphi.gt.180.) dphi=360-dphi
c
c get sum eta
c -----------
	eta1 = 0.5*alog((p4m_cmio(4,i)+p4m_cmio(3,i))/
     >		(p4m_cmio(4,i)-p4m_cmio(3,i)))
	eta2 = 0.5*alog((p4m_cmio(4,j)+p4m_cmio(3,j))/
     >		(p4m_cmio(4,j)-p4m_cmio(3,j)))
	seta = abs(eta1+eta2)
c
c pure back-to-back cut
c ---------------------
	if((dphi.gt.dphi_cut).and.(seta.lt.seta_cut)) then
	  answer=.true.
	  goto 9999
	endif
c
c get had TDC values
c -------------------
	status= blocat(iw,'CMIO',bnk_cmio(i),ind,inddat)
	if(status.eq.yesucc) then
	  tdc1 = rw(inddat+mochat)
	  if(tdc1.eq.-1000.) goto 9999
	  if((tdc1.le.0.00001).and.(tdc1.ge.0.)) goto 9999
	else
	  call erlogr('COSMIC_2CMIO',bnk_cmio(i),status,
     >		'CMIO bank lost')
	  goto 9999
	endif
	status= blocat(iw,'CMIO',bnk_cmio(j),ind,inddat)
	if(status.eq.yesucc) then
	  tdc2 = rw(inddat+mochat)
	  if(tdc2.eq.-1000.) goto 9999
	  if((tdc2.le.0.00001).and.(tdc2.ge.0.)) goto 9999
	else
	  call erlogr('COSMIC_2CMIO',bnk_cmio(j),status,
     >		'CMIO bank lost')
	  goto 9999
	endif
c
c had tdc cut
c -----------
	if(nrun.gt.50000) then
	  tdc_max=tdc_max_1b
	  tdc_min=tdc_min_1b
	  tdc_diff=tdc_diff_1b
	else
	  tdc_max=tdc_max_1a
	  tdc_min=tdc_min_1a
	  tdc_diff=tdc_diff_1a
	endif
	if((dphi.gt.dphi_cut2).and.(seta.lt.seta_cut2)) then
	  if((tdc1.gt.tdc_max).or.(tdc1.lt.tdc_min).or.
     >		(tdc2.gt.tdc_max).or.(tdc2.lt.tdc_min).or.
     >		(abs(tdc1-tdc2).gt.tdc_diff)) then
	    answer = .true.
	    goto 9999
	  endif
	endif
9999	return
	end

