c ===========================================================================
c ===========================================================================
	SUBROUTINE dil_topology(k,answer)
c
c	This routine is called by topfnd.  Its function is to apply the 
c	dilepton selection cuts to see whether a given event would pass
c	the dilepton criteria.  To use this routine.
c		-  Call the standard MUO/MIO/ELE/PEM Fndr Routines
c		-  If you find two lepton candidates, then call this routine.
c		-  This routine will then apply
c			MET, Met Phi
c		        Jet ET -- 2 jets
c			Opposite sign cut
c
c	Inputs	  k, which row in the dil_type matrix we are in
c
c	Outputs   Answer = .TRUE. means passes dilep criteria
c
c                 Cut word with the following bits (1=failed,0=passed)
c                      bit                                  
c                        0.....Two leptons in event(If set,no other cuts tested)
c                        1.....Z Cut (1=this is a Z event)
c                        2.....Opposite Sign Cut
c                        3.....Cosmic Ray Filter
c                        4.....Met > 25 GeV
c                        5.....Lepton Angle for 25<MET<50
c                        6.....Jet Angle for 25<MET<50
c                        7.....Number of Jets >= 2
c
c	Authors
c	  Andrew Martin, Rob Roser
c
c  History:
c       21-MAR-95  Added DILINFO.INC ............................. G. Sullivan
c       15-SEP-95  Added lepton id variables to common block...... R. Roser
c        2-Oct-95  Added DIL_WRD for bit packed cut mask.......... B. Winer
c        6-Oct-95  Modified Cosmic code to look beyond mumu ev.... R. + B.
c
c  =========================================================================
$$IMPLICIT
C
$$INCLUDE 'C$INC:ANACTL.INC'
$$INCLUDE 'C$INC:BCS.INC'
$$INCLUDE 'A_C$LIBRARY:ANERROR.INC'
$$INCLUDE 'C$INC:ERLEVL.INC'
$$INCLUDE 'C$INC:JOBSTA.INC'
$$INCLUDE 'C$INC:LOGUNI.INC'
$$INCLUDE 'YBOS$LIBRARY:BNKTYP.INC'
$$INCLUDE 'YBOS$LIBRARY:ERRCOD.INC'
$$include 'c$inc:pivalu.inc'
$$INCLUDE 'c$inc:ELEINFO.INC'
$$INCLUDE 'c$inc:MUOINFO.INC'
$$INCLUDE 'c$inc:MIOINFO.INC'
$$INCLUDE 'c$inc:JETINFO.INC'
$$INCLUDE 'C$inc:EVTINFO.INC'
$$INCLUDE 'C$inc:DILINFO.INC'
$$INCLUDE 'TOP$FND:DILMORE.INC'
C
c input/output variables
	logical answer
        integer k

c local
 	logical ee,emu,mumu,pem,cmio

        logical jet_et_cor/.false./
	real et1_cut/10./
	real et2_cut/10./
	real metmag_cut1/25./
	real metmag_cut2/50./
	real metphi_cut_l/20./
	real metphi_cut_j/20./
	real min_et/10.0/
	real max_eta/2.4/
        integer jetcone/2/
	real jeta_max/2.0/
                                                 
        integer i,j
        integer ip,jp

	integer status,anptrg
	integer numjets
	integer n1,n2
        real tot_charge
	real metphi1
	real phi1,phi2,dphi1,dphi2
	real phij(10),dphij(10)
	real met_mag
        real metx,mety,metjetx,metjety

	logical cosmic
c ============================================================================
c
c init local variables
	answer=.true.
	ee   = .false.
	emu  = .false.
	mumu = .false.
	pem  = .false.
	cmio  = .false.
	cosmic = .false.
        dil_wrd = 0 
        ele1_ptr = 0
        ele2_ptr = 0
        muo1_ptr = 0
        muo2_ptr = 0


	tot_charge=-666.       

 
c find event type and set flags
	if(dil_type(1,k).ge.1.and.dil_type(1,k).le.3)then
	  ee=.true.
	else if(dil_type(1,k).ge.10.and.dil_type(1,k).le.12)then
	  mumu=.true.
	else if((dil_type(1,k).ge.4.and.dil_type(1,k).le.9))then 
	  emu=.true.
	endif
	if(dil_type(1,k).eq.3 .or. dil_type(1,k).eq.9)pem=.true.
	if(dil_type(1,k).eq.6 .or. dil_type(1,k).eq.12)cmio=.true.

c something should be set... otherwise punt
	if(ee)then
           dil_mode=1
           goto 123
        endif
	if(emu)then
           dil_mode=2
           goto 123
        endif
	if(mumu)then
           dil_mode=3
           goto 123
        endif
	goto 9999


c pointer management -- decoding Brian's convoluted scheme...

123	ip=DIL_TYPE(2,K)
	jp=DIL_TYPE(3,K)
c-------------------------------
c emu	     			|	
c	  ce mu   HPTE_CMU/4/	|
c	  ce mu   HPTE_CMX/5/	|
c	  ce mi   HPTE_CMIO/6/	|
c	  mu ce   HPTM_HPTE/7/	|
c	  mu lce  HPTM_LCEM/8/	|
c	  mu-pe   HPTM_PEM/9/	|
c ee				|
c	  ce ce   HPTE_HPTE/1/	|
c	  ce lce  HPTE_LCEM/2/	|
c	  ce pe   HPTE_PEM/3/	|
c mumu				|
c	  mu mu   HPTM_CMU/10/   |
c	  mu mu   HPTM_CMX/11/  |
c	  mu-mi   HPTM_CMIO/12/ |
c ------------------------------

c set  ee pointers
	if(ee)then
	  ele1_ptr=ind_hpte(ip)
	  if(dil_type(1,k).eq.1)ele2_ptr=ind_hpte(jp)
	  if(dil_type(1,k).eq.2)ele2_ptr=ind_loose_e(jp)
	  if(dil_type(1,k).eq.3)ele2_ptr=ind_pem(jp)
	endif

c set  emu pointers
	if(emu)then
	  if(dil_type(1,k).eq.4)then
	    ele1_ptr=ind_hpte(ip)
	    muo1_ptr=ind_hptm(jp)
	  endif
	  if(dil_type(1,k).eq.5)then
	    ele1_ptr=ind_hpte(ip)
	    muo1_ptr=ind_hptm(jp)
	  endif
	  if(dil_type(1,k).eq.6)then
	    ele1_ptr=ind_hpte(ip)
	    muo1_ptr=ind_cmio(jp)
	  endif
	  if(dil_type(1,k).eq.7)then
	    muo1_ptr=ind_hptm(ip)
	    ele1_ptr=ind_hpte(jp)
	  endif
	  if(dil_type(1,k).eq.8)then
	    muo1_ptr=ind_hptm(ip)
	    ele1_ptr=ind_loose_e(jp)
	  endif
	  if(dil_type(1,k).eq.9)then
	    muo1_ptr=ind_hptm(ip)
	    ele1_ptr=ind_pem(jp)
	  endif
	endif

c set mumu pointers
	if(mumu)then
	  muo1_ptr=ind_hptm(ip)
	  if(dil_type(1,k).eq.12)then
	    muo2_ptr=ind_cmio(jp)
C	    call cosmic_cmio(muo1_ptr,muo2_ptr,cosmic)
	  else
	    muo2_ptr=ind_hptm(jp)
C	    call cosmic_cmuo(muo1_ptr,muo2_ptr,cosmic)
	  endif
      	endif

C Check for cosmic rays for all muons.
c ------------------------------------
        if(emu.or.mumu) then
           do i=1,numcmuo

c Check 1st Muon with all other CMUO's
c ------------------------------------
              if(muo1_ptr.ne.i) call cosmic_cmuo(muo1_ptr,i,cosmic)
  	      if(cosmic) then
                 dil_wrd = ibset(dil_wrd,3)
                 answer = .false.
              endif  

c If a mu-mu event, check second leg with respect to all other CMUO's
c -------------------------------------------------------------------
              if(mumu.and.dil_type(1,k).eq.12) then
                 call cosmic_cmio(i,muo2_ptr,cosmic)
     	         if(cosmic) then
                    dil_wrd = ibset(dil_wrd,3)
                    answer = .false.
                 endif
              elseif(mumu.and.dil_type(1,k).ne.12) then 
                 if(muo2_ptr.ne.i) call cosmic_cmuo(muo2_ptr,i,cosmic)
     	         if(cosmic) then
                    dil_wrd = ibset(dil_wrd,3)
                    answer = .false.
                 endif
              endif
           enddo

           do i=1,num_cmio

c Check 1st Muon with all other CMIO's
c ------------------------------------
              call cosmic_cmio(muo1_ptr,i,cosmic)
  	      if(cosmic) then
                 dil_wrd = ibset(dil_wrd,3)
                 answer = .false.
              endif  

c If a mu-mu event, check second leg with respect to all other CMUO's
c -------------------------------------------------------------------
              if(mumu.and.dil_type(1,k).eq.12) then
                  call cosmic_2cmio(i,muo2_ptr,cosmic)
     	         if(cosmic) then
                    dil_wrd = ibset(dil_wrd,3)
                    answer = .false.
                 endif
              elseif(mumu.and.dil_type(1,k).ne.12) then 
                 if(muo2_ptr.ne.i) call cosmic_cmio(muo2_ptr,i,cosmic)
     	         if(cosmic) then
                    dil_wrd = ibset(dil_wrd,3)
                    answer = .false.
                 endif
              endif

           enddo
        endif


c ----------------------------- Isolation Cut ----------------------------------
c   These cuts are performed in TOPFND before the call to this routine -BLW
c -----------------------------------------------------------------------------
c	if(ee) then
c	  if(iand(cem_iso(ele1_ptr),6).ne.0) answer = .false.
c	endif
c
c	if(mumu) then
c	  if((iand(muo_iso(muo1_ptr),6).ne.0).and.(dil_type(1,k).lt.12)) then
c	    answer=.false.
c	  elseif(((iand(muo_iso(muo1_ptr),6).ne.0).or.
c     >		(iand(mio_iso(muo2_ptr),7).ne.0)).and.
c     >		(dil_type(1,k).eq.12)) then
c	    answer = .false.
c	  endif
c	endif
c
c	if(emu) then
c	  if(dil_type(1,k).lt.6) then
c	    if(iand(cem_iso(ele1_ptr),6).ne.0) answer = .false.
c	  elseif(dil_type(1,k).eq.6) then
c	    if((iand(cem_iso(ele1_ptr),6).ne.0).or.
c     >		(iand(mio_iso(muo1_ptr),7).ne.0)) answer = .false.
c	  else
c	    if(iand(muo_iso(muo1_ptr),6).ne.0) answer = .false.
c	  endif
c	endif
c
c ----------------------------- Opposite Sign Cut ------------------------------
	if(ee)then
	  tot_charge = chg_ele(ele1_ptr)+chg_ele(ele2_ptr)
	endif
                      
	if(emu)then
	  if(cmio)then
	    tot_charge = chg_ele(ele1_ptr)+chrg_cmio(muo1_ptr)
	  else
	    tot_charge = chg_ele(ele1_ptr)+chrg_cmuo(muo1_ptr)
	  endif
	endif

	if(mumu)then
	  if(cmio)then
	    tot_charge = chrg_cmuo(muo1_ptr)+chrg_cmio(muo2_ptr)
	  else
	    tot_charge = chrg_cmuo(muo1_ptr)+chrg_cmuo(muo2_ptr)
	  endif
	endif    

	oppsign_dil=.true.
	if(tot_charge.ne.0.) then
	   oppsign_dil = .false.
	endif
	   

c----------------------------- Remove Z's -------------------------------------
	if(z_event) then
            answer = .false.
            dil_wrd = ibset(dil_wrd,1)
        endif

	if(iand(dil_wrd,2).eq.0) good_dil = .true.

c ----------------------------- Get Met it Pay's ------------------------------
c get MET components
	metx = met(4)*cos(metphi(4))
	mety = met(4)*sin(metphi(4))

c  get electron/Jet number correspondence
	do I=1,ICLUS
	  if(.not.mumu)then
	    IF(IJETBNK(I).EQ.JTS_ELE(ele1_ptr))jtele1_bnk=ijetbnk(i)
	  endif
	  if(ee)then
	    IF(IJETBNK(I).EQ.JTS_ELE(ele2_ptr))jtele2_bnk=ijetbnk(i)
	  endif
	enddo

	call metjet(jtele1_bnk,jtele2_bnk,min_et,max_eta,metx,mety,
     >		metjetx,metjety,metphi1,numjets,jetcone)

c  Ok, now calculate MET shit.
	met_mag = sqrt(max(0.00000001,(metjetx**2+metjety**2)))
	MET_DIL = MET_MAG

	if((metjetx.ne.0).and.(metjety.ne.0)) then
	  metphi1 = (pi+atan2(-metjety,-metjetx))*raddeg
	else
	  metphi1 = 0.
	endif                                        

C --------------------- cut on magnitude of corrected missing Et -------------
	if(met_mag.lt.metmag_cut1) then	! MET<25  PUNT always
	  answer=.false.
          dil_wrd = ibset(dil_wrd,4)
	endif
c	if(met_mag.gt.metmag_cut2) then	! met>50 Always ok do jet cut
c	  goto 666
c	endif


C calculate phi angles 
c
c mu mu
	if(mumu)then
	  phi1 = (pi+atan2(-p4m_cmuo(2,muo1_ptr),
     &                       -p4m_cmuo(1,muo1_ptr)))*raddeg
	  if(cmio)then
	    phi2 = (pi+atan2(-p4m_cmio(2,muo2_ptr),
     &                       -p4m_cmio(1,muo2_ptr)))*raddeg
	  else
	    phi2 = (pi+atan2(-p4m_cmuo(2,muo2_ptr),
     &                       -p4m_cmuo(1,muo2_ptr)))*raddeg
	  endif
	endif 

c ee
	if(ee)then
	  phi1 = phi_ele(ele1_ptr)*raddeg
	  phi2 = phi_ele(ele2_ptr)*raddeg
	endif

	if(emu)then
	  phi1 = phi_ele(ele1_ptr)*raddeg
	  if(cmio)then
	    phi2 = (pi+atan2(-p4m_cmio(2,muo1_ptr),
     &                       -p4m_cmio(1,muo1_ptr)))*raddeg
	  else
	    phi2 = (pi+atan2(-p4m_cmuo(2,muo1_ptr),
     &                       -p4m_cmuo(1,muo1_ptr)))*raddeg
	  endif
	endif

c calculate relative angle
	dphi1 = abs(metphi1-phi1)
	if(dphi1.gt.180.) dphi1=360.-dphi1
	dphil1_dil = dphi1
	dphi2 = abs(metphi1-phi2)
	if(dphi2.gt.180.) dphi2=360.-dphi2
	dphil2_dil = dphi2


C --------------------- cut on direction of corrected missing Et -------------
c met and leptons
	if(met_mag.ge.metmag_cut1.and.met_mag.le.metmag_cut2)then
	  if((dphi1.lt.metphi_cut_l) .or. 
     &       (dphi2.lt.metphi_cut_l)) then
              answer = .false.
              dil_wrd = ibset(dil_wrd,5)
          endif
        endif
C cut on direction of corrected missing Et relative to the uncorrected jets
	j=0
	do i=1,iclus
	  if(   (ijet_et(i).gt.10.).and.
     >	        (abs(ijet_deta(i)).LT.jeta_max).and.
     >		(ijetbnk(i).ne.jtele1_bnk).and.
     >          (ijetbnk(i).ne.jtele2_bnk)  ) then
	    j=j+1
	    phij(j) = ijet_phi(i)*raddeg
	    dphij(j) = abs(metphi1-phij(j))
	    if(dphij(j).gt.180.) dphij(j)=360-dphij(j)
	    if(met_mag.ge.metmag_cut1.and.met_mag.le.metmag_cut2)then
	      if(dphij(j).lt.metphi_cut_j) then
                 answer = .false.
                 dil_wrd = ibset(dil_wrd,6)
              endif
	    endif
	    dphij_dil(j) = dphij(j)
	    njetmet_dil = j
	  endif
	enddo
	
	if(iand(dil_wrd,2**6).eq.0) goodmet_dil = .true.

C ---------------------------- cut on number of jets --------------------------
666	n1=0
	n2=0
	do i=1,iclus
	  if((ABS(IJET_DETA(I)).LT.jeta_max).and.
     >		(ijetbnk(i).ne.jtele1_bnk).and.
     >          (ijetbnk(i).ne.jtele2_bnk))then

	    if(jet_et_cor) then
	      if((jcorf(i,1)*ijet_et(i).gt.et1_cut).and.(n1.eq.0))then
	        n1 = n1+1
	      elseif(jcorf(i,1)*ijet_et(i).gt.et2_cut)then 
	        n2 = n2+1
	      endif
	    else
	      if((ijet_et(i).gt.et1_cut).and.(n1.eq.0))then
	        n1 = n1+1
	      elseif(ijet_et(i).gt.et2_cut)then
	        n2 = n2+1
	      endif
	    endif
	  endif
	enddo

	njet_dil=n1+n2

        if((n1.eq.1).and.(n2.eq.0)) then
             answer = .false.     ! 1 jet -- PUNT
             dil_wrd = ibset(dil_wrd,7)
        endif
        if((n1.eq.0).and.(n2.eq.0)) then
             answer = .false.     ! 0 jet -- PUNT
             dil_wrd = ibset(dil_wrd,7)
        endif

	if(iand(dil_wrd,2**7).eq.0) goodmetjet_dil = .true.

c apply opp sign cut here!
	if(.not.oppsign_dil) then
            answer = .false.
            dil_wrd = ibset(dil_wrd,2)
        endif

C set cmio/plug pointer
         if(cmio)then
           if(dil_mode.eq.2)muo1_ptr = -muo1_ptr
           if(dil_mode.eq.3)muo2_ptr = -muo2_ptr
	 endif
         if(pem)then
           if(dil_mode.eq.2)ele1_ptr = -ele1_ptr
           if(dil_mode.eq.1)ele2_ptr = -ele2_ptr
	 endif

c Whew, Made it -- Congratulations!

9999	continue
C  C'est la vie
c	status=anptrg(answer)
	RETURN
	END

c ===========================================================================
c ===========================================================================
	SUBROUTINE dil_init
c
c	Inputs	  none
c
c	Outputs   none
c
c	Authors
c	  Rob Roser

c
c	Routine initializes all variables in the common dilinfo.inc which
c	is filled by dil_topology.cdf
c
c
c ============================================================================
$$INCLUDE 'C$INC:DILINFO.INC'
$$INCLUDE 'TOP$FND:DILMORE.INC'
c
c init variables
	goodmetjet_dil  = .false.
	goodmet_dil	= .false.
	good_dil	= .false.
	oppsign_dil	= .false.

	njet_dil	= 0
	njetmet_dil	= 0
	dil_mode	= 0

	jtele1_bnk	= 0
	jtele2_bnk	= 0
	muo1_ptr	= 0
	muo2_ptr	= 0
	ele1_ptr	= 0
	ele2_ptr	= 0

	met_dil 	= 0.
	dphil1_dil	= -999.
	dphil2_dil	= -999.
	
	call vzero(dphij_dil,20)

C  C'est la vie
	RETURN
	END
