c ===========================================================================
c  DSTOPS.INC 
c ===========================================================================
c
c     TOPS Bank Structure:
c
c            Header Words:
c
c(I)      0) Number of Words in Header
c(I)      1) Version Number (100)
c(I)      2) Code Word (bit 0: If set, jets were reclustered after removing the
c                             lepton.
c                     (bit 1: If set, jets were counted after correcting the
c                             jet Et for relative detector response.
c
c(R)      3) MET
c(R)      4) MET phi
c(I)      5) Badrun Flag
c(I)      6) Trigger Flag (For whole event)
c(I)      7) Z Flag
c(I)      8) Dilepton Candidate
c
c(I)      9) Number of Electrons (isolated or not)
c(I)     10) Number of Electrons (isolated)
c(I)     11) Number of Electrons (isolated .AND. pass trigger req; NOTE:
c            if a MET trigger is satisfied then all leptons (of the proper
c            type) are classified as passing the trigger.)
c
c(I)     12) Number of Muons     (isolated or not)
c(I)     13) Number of Muons     (isolated)
c(I)     14) Number of Muons     (isolated .AND. pass trigger req; NOTE:
c            if a MET trigger is satisfied then all leptons (of the proper
c            type) are classified as passing the trigger.)
c
c        Only good leptons are counted (Et > 20 and all quality ID cuts).
c        The lepton information is stored as follows:
c            a) Highest Et Lepton which is isolated and passes trigger,
c               then next highest Et of this type.
c            b) Highest Et Lepton which is isolated but doesn't trigger,
c               then next highest Et of this type.
c            c) Highest Et Lepton which is nonisolated,
c               then next highest Et of this type.
c         Only two Leptons are stored; Other are counted but not stored
c         in this bank.
c
c(R)          15) Pt of lepton 1
c(R)          16) Px of Lepton 1
c(R)          17) Py of Lepton 1
c(R)          18) Pz of Lepton 1
c(R)          19) E  of Lepton 1
c(R)          20) Cone Isolation lepton 1
c(I)          21) Type of lepton 1
c(I)          22) Lepton Array pointer to lepton 1
c(I)          23) Bank number of lepton 1 (ELES bank# or CMUO bank #)
c(I)          24) Bank number of JETS  for ele or TRKS for cmu  for lepton 1
c(I)          25) Trigger Flag for lepton 1
c
c(R)          26) Pt of lepton 2
c(R)          27) Px of Lepton 2
c(R)          28) Py of Lepton 2
c(R)          29) Pz of Lepton 2
c(R)          30) E  of Lepton 2
c(R)          31) Cone Isolation lepton 2
c(I)          32) Type of lepton 2
c(I)          33) Lepton Array pointer to lepton 2
c(I)          34) Bank number of lepton 2 (ELES bank# or CMUO bank #)
c(I)          35) Bank number of JETS  for ele or TRKS for cmu  for lepton 2
c(I)          36) Trigger Flag for lepton 2
c
c(I)     37) Number of jets with Et>15 and eta<2.0
c(I)          38) JETS Bank # for jet 1
c(I)          39) JETS Bank # for jet 2
c(I)          40) JETS Bank # for jet 3
c(I)          41) JETS Bank # for jet 4
c(I)          42) JETS Bank # for jet 5
c(I)          43) JETS Bank # for jet 6
c(I)          44) JETS Bank # for jet 7
c(I)          45) JETS Bank # for jet 8
c(I)          46) JETS Bank # for jet 9
c(I)          47) JETS Bank # for jet 10
c(I)     48) Number of jets with Et>8  and eta<2.4 and .NOT. Et>15 and eta<2.0
c(I)          49) JETS Bank # for jet 1
c(I)          50) JETS Bank # for jet 2
c(I)          51) JETS Bank # for jet 3
c(I)          52) JETS Bank # for jet 4
c(I)          53) JETS Bank # for jet 5
C
c(I)     54) Jet Mult for Single Lepton Event      
c(I)     55) Jet Mult for Dilepton Lepton Event      
c(I)     56) Jet Mult for Z Event
c
c  This bank can be unpacked into a common block with TOPSBTC.CDF. It fills
c  the common block in TOPSCOM.INC.  This is the reccomended way to access
c  the information.
c
c     Brian Winer, 13-Sep-1995
c ===============================================================================
C
c  Bank Parameters
      INTEGER TSHEAD
      PARAMETER(TSHEAD=3)
      INTEGER TSINT1
      PARAMETER(TSINT1=3)
      INTEGER TSREA1
      PARAMETER(TSREA1=3)
      INTEGER TSINT2
      PARAMETER(TSINT2=10)
      INTEGER TSREA2
      PARAMETER(TSREA2=6)
      INTEGER TSINT3
      PARAMETER(TSINT3=5)
      INTEGER TSREA3
      PARAMETER(TSREA3=6)
      INTEGER TSINT4
      PARAMETER(TSINT4=25)
      INTEGER LNTOPS
      INTEGER OFFSETL
      PARAMETER(OFFSETL = 16)
      INTEGER OFFSETL2
      PARAMETER(OFFSETL2 = OFFSETL+11)
      INTEGER OFFSET15
      PARAMETER(OFFSET15 = OFFSETL2+11)
      INTEGER OFFSET8
      PARAMETER(OFFSET8  = OFFSET15+11)
      INTEGER VERSION
      PARAMETER(VERSION=100)

c ===========================================================================
