C======================================================================
C  File:  EleVal.Cdf
C======================================================================
C
C===Description:  This routine unpacks the ELES banks and calculates
C                 the electrons variables which will be cut on.  It
C                 fills a common block (ELEVAL.INC) with the electron
C                 varialbes.
C
C      Input variables:  None
C      Input Conditions: ELES and TRKS banks present
C      Output Variables: Fills ELEVAL.INC common block
C
C===Author:
c
C       Brian Winer  04-May-1993 Initial Construction
C       BLW          11-May-1993 Change Standard Conversion Definition
c
c       cgp          sep 1994    add some more usefull quantities and PEM 
c	RR	     8-Feb-1995	 Fixed tracking isolation Req.
C========================================================================

C==============================
      SUBROUTINE ELEVAL(BC)
C==============================
C
C
$$IMPLICIT
      SAVE    !FOR ACP
C
C===Global Declarations:
C
$$INCLUDE 'C$INC:BCS.INC'
$$INCLUDE 'C$INC:ERLEVL.INC'
$$INCLUDE 'C$INC:PIVALU.INC'
$$INCLUDE 'C$INC:LOGUNI.INC'
$$INCLUDE 'C$INC:DSEVTA.INC'
$$INCLUDE 'C$INC:REC152.INC'
$$INCLUDE 'C$INC:DSELES.INC'
$$INCLUDE 'C$INC:JOBSTA.INC'
$$INCLUDE 'C$INC:BTMASK.INC'
$$INCLUDE 'C$INC:TREXPO.INC'
$$INCLUDE 'C$INC:CTCGEO.INC'
$$INCLUDE 'C$INC:CTCAUX.INC'
$$INCLUDE 'YBOS$LIBRARY:ERRCOD.INC'
$$INCLUDE 'UIPACK$LIBRARY:UIPARAM.INC'
$$INCLUDE 'UIPACK$LIBRARY:UIERROR.INC'
$$INCLUDE  'c$inc:ELEINFO.INC'

C    External declarations:
C    ======================
      INTEGER QTRKBM

c Define Local Variables
c ----------------------
      REAL VERTEXZ                    !Z vertex with highest class
      REAL XVERT(5),YVERT(5),ZVERT(5) !x,y,z of vertex
      REAL PRM(5),PRMT(5)             !Track parameters..is different order
      REAL CVM(15)                    !Covariance Matrix values...not used
      REAL XCTC,DXC                   !extrapolated track position and shift
      REAL ZCTC,DZC                   !extrapolated track position and shift
      REAL ZDIFF                      !Diff between z of vertex and ele Z0
      REAL ZVRTX                      !Value of the Z vertex passed to CESALG
      REAL TINY/0.0001/               !A small number or large person
      Real P,PBC(3)                   !track momentum 
      REAL D0,D0_OLD                  !impact parameter
      REAL ZV                         !track Z vertex
      REAL DZ
      REAL ZVC                        !closest vertex

      REAL COR_ELE
      
      INTEGER Q                       !charge of track - from call to QTRKCT
      INTEGER NUMOUT
      INTEGER I,J,K,L                 !Counters
      INTEGER IERR                    !Error flag
      INTEGER VERSION_ELES            !Version number of ELES bank
      INTEGER KCLASS(5),KCLASS_MAX    !Vertex class info
      INTEGER NVERT,NVERT_GOOD        !Number of vertices
      INTEGER CVMASK                  !Conversion parameter return mask
      INTEGER TRKS_NUM                !TRKS bank number for electron
      INTEGER SVXS_NUM                !SVXS bank number for electron
      INTEGER EOW,EOWT                !East/West Flags
      INTEGER WG_ELE,ETA_ELE          !Wedge and Eta numbers of Electrons
      INTEGER BADT                    !Bad tower flag...not used
      INTEGER TQUAL                   !Track quality...not used
      INTEGER STATUS,BLOCAT,INDBNK,INDELE  !YBOS stuff
      INTEGER FIDELE                  !Electron Fiducial Region routine
      INTEGER IETA                    !eta tower of electron

      LOGICAL FAIL                    !Failure Code for FIDELE...not used
      LOGICAL VTC,SVX                 !VTX/SVX track flags      
      LOGICAL FLAG3D
      LOGICAL BC                      ! request beam constrain
      INTEGER BCSTATUS                ! .TRUE. if BC worked.
      

C Zero All Arrays
c ---------------
      CALL VZERO(BNK_ELE,MAXELES)
      CALL VZERO(DET_ELE,MAXELES)
      CALL VZERO(ET_ELE,MAXELES)
      CALL VZERO(P4E_ELE,MAXELES*4)
      CALL VZERO(FID_ELE,MAXELES)
      CALL VZERO(HADEM_ELE,MAXELES)
      CALL VZERO(BTE_ELE,MAXELES)
      CALL VZERO(ISOL_ELE,MAXELES)
      CALL VZERO(CHIS_ELE,MAXELES)
      CALL VZERO(CHIW_ELE,MAXELES)
      CALL VZERO(LSHR_ELE,MAXELES)
      CALL VZERO(EP_ELE,MAXELES)
      CALL VZERO(DELX_ELE,MAXELES)
      CALL VZERO(DELZ_ELE,MAXELES)
      CALL VZERO(CON_ELE,MAXELES)
      CALL VZERO(EZV_ELE,MAXELES)
      CALL VZERO(Z0_ELE,MAXELES)
      CALL VZERO(TRKS_ELE,MAXELES)
      CALL VZERO(EVETA_ELE,MAXELES)
      CALL VZERO(DTETA_ELE,MAXELES)
      CALL VZERO(PHI_ELE,MAXELES)
      CALL VZERO(CHG_ELE,MAXELES)
      CALL VZERO(PELE_ELE,MAXELES)
      CALL VZERO(PTELE_ELE,MAXELES)
      CALL VZERO(SUMPT_ELE,MAXELES)
      CALL VZERO(HIGHPT_ELE,MAXELES)
      CALL VZERO(NASL,MAXELES)
      CALL VZERO(NSSL,MAXELES)
      CALL VZERO(PXBC_ELE,MAXELES)
      CALL VZERO(PYBC_ELE,MAXELES)
      CALL VZERO(PZBC_ELE,MAXELES)
      CALL VZERO(BC_ELE,MAXELES)
      CALL VZERO(ECOR_ELE,MAXELES)
      CALL VZERO(EPBC_ELE,MAXELES)
      CALL VZERO(JTS_ELE,MAXELES)
      CALL VZERO(N3D_ELE,MAXELES)
      CALL VZERO(VTP_ELE,MAXELES)
      CALL VZERO(LSHR2_ELE,MAXELES)
      CALL VZERO(CHI3_ELE,MAXELES)
      CALL VZERO(CHID_ELE,MAXELES)
      CALL VZERO(VRES,MAXELES)
c
C Find the Vertex(ices) of event and decide which is best
C -------------------------------------------------------
      VERTEXZ = 999.
      CALL VTGZZT(5,NVERT,KCLASS,ZVERT,XVERT,YVERT)
      KCLASS_MAX = -1
      DO I=1,NVERT
         IF(KCLASS(I).GT.KCLASS_MAX) THEN
            NVERT_GOOD = I
            KCLASS_MAX = KCLASS(I)
         ENDIF
      ENDDO
      IF(NVERT.GT.0) THEN
         VERTEXZ = ZVERT(NVERT_GOOD)
      ELSE
         VERTEXZ = 999.
      ENDIF

c Get_ele the Electron Quantities
c ===============================
	
C Get_ele Number and List of Bank Numbers for electron
c ------------------------------------------------
      CALL GTELES(MAXELES,NUMELES,BNK_ELE)

C  Loop over electron objects to find electron qualities
C  -----------------------------------------------------
      DO K=1,NUMELES

C Locate ELES Bank
C ----------------
        STATUS = BLOCAT(IW,'ELES',BNK_ELE(K),INDBNK,INDELE)
        IF (STATUS.NE.YESUCC) THEN
          CALL ERLOGR ('WVAL',ERWARN,0,STATUS,
     &        'Can not Find ELES')
          GOTO 100
        ENDIF

C Check to see if this "electron" was a conversion
c ------------------------------------------------
        CALL CONFND(BNK_ELE(K),CON_ELE(K))

C Determine the Detector Position of Electron
c -------------------------------------------
        DET_ELE(K) = IW(INDELE + REGELE)

C Determine the Momentum and the Energy of the "Electron" 
C -------------------------------------------------------
        DO I=1,3
           P4E_ELE(I,K) = RW(INDELE + EXVELE + (I-1))
        ENDDO
        P4E_ELE(4,K) = RW(INDELE + E__ELE)
        ET_ELE(K) = SQRT(P4E_ELE(1,K)**2 + P4E_ELE(2,K)**2)

c Determine Some Tracking Information for the electron
c ----------------------------------------------------
        TRKS_NUM = IW(INDELE+TRPELE)
        TRKS_ELE(K)=TRKS_NUM
        IF(TRKS_NUM.GT.0) THEN
            CALL TRK_INFO('TRKS',TRKS_NUM,PRM,CVM,TQUAL) 
            Z0_ELE(K) = PRM(3)
        ELSE
            Z0_ELE(K) = 999.
        ENDIF

c Get the Z vertex used for this electron (Close Vertex with VClass>=10)
c ----------------------------------------------------------------------
        ZDIFF = 999.
        EZV_ELE(K) = 999.
        DO I=1,NVERT
          IF(KCLASS(I).GE.10) THEN
            IF(ABS(ZVERT(I)-Z0_ELE(K)).LT.ZDIFF) THEN
               EZV_ELE(K) = ZVERT(I)
               ZDIFF = ABS(ZVERT(I)-Z0_ELE(K))
            ENDIF
          ENDIF
        ENDDO


c Determine Isolation of Electron
c -------------------------------
        IF(ET_ELE(K).GT.0.) THEN
            ISOL_ELE(K) = RW(INDELE+ER4ELE)/ET_ELE(K)
        ELSE
            ISOL_ELE(K) = 999.
        ENDIF

C New definition of Had/EM
c ------------------------
        HADEM_ELE(K) = RW(INDELE + HAD3ELE)
        IF(DET_ELE(K).NE.0) HADEM_ELE(K) = RW(INDELE+HADELE)
            
c Calculate the E/P
c -----------------
        EP_ELE(K) = 999.
        IF(RW(INDELE+P__ELE).GT.TINY)
     &     EP_ELE(K) = P4E_ELE(4,K)/RW(INDELE+P__ELE)


c  event/detector eta and phi 
c--------------------------------
        EVETA_ELE(K) = RW(INDELE + ETAELE)
        DTETA_ELE(K) = RW(INDELE + EDTELE)
        PHI_ELE(K) = RW(INDELE + PHIELE)

c Number of 3D tracks
c--------------------
        N3D_ELE(K)   = IW(INDELE + N3DELE)

C  P, PT of track and charge
c---------------------------
        PELE_ELE(K) = RW(INDELE + P__ELE)
        PTELE_ELE(K) = RW(INDELE + PT_ELE)
        CHG_ELE(K) = RW(INDELE + Q__ELE)

c  Track isolation
c  ---------------
c
         CALL TKISO(TRKS_ELE(K),0,0.4,SUMPT_ELE(K),
     *              HIGHPT_ELE(K))

         IF(ET_ELE(K).GT.0.)THEN
           SUMPT_ELE(K)=SUMPT_ELE(K)/ET_ELE(K)
         ELSE
           SUMPT_ELE(K)=999.
         ENDIF

c  Calculate beam constraint parameters
c--------------------------------------
c
 
        IF(TRKS_NUM.GT.0) THEN
          BC_ELE(K) = 0
          CALL TRACKQ(TRKS_NUM,BC,FLAG3D,D0_OLD,D0,
     *              NASL(k),NSSL(k),ZVC,DZ,ZV,PBC,BCSTATUS)
        ENDIF

c  beam constrain succesful, fill qauntities
C  -----------------------------------------
        IF(BCSTATUS.EQ.0) THEN
          PXBC_ELE(K) = PBC(1)
          PYBC_ELE(K) = PBC(2)
          PZBC_ELE(K) = PBC(3)
          P = SQRT(PBC(1)**2 + PBC(2)**2 + PBC(3)**2)
          BC_ELE(K) = 1
        ENDIF

C    Correct electron energy
C---------------------------
        IF(DET_ELE(K).EQ.0) THEN
          CALL CEMFIX(BNK_ELE(K),COR_ELE)
          IF(COR_ELE.GT.0.) 
     *       ECOR_ELE(K) = RW(INDELE + E__ELE)/COR_ELE      
        ELSE IF(DET_ELE(K).EQ.1) THEN
          CALL PEMFIX(BNK_ELE(K),COR_ELE)
          IF(COR_ELE.GT.0.) 
     *       ECOR_ELE(K) = RW(INDELE + E__ELE)/COR_ELE      
        ENDIF

C     Get best E/P value using corrected energy and beam constr.
c--------------------------------------------------------------
          if(bc_ele(k).eq.1.and.p.ne.0.) 
     *          epbc_ele(k) = ecor_ele(k)/p

c bank number of jet containing this cluster NOTE: This is only good for
c a cone of 0.7.  If a different size cone is being used we must do the
c association 'by hand' in JETINFO
c-----------------------------------------------------------------------
        JTS_ELE(K) = IW(INDELE + JETELE)


c VTPC hit information
c---------------------
        VTP_ELE(K) = IW(INDELE + VTPELE)

C Check to see if Electron is in Fuducial Region
c ----------------------------------------------
           STATUS = FIDELE(BNK_ELE(K),0,FAIL,FID_ELE(K))

C Fill quantities specific to central
c ----------------------------------
        IF (DET_ELE(K).EQ.0) THEN


C Get_ele Border Tower Energy
c ---------------------------
           BTE_ELE(K) = RW(INDELE + EBRELE)

C Get_ele the Chi Squared of the Strips
c ---------------------------------
           CHIS_ELE(K) = RW(INDELE + CHSELE)
           CHIW_ELE(K) = RW(INDELE + CHWELE)
      
C Get the Strip Postion Using the corrected position of the Wedges
c ----------------------------------------------------------------
           IF(TRKS_NUM.GT.0) THEN
             ZVRTX = EZV_ELE(K)
             IF(EZV_ELE(K).EQ.999.) ZVRTX = VERTEXZ

C Call the cluster alignment routine (bailout if no trk info)
c -----------------------------------------------------------
             IERR = 999
             IF(ABS(PRM(2)).GT.0.0.AND.Z0_ELE(K).LT.998.) THEN

C First rearrange the storage order (DAMN!...two routines order params dif.)
c -----------------------------------------------------------------------
               PRMT(1) = PRM(2)
               PRMT(2) = PRM(4)
               PRMT(3) = PRM(5)
               PRMT(4) = PRM(1)
               PRMT(5) = PRM(3)

C Get tower and wedge numbers
c ---------------------------
               CALL BADTOW(IW(INDELE+STWELE),WG_ELE,ETA_ELE,EOW,BADT)
               IF(EOW.EQ.1) EOWT = 0  !DAMN...two CDF routines define
               IF(EOW.EQ.0) EOWT = 1  !east and west the opposite way

c Call alginment routine
c ----------------------
               CALL CESALG(PRMT,EOWT,WG_ELE,ZVRTX,
     &                     XCTC,ZCTC,DXC,DZC,IERR)
             ENDIF

c If aligment was succesful then use it otherwise use nonalignment
c ----------------------------------------------------------------
             IF(IERR.EQ.0) THEN
               DELX_ELE(K) = -(XCTC+DXC)-RW(INDELE+XRWELE) !Weird sign stuff
               IF(EOW.EQ.1) THEN
                  DELZ_ELE(K)=-(ZCTC+DZC)-RW(INDELE+ZRWELE)
               ELSE
                  DELZ_ELE(K)= (ZCTC+DZC)-RW(INDELE+ZRWELE)
               ENDIF
             ELSE
               DELX_ELE(K) = RW(INDELE + XTRELE) - RW(INDELE + XRWELE)
               DELZ_ELE(K) = RW(INDELE + ZEXELE) - RW(INDELE + ZRWELE)
             ENDIF

c Make sure Delta are not Outragous
c ---------------------------------
             IF (ABS(DELX_ELE(K)).GT.100.) DELX_ELE(K) = 999.
             IF (ABS(DELZ_ELE(K)).GT.100.) DELZ_ELE(K) = 999.

           ELSE

C No 3D track so match should not occur
c -------------------------------------
             DELX_ELE(K) = 999.
             DELZ_ELE(K) = 999.
           ENDIF             

C  Get  LSHR
c  ---------
           LSHR_ELE(K) = RW(INDELE + LSHELE)
           LSHR2_ELE(K) = RW(INDELE + LSH2ELE)

           CHI3_ELE(K) = 999.
           CHID_ELE(K) = 999.

        ELSEIF(DET_ELE(K).EQ.1) THEN   

c     3*3 chi2 of transverse profile
c-----------------------------------
           CHI3_ELE(K) = RW(INDELE + CH3ELE)
           CHID_ELE(K) = RW(INDELE + CHDELE)
           IETA=IAND(ISHFT(IW(indele+STWELE),-7),'7F'X)
           IF(IETA.EQ.31.OR.IETA.EQ.32 .OR. 
     >        IETA.EQ.53.OR.IETA.EQ.54)
     >          chid_ele(K) = -1

c Set Central quantities to 999. (Future Expansion)
c -------------------------------------------------
           BTE_ELE(K) = 999.
           LSHR_ELE(K) = 999.
           DELX_ELE(K) = 999.
           DELZ_ELE(K) = 999.
           CHIS_ELE(K) = 999.
           CHIW_ELE(K) = 999.

        ELSEIF(DET_ELE(K).EQ.2) THEN  

C Set Non-Forward quanties to 999. (Future Expansion)
c ---------------------------------------------------
           BTE_ELE(K) = 999.
           CHIS_ELE(K) = 999.
           CHIW_ELE(K) = 999.
           LSHR_ELE(K) = 999.
           DELX_ELE(K) = 999.
           DELZ_ELE(K) = 999.
           CHI3_ELE(K) = 999.
           CHID_ELE(K) = 999.

        ENDIF

100   ENDDO !Loop over ELES banks

       RETURN
       END
