C======================================================================
C  File:  EleZee.Cdf
C======================================================================
C
C===Description:
C
C      Input variables:  CEM_ELE, i.e. the mask that needs to
C			 be applied to the cut-word returned by
C			 ELEFND to flag good electrons...currently
C			 defaults to '3FF'X
C			 CONV_CUT, which is used to reject conversions...
C			 currently defaults to 3
c
C      Output Variables: (Contained in TFZINFO.INC common block)
c
C                        LZFLAG......Loose Z Boson Flag (TRUE = Loose Z Found)
C                        TZFLAG......Tight Z Boson Flag (TRUE = Tight Z Found)
C                        ZCNT........Number of Tight or Loose Z's found
C			 MASSZ(3)....Z mass from lepton pair 
C                        LEGZ(2,3)...Index to Leg of Z
C                        ZTYPE(3)....Type of Z (1=loose ee, 2=tight ee)
C                                              (-1=loose mu mu, -2=tight mu mu)
C                        LTYPE(2,3)..Type of Leg (For electrons 0=central,
C                                    1=plug, 2=forward; For Muons, it is the
C                                    CMUO type 1=cmu,2=CMP,3=CMU/CMP,4=CMX)
C                        
C
C===Author:
c
C       Brian Winer  11-May-1993 Initial Construction
c                    12-SEP-1995 Put info into common block and combine
c                                tight and loose cuts into single routine.
c  
C======================================================================

C===============================
      SUBROUTINE ELEZEE(CEM_ELE,CONV_CUT)
C===============================
C
C
$$IMPLICIT
      SAVE    !FOR ACP
C
C===Global Declarations:
C
$$INCLUDE 'C$INC:JOBSTA.INC'
$$INCLUDE 'c$inc:ELEINFO.INC'
$$INCLUDE 'top$fnd:TFZINFO.INC'

c Define Local Variables
c ----------------------
      INTEGER I,J,K                !Counting variables
      REAL MZ                      !Invariant mass of pair
      REAL Z_CUT_HIGH/105./        !Upper Z mass cut
      REAL Z_CUT_LOW/75./          !Lower Z mass cut
      REAL ZET_CUT/10./            !Et cut for second leg
      REAL ZHADEM_CUT/0.12/        !Had/EM Cut for second leg
      REAL ZISOL_CUT/0.2/          !Isolation cut for second leg
      REAL ZEP_CUT/2.0/            !E/P cut for second leg (if central)
      REAL TZHADEM_CUT/0.10/       !TIGHT Had/EM Cut for second leg
      REAL TZISOL_CUT/0.1/         !TIGHT Isolation cut for second leg

      INTEGER CEM_ELE,CONV_CUT
      LOGICAL PAIR_FOUND

c If this is a new event then zero out the arrays
c -----------------------------------------------
      IF(NRUN.NE.LAST_RUN.OR.NEV.NE.LAST_EVT) THEN
         ZCNT = 0
         LZFLAG = .FALSE.
         TZFLAG = .FALSE.
         DO I=1,3
            MASSZ(I) = 0.
            ZTYPE(I) = 0
            LEGZ(1,I) = 0
            LEGZ(2,I) = 0
            LTYPE(1,I) = 0
            LTYPE(2,I) = 0
         ENDDO
         LAST_RUN = NRUN
         LAST_EVT = NEV
      ENDIF

c For every good Electron try to partner to form Z mass
c -----------------------------------------------------
      DO I=1,NUMELES
	 IF (IAND(CEM_WRD(I),CEM_ELE).EQ.0.AND.
     &      IAND(CON_ELE(I),CONV_CUT).EQ.0) THEN

            DO J=1,NUMELES
               IF(I.NE.J) THEN

C Put quality cuts on the second leg of the Z
c -------------------------------------------
                  IF(ET_ELE(J).GT.ZET_CUT) THEN
                     IF(HADEM_ELE(J).LT.ZHADEM_CUT) THEN
                         IF(ISOL_ELE(J).LT.ZISOL_CUT) THEN
                          IF(DET_ELE(J).NE.0.OR.(DET_ELE(J).EQ.0.
     &                       AND.EP_ELE(J).LT.ZEP_CUT)) THEN

C Found a good candidate for the second leg now form invariant mass
c -----------------------------------------------------------------
                             MZ=(P4E_ELE(4,I)+P4E_ELE(4,J))**2 -
     &                             (P4E_ELE(1,I)+P4E_ELE(1,J))**2 -
     &                             (P4E_ELE(2,I)+P4E_ELE(2,J))**2 -
     &                             (P4E_ELE(3,I)+P4E_ELE(3,J))**2

                             MZ=SQRT(MAX(MZ,0.))

c Make sure we have not found this combination already
c ----------------------------------------------------
                             PAIR_FOUND = .FALSE.
                             DO K=1,ZCNT
                                IF(I.EQ.LEGZ(2,K).AND.
     &                             J.EQ.LEGZ(1,K).AND.
     &                             ZTYPE(K).GT.0) 
     &                             PAIR_FOUND = .TRUE.
                             ENDDO

C Now cut on the value of the Z mass
c ----------------------------------
                             IF(MZ.LT.Z_CUT_HIGH.AND.
     &                          MZ.GT.Z_CUT_LOW.AND.
     &                          .NOT.PAIR_FOUND) THEN


C This statisfies loose Z cuts
c ----------------------------  
                                ZCNT = ZCNT + 1
                                LZFLAG = .TRUE.
                                IF(ZCNT.LE.3) THEN
                                   MASSZ(ZCNT)    = MZ
                                   LEGZ(1,ZCNT)   = I
                                   LEGZ(2,ZCNT)   = J
                                   ZTYPE(ZCNT)    = 1
                                   LTYPE(1,ZCNT)  = DET_ELE(I)
                                   LTYPE(2,ZCNT)  = DET_ELE(J)
                                ENDIF


C Now check to see if this is a tight Z (Make sure Primary is isolated too)
c -------------------------------------------------------------------------
                                IF((HADEM_ELE(J).LT.TZHADEM_CUT).AND.
     &                             (ISOL_ELE(J).LT.TZISOL_CUT).AND.
     &                             (CHG_ELE(J)*CHG_ELE(I).NE.1.).AND.
     &                             (ISOL_ELE(I).LT.TZISOL_CUT)  ) THEN

                                      TZFLAG = .TRUE.
                                      IF(ZCNT.LE.3) ZTYPE(ZCNT) = 2

                                ENDIF

                             ENDIF

                          ENDIF
                        ENDIF
                     ENDIF
                  ENDIF
               ENDIF
            ENDDO
         ENDIF
      ENDDO
      
      RETURN
      END
