	integer function GOODCMX(cmxbnk)
c========================================================================
c
c       GOODCMX: checks if CMXBNK passes CMX quality cuts
c
c       input:   cmxbnk = CMUO bank number
c
c       output:
c                     1 = PASSES matching cuts
c                    -1 = fails matching quality cuts
c		     -2 = couldn't BLOCATE bank
c                    -3 = not a CMX muon
C		     -4 = Pt < 2
C		     -5 = Fails track quality cuts
c		
c
c       uses claudio's TRKTYP to check track quality
c       and KOADS which is a slightly modified version of CMUSWM
c       that has some changes for distance to edge of CMX,
c	also currently refits using Jorge Troconiz's cmxfix
c       which corrects the return field and drift velocity
c       for run1a.
c
c
c
c       david kestenbaum, April 9, 1994
c                                                                     
c=======================================================================


$$implicit none

	integer status,cmxbnk,inddat,index,blocat
	integer itype,trks_numb,track_type
	integer ierr,mchg,mtow
	real pxuo,pyuo,pzuo,xxuo,yyuo,zzuo,ph1o,ph2o,th1o,th2o
	real delp,delx,delt,delz,chpx,chxx,chtz,chzz,ptuo

$$INCLUDE 'YBOS$LIBRARY:ERRCOD.INC' 
$$INCLUDE 'C$INC:BCS.INC'
$$include 'c$inc:cmuo92.inc'
$$INCLUDE 'C$INC:JOBSTA.INC'
$$INCLUDE 'C$INC:ANACTL.INC'

c Blocate bank
        status=Blocat(iw,'CMUO',cmxbnk,index,inddat)
	if(status.ne.yesucc)then
	  goodcmx=-2
	  return
	endif

c Check to see if CMX
	if(iand(iw(inddat+mostyp),4).ne.4)then     	
	  goodcmx=-3
	  return
	endif

c Pt Cut
	If (rw(inddat+mopt) .lt. 2.0) then
	    goodcmx=-4
	    return
	end if

c Track Quality Cuts
        trks_numb = ishft(iand(iw(inddat+motrks),'FF00'x),-8)

        itype     = track_type(trks_numb)
        if (itype .le. 0) then
	   goodcmx = -5
	   return
	end if

C- If Monte carlo, Take it
	If (LMONTE) then
	    goodcmx = 1
	    return
	end if

c Get matching variables

	mchg=iw(inddat+mochrg)   ! charge
	mtow=iw(inddat+mocxtw)   ! cmx tower
        pxuo=rw(inddat+mopx  )   ! px
        pyuo=rw(inddat+mopy  )   ! py
        pzuo=rw(inddat+mopz  )   ! pz
        xxuo=rw(inddat+mox0  )   ! x vertex
        yyuo=rw(inddat+moy0  )   ! y vertex
        zzuo=rw(inddat+moz0  )   ! z vertex
        ph1o=rw(inddat+moaphx)   ! dif in r-phi angle CTC/CMX
        ph2o=rw(inddat+morphx)   ! dif in r-phi position CTC/CMX
        th1o=rw(inddat+moathx)   ! dif in theta angle CTC/CMX
        th2o=rw(inddat+morthx)   ! dif in z postition  CTC/CMX
        ptuo=rw(inddat+mopt  )   ! pt
        
c refit using JT's fix.  corrects for drift velocity and b field change
cdk ONLY do corrections for 1a now.... we assume 1b CMUO banks have been
cdk remade with offline drift corrections from database. - oct 20 1995

	if(nrun.lt.50000)then
            call cmxfix(mchg,mtow,pxuo,pyuo,pzuo,xxuo,yyuo,zzuo,
     >              ph1o,ph2o,th1o,th2o,delp,delx,delt,delz,
     >              chpx,chxx,chtz,chzz,ierr)
	else
	  chxx=rw(inddat+moxftx)    ! delta-x chi-square
	  chzz=rw(inddat+mozftx)    ! delta-z chi-square
	  chpx=rw(inddat+motftx)    ! detta x-phi chi-square
	endif

c cut cut cut
	goodcmx=-1
	if( (chzz.lt.9).and.
     >      (chxx.lt.9).and.
     >      ( (ptuo.le.5.and.chpx.lt.9).or.
     >        (ptuo.gt.5.and.abs(ph1o).lt..1)  ) )goodcmx=1
        
	return
	end
