C======================================================================
C  File:  MuoZee.Cdf
C======================================================================
C
C===Description:
C
C      Input variables:  CMU_MUON, i.e. the mask that is used to determine
C			 whether a given muon is good...currently default
C			 should be 'DFE'X
c
C      Output Variables: (Contained in TFZINFO.INC common block)
c
C                        LZFLAG......Loose Z Boson Flag (TRUE = Loose Z Found)
C                        TZFLAG......Tight Z Boson Flag (TRUE = Tight Z Found)
C                        ZCNT........Number of Tight or Loose Z's found
C			 MASSZ(3)....Z mass from lepton pair 
C                        LEGZ(2,3)...Index to Leg of Z
C                        ZTYPE(3)....Type of Z (1=loose ee, 2=tight ee)
C                                              (-1=loose mu mu, -2=tight mu mu)
C                        LTYPE(2,3)..Type of Leg (For electrons 0=central,
C                                    1=plug, 2=forward; For Muons, it is the
C                                    CMUO type 1=cmu,2=CMP,3=CMU/CMP,4=CMX)
C
C===Author:
c
C       Tony Liss  12-May-1993 Stolen code (Thanks Brian)
C		    7-JUN-1993 Get SVX or BC track params for CMIOs
C		   13-JUL-1993 fix GETTRK call		-AJM
C		   28-JUL-1993 include CMX leg of Zs
C		   29-JUL-1993 Explicitly set CSCALE(0)=0 for GETTRK
C                  12-SEP-1995 Put info into common block and combine
c                              loose and tight cuts into single routine...BLW
c                      
C======================================================================

C===============================
      SUBROUTINE MUOZEE(CMU_MUON)
C===============================
C
C
$$IMPLICIT 
      SAVE    !FOR ACP
C
C===Global Declarations:
C
$$INCLUDE 'C$INC:TRKSCO.INC'
$$INCLUDE 'YBOS$LIBRARY:ERRCOD.INC'
$$INCLUDE 'C$INC:BCS.INC'
$$INCLUDE 'C$INC:CMUO92.INC'
$$INCLUDE 'C$INC:CTCGEO.INC'
$$INCLUDE 'C$INC:CTCCON.INC'
$$INCLUDE 'C$INC:JOBSTA.INC'
$$INCLUDE 'c$inc:MUOINFO.INC'
$$INCLUDE 'c$inc:MIOINFO.INC'
$$INCLUDE 'top$fnd:TFZINFO.INC'
C
C -Externals
C
	INTEGER QTRKUN
C
c Define Local Variables
c ----------------------
      INTEGER CMU_MUON
      INTEGER BLOCAT,IND,INDDAT
      INTEGER I,J,II,K             !Counting variables
      INTEGER BNKNUM		   !CMIO Bank #
      INTEGER TQUAL		   !Track quality (not used)
      INTEGER TRKS_NUM		   !TRKS bank number for muon
      INTEGER  NHIT                ! SVX HITS 
      INTEGER IERR_BM,VRTX
      REAL MZ                   !Invariant mass of pair
      REAL PHIMU,ETAMU,P,PX,PY,PZ  !CMIO things
      REAL PT,E4		   !Ditto
      REAL CVM(15)		   !Covariance matrix (not used)
      REAL PRM(5)		   !Track parameters from TRKS
      REAL PF(5), DUMMY(5,5)       ! RETURN BEAM FIT 
      REAL Z_CUT_HIGH/105./        !Upper Z mass cut
      REAL Z_CUT_LOW/75./          !Lower Z mass cut
      REAL ZPT_CUT/10./            !Et cut for second leg
      REAL ZHAD_CUT_CMUO/10.0/     !Had Tower Cut for second leg
      REAL ZEM_CUT_CMUO/5.0/	   !EM Tower Cut for second leg
      REAL ZHAD_CUT_CMIO/6.0/      !Had Tower Cut for second leg
      REAL ZEM_CUT_CMIO/2.0/	   !EM Tower Cut for second leg
      REAL ZISOL_CUT/0.2/          !Isolation cut for second leg
      REAL ZDX_CUT/5.0/            !CMU/CMX/CMP matching cut for 2nd leg
      REAL ZETA_CUT/1.2/	   !Eta cut for CMIOs only
      REAL TZHAD_CUT_CMUO/6.0/     !Tight Had Tower Cut for second leg
      REAL TZEM_CUT_CMUO/2.0/	   !Tight EM Tower Cut for second leg
      REAL TZISOL_CUT/0.1/         !Tight Isolation cut for second leg
      REAL TZETA_CUT/1.1/	   !Tight Eta cut for CMIOs only
      LOGICAL SVXTRK		   !Use SVX or BC track for CMIOs
      LOGICAL PAIR_FOUND           !lepton pair already located
      REAL CDFPHI
C=============================


c If this is a new event then zero out the arrays
c -----------------------------------------------
      IF(NRUN.NE.LAST_RUN.OR.NEV.NE.LAST_EVT) THEN
         ZCNT = 0
         LZFLAG = .FALSE.
         TZFLAG = .FALSE.
         DO I=1,3
            MASSZ(I) = 0.
            ZTYPE(I) = 0
            LEGZ(1,I) = 0
            LEGZ(2,I) = 0
            LTYPE(1,I) = 0
            LTYPE(2,I) = 0
         ENDDO
         LAST_RUN = NRUN
         LAST_EVT = NEV
      ENDIF

C
c For every good muon try to partner to form Z mass
c -----------------------------------------------------
      DO I=1,MIN(NUMCMUO,MAXCMUO)
         IF(IAND(MUO_WRD(I),CMU_MUON).EQ.0) THEN	!1st one must pass cuts
            DO J=1,NUMCMUO
               IF(I.NE.J) THEN

C Put quality cuts on the second leg of the Z
c -------------------------------------------
                  IF(PT_CMUO(J).GT.ZPT_CUT) THEN
                    IF(HAD_CMUO(J).LT.ZHAD_CUT_CMUO) THEN
              	      IF(EM_CMUO(J).LT.ZEM_CUT_CMUO) THEN
                        IF(ISOL_CMUO(J).LT.ZISOL_CUT) THEN
			  IF( ((ABS(DX_CMU(J)).LT.ZDX_CUT).OR.
     >			       (ABS(DX_CMP(J)).LT.ZDX_CUT)).AND.
     >			       (ABS(DX_CMX(J)).LT.ZDX_CUT)) THEN

C Found a good candidate for the second leg now form invariant mass
c -----------------------------------------------------------------
                             MZ=(P4M_CMUO(4,I)+P4M_CMUO(4,J))**2 -
     &                             (P4M_CMUO(1,I)+P4M_CMUO(1,J))**2 -
     &                             (P4M_CMUO(2,I)+P4M_CMUO(2,J))**2 -
     &                             (P4M_CMUO(3,I)+P4M_CMUO(3,J))**2

                             MZ=SQRT(MAX(MZ,0.))

c Make sure we have not found this combination already
c ----------------------------------------------------
                             PAIR_FOUND = .FALSE.
                             DO K=1,ZCNT
                                IF(I.EQ.LEGZ(2,K).AND.
     &                             J.EQ.LEGZ(1,K).AND.
     &                             ZTYPE(K).LT.0) 
     &                             PAIR_FOUND = .TRUE.
                             ENDDO

C Now cut on the value of the Z mass
c ----------------------------------
                             IF(MZ.LT.Z_CUT_HIGH.AND.
     &                          MZ.GT.Z_CUT_LOW.AND.
     &                          .NOT.PAIR_FOUND) THEN

                        
C This statisfies loose Z cuts
c ----------------------------  
                                ZCNT = ZCNT + 1
                                LZFLAG = .TRUE.
                                IF(ZCNT.LE.3) THEN
                                   MASSZ(ZCNT)    = MZ
                                   LEGZ(1,ZCNT)   = I
                                   LEGZ(2,ZCNT)   = J
                                   ZTYPE(ZCNT)    = -1
                                   LTYPE(1,ZCNT)  = DET_CMUO(I)
                                   LTYPE(2,ZCNT)  = DET_CMUO(J)
                                ENDIF


C Now check to see if this is a tight Z (Make sure tight leg is isolated too)
c ---------------------------------------------------------------------------
                                IF((HAD_CMUO(J).LT.TZHAD_CUT_CMUO).AND.
     &                             (EM_CMUO(J).LT.TZEM_CUT_CMUO).AND.
     &                             (ISOL_CMUO(J).LT.TZISOL_CUT).AND.
     &                             (CHRG_CMUO(J)*CHRG_CMUO(I).NE.1).AND. 
     &                             (ISOL_CMUO(I).LT.TZISOL_CUT) ) THEN

                                      TZFLAG = .TRUE.
                                      IF(ZCNT.LE.3) ZTYPE(ZCNT) = -2

                                ENDIF


                             ENDIF
			   ENDIF
                         ENDIF
                      ENDIF
           	    ENDIF
                  ENDIF
               ENDIF
            ENDDO
         ENDIF
      ENDDO
C
C Now do the same loop for CMIOs
C ------------------------------
C  
      DO I=1,MIN(NUMCMUO,MAXCMUO)
         IF(IAND(MUO_WRD(I),CMU_MUON).EQ.0) THEN	!1st one must pass cuts
            DO 777 J=1,MIN(NUMCMIO,MAXCMIO)

C Put quality cuts on the second leg of the Z
c -------------------------------------------
		IF (PT_CMIO(J)  .LT. ZPT_CUT)        GOTO 777
		IF (EM_CMIO(J)  .GT. ZEM_CUT_CMIO)   GOTO 777
		IF (HAD_CMIO(J) .GT. ZHAD_CUT_CMIO)  GOTO 777
                IF (ISOL_CMIO(J).GT. ZISOL_CUT)      GOTO 777

C Cut on rapidity
c ---------------
		PX = P4M_CMIO(1,J)
		PY = P4M_CMIO(2,J)
		PZ = P4M_CMIO(3,J)
		P  = SQRT(PX**2+PY**2+PZ**2)
		PHIMU=ATAN2(PY,PX)
		PHIMU=CDFPHI(PHIMU)
		ETAMU=ACOS(PZ/P)
		ETAMU=-ALOG(MAX(TAN(ETAMU/2),0.0000001))
		IF(ABS(ETAMU).GT.ZETA_CUT) GOTO 777

C Found a good candidate for the second leg now form invariant mass
c -----------------------------------------------------------------
                             MZ=(P4M_CMUO(4,I)+P4M_CMIO(4,J))**2 -
     &                             (P4M_CMUO(1,I)+P4M_CMIO(1,J))**2 -
     &                             (P4M_CMUO(2,I)+P4M_CMIO(2,J))**2 -
     &                             (P4M_CMUO(3,I)+P4M_CMIO(3,J))**2

                             MZ=SQRT(MAX(MZ,0.))

C Now cut on the value of the Z mass
c ----------------------------------
                             IF(MZ.LT.Z_CUT_HIGH.AND.
     &                          MZ.GT.Z_CUT_LOW) THEN

C This statisfies loose Z cuts
c ----------------------------  
                                ZCNT = ZCNT + 1
                                LZFLAG = .TRUE.
                                IF(ZCNT.LE.3) THEN
                                   MASSZ(ZCNT)    = MZ
                                   LEGZ(1,ZCNT)   = I
                                   LEGZ(2,ZCNT)   = J
                                   ZTYPE(ZCNT)    = -1
                                   LTYPE(1,ZCNT)  = DET_CMUO(I)
                                   LTYPE(2,ZCNT)  = 10
                                ENDIF


C Now check to see if this is a tight Z
c -------------------------------------
                                IF((ISOL_CMIO(J).LT.TZISOL_CUT).AND.
     &                             (ABS(ETAMU).LT.TZETA_CUT).AND.
     &                             (CHRG_CMIO(J)*CHRG_CMUO(I).NE.1) 
     &                             ) THEN

                                      TZFLAG = .TRUE.
                                      IF(ZCNT.LE.3) ZTYPE(ZCNT) = -2

                                ENDIF
                             ENDIF

777	    CONTINUE
	END IF
      ENDDO 
      RETURN
      END
