	SUBROUTINE SIM_MUTRIG(TRIG_WORD,ANY_MUTRIG)
C
C	Subroutine to simulate 1B Rube Goldberg muon trigger, including
C 	prescales and MET,JET trigger turn on curves (which are parameterized)
C	16 bit trigger word (defined below) is returned.  
C	
C	This works only for high Pt mu's, since the mu trigger turn on 
C	is not currently included (could be), only the efficiency plateau 
C	is used.
C
C	REQUIRES TOWE and CMUO .
C
C	NOTE: This routine uses side/wedge/tower info from the CMUO
C	bank (MOCMTW).  Some older Monte Carlo samples may not have this.
C	
C
C   Returns:
C	ANY_MUTRIG  = .TRUE. if any muon/met_35 trigger passed
C		    = .FALSE. if no muon/met_35 trigger passed
C
C	TRIG_WORD : Bit word of trigger passed (see list below)
C	PRESCL(0:15) IS THE CORRESPONDING PRESCALE FOR THE TRIGGER.
C	THE TRIGGER LIST IS AS FOLLOWS (SAME AS GET_TRIG_INFO):
C
C	0	CEM_16_CFT_12*		!DUMMY HERE
C	1	CEM_23_ISO_XCES*	!DUMMY HERE
C	2	MET_20*			!DUMMY HERE
C	3	MET_35_TWO*
C	4	MET_35_TEX*
C	5	CMNP_CFT_12_5DEG_V*
C	6	CMUP_CFT_12_5DEG_V*
C	7	CMNP_JET*
C	8	CMUP_JET*
C	9	CMU_CMP_JET*
C	10	CMNP_CFT_12_5DEG_M*
C	11	CMUP_CFT_12_5DEG_M*
C	12	CMX_CFT_12_5DEG_V*
C	13	CMX_JET*
C	14	CMX_CFT_12_5DEG_M*
C	15	CMX_CFT_12_5DEG_E*
C
C===Author:
C
C	Tony Liss
C	University of Illinois
C	5-FEB-1995		-Birth       
C	8-JAN-1996		-Updated prescales etc. for 110 pb-1
C 
$$IMPLICIT
C
$$INCLUDE 'C$INC:BCS.INC'
$$INCLUDE 'C$INC:JOBSTA.INC'
$$INCLUDE 'YBOS$LIBRARY:BNKTYP.INC'
$$INCLUDE 'YBOS$LIBRARY:ERRCOD.INC'
$$INCLUDE 'C$INC:CMUO92.INC'
$$INCLUDE 'c$inc:JETINFO.INC'
$$INCLUDE 'c$inc:EVTINFO.INC'
$$INCLUDE 'c$inc:MUOINFO.INC'
C
	REAL RNDM
	INTEGER BLOCAT,STATUS,IND,INDDAT
	INTEGER TRIG_WORD
	INTEGER TRGBIT(0:15),HITMSK
	INTEGER II,CMU_TOWER,IBIN
	INTEGER SIDE,WEDGE,TOWER
	REAL X,Y,MET_EFF,JET_EFF,JETBIN,METBIN,ET_WEDGE
	REAL EFF_JET(50),EFF_MET(12),PRESCL(0:15),DX,SLOPE
C	REAL L1_CMUEFF,L2_CMUEFF,L1_CMXEFF,L2_CMXEFF
	REAL L1L2_CMUEFF,L1L2_CMXEFF,BIT14_FRAC,BIT15_FRAC
	REAL MOMMUO(3),XSTRT(3),CHARGE
	REAL DXCMU,CHIXCM,DZCMU,CHIZCM,DXCMP,CHIXCP,DZCMP,CHIZCP
	REAL DXCMX,CHIXCX,DZCMX,CHIZCX
	LOGICAL DENDX,MULSCT
	LOGICAL NO_CMP,JET_TRIG,PRESCL_OK(0:15),ANY_MUTRIG
	LOGICAL BIT14_PART,BIT15_PART
C
	DATA JETBIN/2./
	DATA METBIN/10./
	DATA EFF_JET/5*0.,.017,.034,.047,.166,.393,.577,.752,.885,
     >	.901,.972,.959,.970,.987,.992,.978,1.,.985,.983,.975,26*1./
	DATA EFF_MET/0.,.0004,.0015,.1027,.5,.8194,.8261,.9143,.9524
     >	,.9000,1.,1./
C
C	Old (67pb-1) prescales:
C
C	DATA PRESCL/5*1.,0.59,3*1.,0.61,0.62,1.,0.60,1.,0.38,0.19/  !From Carla
C
C	New prescales for 110 pb-1:
C	N.B.  Last two triggers are non-overlapping - bit 14 for first 1/4 of
C	run, bit 15 for last 3/4 .  This is taken care of below.
C
	DATA PRESCL/5*1.,0.35,3*1.,0.35,0.44,1.,0.37,1.,0.88,0.62/
C	DATA L1_CMUEFF/0.92/
C	DATA L2_CMUEFF/0.87/
C	DATA L1_CMXEFF/0.80/
C	DATA L2_CMXEFF/0.77/	
C
	DATA BIT14_FRAC/0.17/
	DATA BIT15_FRAC/0.74/
C
C	CMU,CMX L1,L2 efficiencies now entered as product.  Latest numbers
C	are taken from CDF2954 with CMP numbers as weighted average of
C	CMUP and CMNP (they're almost the same anyway).
C
	DATA L1L2_CMUEFF/0.863/
	DATA L1L2_CMXEFF/0.696/
C
C	Decide which part of run it is for bit 14,15 discrimination
C
	bit14_part=.false.
	bit15_part=.false.
	y=rndm(x)
	if(y.le.bit14_frac) then
	   bit14_part=.true.
	elseif(y.le.(bit14_frac+bit15_frac)) then
	   bit15_part=.true.
	endif

C
C 	Initialize and see which prescales are passed:
C
	do ii=0,13
	  trgbit(ii)=0
	  prescl_ok(ii)=.false. 
	  x=float(ii)
	  y=rndm(x)
	  if(y.le.prescl(ii)) prescl_ok(ii)=.true.
	enddo
C
C	Now do bits 14,15
C
	trgbit(14)=0
	trgbit(15)=0
	prescl_ok(14)=.false.
	prescl_ok(15)=.false.
	y=rndm(x)
	if(bit14_part) then
	  if(y.le.prescl(14)) prescl_ok(14)=.true.
	elseif(bit15_part) then
	  if(y.le.prescl(15)) prescl_ok(15)=.true.
	endif
C
	no_cmp=.false.
	jet_trig=.false.
	any_mutrig=.false.
	dendx     = .false.
	mulsct    = .false.


C RMR -- Commented out since we are now calling this routine within topfnd.
c -------------------------------------------------------------------------
C  	call TOPfnd(.TRUE.,.TRUE.,.TRUE.)

C
C	Get jet trig efficiency for this event.
C	This is done using Claudio's efficiency curves from CDF-2821
C
	do ii=1,iclus
	  if(ijet_et(ii).gt.50) then
	    jet_trig=.true.		!100% efficient here
	  else
	    ibin=ifix(ijet_et(ii)/jetbin)+1
	    dx=ijet_et(ii)-(ibin-0.5)*jetbin
	    if(dx.ge.0.) then			!> bin center
		slope=(eff_jet(ibin+1)-eff_jet(ibin))/jetbin
	    elseif(ibin.gt.1) then
		slope=(eff_jet(ibin)-eff_jet(ibin-1))/jetbin
	    else
		slope=0.
	    endif
	    jet_eff=eff_jet(ibin)+slope*dx
	    x=float(ii)
	    y=rndm(x)
	    if(y.le.jet_eff) jet_trig=.true.	
	  endif
	end do
C
C	Get raw met and calc met_35 efficiency for this event
C 	Efficiency done using G. Sullivan's MET turn on curves.
C
	if(met(1).ge.120.) then				!MET(1) is raw
	  met_eff=1.0					!100% efficient here
	else
	 ibin=ifix(met(1)/metbin)+1
	 dx=met(1)-(ibin-0.5)*metbin
	 if(dx.ge.0) then				!> bin center
	   slope=(eff_met(ibin+1)-eff_met(ibin))/metbin
	 elseif(ibin.gt.1) then
	   slope=(eff_met(ibin)-eff_met(ibin-1))/metbin
	 else
	   slope=0.
	 endif
	 met_eff=eff_met(ibin)+dx*slope
	endif
C
C	Now check for MET trigs
C
	y=rndm(x)
	if(y.lt.met_eff) then
		trgbit(3)=1		!MET_35_TWO*	(These two
		trgbit(4)=1		!MET_35_TEX*	 indistinguishable here)
	endif
C
C	Loop ever CMUO banks and check for trigger
C
	do ii=1,numcmuo
	 if(iand(muo_wrd(ii),'DFE'x).ne.0) goto 2	!NEXT!
	 if(det_cmuo(ii).eq. 2) goto 2			!Have to do MET for this
	 if(det_cmuo(ii).lt.4) then			!CMU+
	  x=float(ii)
	  y=rndm(x)
	  if(y.gt.l1l2_cmueff) goto 2			!No L1,L2 this mu
C	  if(y.gt.l1_cmueff) goto 2			!No L1, this mu
C	  y=rndm(x)
C	  if(y.gt.l2_cmueff) goto 2			!No L2, this mu
C
C	OK, have a valid l1/l2 CMU or CMU/CMP.  Adjust for
C	prescales and other trigger combobulations
C
C	First is CMU_CMP_JET* which takes any det_cmuo<4, regardless
C	of phi, and therefore picks up eta gaps.
C
	  if(jet_trig.and.prescl_ok(9)) trgbit(9)=1
C
	  if(det_cmuo(ii).eq.3) then			!CMU/CMP, no prescale
	   if(prescl_ok(6)) trgbit(6)=1			!Inclusive 12 GeV
	   if(jet_trig.and.prescl_ok(8)) trgbit(8)=1	!CMU/CMP w/ jet
C
C  Do Imin trigger w/Claudio's routine
C
	   call get_level2_wedge(bnk_cmuo(ii),et_wedge)
	   if((et_wedge.lt.3.).and.(prescl_ok(11))) trgbit(11)=1
C
	  elseif(det_cmuo(ii).eq.1) then		!CMU_ONLY (CMNP)
C
C	See if this is in CMP phi gap region (must be for valid L1,L2 mu trig)
C
	   status=blocat(iw,'CMUO',bnk_cmuo(ii),ind,inddat)
	   cmu_tower=iw(inddat+mocmtw)
	   side = cmu_tower/144
	   wedge = (cmu_tower-(side*144))/6
	   tower = (cmu_tower-side*144-wedge*6)
	   if(tower.le.1) then
		tower=0
	   elseif(tower.le.3) then
		tower=1
	   else
		tower=2
	   endif
	   cmu_tower = 3*wedge+tower
	   if((cmu_tower.ge.8).and.(cmu_tower.le.10)) no_cmp=.true.
	   if((cmu_tower.ge.15).and.(cmu_tower.le.20)) no_cmp=.true.
	   if((cmu_tower.ge.25).and.(cmu_tower.le.27)) no_cmp=.true.
	   if((cmu_tower.ge.51).and.(cmu_tower.le.56)) no_cmp=.true.
	   if((cmu_tower.eq.43).or.(cmu_tower.eq.64)) no_cmp=.true.
C
	   if(no_cmp) then			!Valid CMU_ONLY region
		if(prescl_ok(5)) trgbit(5)=1	!Inclusive CMU_ONLY 12 GeV
		if(jet_trig.and.prescl_ok(7)) trgbit(7)=1
C
C  Do Imin trigger w/Claudio's routine
C
	  	call get_level2_wedge(bnk_cmuo(ii),et_wedge)
	  	if((et_wedge.lt.3.).and.(prescl_ok(10))) trgbit(10)=1
C
	   endif     !End no_cmp
	  endif
	 endif      !End det_cmuo <4
C
C	Do CMX now
C
	 if(det_cmuo(ii).eq.4) then
	  y=rndm(x)
	  if(y.gt.l1l2_cmxeff) goto 2			!No L1,L2 this mu
C	  if(y.gt.l1_cmxeff) goto 2			!No L1, this mu
C	  y=rndm(x)
C	  if(y.gt.l2_cmxeff) goto 2			!No L2, this mu
C
C  Check if track goes through SL8 (ala Claudio. sl8_radius=125.,z_ctc=150.)
C
	  if((z0_cmuo(ii)+
     >	  (p4m_cmuo(3,ii)/pt_cmuo(ii))*125.).gt.150.) goto 2
C
	  if(prescl_ok(12)) trgbit(12)=1		!Inclusive 12 GeV CMX
C
C  Do Imin trigger w/Claudio's routine
C
	  call get_level2_wedge(bnk_cmuo(ii),et_wedge)
	  if((et_wedge.lt.3.).and.(prescl_ok(14))) trgbit(14)=1
C
C  Do the Et>0 trig (CMX...E*)
C
	  if((et_wedge.gt.0.).and.(prescl_ok(15))) trgbit(15)=1
C
	  if(jet_trig.and.prescl_ok(13)) trgbit(13)=1	!CMX_JET*
	 endif
C
    2	 continue
	end do				!Do loop over CMUOs
C
	do ii=3,15
	  if(trgbit(ii).eq.1) any_mutrig=.true.
	end do
C
	trig_word = 0
	Do II = 0, 15
	   If (trgbit(ii).eq.1) trig_word = trig_word + 2**ii
	Enddo
C
	return
	end
