c  ==========================================================================
c  ==========================================================================
	subroutine slt_top_cuts(p_primary,emu_ty,z_primary,
     >                          q_primary,ntags,qtr_slt,
     >                          num_pass,pass)
c
c       This routine applies the SLT cuts for the top analysis not
c       implemented in the offline DATA_TAGGER routine which is
c       intended to be a general purpose routine.  Here we remove
c     	      o   Drell-yan candidates
c             o   Upsilon candidates
c             o   b sequential decays and psis (low mass) candidates
c             o   z->mu+mu with looser cuts than have already been
c                 applied to the sample.
c             o   |Z_slt - Z_primary_lepton|< 5cm.
c
c	INPUT:    p_primary(3)    -   3 vector momentum of primary
c                 emu_ty	  -   event type of primary lepton [integer]
c                 z_primary       -   z vertex of primary lepton
c 		  q_primary	  -   charge of primary lepton [integer]
c                 ntags           -   number of slt tags
c                 qtr_slt         -   qtrk # of slt [integer]
c                                     if positive, ele, neg, muon
c
c	OUTPUT:   pass		 =   0 if passes all cuts, otherwise
c				    -1 (fails Drell-yan)
c				    -2 (fails Upsilon)
c				    -3 (fails low mass)
c				    -4 (fails z->mu-mu)
c				    -5 (fails z vertex)
c				    -999 (some horrible error)
c
c		num_pass	-    number of slt tags which pass filter
c
c       WHAT YOU HAVE TO DO AHEAD
c            make TOWE, fill MUOINFO, i think thats it
c
c	David Kestenbaum (SLT slave)
c       Nov 2, 1995
c
c       Revisions
c	Rob Roser	Nov. 3, 95	Minor changes to make it fit into code
c
c=======================================================================
c=======================================================================

$$IMPLICIT
$$include 'C$INC:BCS.INC'
$$include 'C$INC:ERLEVL.INC'
$$include 'C$INC:SOFTELE.INC'
$$include 'C$INC:TRAKCM.INC'
$$include 'C$INC:MUOINFO.INC'
$$include 'C$INC:TRKSCO.INC'
$$include 'C$INC:DSFITP.INC'
$$include 'YBOS$LIBRARY:ERRCOD.INC'
$$include 'c$inc:trexpo.INC'
$$include 'c$inc:dstowe.INC'

	real p_primary(3),z_primary
	real param(5),XYZ(6),DXYZ(6)
        integer ii,num_pass
	integer emu_ty,emu_primary,emu_slt
        integer pass(5),qtrk_slt,qtr_slt(5),ntags
	integer indbnk,inseta,trkid,i,q_primary
	real phi_center,eta_center,emass,mumass,isoslt
        real et4slt,ptslt,pxslt,pyslt,pzslt,eslt
	real p_mass,e_primary,z0_slt
	real inv_mass,cdfphi,cc_econe
	integer qslt,status,trksun,ierr,blocat
        parameter (EMASS  = 0.000511)
        parameter (MUMASS = 0.1056584)

c  ==========================================================================
c  ==========================================================================

c--
c initialize with pessimism
c--
        num_pass=0
	pass(1)=-999
	pass(2)=-999
	pass(3)=-999
	pass(4)=-999
	pass(5)=-999

c--
c blow do clambake if no slt tags -- why am I here anyway?
c--
	if(ntags.eq.0)goto 999

c--
c figure out whether type is muon or electron
c--
	if(emu_ty.gt.0 .and. emu_ty.lt.30)then
           emu_primary=1
        elseif(emu_ty.gt.30. .and. emu_ty.lt.50)then
           emu_primary=2
        else
           goto 999
        endif

c--
c calculate energy of primary lepton
c--
	 if(emu_primary.eq.1)then
	  p_mass=emass
	 else
	  p_mass=mumass
	 endif         
         e_primary=sqrt(p_primary(1)**2+p_primary(2)**2+
     >                  p_primary(3)**2+p_mass**2)


c--
c Now look over SLT tags
c--

      do ii=1,min(ntags,5)
c--
c set type by sign of qtrk
c--      
         qtrk_slt=qtr_slt(ii)
         if(qtrk_slt.gt.0)then
          emu_slt=1
         elseif(qtrk_slt.lt.0)then
          emu_slt=2
          qtrk_slt=abs(qtrk_slt)
         elseif(qtrk_slt.eq.0)then
          goto 99
         endif

c--
c first get isolation information for slt
c--
	et4slt=-999.
	ptslt =-999.
	trkid =-999

        if(emu_slt.eq.1)then
	  
c--
c for electron, propagate track out to calorimeter
c and get energy in a cone of 0.4
c--

	     CALL UNPTRK(2)	!already done?
	     do i=1,n_track
	       if(qtrkbnk(i).eq.qtrk_slt)trkid=i
	     enddo
	     if(trkid.eq.-999)goto 99
             CALL GET_SOFT_ELECTRON(qtrk_slt,2)
             STATUS = TRKSUN(qtrk_slt,1,0)
	     if(status.ne.0)goto 99
             PARAM(1) = TRKSFP(CUFP) ! CU
             PARAM(2) = TRKSFP(D0FP) ! D
             PARAM(3) = TRKSFP(P0FP) ! PHI0
             PARAM(4) = TRKSFP(CTFP) ! CTG
             PARAM(5) = TRKSFP(Z0FP) ! Z0
             CALL TRPROP_AY(1,PARAM,TRPOEC,XYZ,DXYZ,IERR)
	     if(ierr.ne.0)goto 99
             STATUS = BLOCAT(IW,'SETA',1,INDBNK,INSETA)
	     if(status.ne.yesucc)goto 99
             PHI_CENTER = CDFPHI(ATAN2(XYZ(2),XYZ(1)))
             ETA_CENTER = RW(INSETA+ETA_SOFT+2*TENETA)
             et4slt   = CC_ECONE(ETA_CENTER,PHI_CENTER,0.4) - EEM
	     ptslt    = eem	    !Et actually for electron
	     if(eem.le.0)goto 99
	     isoslt   = et4slt/eem
	     qslt     = chtr(trkid)
	     pxslt    = pxtr(trkid)
	     pyslt    = pytr(trkid)
	     pzslt    = pztr(trkid)
	     eslt     = sqrt(pxslt**2+pyslt**2+pzslt**2+emass**2)
	     z0_slt   = z0tr(trkid)

       else

c--
c for muons, get energy in cone 0.4 from CMUO
c--

	  do i=1,numcmuo
	    if(trks_cmuo(i).eq.qtrk_slt)then
	      ptslt=pt_cmuo(i)
	      isoslt=isol_cmuo(i)
	      et4slt=isoslt*ptslt
	      pxslt=p4m_cmuo(1,i)
	      pyslt=p4m_cmuo(2,i)
	      pzslt=p4m_cmuo(3,i)
	      eslt =p4m_cmuo(4,i)
	      qslt=chrg_cmuo(i)
	      z0_slt=z0_cmuo(i)
	    endif
	  enddo

	endif	    !  e or mu slt type


	if(et4slt.lt.-900)then
           call erlogr('SLT_TOP_CUTS',ERWARN,0,0,'COULDNT GET SLT ISO')
	   goto 99
	endif		

c-
c calculate invariant mass
c-

	 inv_mass=    (   (Eslt +e_primary)**2 
     >                  - (pxslt+p_primary(1))**2 
     >                  - (pyslt+p_primary(2))**2 
     >                  - (pzslt+p_primary(3))**2   )


	 if(inv_mass.gt.0)then
	    inv_mass=sqrt(inv_mass)
	 else
	    goto 99
	 endif

c--
c NOW we apply the cuts
c--
	if(qslt*q_primary.lt.0)then	!check opposite sign

	  if (emu_primary.eq.emu_slt) then


c--
c now test for Drell-yan
c--
	     if(ptslt.gt.20.and.isoslt.lt.0.1)then
	       pass(ii)=-1
	       goto 99
	     elseif(ptslt.le.20.and.et4slt.lt.2.0)then
	       pass(ii)=-1
	       goto 99
	     endif

c--
c now test for upsilon
c--
	     if(inv_mass.gt.8.and.inv_mass.lt.11)then
	       pass(ii)=-2
	       goto 99
	     endif

c--
c now test for Z->mu+mu
c--
	     if(inv_mass.gt.70.0.and.inv_mass.lt.110)then
	       pass(ii)=-4
	       goto 99
	     endif

	  endif	    !same flavor

c--
c now do low mass test
c--
	  if(inv_mass.lt.5)then
	     pass(ii)=-3
	     goto 99
	  endif

	endif	    !opposite sign

c--
c finally, the z matching cut
c--
	if(abs(z0_slt-z_primary).gt.5.0)then
	  pass(ii)=-5
          goto 99
	endif


c--
c if we made it this far, it passes all cuts
c--
	pass(ii)=0
        num_pass=num_pass+1
c--
c end loop
c--
99     continue
      enddo

c--
c The pool, the pond -- the pond would be good for you
c--
999   return

      END
c  ==========================================================================
c  ==========================================================================
