c ==============================================================================
c ==============================================================================
      SUBROUTINE TAGFILT(ptags,svx_ptags,ntags,svx_ntags,
     &                   nslttg,nsltps,qtr_slt,pass)
c
c
c
c       This routine looks at the BTGS Banks to get the "generic"
c       SVX and SLT tags.  This routine applies the cuts required
c       for the top analysis.
c		SVX Tags	-  Make sure the tag is in the primary
c				   electron jet (for e+jet events)
c			        -  Makes sure tag has pos. decay length.
c
c		SLT Tags	-  SLT tag can not be the primary lepton
c				-  DY cuts
c				-  Upsilon Cuts
c				-  low mass candidates
c				-  z->mu+mu
c				-  z vertex cut
c
c		Input:		requires TOWE banks be present
c	
c		Output:(SVX)    ptags		Number of +SIP SVX tags
c				svx_ptags(5) 	Jets number of + tagged jets
c			        ntags		Number of -SIP SVX tags
c				svx_ntags(5) 	Jets number of - tagged jets
c
c		       (SLT)    NSLTTG		Number of SLT tags prior top cuts
c				NSLTPS		Number of good top SLT tags 
c				QTR_SLT(5)	trks number of SLT tags
c						  pos trks num -- ele tag 
c                                                 neg trks num -- mu tag
c       			pass(5)         0 if passes all cuts, otherwise
c                       	            	-1 (fails Drell-yan)
c                       	            	-2 (fails Upsilon)
c                       	            	-3 (fails low mass)
c                                   		-4 (fails z->mu-mu)
c                                   		-5 (fails z vertex)
c                                  	 	-999 (some horrible error)
c
c	Written:  	Robert Roser	November 4, 1995
c       Revisions:	RR		1/9/96		Fixed SVX bug
c
c =============================================================================

      IMPLICIT NONE

$$INCLUDE 'SYSTEM_INTERFACE$LIBRARY:NOARGINC.INC'
$$INCLUDE 'C$INC:BCS.INC'
$$INCLUDE 'C$INC:ERLEVL.INC'
$$INCLUDE 'YBOS$LIBRARY:ERRCOD.INC'
$$INCLUDE 'C$INC:JOBSTA.INC'
$$INCLUDE 'UIPACK$LIBRARY:UIPARAM.INC'
$$INCLUDE 'UIPACK$LIBRARY:UIERROR.INC'
C
$$INCLUDE 'C$INC:BTGSCM.INC'
$$INCLUDE 'C$INC:ELEINFO.INC'
$$INCLUDE 'C$INC:MUOINFO.INC'
$$INCLUDE 'C$INC:MIOINFO.INC'
$$INCLUDE 'C$INC:SECVTX.INC'
$$INCLUDE 'c$inc:JETINFO.INC'
$$INCLUDE 'c$inc:EVTINFO.INC'
$$INCLUDE 'c$inc:DILINFO.INC'
$$INCLUDE 'top$fnd:DILMORE.INC'
$$INCLUDE 'top$fnd:TOPSCOM.INC'

      
      INTEGER I,J,K 
      INTEGER ANPTRG,STATUS,UIGTLO,UIUSGP,UIGTVB,UIDFFI
      INTEGER SLT_E,SLT_M
      integer svx_ptags(5),svx_ntags(5),ptags,ntags
      integer q_primary,nslttg,qtr_slt(5),pass(5),nsltps
      integer jbank
      integer counter

      real    good_etag(5,2),good_mtag(5,2)
      real    etajt,phijt
      real    etatrk,phitrk
      real    delr
      real    p_primary(3),z_primary

      logical slte,sltm

c  ===========================================================================
c  ===========================================================================

c initialize variables
c --------------------
          call vzero(good_etag,5)
          call vzero(good_mtag,5)
          call vzero(svx_ptags,5)
          call vzero(svx_ntags,5)

          j=1
          k=1
                            
c   svx tag info
         ptags=0
         ntags=0

c   slt tag info
          SLT_E=0
          SLT_M=0

C unpack banks into common blocks
c (we need to call topfnd to get muo/ele info banks filled for SLT shit)
c -----------------------------------------------------------------------
      CALL TOPSBTC(-1)
      CALL BTGBTC
        if(nrun.gt.1000)then
           CALL TOPFND(.TRUE., .TRUE., .TRUE.)
        else
           CALL TOPFND(.TRUE., .FALSE., .FALSE.)
        endif


c --------------------------------------------------------------------------
C ---------------------------------- SVX tags -----------------------------
c --------------------------------------------------------------------------

      DO I=1,TS_NJ15
         DO J=1,BTNNJET

c pos tags
c --------
             IF(TS_NJ15_BNK(I).EQ.BTJETB(J).AND.SCTAG(J).EQ.1
     &         .AND.SC2DLEN(J).GT.0.) THEN
      
                  IF(TS_NJ15_BNK(I).EQ.TS_L1_LK.AND.
     &               TS_L1_TY.EQ.20) THEN
                     continue
                  ELSE
                        ptags=ptags+1
                        svx_ptags(ptags)=TS_NJ15_BNK(I)
                  ENDIF                  
             ENDIF
c neg tags
c --------
             IF(TS_NJ15_BNK(I).EQ.BTJETB(J).AND.SCTAG(J).EQ.1
     &         .AND.SC2DLEN(J).LT.0.) Then

                  IF(TS_NJ15_BNK(I).EQ.TS_L1_LK.AND.
     &               TS_L1_TY.EQ.20) THEN
                     continue
                  ELSE
                        ntags=ntags+1
                        svx_ntags(ntags)=TS_NJ15_BNK(I)
                  ENDIF                  
             ENDIF
         ENDDO
      ENDDO

c --------------------------------------------------------------------------
C ---------------------------------- SLT tags -----------------------------
c --------------------------------------------------------------------------

c first count how many "RAW" SLT tags we have
c -------------------------------------------
       DO I=1,5
          SLTE=.FALSE.
          SLTM=.FALSE.

	  IF(BTNELE(I).GT.0)THEN
               SLT_E  = SLT_E  + 1
               SLTE=.TRUE.
          ENDIF
	  IF(BTNMUO(I).GT.0)THEN
               SLT_M = SLT_M + 1
               SLTM=.TRUE.
          ENDIF

	  IF(BTNMUO(I).EQ.0 .AND. BTNELE(I).EQ.0)GOTO 2

	  IF(TS_L1_TY.EQ.20  .AND.
     &        BTNELE(I).GT.0  .AND. 
     &        BTNELE(I).EQ.TRKS_ELE(TS_L1_IN))THEN
                SLT_E=SLT_E-1
                SLTE=.FALSE.
          ENDIF 

	  IF(TS_L1_TY.GT.30  .AND.
     &        BTNMUO(I).GT.0  .AND. 
     &        BTNMUO(I).EQ.TRKS_CMUO(TS_L1_IN))THEN
                SLT_M=SLT_M-1
                SLTM=.FALSE.
          ENDIF

          if(ts_dilcd)then
	     IF(TS_L2_TY.EQ.20  .AND.
     &          BTNELE(I).GT.0  .AND. 
     &          BTNELE(I).EQ.TRKS_ELE(TS_L2_IN))THEN
                   SLT_E=SLT_E-1
                   SLTM=.FALSE.
             ENDIF
	        IF(TS_L2_TY.GE.30  .AND.
     &	           TS_L2_TY.LT.35  .AND.
     &            BTNMUO(I).GT.0  .AND. 
     &            BTNMUO(I).EQ.TRKS_CMUO(TS_L2_IN))THEN
                   SLT_M=SLT_M-1
                   SLTM=.FALSE. 
                ENDIF

	        IF(TS_L2_TY.EQ.40  .AND.
     &            BTNMUO(I).GT.0  .AND. 
     &            BTNMUO(I).EQ.TRKS_CMIO(TS_L2_IN))THEN
                   SLT_M=SLT_M-1
                   SLTM=.FALSE.
                ENDIF
          endif    

          IF(SLTE)then
             GOOD_eTAG(J,1)=BTNELE(I)
             j=j+1
          endif
          IF(SLTm)then
             GOOD_mTAG(k,1)=BTNMUO(I)
             k=k+1
          endif

2       CONTINUE
       ENDDO

c calculate total number of tags
c ------------------------------
	nslttg = slt_e + slt_m

c determine with trepidation whether given SLT tags pass top cuts
c --------------------------------------------------------------- 
        p_primary(1)=TS_L1_PX
        p_primary(2)=TS_L1_PY
        p_primary(3)=TS_L1_PZ

        if(ts_l1_ty.gt.0 .and. ts_l1_ty.lt.30)then
           z_primary =  EZV_ELE(ts_l1_in)
           q_primary =  CHG_ELE(ts_l1_in)
        elseif(ts_l1_ty.gt.30. .and. ts_l1_ty.lt.50)then
c           z_primary =  MUZV_CMUO(ts_l1_in)
           z_primary =  z0_cmuo(ts_l1_in)
           q_primary =  CHRG_CMUO(ts_l1_in)
        endif


        counter=0

        do i=1,5
         if(good_etag(i,1).gt.0)then
             counter=counter+1
             qtr_slt(counter)=good_etag(i,1)
         endif
         if(good_mtag(i,1).gt.0)then
             counter=counter+1
             qtr_slt(counter)=-good_mtag(i,1)
         endif
        enddo	

        CALL slt_top_cuts(p_primary,
     &                    ts_l1_ty,
     &                    z_primary,
     &                    q_primary,
     &                    nslttg,
     &                    qtr_slt,
     &                    nsltps,
     &                    pass)



      RETURN
      END

c  ===========================================================================
c  ===========================================================================

	subroutine eta_trk(trks_num,t_eta,t_phi)
c
c  calculate eta/phi of a track starting with a trks bank number
c
c	Rob Roser		March 25, 1995
c
$$implicit
C
$$include 'c$inc:jobsta.inc'
$$include 'ybos$library:errcod.inc'
$$include 'c$inc:pivalu.inc'
$$INCLUDE 'C$INC:BCS.INC'
$$INCLUDE 'C$INC:CTCGEO.INC'
$$Include 'c$inc:svsbtc.inc'             ! common for SVX tracks

      real    pxt,pyt,pzt,pt,pmag
      real    t_theta,t_eta,t_phi
      real    denom
      REAL    PR(5)                     !Track parameters..is different order
      REAL    CVM(15)                    !Covariance Matrix values...not used
      INTEGER TQUAL                      !Track quality...not used
      INTEGER TRKS_NUM                   !TRKS bank number for electron


c ==============================================================================
c ==============================================================================


c call routine to get tracking parameters
c ----------------------------------------
               CALL TRK_INFO('TRKS',TRKS_NUM,PR,CVM,TQUAL)

c get 4 momentum
c --------------
               if(pr(2).ne.0.)then
                  Pt = 1.49898E-4*bmagnt/ABS(pr(2))
               else
                  pt = -999.
               endif
               Pxt = Pt * Cos(Pr(5))
               Pyt = Pt * Sin(Pr(5))
               Pzt = Pt * Pr(1)
               Pmag = Sqrt(Pt**2 + Pzt**2)

c calculate stuff
c ---------------
                denom = sqrt(pxt**2 + pyt**2 + pzt**2)
               if(denom.ne.0.)then
                t_theta = acos(pzt /denom)
                t_eta = -alog(max(tan(t_theta/2),0.0000001))
               else
                t_theta = -999.
                t_eta = -999.
               endif

               if(pxt.eq.0. .and. pyt.eq.0.)then
                 t_phi=0.
               else
                 t_phi= atan2(-pyt,-pxt)+atan2(0.0,-1.0)
               endif

c C'est la Vie
c ------------

999	return
	end
c ==============================================================================
c ==============================================================================


