c =========================================================================
c    TOPFND.CDF
c =========================================================================
c
c
c    Purpose:  This Routine determines all the information used for
c              event selection in the Top Analysis.  The information
c              is gathered from various other subroutines and summarized
c              by this routine.  This routine has been tested on data.
c              The testing on Monte Carlo has not been completed.  Use
c              on Monte Carlo with caution.
c -------------------------------------------------------------------------
c    Information:
c
c        Electron Quantities:
c
c               Central Electrons:  
c                      - Electron quantities are stored in
c                        ELEINFO common which is filled by ELEVAL.
c                      - The routine ELEFND selects good primary
c                        and secondary electrons (for the dilepton 
c                        analysis).
c
c               Plug Electrons:
c                      - Electron quantities are stored in 
c                        ELEINFO common which is filled by ELEVAL.
c                      - The routine PEMFND selects good plug electrons
c                        (currently only used for the second lepton
c                         of the dilepton analysis.)
c
c       Muon Quantities:
c
c               Central Muons:
c                      - Muon quantities are stored in MUOINFO 
c                        common which is filled by MUOFND.
c                      - The routine MUOFND selects good primary
c                        and secondary muons.
c
c               Minimum Ionizing Particles:
c                      - CMIO quantities are stored in MIOINFO
c                        common which is filled by MIOFND.
c                      - The routine MIOFND selects good CMIO's.
c
c       Jet Quantities:
c
c                      - Jet quantities are stored in JETINFO common
c                        block which is filled by JETINFO. Jet counting
c                        is performed by JETFND.
c
c       Event Quantites:
c
c                       - Event quantities are stored in EVTINFO
c                         common block which is filled by this routine
c                         and a few others.
c ---------------------------------------------------------------------------
c
c     Initial Requirements:
c
c              - Presents of ELES,CMUO,CMIO,METS,JETS,QTRK,VTVZ,CLSL
c
c     Inputs:
c
c     Outputs: All outputs are contained in EVTINFO common and associated
c              common blocks from other routines.
c
c ---------------------------------------------------------------------------
c  History:   
c   
c      Brian L Winer, 1-Nov-1994            Initial Creation
c
c ============================================================================
      SUBROUTINE TOPFND(MET_JET_FLAG,BR_FLAG,TRG_FLAG)
C ----------------------------------------------------------------------------

$$IMPLICIT

C Common Blocks/Include Files (Standard First)
c --------------------------------------------
$$INCLUDE 'C$INC:ANACTL.INC'
$$INCLUDE 'C$INC:JOBSTA.INC'
$$INCLUDE 'C$INC:ERLEVL.INC'
$$INCLUDE 'C$INC:BCS.INC'
$$INCLUDE 'YBOS$LIBRARY:ERRCOD.INC'
$$INCLUDE 'UIPACK$LIBRARY:UIPARAM.INC'
$$INCLUDE 'UIPACK$LIBRARY:UIERROR.INC'
$$INCLUDE 'c$inc:ELEINFO.INC'
$$INCLUDE 'c$inc:MUOINFO.INC'
$$INCLUDE 'c$inc:MIOINFO.INC'
$$INCLUDE 'c$inc:JETINFO.INC'
$$INCLUDE 'C$INC:EVTINFO.INC'
$$INCLUDE 'top$fnd:TFZINFO.INC'
$$INCLUDE 'C$INC:DILINFO.INC'
$$INCLUDE 'TOP$FND:DILMORE.INC'

c Local Variables
c ---------------
      INTEGER CEM_ELE,CMU_MUON,CEM_LOOSE,PEM_ELE,MIO_MUON,MIO_MUON_MET
      INTEGER CONV_CUT,DIL_ISO,SGL_ISO
      INTEGER HPTE_HPTE,HPTE_LCEM,HPTE_PEM,HPTE_CMU,HPTE_CMX,HPTE_CMIO
      INTEGER HPTM_HPTE,HPTM_LCEM,HPTM_PEM,HPTM_CMU,HPTM_CMX,HPTM_CMIO
      INTEGER HPTMU,HPTMX,HPTE
      INTEGER IERR
      INTEGER HPTE_TRIG_MASK
      INTEGER HPT_CMU_TRIG_MASK
      INTEGER HPT_CMP_TRIG_MASK
      INTEGER HPT_CMX_TRIG_MASK
      INTEGER STREAM,BR_CODE
      INTEGER I,J,STATUS
      INTEGER BLOCAT,JTOPS,JDTOPS
      LOGICAL PRIM_DIL
      LOGICAL BC/.TRUE./
      LOGICAL MET_JET_FLAG,BR_FLAG,TRG_FLAG
      LOGICAL DUP_DIL(10)
      LOGICAL ANY_MUTRIG

      INTEGER KK
      LOGICAL ANSWER

c Define all the cut Masks 
c ------------------------
      DATA CEM_ELE           /'3FF'X/
      DATA CEM_LOOSE         /'F33'X/
      DATA PEM_ELE            /'FF'X/
      DATA CMU_MUON          /'DFE'X/
      DATA MIO_MUON         /'1F1E'X/
      DATA MIO_MUON_MET      /'F1E'X/
      DATA CONV_CUT               /3/
      DATA DIL_ISO                /6/
      DATA SGL_ISO                /1/

c Note: Currently these trigger masks are only good for Run 1b
c ------------------------------------------------------------
      DATA HPTE_TRIG_MASK       /'5'X/
      DATA HPT_CMU_TRIG_MASK  /'FF8'X/
      DATA HPT_CMP_TRIG_MASK   /'18'X/
      DATA HPT_CMX_TRIG_MASK /'F018'X/

C Define Dileption and Single Lepton Types
c ----------------------------------------
      DATA HPTE_HPTE/1/
      DATA HPTE_LCEM/2/
      DATA HPTE_PEM/3/
      DATA HPTE_CMU/4/
      DATA HPTE_CMX/5/
      DATA HPTE_CMIO/6/
      DATA HPTM_HPTE/7/
      DATA HPTM_LCEM/8/
      DATA HPTM_PEM/9/
      DATA HPTM_CMU/10/
      DATA HPTM_CMX/11/
      DATA HPTM_CMIO/12/

      DATA HPTMU/20/
      DATA HPTMX/21/
      DATA HPTE/30/      

c Hello, Mr. Gopher, don't mind the wires...Dr's orders and such
c --------------------------------------------------------------
      

C Start by Collecting all the Information
c =======================================
      NDIL = 0
      NSGL = 0
      NUM_HPTE = 0
      NUM_LOOSE_E = 0
      NUM_PEM = 0
      NUM_HPTM = 0
      NUM_CMIO = 0
      ONE_LEPTON = .FALSE.
      TWO_LEPTON = .FALSE.
      DIL_CAND=.FALSE.
      ANSWER=.FALSE.
      CALL VZERO(IND_HPTE,5)
      CALL VZERO(IND_LOOSE_E,5)
      CALL VZERO(IND_PEM,5)
      CALL VZERO(IND_HPTM,5)
      CALL VZERO(IND_CMIO,5)
      CALL VZERO(DIL_TYPE,30)
      CALL VZERO(SGL_TYPE,2)
      CALL DIL_INIT      

c Fill central Electron Information
c ---------------------------------
      CALL ELEVAL(BC)

c Fill central Electron: perform cuts
c -----------------------------------
      CALL ELEFND

c Fill plug Electron Information and perform cuts
c -----------------------------------------------
      CALL PEMFND

c Fill Muon Information
c ---------------------
      CALL MUOVAL(BC)
      CALL MIOVAL(BC)

c Fill central muon Information and perform cuts
c ----------------------------------------------
      CALL MUOFND

c Fill Min. Ionizing Information and perform cuts
c -----------------------------------------------
      CALL MIOFND


c Now decide what we got with the leptons
c =======================================

c Look for good electrons
c -----------------------
      DO I=1,min(maxeles,NUMELES)

C Check for good High Pt Electron
c -------------------------------
         IF(IAND(CEM_WRD(I),CEM_ELE).EQ.0.AND.
     &      IAND(CON_ELE(I),CONV_CUT).EQ.0) THEN
            NUM_HPTE = NUM_HPTE + 1
            IND_HPTE(NUM_HPTE) = I
         ENDIF   

C Also, check for any loose electron for the dileptons
c ----------------------------------------------------
         IF(IAND(CEM_WRD(I),CEM_LOOSE).EQ.0.AND.
     &      IAND(CON_ELE(I),CONV_CUT).EQ.0) THEN
            NUM_LOOSE_E = NUM_LOOSE_E + 1
            IND_LOOSE_E(NUM_LOOSE_E) = I
         ENDIF   

C Check for good PEM electrons
c ----------------------------
         IF(IAND(PEM_WRD(I),PEM_ELE).EQ.0.AND.
     &      IAND(PEM_ISO(I),1).EQ.0) THEN
            NUM_PEM = NUM_PEM + 1
            IND_PEM(NUM_PEM) = I
         ENDIF
      ENDDO


c Now look for Central Muons
c --------------------------
      DO I=1,min(maxcmuo,NUMCMUO)

C Check for Good High Pt Muons (no isolation requirement)
c -------------------------------------------------------
         IF(IAND(MUO_WRD(I),CMU_MUON).EQ.0) THEN
            NUM_HPTM = NUM_HPTM + 1
            IND_HPTM(NUM_HPTM) = I
         ENDIF   

      ENDDO


c Now look for CMIO's
c -------------------
      DO I=1,min(maxcmio,NUMCMIO)

C Check for good high Pt CMIO
c ---------------------------
         IF(IAND(MIO_WRD(I),MIO_MUON).EQ.0.AND.
     &      IAND(MIO_ISO(I),5).EQ.0) THEN
            NUM_CMIO = NUM_CMIO + 1
            IND_CMIO(NUM_CMIO) = I
         ENDIF
      ENDDO


c ======================================================================
c   Begin by looking for an isolated (by dilepton standards) lepton
c   Next, look for a second lepton.  If there is a second lepton then
c   clasify.  If no second lepton can be found, determine if the initial
c   lepton is isolated enough to clasify as a lepton for the lepton plus
c   jets analysis.  NOTE: For the dilepton clasification it is possible
c   to get the same dilepton pair giving different clasifications.
c ======================================================================

C Start by looping of the electrons
c ---------------------------------
      DO I=1,NUM_HPTE
      
         IF(IAND(CEM_ISO(IND_HPTE(I)),DIL_ISO).EQ.0) THEN

C Now look for another tight CEM Electron
c ---------------------------------------
            DO J=1,NUM_HPTE
               IF(J.NE.I) THEN
                   NDIL = NDIL + 1
                   DIL_TYPE(1,NDIL) = HPTE_HPTE
                   DIL_TYPE(2,NDIL) = I
                   DIL_TYPE(3,NDIL) = J
               ENDIF
            ENDDO

C Look for a loose central electron
c ---------------------------------
            DO J=1,NUM_LOOSE_E
               IF(IND_LOOSE_E(J).NE.IND_HPTE(I)) THEN
                  NDIL = NDIL + 1
                  DIL_TYPE(1,NDIL) = HPTE_LCEM
                  DIL_TYPE(2,NDIL) = I
                  DIL_TYPE(3,NDIL) = J
               ENDIF
            ENDDO

C Look for a loose plug
c ---------------------
            DO J=1,NUM_PEM
              IF(IND_PEM(J).NE.IND_HPTE(I))THEN
                 NDIL = NDIL + 1
                 DIL_TYPE(1,NDIL) = HPTE_PEM
                 DIL_TYPE(2,NDIL) = I
                 DIL_TYPE(3,NDIL) = J
              ENDIF
            ENDDO

c Look for a CMU/CMX muon
c -----------------------
            DO J=1,NUM_HPTM
              IF(IAND(MUO_WRD(IND_HPTM(J)),1).EQ.0) THEN
                 NDIL = NDIL + 1
                 DIL_TYPE(1,NDIL) = HPTE_CMU
                 DIL_TYPE(2,NDIL) = I
                 DIL_TYPE(3,NDIL) = J
              ELSEIF(IAND(MUO_WRD(IND_HPTM(J)),1).EQ.1) THEN
                 NDIL = NDIL + 1
                 DIL_TYPE(1,NDIL) = HPTE_CMX
                 DIL_TYPE(2,NDIL) = I
                 DIL_TYPE(3,NDIL) = J
              ENDIF
            ENDDO

c Look for a CMIO as a partner
c ----------------------------
            DO J=1,NUM_CMIO
                 NDIL = NDIL + 1
                 DIL_TYPE(1,NDIL) = HPTE_CMIO
                 DIL_TYPE(2,NDIL) = I
                 DIL_TYPE(3,NDIL) = J
            ENDDO


        ENDIF  !If the lepton is isolated enough for a dilepton candidate
      
c Check to see if this lepton is good enough for lepton plus jets analysis
c ------------------------------------------------------------------------
        IF(IAND(CEM_ISO(IND_HPTE(I)),SGL_ISO).EQ.0) THEN
            NSGL = NSGL + 1
            SGL_TYPE(1) = HPTE
            SGL_TYPE(2) = I
        ENDIF


      ENDDO  !end loop over high pt CEM electrons


c Look for a Tight Muons
c ----------------------
      DO I=1,NUM_HPTM

         IF(IAND(MUO_ISO(IND_HPTM(I)),DIL_ISO).EQ.0) THEN
        
C Now look for another tight CEM Electron
c ---------------------------------------
            DO J=1,NUM_HPTE
               NDIL = NDIL + 1
               DIL_TYPE(1,NDIL) = HPTM_HPTE
               DIL_TYPE(2,NDIL) = I
               DIL_TYPE(3,NDIL) = J
            ENDDO

C Look for a loose central electron
c ---------------------------------
            DO J=1,NUM_LOOSE_E
                  NDIL = NDIL + 1
                  DIL_TYPE(1,NDIL) = HPTM_LCEM
                  DIL_TYPE(2,NDIL) = I
                  DIL_TYPE(3,NDIL) = J
            ENDDO

C Look for a loose plug
c ---------------------
            DO J=1,NUM_PEM
                 NDIL = NDIL + 1
                 DIL_TYPE(1,NDIL) = HPTM_PEM
                 DIL_TYPE(2,NDIL) = I
                 DIL_TYPE(3,NDIL) = J
            ENDDO

c Look for a CMU/CMX muon
c -----------------------
            DO J=1,NUM_HPTM
              IF(IND_HPTM(I).NE.IND_HPTM(J)) THEN
                IF(IAND(MUO_WRD(IND_HPTM(J)),1).EQ.0) THEN
                  NDIL = NDIL + 1
                  DIL_TYPE(1,NDIL) = HPTM_CMU
                  DIL_TYPE(2,NDIL) = I
                  DIL_TYPE(3,NDIL) = J
                ELSEIF(IAND(MUO_WRD(IND_HPTM(J)),1).EQ.1) THEN
                  NDIL = NDIL + 1
                  DIL_TYPE(1,NDIL) = HPTM_CMX
                  DIL_TYPE(2,NDIL) = I
                  DIL_TYPE(3,NDIL) = J
                ENDIF
              ENDIF
            ENDDO

c Look for a CMIO as a partner
c ----------------------------
            DO J=1,NUM_CMIO
                 NDIL = NDIL + 1
                 DIL_TYPE(1,NDIL) = HPTM_CMIO
                 DIL_TYPE(2,NDIL) = I
                 DIL_TYPE(3,NDIL) = J
            ENDDO


        ENDIF  !If the lepton is isolated enough for a dilepton candidate
      
c Check to see if this lepton is good enough for lepton plus jets analysis
c ------------------------------------------------------------------------
        IF(IAND(MUO_ISO(IND_HPTM(I)),SGL_ISO).EQ.0) THEN
            NSGL = NSGL + 1
            IF(IAND(MUO_WRD(IND_HPTM(I)),1).EQ.1) SGL_TYPE(1) = HPTMX
            IF(IAND(MUO_WRD(IND_HPTM(I)),1).EQ.0) SGL_TYPE(1) = HPTMU
            SGL_TYPE(2) = I
        ENDIF


      ENDDO  


c Eliminate Duplicate Dilepton Combinations
c -----------------------------------------
      IF(NDIL.GE.2) THEN

         DO I=1,10
            DUP_DIL(I) = .FALSE.
         ENDDO

C Eliminate Same typed pairs
c --------------------------
         DO I=1,NDIL-1
            DO J=I+1,NDIL

C Eliminate Same typed pairs
c --------------------------
               IF(DIL_TYPE(1,I).EQ.DIL_TYPE(1,J).AND.
     &            DIL_TYPE(2,I).EQ.DIL_TYPE(3,J).AND.
     &            DIL_TYPE(3,I).EQ.DIL_TYPE(2,J)) DUP_DIL(J) = .TRUE.

C Now eliminate similar but not identically classified pairs
C Start with tight e - loose e
c ----------------------------------------------------------
               IF(DIL_TYPE(1,I).EQ.1.AND.(DIL_TYPE(1,J).EQ.2).AND.
     &            DIL_TYPE(2,I).EQ.DIL_TYPE(3,J).AND.
     &            DIL_TYPE(3,I).EQ.DIL_TYPE(2,J)) DUP_DIL(J) = .TRUE.

               IF(DIL_TYPE(1,I).EQ.2.AND.(DIL_TYPE(1,J).EQ.1).AND.
     &            DIL_TYPE(2,I).EQ.DIL_TYPE(3,J).AND.
     &            DIL_TYPE(3,I).EQ.DIL_TYPE(2,J)) DUP_DIL(J) = .TRUE.

c Now e-mu versus mu-e
c -------------------
               IF( (DIL_TYPE(1,I).EQ.4.OR.DIL_TYPE(1,I).EQ.5).AND.
     &             (DIL_TYPE(1,J).EQ.7.OR.DIL_TYPE(1,J).EQ.8).AND.
     &            DIL_TYPE(2,I).EQ.DIL_TYPE(3,J).AND.
     &            DIL_TYPE(3,I).EQ.DIL_TYPE(2,J)) DUP_DIL(J) = .TRUE.

               IF( (DIL_TYPE(1,I).EQ.7.OR.DIL_TYPE(1,I).EQ.8).AND.
     &             (DIL_TYPE(1,J).EQ.4.OR.DIL_TYPE(1,J).EQ.5).AND.
     &            DIL_TYPE(2,I).EQ.DIL_TYPE(3,J).AND.
     &            DIL_TYPE(3,I).EQ.DIL_TYPE(2,J)) DUP_DIL(J) = .TRUE.

c Now mu-tight e versus mu-loose e
c --------------------------------
               IF(DIL_TYPE(1,I).EQ.7.AND.(DIL_TYPE(1,J).EQ.8).AND.
     &            DIL_TYPE(2,I).EQ.DIL_TYPE(3,J).AND.
     &            DIL_TYPE(3,I).EQ.DIL_TYPE(2,J)) DUP_DIL(J) = .TRUE.

               IF(DIL_TYPE(1,I).EQ.8.AND.(DIL_TYPE(1,J).EQ.7).AND.
     &            DIL_TYPE(2,I).EQ.DIL_TYPE(3,J).AND.
     &            DIL_TYPE(3,I).EQ.DIL_TYPE(2,J)) DUP_DIL(J) = .TRUE.

c Now cmu-cmx versus cmx-cmu
c --------------------------
               IF(DIL_TYPE(1,I).EQ.10.AND.(DIL_TYPE(1,J).EQ.11).AND.
     &            DIL_TYPE(2,I).EQ.DIL_TYPE(3,J).AND.
     &            DIL_TYPE(3,I).EQ.DIL_TYPE(2,J)) DUP_DIL(J) = .TRUE.

               IF(DIL_TYPE(1,I).EQ.11.AND.(DIL_TYPE(1,J).EQ.10).AND.
     &            DIL_TYPE(2,I).EQ.DIL_TYPE(3,J).AND.
     &            DIL_TYPE(3,I).EQ.DIL_TYPE(2,J)) DUP_DIL(J) = .TRUE.

            ENDDO
         ENDDO

c Now remove the duplicates (First one cannot be a duplicate)
c -----------------------------------------------------------
         DO I=2,NDIL
            IF(DUP_DIL(I)) THEN
               DO J=I,NDIL-1
                  DIL_TYPE(1,J) = DIL_TYPE(1,J+1)
                  DIL_TYPE(2,J) = DIL_TYPE(2,J+1)
                  DIL_TYPE(3,J) = DIL_TYPE(3,J+1)
                  DUP_DIL(J)    = DUP_DIL(J+1)
               ENDDO
               NDIL = NDIL - 1
            ENDIF
         ENDDO

      ENDIF



C General classification of event
c -------------------------------
      IF(NDIL.GT.0) THEN
            TWO_LEPTON = .TRUE.
      ELSEIF(NSGL.EQ.1) THEN
            ONE_LEPTON = .TRUE.
      ELSEIF(NSGL.GT.1) THEN
            PRINT *,'WARNING: THIS SHOULD BE A TWO LEPTON EVT'
      ENDIF
      

C Now Fill other Event Information
c ================================
      NJET_10 = 0
      NJET_15 = 0
      NJET_20 = 0
      NJET_25 = 0
      IF(MET_JET_FLAG) THEN

c Fill Jet Information
c --------------------
         CALL JETFND

c Get corrected MET
c -----------------
         CALL GET_CORR_MET(CMU_MUON,MIO_MUON_MET,MET,METPHI,IERR)
         IF(IERR.EQ.1) 
     &       CALL ERLOGR('TOPFND',ERWARN,0,IERR,
     &        'TOPFND--NO METS BANK')
      

      ENDIF

C Get Trigger Information
c -----------------------
      GOOD_TRIG = .FALSE.
      TRWRD = 0
      CALL VZERO(MU_TRWRD,MAXCMUO)
      IF(TRG_FLAG) THEN
         IERR=0
         IF(LMONTE) THEN
           CALL SIM_MUTRIG(TRWRD,ANY_MUTRIG)
	 ELSE
           CALL GET_TRIG_INFO(TRWRD,IERR)
	 ENDIF

         IF(IERR.EQ.0) THEN

c For the Muons get the triggering information for each muon
c ----------------------------------------------------------
             DO I=1,min(maxcmuo,NUMCMUO)

                CALL MU_TRIG_MATCH(BNK_CMUO(I),TRWRD,MU_TRWRD(I),IERR)
                IF(IERR.EQ.1.OR.IERR.EQ.78.OR.IERR.EQ.999) THEN
                   CALL ERLOGR('TOPFND',ERWARN,0,IERR,
     &               'TOPFND--PROBLEM WITH MU_TRIG_MATCH.')
                ENDIF
      
             ENDDO

         ELSEIF(IERR.EQ.-2) THEN

             CALL ERLOGR('TOPFND',ERWARN,0,STATUS,
     &                   'TOPFND--Unable to locate TAGC bank.')

         ENDIF

c Now set the trigger flag
c ------------------------
         IF( ( (ONE_LEPTON.AND.SGL_TYPE(1).EQ.HPTE).OR.
     &         (TWO_LEPTON.AND.DIL_TYPE(1,1).LE.6) ).AND.
     &          IAND(TRWRD,HPTE_TRIG_MASK).NE.0) GOOD_TRIG = .TRUE.

C Trigger flag for the muons...this ain't easy.
c ---------------------------------------------
         DO I=1,NUM_HPTM
            PRIM_DIL = .FALSE.
            DO J=1,NDIL
               IF(DIL_TYPE(1,J).GE.7.AND.DIL_TYPE(2,J).EQ.I) 
     &            PRIM_DIL = .TRUE.
            ENDDO
            IF( (TWO_LEPTON.AND.PRIM_DIL).OR.
     &          (ONE_LEPTON.AND.SGL_TYPE(1).NE.HPTE.AND.
     &           SGL_TYPE(2).EQ.I) )THEN

C If it is a CMU check that it triggers the CMU
c ---------------------------------------------
                IF(IAND(DET_CMUO(IND_HPTM(I)),1).NE.0) THEN
                   IF(IAND(MU_TRWRD(IND_HPTM(I)),HPT_CMU_TRIG_MASK)
     &                .NE.0) GOOD_TRIG = .TRUE.    
                ENDIF

C If it is a CMP Only then it MUST come in on a MET Trigger
c ---------------------------------------------------------
                IF(IAND(DET_CMUO(IND_HPTM(I)),2).NE.0.AND.
     &             IAND(DET_CMUO(IND_HPTM(I)),1).EQ.0) THEN
                   IF(IAND(MU_TRWRD(IND_HPTM(I)),HPT_CMP_TRIG_MASK)
     &                .NE.0) GOOD_TRIG = .TRUE.    
                ENDIF

C If it is a CMX check that it triggers the CMX
c ---------------------------------------------
                IF(IAND(DET_CMUO(IND_HPTM(I)),4).NE.0) THEN
                   IF(IAND(MU_TRWRD(IND_HPTM(I)),HPT_CMX_TRIG_MASK)
     &                .NE.0) GOOD_TRIG = .TRUE.    
                ENDIF
            ENDIF
         ENDDO
      ENDIF


C Get Bad Run Information
c -----------------------
      GOOD_RUN = .FALSE.
      IF(BR_FLAG) THEN
         STREAM = 2
         CALL BADRUN(NRUN,STREAM,BR_CODE)

C Set BADRUN flag 
c ---------------
         IF(BR_CODE.EQ.15.OR.BR_CODE.EQ.-1) GOOD_RUN = .TRUE.
      ENDIF


c Check to see if this event is a Z
c ---------------------------------
      Z_EVENT = .FALSE.
      CALL ELEZEE(CEM_ELE,CONV_CUT)
      CALL MUOZEE(CMU_MUON)
      Z_EVENT = LZFLAG

C Apply the Topology Cuts for the dilepton events
c -----------------------------------------------
      IF(TWO_LEPTON) THEN
         DO KK=1,NDIL
            CALL DIL_TOPOLOGY(KK,ANSWER)
            IF(ANSWER)GOTO 999
         ENDDO

999      IF(ANSWER .AND. .NOT.Z_EVENT)DIL_CAND=.TRUE.
      ELSE
         DIL_WRD = IBSET(DIL_WRD,0)
      ENDIF

c Check to see if TOPS bank exists, if not create it
c --------------------------------------------------
      STATUS=BLOCAT(IW,'TOPS',-1,JTOPS,JDTOPS)
      IF(STATUS.NE.YESUCC) CALL TOPSCR

C Now unpack the TOPS bank
c ------------------------
      CALL TOPSBTC(-1)      


c Au Revoir Gopher
c ----------------

      RETURN
      END      

