      SUBROUTINE TOPSCR
C =====================================================================
C    This routine creates and fills the TOPS bank
C    containing information about top event selection.  
c    The information is taken from the include files 
c    filled by TOPFND and its subroutines.
C
C    Brian Winer             Aug 23, 1995
c
c ======================================================================
c       TOPS Bank Structure:
c
c            Header Words:
c
c(I)      0) Number of Words in Header
c(I)      1) Version Number (100)
c(I)      2) Code Word (bit 0: If set, jets were reclustered after removing the
c                             lepton.
c                     (bit 1: If set, jets were counted after correcting the
c                             jet Et for relative detector response.
c
c(R)      3) MET
c(R)      4) MET phi
c(R)      5) HT
c(I)      6) Badrun Flag
c(I)      7) Trigger Flag (For whole event)
c(I)      8) Z Flag
c(I)      9) Dilepton Candidate
c
c(I)     10) Number of Electrons (isolated or not)
c(I)     11) Number of Electrons (isolated)
c(I)     12) Number of Electrons (isolated .AND. pass trigger req; NOTE:
c            if a MET trigger is satisfied then all leptons (of the proper
c            type) are classified as passing the trigger.)
c
c(I)     13) Number of Muons     (isolated or not)
c(I)     14) Number of Muons     (isolated)
c(I)     15) Number of Muons     (isolated .AND. pass trigger req; NOTE:
c            if a MET trigger is satisfied then all leptons (of the proper
c            type) are classified as passing the trigger.)
c
c        Only good leptons are counted (Et > 20 and all quality ID cuts).
c        The lepton information is stored as follows:
c            a) Highest Et Lepton which is isolated and passes trigger,
c               then next highest Et of this type.
c            b) Highest Et Lepton which is isolated but doesn't trigger,
c               then next highest Et of this type.
c            c) Highest Et Lepton which is nonisolated,
c               then next highest Et of this type.
c         Only two Leptons are stored; Other are counted but not stored
c         in this bank.
c
c(R)          16) Pt of lepton 1
c(R)          17) Px of Lepton 1
c(R)          18) Py of Lepton 1
c(R)          19) Pz of Lepton 1
c(R)          20) E  of Lepton 1
c(R)          21) Cone Isolation lepton 1
c(I)          22) Type of lepton 1
c(I)          23) Lepton Array pointer to lepton 1
c(I)          24) Bank number of lepton 1 (ELES bank# or CMUO bank #)
c(I)          25) Bank number of JETS  for ele or TRKS for cmu  for lepton 1
c(I)          26) Trigger Flag for lepton 1
c               
c(R)          27) Pt of lepton 2
c(R)          28) Px of Lepton 2
c(R)          29) Py of Lepton 2
c(R)          30) Pz of Lepton 2
c(R)          31) E  of Lepton 2
c(R)          32) Cone Isolation lepton 2
c(I)          33) Type of lepton 2
c(I)          34) Lepton Array pointer to lepton 2
c(I)          35) Bank number of lepton 2 (ELES bank# or CMUO bank #)
c(I)          36) Bank number of JETS  for ele or TRKS for cmu  for lepton 2
c(I)          37) Trigger Flag for lepton 2
c
c(I)     38) Number of jets with Et>15 and eta<2.0
c(I)          39) JETS Bank # for jet 1
c(I)          40) JETS Bank # for jet 2
c(I)          41) JETS Bank # for jet 3
c(I)          42) JETS Bank # for jet 4
c(I)          43) JETS Bank # for jet 5
c(I)          44) JETS Bank # for jet 6
c(I)          45) JETS Bank # for jet 7
c(I)          46) JETS Bank # for jet 8
c(I)          47) JETS Bank # for jet 9
c(I)          48) JETS Bank # for jet 10
c(I)     49) Number of jets with Et>8  and eta<2.4 and .NOT. Et>15 and eta<2.0
c(I)          50) JETS Bank # for jet 1
c(I)          51) JETS Bank # for jet 2
c(I)          52) JETS Bank # for jet 3
c(I)          53) JETS Bank # for jet 4
c(I)          54) JETS Bank # for jet 5
C
c(I)     55) Jet Mult for Single Lepton Event      
c(I)     56) Jet Mult for Dilepton Lepton Event      
c(I)     57) Jet Mult for Z Event
C
c
c ===============================================================================
C
C
$$IMPLICIT
C
C    Global declarations.
$$INCLUDE 'C$INC:JOBSTA.INC'
$$INCLUDE 'C$INC:ERLEVL.INC'
$$INCLUDE 'YBOS$LIBRARY:ERRCOD.INC'
$$INCLUDE 'YBOS$LIBRARY:BNKTYP.INC'
$$INCLUDE 'C$INC:BCS.INC'
$$INCLUDE 'C$INC:ELEINFO.INC'
$$INCLUDE 'C$INC:MUOINFO.INC'
$$INCLUDE 'C$INC:MIOINFO.INC'
$$INCLUDE 'C$INC:JETINFO.INC'
$$INCLUDE 'C$INC:EVTINFO.INC'
$$INCLUDE 'C$INC:DILINFO.INC'
$$INCLUDE 'top$fnd:DILMORE.INC'
$$INCLUDE 'top$fnd:TFZINFO.INC'
$$INCLUDE 'top$fnd:DSTOPS.INC'
c
C
C Local Variables
C
      INTEGER I,J
      INTEGER BR_CODE
      LOGICAL GOOD_MU_TRIG(MAXCMUO),GOOD_E_TRIG
      INTEGER HPTE_TRIG_MASK/'5'X/
      INTEGER HPT_CMU_TRIG_MASK/'FF8'X/
      INTEGER HPT_CMP_TRIG_MASK/'18'X/
      INTEGER HPT_CMX_TRIG_MASK/'F018'X/
      INTEGER FIRST_LEP,SECOND_LEP,FIRST_CL,SECOND_CL
      REAL FIRST_ET,SECOND_ET
      REAL ZDIFF
      INTEGER ZJET
      REAL PX_M,PY_M,PZ_M,PTOT,MAX_CTH,COS_TH
      INTEGER MJET
      INTEGER ZLEG_1,ZLEG_2,ZPNT
      INTEGER PEM
      LOGICAL CMIO
      REAL    LEP_EN,JET_EN,MET_EN,ET_JET_COR,HT

      INTEGER NUM_E,NUM_IE,NUM_ITE
      INTEGER NUM_M,NUM_IM,NUM_ITM
      INTEGER NJ15,NJ8
      INTEGER HPTE/20/,HPTM/30/
      INTEGER JTOPS,JDTOPS,BNKTYP(9),STATUS

C
C    External declarations.
C
      INTEGER BLOCAT,BTPACK,BMAKE,GTRGNU,BNUMB,BNEXT,BDATA,BDROP
C
C    First, make the TOPS bank.
C
      STATUS=BTPACK(BNKTMX,0,7,BNKTYP(1))
      STATUS=BTPACK(BNKTI4,0,TSINT1,BNKTYP(2))
      STATUS=BTPACK(BNKTR4,0,TSREA1,BNKTYP(3))
      STATUS=BTPACK(BNKTI4,0,TSINT2,BNKTYP(4))
      STATUS=BTPACK(BNKTR4,0,TSREA2,BNKTYP(5))
      STATUS=BTPACK(BNKTI4,0,TSINT3,BNKTYP(6))
      STATUS=BTPACK(BNKTR4,0,TSREA3,BNKTYP(7))
      STATUS=BTPACK(BNKTI4,0,TSINT4,BNKTYP(8))
c      STATUS=BTPACK(BNKTR4,0,TSREA4,BNKTYP(9))
      LNTOPS=TSINT1+TSINT2+TSINT3+TSINT4+TSREA1+TSREA2+TSREA3
C
C
C
      STATUS=BDROP(IW,'TOPS')
      STATUS=BMAKE(IW,'TOPS',1,LNTOPS,BNKTYP,JTOPS,JDTOPS)
      IF(STATUS.NE.YESUCC) THEN
          CALL ERLOGR('TOPSCR',ERWARN,0,STATUS,
     +                'Unable to make TOPS bank.')
          RETURN
      ENDIF
         
c Fill the information (Global Event Info first)
c ----------------------------------------------
      IW(JDTOPS+0)   = TSHEAD
      IW(JDTOPS+1)   = VERSION
      IW(JDTOPS+2)   = 0
      RW(JDTOPS+3)   = MET(4)
      RW(JDTOPS+4)   = METPHI(4)
      CALL BADRUN(NRUN,2,BR_CODE)
      IW(JDTOPS+6)   = BR_CODE
      IF(LZFLAG) IW(JDTOPS+8) = 1
      IF(TZFLAG) IW(JDTOPS+8) = 2
      IF (DIL_CAND) THEN
       IW(JDTOPS+9)=1
      ELSE
       IW(JDTOPS+9)=0
      ENDIF


C Get some trigger information for sorting of the leptons
c -------------------------------------------------------
      GOOD_E_TRIG = .FALSE.
      IF(IAND(TRWRD,HPTE_TRIG_MASK).NE.0) GOOD_E_TRIG = .TRUE.

      DO I=1,NUM_HPTM
         GOOD_MU_TRIG(IND_HPTM(I)) = .FALSE.

C If it is a CMU check that it triggers the CMU
c ---------------------------------------------
         IF(IAND(DET_CMUO(IND_HPTM(I)),1).NE.0) THEN
            IF(IAND(MU_TRWRD(IND_HPTM(I)),HPT_CMU_TRIG_MASK).NE.0) 
     &            GOOD_MU_TRIG(IND_HPTM(I)) = .TRUE.    
         ENDIF

C If it is a CMP Only then it MUST come in on a MET Trigger
c ---------------------------------------------------------
         IF(IAND(DET_CMUO(IND_HPTM(I)),2).NE.0.AND.
     &      IAND(DET_CMUO(IND_HPTM(I)),1).EQ.0) THEN
            IF(IAND(MU_TRWRD(IND_HPTM(I)),HPT_CMP_TRIG_MASK).NE.0) 
     &            GOOD_MU_TRIG(IND_HPTM(I)) = .TRUE.    
         ENDIF

C If it is a CMX check that it triggers the CMX
c ---------------------------------------------
         IF(IAND(DET_CMUO(IND_HPTM(I)),4).NE.0) THEN
            IF(IAND(MU_TRWRD(IND_HPTM(I)),HPT_CMX_TRIG_MASK).NE.0) 
     &            GOOD_MU_TRIG(IND_HPTM(I)) = .TRUE.    
         ENDIF
      ENDDO

C Sort out the leptons, put them in order of Et for isolated triggered
c leptons, isolated nontrigger leptons, and nonisolated leptons.
c 
C First sort electrons by trigger & isolation
c -------------------------------------------
      FIRST_LEP = 0
      FIRST_ET  = 0
      FIRST_CL  = 0
      SECOND_LEP = 0
      SECOND_ET  = 0
      SECOND_CL  = 0

      NUM_E = 0
      NUM_IE = 0
      NUM_ITE = 0
      DO I=1,NUM_HPTE
         IF(IAND(CEM_ISO(IND_HPTE(I)),1).EQ.0) THEN
            IF(GOOD_E_TRIG) THEN
               NUM_ITE = NUM_ITE + 1
               IF(ET_ELE(IND_HPTE(I)).GT.FIRST_ET.OR.FIRST_CL.LT.3) THEN
                  SECOND_LEP = FIRST_LEP
                  SECOND_ET  = FIRST_ET
                  SECOND_CL  = FIRST_CL
                  FIRST_LEP  = IND_HPTE(I)
                  FIRST_ET   = ET_ELE(FIRST_LEP)
                  FIRST_CL   = 3
               ELSEIF(ET_ELE(IND_HPTE(I)).GT.SECOND_ET.OR.
     &                SECOND_CL.LT.3) THEN
                  SECOND_LEP = IND_HPTE(I)
                  SECOND_ET  = ET_ELE(SECOND_LEP)
                  SECOND_CL  = 3
               ENDIF

            ELSE
               NUM_IE = NUM_IE + 1
               IF((ET_ELE(IND_HPTE(I)).GT.FIRST_ET.AND.FIRST_CL.EQ.2)
     &            .OR.(FIRST_CL.LT.2)) THEN
                  SECOND_LEP = FIRST_LEP
                  SECOND_ET  = FIRST_ET
                  SECOND_CL  = FIRST_CL
                  FIRST_LEP  = IND_HPTE(I)
                  FIRST_ET   = ET_ELE(FIRST_LEP)
                  FIRST_CL   = 2
               ELSEIF((ET_ELE(IND_HPTE(I)).GT.SECOND_ET.AND.
     &            SECOND_CL.EQ.2).OR.(SECOND_CL.LT.2)) THEN
                  SECOND_LEP = IND_HPTE(I)
                  SECOND_ET  = ET_ELE(SECOND_LEP)
                  SECOND_CL  = 2
               ENDIF
            ENDIF

         ELSE
            NUM_E = NUM_E + 1
            IF(ET_ELE(IND_HPTE(I)).GT.FIRST_ET.AND.FIRST_CL.LE.1) THEN
                  SECOND_LEP = FIRST_LEP
                  SECOND_ET  = FIRST_ET
                  SECOND_CL  = FIRST_CL
                  FIRST_LEP  = IND_HPTE(I)
                  FIRST_ET   = ET_ELE(FIRST_LEP)
                  FIRST_CL   = 1
            ELSEIF(ET_ELE(IND_HPTE(I)).GT.SECOND_ET.AND.SECOND_CL.LE.1) 
     &            THEN
                  SECOND_LEP = IND_HPTE(I)
                  SECOND_ET  = ET_ELE(SECOND_LEP)
                  SECOND_CL  = 1
            ENDIF
         ENDIF
      ENDDO

C First sort Muons  by trigger & isolation
c ---------------------------------------- 
      NUM_M = 0
      NUM_IM = 0
      NUM_ITM = 0
      DO I=1,NUM_HPTM
         IF(IAND(MUO_ISO(IND_HPTM(I)),1).EQ.0) THEN
            IF(GOOD_MU_TRIG(IND_HPTM(I))) THEN
               NUM_ITM = NUM_ITM + 1
               IF(PT_CMUO(IND_HPTM(I)).GT.FIRST_ET.OR.FIRST_CL.LT.3)
     &            THEN
                  SECOND_LEP = FIRST_LEP
                  SECOND_ET  = FIRST_ET
                  SECOND_CL  = FIRST_CL
                  FIRST_LEP  = -IND_HPTM(I)
                  FIRST_ET   = PT_CMUO(-FIRST_LEP)
                  FIRST_CL   = 3
               ELSEIF(PT_CMUO(IND_HPTM(I)).GT.SECOND_ET.OR.
     &            SECOND_CL.LT.3) THEN
                  SECOND_LEP = -IND_HPTM(I)
                  SECOND_ET  = PT_CMUO(-SECOND_LEP)
                  SECOND_CL  = 3
               ENDIF

            ELSE
               NUM_IM = NUM_IM + 1
               IF((PT_CMUO(IND_HPTM(I)).GT.FIRST_ET.AND.FIRST_CL.EQ.2)
     &            .OR.(FIRST_CL.LT.2)) THEN
                  SECOND_LEP = FIRST_LEP
                  SECOND_ET  = FIRST_ET
                  SECOND_CL  = FIRST_CL
                  FIRST_LEP  = -IND_HPTM(I)
                  FIRST_ET   = PT_CMUO(-FIRST_LEP)
                  FIRST_CL   = 2
               ELSEIF((PT_CMUO(IND_HPTM(I)).GT.SECOND_ET.AND.
     &            SECOND_CL.EQ.2).OR.(SECOND_CL.LT.2)) THEN
                  SECOND_LEP = -IND_HPTM(I)
                  SECOND_ET  = PT_CMUO(-SECOND_LEP)
                  SECOND_CL  = 2
               ENDIF
            ENDIF

         ELSE
            NUM_M = NUM_M + 1
            IF(PT_CMUO(IND_HPTM(I)).GT.FIRST_ET.AND.FIRST_CL.LE.1) THEN
                  SECOND_LEP = FIRST_LEP
                  SECOND_ET  = FIRST_ET
                  SECOND_CL  = FIRST_CL
                  FIRST_LEP  = -IND_HPTM(I)
                  FIRST_ET   = PT_CMUO(-FIRST_LEP)
                  FIRST_CL   = 1
            ELSEIF(PT_CMUO(IND_HPTM(I)).GT.SECOND_ET.AND.SECOND_CL.LE.1)
     &            THEN
                  SECOND_LEP = -IND_HPTM(I)
                  SECOND_ET  = PT_CMUO(-SECOND_LEP)
                  SECOND_CL  = 1
            ENDIF
         ENDIF
      ENDDO

C Store the number and type of various leptons
c --------------------------------------------
      IW(JDTOPS+10) = NUM_HPTE      
      IW(JDTOPS+11) = NUM_IE+NUM_ITE      
      IW(JDTOPS+12) = NUM_ITE      
      IW(JDTOPS+13) = NUM_HPTM      
      IW(JDTOPS+14) = NUM_IM+NUM_ITM
      IW(JDTOPS+15) = NUM_ITM      

c Force Dilepton Events to fill both leptons in this common block...
c -----------------------------------------------------------------
        PEM=0
        CMIO=.FALSE.
      IF(DIL_CAND)THEN
c  ee
        IF(DIL_MODE.EQ.1)THEN
           FIRST_LEP  = ELE1_PTR
           IF(ELE2_PTR.LT.0)THEN
             SECOND_LEP = -ELE2_PTR
             PEM=1
           ELSE
             SECOND_LEP =  ELE2_PTR
           ENDIF
        ENDIF
c  emu
        IF(DIL_MODE.EQ.2)THEN
           IF(ELE1_PTR.LT.0)THEN
             FIRST_LEP  = -ELE1_PTR
             PEM=1
           ELSE
             FIRST_LEP  = ELE1_PTR
           ENDIF
           IF(MUO1_PTR.LT.0.)THEN
             SECOND_LEP =MUO1_PTR
             CMIO=.TRUE.
           ELSE
            SECOND_LEP =-MUO1_PTR
           ENDIF
        ENDIF
c  mumu
        IF(DIL_MODE.EQ.3)THEN
           FIRST_LEP  =-MUO1_PTR
           IF(MUO2_PTR.LT.0.)THEN
             SECOND_LEP =MUO2_PTR
             CMIO=.TRUE.
           ELSE
            SECOND_LEP =-MUO2_PTR
           ENDIF
        ENDIF
c  now deal with cmio -- LEP TYPE = 40
        IF(CMIO)THEN
             RW(JDTOPS+OFFSETL2+0)  = PT_CMIO(-SECOND_LEP)
             RW(JDTOPS+OFFSETL2+1)  = P4M_CMIO(1,-SECOND_LEP)
             RW(JDTOPS+OFFSETL2+2)  = P4M_CMIO(2,-SECOND_LEP)
             RW(JDTOPS+OFFSETL2+3)  = P4M_CMIO(3,-SECOND_LEP)
             RW(JDTOPS+OFFSETL2+4)  = P4M_CMIO(4,-SECOND_LEP)
             RW(JDTOPS+OFFSETL2+5)  = ISOL_CMIO(-SECOND_LEP)
             IW(JDTOPS+OFFSETL2+6)  = 40
             IW(JDTOPS+OFFSETL2+7)  = -SECOND_LEP
             IW(JDTOPS+OFFSETL2+8)  = BNK_CMIO(-SECOND_LEP)
             IW(JDTOPS+OFFSETL2+9)  = TRKS_CMIO(-SECOND_LEP)
             IF (GOOD_MU_TRIG(-SECOND_LEP)) THEN
               IW(JDTOPS+OFFSETL2+10) = 1
             ELSE
               IW(JDTOPS+OFFSETL2+10) = 0
             ENDIF 
             SECOND_LEP=0
	ENDIF
      ENDIF


C Now fill the PRIMARY Lepton Information
c ---------------------------------------
      IF(FIRST_LEP.GT.0) THEN
         RW(JDTOPS+OFFSETL+0)  = ET_ELE(FIRST_LEP)
         RW(JDTOPS+OFFSETL+1)  = P4E_ELE(1,FIRST_LEP)
         RW(JDTOPS+OFFSETL+2)  = P4E_ELE(2,FIRST_LEP)
         RW(JDTOPS+OFFSETL+3)  = P4E_ELE(3,FIRST_LEP)
         RW(JDTOPS+OFFSETL+4)  = P4E_ELE(4,FIRST_LEP)
         RW(JDTOPS+OFFSETL+5)  = ISOL_ELE(FIRST_LEP)
         IW(JDTOPS+OFFSETL+6)  = HPTE+PEM
         IW(JDTOPS+OFFSETL+7)  = FIRST_LEP
         IW(JDTOPS+OFFSETL+8)  = BNK_ELE(FIRST_LEP)
         IW(JDTOPS+OFFSETL+9)  = JTS_ELE(FIRST_LEP)
         IF (GOOD_E_TRIG) THEN
            IW(JDTOPS+OFFSETL+10) = 1
         ELSE
            IW(JDTOPS+OFFSETL+10) = 0
         ENDIF

c Fill the event trigger flag with the primary's Flag
c ---------------------------------------------------
          IF (GOOD_E_TRIG) THEN
            IW(JDTOPS+7)=1
          ELSE
            IW(JDTOPS+7)=0
          ENDIF

      ELSEIF(FIRST_LEP.LT.0) THEN
         RW(JDTOPS+OFFSETL+0)  = PT_CMUO(-FIRST_LEP)
         RW(JDTOPS+OFFSETL+1)  = P4M_CMUO(1,-FIRST_LEP)
         RW(JDTOPS+OFFSETL+2)  = P4M_CMUO(2,-FIRST_LEP)
         RW(JDTOPS+OFFSETL+3)  = P4M_CMUO(3,-FIRST_LEP)
         RW(JDTOPS+OFFSETL+4)  = P4M_CMUO(4,-FIRST_LEP)
         RW(JDTOPS+OFFSETL+5)  = ISOL_CMUO(-FIRST_LEP)
         IW(JDTOPS+OFFSETL+6)  = HPTM + DET_CMUO(-FIRST_LEP)
         IW(JDTOPS+OFFSETL+7)  = -FIRST_LEP
         IW(JDTOPS+OFFSETL+8)  = BNK_CMUO(-FIRST_LEP)
         IW(JDTOPS+OFFSETL+9)  = TRKS_CMUO(-FIRST_LEP)

         IF (GOOD_MU_TRIG(-FIRST_LEP)) THEN
           IW(JDTOPS+OFFSETL+10)=1
         ELSE
           IW(JDTOPS+OFFSETL+10)=0
         ENDIF

c Fill the event trigger flag with the primary's Flag
c ---------------------------------------------------
         IF (GOOD_MU_TRIG(-FIRST_LEP)) THEN
           IW(JDTOPS+7)=1
         ELSE
           IW(JDTOPS+7)=0
         ENDIF
      ENDIF


C Now fill the SECONDARY Lepton Information
c -----------------------------------------
      IF(SECOND_LEP.GT.0) THEN
         RW(JDTOPS+OFFSETL2+0)  = ET_ELE(SECOND_LEP)
         RW(JDTOPS+OFFSETL2+1)  = P4E_ELE(1,SECOND_LEP)
         RW(JDTOPS+OFFSETL2+2)  = P4E_ELE(2,SECOND_LEP)
         RW(JDTOPS+OFFSETL2+3)  = P4E_ELE(3,SECOND_LEP)
         RW(JDTOPS+OFFSETL2+4)  = P4E_ELE(4,SECOND_LEP)
         RW(JDTOPS+OFFSETL2+5)  = ISOL_ELE(SECOND_LEP)
         IW(JDTOPS+OFFSETL2+6)  = HPTE+PEM
         IW(JDTOPS+OFFSETL2+7)  = SECOND_LEP
         IW(JDTOPS+OFFSETL2+8)  = BNK_ELE(SECOND_LEP)
         IW(JDTOPS+OFFSETL2+9)  = JTS_ELE(SECOND_LEP)
         IF (GOOD_E_TRIG) THEN
            IW(JDTOPS+OFFSETL2+10)=1
         ELSE
            IW(JDTOPS+OFFSETL2+10)=0
         ENDIF
      ELSEIF(SECOND_LEP.LT.0) THEN
         RW(JDTOPS+OFFSETL2+0)  = PT_CMUO(-SECOND_LEP)
         RW(JDTOPS+OFFSETL2+1)  = P4M_CMUO(1,-SECOND_LEP)
         RW(JDTOPS+OFFSETL2+2)  = P4M_CMUO(2,-SECOND_LEP)
         RW(JDTOPS+OFFSETL2+3)  = P4M_CMUO(3,-SECOND_LEP)
         RW(JDTOPS+OFFSETL2+4)  = P4M_CMUO(4,-SECOND_LEP)
         RW(JDTOPS+OFFSETL2+5)  = ISOL_CMUO(-SECOND_LEP)
         IW(JDTOPS+OFFSETL2+6)  = HPTM + DET_CMUO(-SECOND_LEP)
         IW(JDTOPS+OFFSETL2+7)  = -SECOND_LEP
         IW(JDTOPS+OFFSETL2+8)  = BNK_CMUO(-SECOND_LEP)
         IW(JDTOPS+OFFSETL2+9)  = TRKS_CMUO(-SECOND_LEP)
         IF (GOOD_MU_TRIG(-SECOND_LEP)) THEN
            IW(JDTOPS+OFFSETL2+10)=1
         ELSE
            IW(JDTOPS+OFFSETL2+10)=0
         ENDIF
      ENDIF


c Store All Jets which pass conditions (These can be the electron jet)
c --------------------------------------------------------------------
      NJ15 = 0
      NJ8 = 0
      DO I=1,ICLUS
        IF(IJET_ET(I).GT.15.AND.ABS(IJET_DETA(I)).LT.2.0
     &    .AND.NJ15.LT.10) THEN

             NJ15 = NJ15 + 1
             IW(JDTOPS+OFFSET15+NJ15) = IJETBNK(I)

        ELSEIF(IJET_ET(I).GT.8.AND.ABS(IJET_DETA(I)).LT.2.4
     &         .AND.NJ8.LT.5) THEN

             NJ8 = NJ8 + 1
             IW(JDTOPS+OFFSET8+NJ8) = IJETBNK(I)

        ENDIF
      ENDDO
      IW(JDTOPS+OFFSET15) = NJ15
      IW(JDTOPS+OFFSET8)  = NJ8

c Now try to count the jets for the different analysis cases
c ----------------------------------------------------------
      IF(FIRST_LEP.GT.0) THEN

C Don't count the electron jet
c ----------------------------
          IW(JDTOPS+OFFSET8+6) = NJ15 - 1

      ELSEIF(FIRST_LEP.LT.0) THEN

c In the case where the primary muon is not isolated don't count the muon jet
c ---------------------------------------------------------------------------
          MJET = 0
          IF(NUM_IM+NUM_ITM.EQ.0.AND.NUM_M.GT.0) THEN
            
C Check to see if primary nonisolated muon has a jet associated with it
c ---------------------------------------------------------------------
            PX_M = P4M_CMUO(1,-FIRST_LEP)
            PY_M = P4M_CMUO(2,-FIRST_LEP)
            PZ_M = P4M_CMUO(3,-FIRST_LEP)
            PTOT = SQRT(PX_M**2+PY_M**2+PZ_M**2)
            MAX_CTH = 0.9
            DO J=1,ICLUS
                IF(PTOT*SQRT(IJET_P(1,J)**2+IJET_P(2,J)**2+
     &                               IJET_P(3,J)**2).GT.0.00001) THEN

                   COS_TH = (IJET_P(1,J)*PX_M+IJET_P(2,J)*
     &                       PY_M+IJET_P(3,J)*PZ_M)/
     &                      (PTOT*SQRT(IJET_P(1,J)**2+
     &                       IJET_P(2,J)**2+IJET_P(3,J)**2))
                ENDIF
                IF(COS_TH.GT.MAX_CTH.AND.IJET_ET(J).GT.15.0.AND.
     &             ABS(IJET_DETA(J)).LT.2.0) THEN
                      MAX_CTH = COS_TH
                      MJET = 1
                ENDIF
            ENDDO
          ENDIF
          IW(JDTOPS+OFFSET8+6) = NJ15 - MJET

C If no lepton found then just fill the number of jets.
c -----------------------------------------------------
      ELSE 

          IW(JDTOPS+OFFSET8+6) = NJ15

      ENDIF

C If this is a dilepton event, store the number of jets.
c -----------------------------------------------------
      IF(DIL_CAND) THEN
         IW(JDTOPS+OFFSET8+7) = NJET_DIL
      ELSE
         IW(JDTOPS+OFFSET8+7) = -1
      ENDIF
      

C If this is a Z event, store the number of jets
c ----------------------------------------------
      IF(LZFLAG) THEN
         ZDIFF = 999.
         DO I=1,MIN(ZCNT,3)
            IF(ABS(MASSZ(I)-91.).LT.ZDIFF) THEN
               ZDIFF = ABS(MASSZ(I)-91.)
               ZLEG_1 = LEGZ(1,I)
               ZLEG_2 = LEGZ(2,I)
               ZPNT   = I
               IF(ZTYPE(I).GT.0) THEN
                  
C Look for second leg and decide if it is over threshold
c ------------------------------------------------------
                  ZJET = 1
                  DO J=1,ICLUS
                     IF(IJETBNK(J).EQ.JTS_ELE(LEGZ(2,I)).AND.
     &                  ABS(IJET_DETA(J)).LT.2.0.AND.
     &                      IJET_ET(J).GT.15.)  ZJET = 2
                  ENDDO
                  IW(JDTOPS+OFFSET8+8) = NJ15 - ZJET 
               ELSE
                  IW(JDTOPS+OFFSET8+8) = NJ15
               ENDIF
            ENDIF
         ENDDO
      ELSE
         IW(JDTOPS+OFFSET8+8) = -1
      ENDIF


C If this is a Z event store the second leg in the second lepton slot
c -------------------------------------------------------------------
      IF(LZFLAG) THEN

         SECOND_LEP = 0
         IF(FIRST_LEP.EQ.ZLEG_1.AND.ZTYPE(ZPNT).GT.0) THEN
            SECOND_LEP = ZLEG_2
         ELSEIF(FIRST_LEP.EQ.-ZLEG_1.AND.ZTYPE(ZPNT).LT.0) THEN
            SECOND_LEP = -ZLEG_2
         ELSEIF(FIRST_LEP.EQ.ZLEG_2.AND.ZTYPE(ZPNT).GT.0) THEN
            SECOND_LEP = ZLEG_1
         ELSEIF(FIRST_LEP.EQ.-ZLEG_2.AND.ZTYPE(ZPNT).LT.0) THEN
            SECOND_LEP = -ZLEG_1
         ENDIF

C Now fill the SECONDARY Lepton Information
c -----------------------------------------
         IF(SECOND_LEP.GT.0) THEN
            RW(JDTOPS+OFFSETL2+0)  = ET_ELE(SECOND_LEP)
            RW(JDTOPS+OFFSETL2+1)  = P4E_ELE(1,SECOND_LEP)
            RW(JDTOPS+OFFSETL2+2)  = P4E_ELE(2,SECOND_LEP)
            RW(JDTOPS+OFFSETL2+3)  = P4E_ELE(3,SECOND_LEP)
            RW(JDTOPS+OFFSETL2+4)  = P4E_ELE(4,SECOND_LEP)
            RW(JDTOPS+OFFSETL2+5)  = ISOL_ELE(SECOND_LEP)
            IW(JDTOPS+OFFSETL2+6)  = HPTE+PEM
            IW(JDTOPS+OFFSETL2+7)  = SECOND_LEP
            IW(JDTOPS+OFFSETL2+8)  = BNK_ELE(SECOND_LEP)
            IW(JDTOPS+OFFSETL2+9)  = JTS_ELE(SECOND_LEP)
            IF (GOOD_E_TRIG) THEN
               IW(JDTOPS+OFFSETL2+10)=1
            ELSE
               IW(JDTOPS+OFFSETL2+10)=0
            ENDIF
         ELSEIF(SECOND_LEP.LT.0.AND.LTYPE(2,ZPNT).NE.10) THEN
            RW(JDTOPS+OFFSETL2+0)  = PT_CMUO(-SECOND_LEP)
            RW(JDTOPS+OFFSETL2+1)  = P4M_CMUO(1,-SECOND_LEP)
            RW(JDTOPS+OFFSETL2+2)  = P4M_CMUO(2,-SECOND_LEP)
            RW(JDTOPS+OFFSETL2+3)  = P4M_CMUO(3,-SECOND_LEP)
            RW(JDTOPS+OFFSETL2+4)  = P4M_CMUO(4,-SECOND_LEP)
            RW(JDTOPS+OFFSETL2+5)  = ISOL_CMUO(-SECOND_LEP)
            IW(JDTOPS+OFFSETL2+6)  = HPTM + DET_CMUO(-SECOND_LEP)
            IW(JDTOPS+OFFSETL2+7)  = -SECOND_LEP
            IW(JDTOPS+OFFSETL2+8)  = BNK_CMUO(-SECOND_LEP)
            IW(JDTOPS+OFFSETL2+9)  = TRKS_CMUO(-SECOND_LEP)
            IF (GOOD_MU_TRIG(-SECOND_LEP)) THEN
              IW(JDTOPS+OFFSETL2+10)=1
            ELSE
              IW(JDTOPS+OFFSETL2+10)=0
            ENDIF
         ELSEIF(SECOND_LEP.LT.0.AND.LTYPE(2,ZPNT).EQ.10) THEN
             RW(JDTOPS+OFFSETL2+0)  = PT_CMIO(-SECOND_LEP)
             RW(JDTOPS+OFFSETL2+1)  = P4M_CMIO(1,-SECOND_LEP)
             RW(JDTOPS+OFFSETL2+2)  = P4M_CMIO(2,-SECOND_LEP)
             RW(JDTOPS+OFFSETL2+3)  = P4M_CMIO(3,-SECOND_LEP)
             RW(JDTOPS+OFFSETL2+4)  = P4M_CMIO(4,-SECOND_LEP)
             RW(JDTOPS+OFFSETL2+5)  = ISOL_CMIO(-SECOND_LEP)
             IW(JDTOPS+OFFSETL2+6)  = 40
             IW(JDTOPS+OFFSETL2+7)  = -SECOND_LEP
             IW(JDTOPS+OFFSETL2+8)  = BNK_CMIO(-SECOND_LEP)
             IW(JDTOPS+OFFSETL2+9)  = TRKS_CMIO(-SECOND_LEP)
             IF (GOOD_MU_TRIG(-SECOND_LEP)) THEN
                IW(JDTOPS+OFFSETL2+10) = 1
             ELSE
                IW(JDTOPS+OFFSETL2+10) = 0
             ENDIF
         ELSEIF(SECOND_LEP.EQ.0) THEN

C Primary Lepton is not one of the Z legs....write out -999 for values
c --------------------------------------------------------------------
            RW(JDTOPS+OFFSETL2+0)  = -999.
            RW(JDTOPS+OFFSETL2+1)  = -999.
            RW(JDTOPS+OFFSETL2+2)  = -999.
            RW(JDTOPS+OFFSETL2+3)  = -999.
            RW(JDTOPS+OFFSETL2+4)  = -999.
            RW(JDTOPS+OFFSETL2+5)  =  99.
            IW(JDTOPS+OFFSETL2+6)  =  99
            IW(JDTOPS+OFFSETL2+7)  =  99
            IW(JDTOPS+OFFSETL2+8)  =  99
            IW(JDTOPS+OFFSETL2+9)  =  99
            IW(JDTOPS+OFFSETL2+10) =  99
         ENDIF
      ENDIF


c Calculate Ht Variable:
c
c    Ht  =  Et(e) + Pt(mu) + MET + Jet(1) + Jet(2) + ...
c -------------------------------------------------------

        LEP_EN=0.
        JET_EN=0.
        MET_EN=0.
        HT    =0.

       DO I=1,ICLUS
        ET_JET_COR = JCORF(I,2)*IJET_ET(I)

        IF(IJET_ET(I).GT.8. .AND.
     &     ABS(IJET_DETA(I)).LT.2.4)THEN

            IF(IW(JDTOPS+OFFSETL+6).GE.20 .AND. 
     &         IW(JDTOPS+OFFSETL+6).LT.30 .AND.
     &            IJETBNK(I).EQ.JTS_ELE(FIRST_LEP))GOTO 666
            
            IF(DIL_CAND.AND.
     &         IW(JDTOPS+OFFSETL2+6).GE.20 .AND.
     &         IW(JDTOPS+OFFSETL2+6).LT.30 .AND.
     &         IJETBNK(I).EQ.JTS_ELE(SECOND_LEP))GOTO 666

                  JET_EN = JET_EN + ET_JET_COR

666         CONTINUE
        ENDIF
       ENDDO

        IF(.NOT.DIL_CAND .AND. IW(JDTOPS+OFFSETL+6).GT.30)THEN
            LEP_EN = PT_CMUO(-FIRST_LEP)
            MET_EN = MET(4)
        ENDIF
        IF(.NOT.DIL_CAND .AND. 
     &        IW(JDTOPS+OFFSETL+6).GE.20 .AND.
     &        IW(JDTOPS+OFFSETL+6).LT.30)THEN
            LEP_EN = ET_ELE(FIRST_LEP)
            MET_EN = MET(4)
        ENDIF
        IF(DIL_CAND)THEN
            MET_EN = MET(4)
            ELE2_PTR=ABS(ELE2_PTR)
          IF(DIL_MODE.EQ.1)LEP_EN = ET_ELE(ELE1_PTR)+ET_ELE(ELE2_PTR)
          IF(DIL_MODE.EQ.2)THEN
           IF(MUO1_PTR.LT.0.)LEP_EN=ET_ELE(ELE1_PTR)+PT_CMIO(-MUO1_PTR)
           IF(MUO1_PTR.GT.0.)LEP_EN=ET_ELE(ELE1_PTR)+PT_CMUO(MUO1_PTR)
          ENDIF
          IF(DIL_MODE.EQ.3)THEN
           IF(MUO2_PTR.LT.0.)LEP_EN=PT_CMUO(MUO1_PTR)+PT_CMIO(-MUO2_PTR)
           IF(MUO2_PTR.GT.0.)LEP_EN=PT_CMUO(MUO1_PTR)+PT_CMUO(MUO2_PTR)
          ENDIF
        ENDIF

        HT =  LEP_EN + MET_EN + JET_EN
        RW(JDTOPS+5)   = HT

      RETURN
      END
