	SUBROUTINE ANGLE_CORR_BM(OVER,PXNEW,PYNEW,PZNEW,ETNEW)
C----------------------------------------------------------
C This subroutine calculates the Px,Py,Pz,Et of
C a central electron candidate using the angle from
C the tracking and the energy from the calorimetry
C (ELES uses energy from calorimetry and angles from
C strip chambers).  The new PX,PY,PZ,ET are returned and,
C if OVER=.TRUE., this subroutine OVERWRITES the
C quantities PXELE,PYELE,PZELE,ETELE in the ELES common
C block (NOT in the ELES bank)
C  It uses the PX,PY,PZ of a beam constrained track fit!!!!
C----------------------------------------------------------
C
	IMPLICIT NONE
	INCLUDE 'UC$ANA:ELESCM.INC'

	LOGICAL OVER
	INTEGER I,J,IMAX
	REAL PTMAX,PXNEW(MAXELE),PYNEW(MAXELE),PZNEW(MAXELE),ETNEW(MAXELE)
C
C            NTRK        Number of tracks.
C            JTRK(J)     Track number of Jth track.
C            XTRK1(3,J)  Position (x,y,z) at point of closest
C                            approach to beam line of Jth track.
C            XTRK2(3,J)  Position (x,y,z) at calorimeter of
C                            Jth track.  Position is at R=184.15
C                            for CEM, Z = +-190 for PEM and
C                            Z = +-640 for FEM.
C            DTRK1(3,J)  Direction cosines of Jth track at closest
C                            approach to beam line.
C            DTRK2(3,J)  Direction cosines of Jth track at calorimeter.
C            PTRK(I,J)   I=1 => Momentum times charge for
C                                                unconstrained fit.
C                        I=2 => Error in momentum for
C                                                unconstrained fit.
C                        I=3 => Momentum times charge for vertex
C                                                 constrained fit.
C                        I=4 => Error in momentum for vertex
C                                                 constrained fit.
C            IQUAL(J)    Track type:   0 = Unrecognized fit code
C                                      1 = 3D CTC
C                                      2 = 2D CTC
C                                      3 = 3D VTPC
C                                      4 = 2D VTPC
C                                      5 = 3D FTC
C                                      6 = 2D FTC
C                                      7 = 3D VTPC, FTC
C
C
C    Argument declarations.
      INTEGER NTRKMX
      PARAMETER (NTRKMX=20)
      INTEGER JCAL,NTRK,JTRK(NTRKMX),IQUAL(NTRKMX)
      REAL XTRK1(3,NTRKMX),XTRK2(3,NTRKMX),PTRK(4,NTRKMX)
      REAL DTRK1(3,NTRKMX),DTRK2(3,NTRKMX)

C
C
C    Argument declarations.
	REAL    	STPXYZ(3),IMPCT,PVEC(4),zvertx
	real 		sinthe,trkphi
	real		cdfphi
	integer		flag1

C---------------------------------------------------------------
C Loop over ELES banks, skip if not central or if no 3d track
C---------------------------------------------------------------
	DO 100 I=1,NELE
	   PXNEW(I) = PXELE(I)
	   PYNEW(I) = PYELE(I)
	   PZNEW(I) = PZELE(I)
	   ETNEW(I) = ETELE(I)
	   IF (WHERE(I) .NE. 0) GOTO 100
	   IF (N3DTR(I) .EQ. 0) GOTO 100
c
c  call routine to find track parameters, including the new vertex
c  from constraint code
c
	   call eletrk(elindx(i),pvec,stpxyz,zvertx,impct,flag1)
c
c  if it worked, then use the info to continue
c
	   if (flag1.eq.0) then
c
c  now find theta, phi and correct px,py,pz,et from calorimetry
c
	     trkphi = cdfphi(atan2(pvec(2),pvec(1)))
	     sinthe = sqrt(pvec(1)**2+pvec(2)**2)/
     >		sqrt(pvec(1)**2+pvec(2)**2+pvec(3)**2)
c  
c  px = sin theta cos phi * energy
c  py = sin theta sin phi * energy
c  pz = cos theta * energy = (trackpz/abs(trackpz)*sqrt(1-sintheta**2) * energy
c  et = sin theta * energy
c
	     PXNEW(I) = sinthe*cos(trkphi)*ENELE(I)
	     PYNEW(I) = sinthe*sin(trkphi)*ENELE(I)
	     PZNEW(I) = (pvec(3)/abs(pvec(3)))*sqrt(1-sinthe**2)*ENELE(I)
	     ETNEW(I) = sinthe * ENELE(I)
	     IF (OVER) THEN
	       PXELE(I) = PXNEW(I)
	       PYELE(I) = PYNEW(I)
	       PZELE(I) = PZNEW(I)
	       ETELE(I) = ETNEW(I)
	     END IF
	  else
c  
c  constraint didn't work, just use unconstrained parameters
c
C---------------------------------------------------------------
C Unpack ELET bank
C---------------------------------------------------------------
	   CALL TRKLST(CALSBN(I),NTRK,JTRK,XTRK1,XTRK2,DTRK1,DTRK2,PTRK,IQUAL)
	   IF (NTRK.EQ.0) GOTO 100		! should never happen
C---------------------------------------------------------------
C Find highest Pt 3d-track for this cluster 
C---------------------------------------------------------------
	   IMAX = 0				! pointer to high Pt track
	   PTMAX = 0.0
	   DO 50 J=1,NTRK
	      IF (IQUAL(J) .NE. 1) GOTO 50	! want 3D track
	      IF (ABS(PTRK(1,J))*SQRT(DTRK1(1,J)**2+DTRK1(2,J)**2)
	1				.GT. PTMAX) THEN
		 IMAX = J
		 PTMAX = ABS(PTRK(1,J))*SQRT(DTRK1(1,IMAX)**2+DTRK1(2,IMAX)**2)
	      END IF
50	   CONTINUE
	   IF (IMAX .EQ. 0) GOTO 100		! should never happen
C---------------------------------------------------------------
C Now correct the electron four vectors
C---------------------------------------------------------------
	   PXNEW(I) = DTRK1(1,IMAX)*ENELE(I)
	   PYNEW(I) = DTRK1(2,IMAX)*ENELE(I)
	   PZNEW(I) = DTRK1(3,IMAX)*ENELE(I)
	   ETNEW(I) = SQRT(DTRK1(1,IMAX)**2+DTRK1(2,IMAX)**2)*ENELE(I)
	   IF (OVER) THEN
	      PXELE(I) = PXNEW(I)
	      PYELE(I) = PYNEW(I)
	      PZELE(I) = PZNEW(I)
	      ETELE(I) = ETNEW(I)
	   END IF
	  endif
C---------------------------------------------------------------
C Now try the next electron
C---------------------------------------------------------------
100	CONTINUE
	RETURN
	END
