      INTEGER FUNCTION BADRUN(LRUN, FAILED)
C
C  Description:
C
C  This function determines whether the run number of the current
C  event is a bad run.
C
C  The bad runs are determined from the following sources:
C	Source					Run Numbers
C	------					-----------
C	Large MET means or sigmas 
C	found from Min Bias data		16785
C
C	Large number of PEM dead		17469,17475,17512
C	channels (> 30)				18869-18947
C
C	Problems with luminosity		17512-17516
C
C  Preconditions necessary before call:
C
C	None
C
C  Input Arguments:
C
C	None
C
C  Output Arguments:
C
C  LRUN   (Logical)             True if run is NOT a bad run.
C  FAILED (Integer*4)           Bit pattern of cuts that fail.
C                                 0	failed MET bad run
C                                 1	failed PEM channel bad run
C                                 2     failed luminosity bad run
C
C  Function Return Value:
C
C  ERSUCC       Normal execution
C
C  Author:
C
C	Marshall Miller
C	University of Chicago
C	CDF 
C
C  Revision History:
C
C	29-Jan-1989	Original
C
C  Implicit None Declaration:
C
$$IMPLICIT
C
C  Global Declarations:
C
$$Include 'YBOS$LIBRARY:ERRCOD.INC'
$$Include 'Error_Logger$Library:ERLEVL.INC'
$$Include 'C$INC:BCS   .INC'
$$Include 'C$INC:JOBSTA.INC'
C
C  Local Declarations:
C
        INTEGER         LUNIT
        LOGICAL         LRUN, MET, PEM, LUM
        INTEGER         FAILED
        INTEGER         STATUS
C
C  External Declarations:
C
C  Data Statements:
C
C  Executable Code:
C
        BADRUN = ERSUCC
        LRUN   = .FALSE.
        FAILED = '1FFF'X
C--
C  Initialize logical cuts
C--
	MET = .FALSE.
	PEM = .FALSE.
	LUM = .FALSE.
C--
C  Check if a bad MET run
C--	
	MET = (NRUN.EQ.16785)
C--
C  Check if a bad PEM channel run
C--	
	PEM = (NRUN.EQ.17469 .OR.
     >		    NRUN.EQ.17475 .OR.
     >		    NRUN.EQ.17512 .OR.
     >		    (NRUN.GE.18869 .AND. NRUN.LE.18947))
C--
C  Check if a bad luminosity run
C--	
	LUM = (NRUN.GE.17512 .AND. NRUN.LE.17516)
C--
C  Now set the output flags.
C--
        FAILED = 0
        IF (MET)		   FAILED = IBSET(FAILED,0)
        IF (PEM)		   FAILED = IBSET(FAILED,1)
        IF (LUM)		   FAILED = IBSET(FAILED,2)
        LRUN = (FAILED.EQ.0)
C==
        RETURN
        END
