C=======================================================================
      SUBROUTINE ELETRK(IELS,pvec,stpxyz,zvrtx,impct,flag)
C=======================================================================
C Description:
C
C  Routine to define electron quality cuts. 
C  Input  : IELS      -  Data pointer for ELES bank.
C  Output : PVEC(4)   -  Track info: 
C                           (1) Px
C                           (2) Py
C                           (3) Pz
C                           (4) Electric Charge
C         STPXYZ(3) -  Track interception with strip chamber :
C                           (1) x
C                           (2) y
C                           (3) z
C         ZVRTX     -  TRACK Z0 - if constraint worked, it = vertex Z.
C         IMPCT     -  Impact param. 
C         FLAG      -  Return code:
C                            0  all is well.
C                            1  Track momenta, no beam fit.
C                            2  no track found.
C
C Implicit none declaration:
$$IMPLICIT 
C Global Declarations:
$$INCLUDE 'C$INC:BCS.INC'
$$INCLUDE 'YBOS$LIBRARY:ERRCOD.INC'
$$INCLUDE 'C$INC:ERLEVL.INC'
$$INCLUDE 'C$INC:JOBSTA.INC'
$$INCLUDE 'C$INC:DSELES.INC'
!$$INCLUDE 'C$INC:DSQTRK.INC'
$$INCLUDE 'C$INC:PIVALU.INC'
$$INCLUDE 'C$INC:CTCCOM.INC'
$$INCLUDE 'C$INC:LOGUNI.INC'
$$INCLUDE 'C$INC:BTMASK.INC'
$$INCLUDE 'C$INC:TREXPO.INC'
C   Ybos pointers
	INTEGER   IELS
C External function     
	INTEGER QTRKBM
	INTEGER QTRLUN
	INTEGER BLOCAT
	REAL INVTAN
C   temporary storage
	LOGICAL FLAG3D,SECVTX,VTPC,CTC,VTXCON
	INTEGER   REGION,N3DT,FLAG
	INTEGER TRKNM,BEAMF
	INTEGER DUMMY1,DUMMY2,I                    
	INTEGER CHARGE,HITLAY,CTCSN,IERR,FITQ,SELECT
	INTEGER QTKIN,QTKIND,CTCIN,CTCIND,STATUS
	REAL      PX_E,PY_E,PZ_E,PT
	REAL      X2 
	REAL    STPXYZ(3),ZVRTX,IMPCT,PVEC(4)
	REAL    PX,PY,PZ,ZV,IMPACT                  
	REAL    PARAM(5),XYZ(6),DXYZ(6)
C-----------------------------------------------------------------------
c
c  since we are using QTRK, check to see if it's around.  If not, then
c  check on CTCS and make it.  If not, then log an error and set flag = 2
c
	status = blocat(iw,'QTRK',-1,qtkin,qtkind)	! check QTRK
	if (status.ne.yesucc) then			! not there!
	  status = blocat(iw,'CTCS',-1,ctcin,ctcind)	! check CTCS
	  if (status.eq.yesucc) then			! found it
	    call ctcpak					! make QTRK
	    call qtrlcr					! and the link banks
	  else
	    CALL ERLOGR('ELETRK',ERWARN,0,0,'No QTRK bank')	    
	    pvec(1) = RW(IELS+PX_ELE)	! set to default values
	    pvec(2) = RW(IELS+PY_ELE)
	    pvec(3) = RW(IELS+PZ_ELE)
	    pvec(4) = 0.		! INCLUDE CHARGE....
	    FLAG = 2
	    return			! and bail out
	  endif
	endif    
	N3DT = IW(IELS+N3DELE)		! number of 3D tracks
c
c  get bank number of CTCS bank associated with TRKS bank with highest
c   momentum match to the ELES cluster.  This will be passed on to
c   compare to QTRK, which has CTCS bank numbers.
c
	TRKNM  = QTRLUN(IW(IELS+TRPELE),'TRK','CTC') ! bank number
c
c  if there are no 3D tracks, don't bother, just use calorimetry
c
	IF (N3DT .GE. 1 .AND. TRKNM .GT. 0) THEN  
	  TRKNM = -TRKNM	! negative to refer to CTCS bank #
C... Get track info :                                          
	  CALL QTRKUN(DUMMY1,DUMMY2,                       
     &            TRKNM,CHARGE,PX,PY,PZ,ZV,IMPACT,HITLAY,CTCSN,   
     &            FLAG3D,SECVTX,SELECT,VTPC,CTC,VTXCON,FITQ,IERR) 
	  PX_E = PX
	  PY_E = PY
	  PZ_E = PZ
	  ZVRTX   = ZV		! save old track Z0
	  IMPCT   = IMPACT	! and impact parameter position
	  X2 = 10.0		! chi2 for vertex matching in QTRKBM
	  FLAG = 1		! Have a 3D track
	  BEAMF = 9		! ???????            
c
c  do beam constraint....
c
	  BEAMF = QTRKBM(X2,HITLAY,CHARGE,PX,PY,PZ,ZV,IMPACT)
C... Electron quantities :
	  IF (BEAMF  .EQ. 0) THEN	! constraint worked
	    PX_E = PX			! so get px,py,pz
	    PY_E = PY			! and set flag
	    PZ_E = PZ
	    FLAG = 0
	    ZVRTX = ZV			! new track Z0
	  END IF
	ELSE 				! no 3D tracks/TRKS/CTCS 
	  PX_E = RW(IELS+PX_ELE)	! so use calorimetry???
	  PY_E = RW(IELS+PY_ELE)
	  PZ_E = RW(IELS+PZ_ELE)
	  FLAG = 2
	END IF
C 
c  if there is a 3D track, get other parameters
c
	IF ( FLAG .LE. 1 ) THEN 
c
c  Param is passed to TRPROP - zero them and then fill it in
c
	  DO I=1,5
	    PARAM(I)=0.
	  END DO
c
c...track curvature from pt,charge,field
c
	  PT = SQRT(PX_E**2+PY_E**2)
	  IF(PT.GT.0.0001) PARAM(1)=REAL(CHARGE)*BMAGNT*2.9979E-4/2./PT
c
c...impact parameter -- = 0 if constraint worked
c
	  PARAM(2) = IMPACT
c
C...phi of track
c
	  PARAM(3) = INVTAN(PY_E,PX_E)
c
c...cotan theta
c
	  PARAM(4)=99.
	  IF(PT.GT.0.0001) PARAM(4)=PZ_E/PT
c
c...z vertex - this is the one we constrained to.
c
	  PARAM(5) = ZVRTX
c
c propagate to strip chamber radius
c
	  CALL TRPROP(1,PARAM,TRPOEC,XYZ,DXYZ,IERR)  
c
c  if it worked then return the strip chamber values in global coordinates
c
	  IF ( IERR .EQ. 0 ) THEN
	    DO I = 1, 3
	      STPXYZ(I) = XYZ(I)
	    END DO
	  ELSE 
c
c  otherwise set them slightly ridiculous
c
	    DO I = 1, 3
	      STPXYZ(I) = 999.
	    END DO
	  END IF     
	END IF
C
c  set return parameters of momentum vector at the beam
c
	PVEC(1) = PX_E
	PVEC(2) = PY_E
	PVEC(3) = PZ_E
	PVEC(4) = REAL(CHARGE)	! INCLUDE CHARGE....
C
	RETURN
	END       
