C DEC/CMS REPLACEMENT HISTORY, Element STRFIX.CDF
C *3    22-MAY-1989 16:20:16 PEKKA "Handle case of ELES bank with no CTC track."
C *2    19-MAY-1989 16:47:55 PEKKA "Now handles MC and 5.1 ELES banks."
C *1    17-MAY-1989 03:42:03 PEKKA "Recalculates track-strip match using CES offsets for ELES banks."
C DEC/CMS REPLACEMENT HISTORY, Element STRFIX.CDF
      INTEGER FUNCTION STRFIX(ELESBN, DELX, DELZ)
C
C===Description:
C
C  This function recalculates the track-strip chamber cluster match
C  for the electron candidate in ELES bank number ELESBN.
C  The match is recalculated using Barry Wicklund's CESALG
C  routine.
C
C===Preconditions necessary before call:
C
C  ELES and QTRK bank present.
C
C===Input Arguments:
C
C  ELESBN (Integer*4)           ELES/CALS bank number of cluster.
C
C===Output Arguments:
C
C  DELX (Real*4)                Track-Strip R-Phi position (cm)
C  DELZ (Rela*4)                Track-Strip Z position (cm)
C
C===Function Return Value:
C
C  ERSUCC       Normal execution
C  ERWARN       ELES bank not found, or CESALG error.
C
C===Author:
C
C       Pekka K. Sinervo
C       University of Pennsylvania and CDF
C       15-MAY-1989
C
C===Revisions:
C
C   5/19/89     Pekka   Don't correct MC data, and deal with the event
C                       vertex correctly.
C   3/1/90	Paul Derwent   Get vertex, extrapolated position from Beam
C			       Constrained track.
C
C===Implicit None Declaration:
C
$$IMPLICIT
C
C===Global Declarations:
C
$$Include 'YBOS$LIBRARY:ERRCOD.INC'
$$Include 'Error_Logger$Library:ERLEVL.INC'
$$Include 'C$INC:BCS   .INC'
$$Include 'C$INC:JOBSTA.INC'
$$Include 'C$INC:DSELES.INC'
$$Include 'C$INC:DSEVTA.INC'
$$Include 'C$INC:LOGUNI.INC'
$$Include 'C$INC:PIVALU.INC'
$$Include 'C$INC:TREXPO.INC'
C
C===Local Declarations:
C
        INTEGER         ELESBN
        REAL            DELX,   DELZ
C
        INTEGER         STATUS,flag
	REAL    	STPXYZ(3),IMPCT,PVEC(4)
        REAL            PHITRK, DELPHI
        REAL            PARIN(5),       ZVERTX
        REAL            XCTC,   ZCTC,   DXCTC,  DZCTC
        INTEGER         IEAST,  IPHI,   IERR
        INTEGER         ITRK
        INTEGER         DUMMY
        REAL            PHISTR, ZSTRGL, ZCTCGL
        REAL            ZSTR
        LOGICAL         STRCLS, WIRCLS
        LOGICAL         LMC
C--
C  YBOS bank pointers.
C--
        INTEGER         IBEVTA, IDEVTA
        INTEGER         IBELES, IDELES
C
C===External Declarations:-
C
        REAL            CDFPHI
        INTEGER         BLOCAT
        CHARACTER*8     ERRSTR
C
C===Data Statements:-
C
C       None
C
C===ASF's
C
C       None
C
C===Executable Code:
C
        STRFIX = ERSUCC
        DELX   = 999.
        DELZ   = 999.
C--
C  Check if this is MC data.
C--
        LMC = (EXPTYP.EQ.4 .OR. NRUN.LE.100)
C--
C  Find the corresponding ELES bank first.
C--
        STATUS = BLOCAT(IW, 'ELES', ELESBN, IBELES, IDELES)
        IF (STATUS.NE.YESUCC) THEN
          WRITE(ERRSTR(1:4),'(I4)') ELESBN
          CALL ERLOGR('STRFIX', ERWARN, 0, 0,
     >        'Could not locate ELES bank '//ERRSTR(1:4))
          STRFIX = ERWARN
          RETURN
        ENDIF
C--
C  Only look at CEM candidates that have 3D tracks associated with
C  them.
C--
        IF (IW(IDELES + REGELE).EQ.0 .AND.
     >      IW(IDELES + N3DELE).GT.0) THEN
C--
C  Check for the existence of a strip and wire cluster by checking
C  their energies.
C--
          STRCLS = (RW(IDELES + ESCELE).NE.0.)
          WIRCLS = (RW(IDELES + EWCELE).NE.0.)
C--
C  For MC data, simply fetch the nominal track position.
C  In cases where there is no track momentum stored in ELES,
C  it is probably a VTCS track that just managed to hit the
C  CEM.  For these, we do not try to correct since we do not
C  have a QTRK entry for them.
C--
          IF (LMC .OR.
     >           RW(IDELES + P__ELE).EQ.0.0) THEN
            ZCTCGL = RW(IDELES + ZEXELE)
            PHITRK = CDFPHI(ATAN2(RW(IDELES + YEXELE),
     +                               RW(IDELES + XEXELE)))
C--
C  For real data, set up to call CESALG to get the track position.
C  Get the phi of the track at the strip chamber, and the wedge
C  number in phi and eta.
C--
          ELSE
            PHITRK = CDFPHI(ATAN2(RW(IDELES + YEXELE),
     >                        RW(IDELES + XEXELE)))
            IPHI   = INT(24 * PHITRK/TWOPI)
            ZSTR   = RW(IDELES + ZSTELE)
            IF (ZSTR.GT.0.0) THEN
              IEAST = 1
            ELSE
              IEAST = 0
            ENDIF
C--
C  Find the Z vertex for this electron candidate.  For ELES banks
C  with version 300, this is the EVTA vertex.  For versions 301 on,
C  this is in the ELES bank itself.
C--
            IF (IW(IDELES + VRDELE).GE.301) THEN
c
c get vertex, extrapolated track position from the new routine which
c  uses beam constrained tracks.  If it fails, then use what is in ELES.
c
	      call eletrk(ideles,pvec,stpxyz,zvertx,impct,flag)
	      if (flag.ne.0) then
                ZVERTX = RW(IDELES + ZV_ELE)
	      endif
            ELSE
              STATUS = BLOCAT(IW, 'EVTA', 1, IBEVTA, IDEVTA)
              IF (STATUS.NE.YESUCC) THEN
                CALL ERLOGR('STRFIX', ERWARN, 0, STATUS,
     >                        'Failed to find EVTA bank')
                STRFIX = ERWARN
                GOTO 500
              ELSE
                ZVERTX = RW(IDEVTA + Z0EV)   ! event Z vertex
              ENDIF
            ENDIF
C--
C  Get the track parameters from QTRK.
C  First find the right QTRK track.
C--
            IERR = 0
            ITRK = 0
            CALL GET_TRACK(ELESBN, ITRK)
            IF (ITRK.LE.0) THEN
              WRITE(ERRSTR(1:4),'(I4)') ELESBN
              CALL ERLOGR('STRFIX', ERWARN, 0, 0,
     >        'Could not find QTRK track for ELES bank '//ERRSTR(1:4))
              STRFIX = ERWARN
            ELSE
C--
C  Fetch the track parameters.
C--
              CALL GET_PARAM(IW, DUMMY, ITRK, PARIN, IERR)
              IF (IERR.NE.0) THEN
                WRITE(ERRSTR(1:4),'(I4)') ELESBN
                CALL ERLOGR('STRFIX', ERWARN, 0, 0,
     >           'Could not get QTRK parameters for ELES bank '//
     >                  ERRSTR(1:4))
                STRFIX = ERWARN
              ENDIF
            ENDIF
C--
C  If we failed for some reason to find a QTRK track, just assume
C  the previously calculated position for now.
C--
            IF (IERR.NE.0 .OR. ITRK.LE.0) THEN
              ZCTCGL = RW(IDELES + ZEXELE)
              PHITRK = CDFPHI(ATAN2(RW(IDELES + YEXELE),
     +                               RW(IDELES + XEXELE)))
C--
C  Get the track position, corrected for alignment offsets.
C  If CESALG fails, default to the old track position.
C--
            ELSE
              CALL CESALG( PARIN,IEAST,IPHI,ZVERTX,
     >                      XCTC,ZCTC,DXCTC,DZCTC, IERR)
              IF (IERR.NE.0) THEN
                WRITE(ERRSTR(1:4),'(I4)') ELESBN
                CALL ERLOGR('STRFIX', ERWARN, 0, 0,
     >            'CESALG failed for for ELES bank '//ERRSTR(1:4))
                STRFIX = ERWARN
                ZCTCGL = RW(IDELES + ZEXELE)
                PHITRK = CDFPHI(ATAN2(RW(IDELES + YEXELE),
     +                                RW(IDELES + XEXELE)))
              ELSE
C--
C  Compute the new match in global coordinates.  CESALG has
C  given us the track position in the wedge coordinate system
C  (ie. -24 < X < 24 cm  and 0 > Z > 280 cm).  We have to transform
C  this back to global coordinates to make a meaningful match
C  with the strip chamber position in ELES.
C--
                PHITRK = (XCTC + DXCTC)/TRPOEC +
     >                            (TWOPI/24)*(FLOAT(IPHI) + 0.5)
                PHITRK = CDFPHI(PHITRK)
                ZCTCGL = ZCTC + DZCTC
                IF (IEAST.EQ.0) ZCTCGL = -ZCTCGL
              ENDIF
            ENDIF
          ENDIF
C--
C  Fetch the strip phi and Z and compute the match when the clusters
C  exist.
C--
          PHISTR = RW(IDELES + PSCELE)
          ZSTRGL = RW(IDELES + ZSTELE)
c
c  use the information we got from ELETRK above for the new extrapolated
c  position.  it is in STPXYZ, so convert to phi.
c
c	  phistr = cdfphi(atan2(stpxyz(2),stpxyz(1)))
c	  zstrgl = stpxyz(3)
C
          IF (WIRCLS) THEN
            DELPHI = PHITRK - PHISTR
            IF (DELPHI.GT. PI) DELPHI = DELPHI - TWOPI
            IF (DELPHI.LT.-PI) DELPHI = DELPHI + TWOPI
            DELX = DELPHI*TRPOEC
          ENDIF
C
          IF (STRCLS) THEN
            DELZ = ZCTCGL - ZSTRGL
          ENDIF
C==
C$$          WRITE(LUPRT, 1000) ELESBN, ZVERTX, IPHI, IEAST,
C$$     >                          PHISTR, ZSTRGL,
C$$     >                          PHITRK, ZCTC, XCTC, DZCTC, DXCTC,
C$$     >                          DELX, DELZ
C$$ 1000     FORMAT(/,' STRFIX: ELES ',I3,' Zvtx = ',F7.3,
C$$     >                  '  Wedge ',2I4,
C$$     >           /,'         Strip Phi, Z,   = ',2F8.3,
C$$     >           /,'         Track Phi, Z, X = ',3F8.3,
C$$     >             '  Corr X, Z = ',2F8.3,
C$$     >           /,'         Delta X,Z = ',2F8.3)
C==
        ENDIF
C
 500    CONTINUE
        RETURN
        END
