C======================================================================
	SUBROUTINE TRGELC(ICALL,TRIG_PRES,TRIG_SAT)
C======================================================================
C
C  Description:
C  ============
C  Gets the TAGC bank and looks for the three triggers relevant for central
C  W's: ELECTRON_EMC_12_6_PREREQ_*,MISSING_ET_25_TEX_8_NOT_FWD,MISSING_ET_
C  30_L1_PHOTON. Returns flags for whether they are present and satisfied,
C  and fills and returns the cumulative matrices correlating the three as 
C  to whether they are present and satisfied. NOTE: only the first 22 char.
C  are compared of the names. (good enough as of March,1989)
C
C  Preconditions necessary before call:
C  ====================================
C   Claudio's EWUNPK module has to be in the path upstream of this
C
C  Input argument
C  ==============
C	ICALL		! 0 to initialize, 1 for an event to fill flags 
C			  and matrices, 2 to print the matrices
C
C  Output arguments
C  ==============
C	TRIG_PRES(3)	! 1 if electron,missing_et_25,missing_et_30 is present
C	TRIG_SAT(3)	! 1 if electron,missing_et_25,missing_et_30 is satisfied

C  Author:
C  =======
C  H.J. Frisch
C
C  Revision History:
C  =================
C  15-Mar-1989 - Original creation 
C  4-Aug-1989	PFD	Changing to also look back before EM cluster finding 
C			was implemented
C  10-Dec-1991  CC	Look at the correct TAGC bank number
C
C======================================================================
$$IF VAX
	IMPLICIT NONE
$$ENDIF
C
$$INCLUDE 'C$INC:ERLEVL.INC'
$$INCLUDE 'C$INC:JOBSTA.INC'
$$INCLUDE 'C$INC:BCS   .INC'
$$INCLUDE 'C$INC:BTMASK.INC'
$$INCLUDE 'C$INC:PIVALU.INC'
$$INCLUDE 'YBOS$LIBRARY:ERRCOD.INC'
$$INCLUDE 'C$INC:LOGUNI.INC'
$$INCLUDE 'UC$ANA:ELESCM.INC'		! Claudio's commn blcks filled in UNPEVT
$$INCLUDE 'UC$ANA:TAGCCM.INC/LIST'	! Claudio's commn blcks filled in UNPEVT
$$INCLUDE 'UC$ANA:UNFLAGS.INC'

C Argument Declarations:
C =====================

	INTEGER ICALL		! 0 to initialize, 1 for an event to fill flags 
C				! and matrices, 2 to print the matrices
	INTEGER TRIG_PRES(3)	! 1 if electron,missing_et_25,missing_et_30 
C				! is present
	INTEGER	TRIG_SAT(3)	! 1 if electron,missing_et_25,missing_et_30 
C				! is satisfied

C External Declarations:
C =====================
	INTEGER STR$CASE_BLIND_COMPARE 

C Local Declarations:
C ==================
	INTEGER I,J,ITRIG		! Misc loop indices
	INTEGER IERR			! Error flag = 0 normally
	INTEGER RESULT			! Comparison with trigger name: 0=true
	INTEGER MAT_PRES(3,3)	! Cumulative matrix of trggers being present
	INTEGER MAT_SAT(3,3)	! Cumulative matrix of trggers being satisfied
	CHARACTER*32 TRIG_NAME(3)	! Names to be matched
	CHARACTER*32 TRIG_NAME2(2)	! Names to be matched

C Data Declarations
C ==================
	DATA TRIG_NAME/
     &   'ELECTRON_EMC_12_6_PREREQ        ',	
     &   'MISSING_ET_25_TEX_8_NOT_FWD     ',	
     &   'MISSING_ET_30_L1PHOTON          '/
	DATA TRIG_NAME2/
     &   'ELECTRON_12                      ',	
     &   'MISSING_ET_                      '/

C Start of Code
C ===============
C----------------------------------------------------------------------------
C	Initialize
C----------------------------------------------------------------------------
	IF (ICALL.EQ.0) THEN			! Initialize
	  DO I=1,3
	    DO J=1,3
	      MAT_PRES(I,J)= 0	! Cumulative matrix of trggers being present
	      MAT_SAT(I,J) = 0	! Cumulative matrix of trggers being satisfied
	    ENDDO	
	  ENDDO
C----------------------------------------------------------------------------
C	Event call: test the three trigger for presence in trigger table etc.
C----------------------------------------------------------------------------
	ELSEIF (ICALL.EQ.1) THEN		! Event: fill flags and matrices
	  DO 50 I = 1,3
	    TRIG_PRES(I) = 0			
	    TRIG_SAT(I) = 0
50        CONTINUE
C
	  CALL TAGBTC(TAGC_NUMBER)		! Will normally be in UNPK
C
	  DO 100 ITRIG = 1,TAGC_NTRIG			! Number of triggers
	    IF (TAGC_LEVEL(ITRIG).NE.2) GO TO 100  	! Level 2 only
	    if (nrun.lt.16300) then		!different for pre EM clus find
	      do 89 i = 1,2			! just check two of them
	        result = str$case_blind_compare(	
	1			tagc_name(itrig)(1:11),
	1			trig_name2(i)(1:11))
	        if (result .eq. 0) then
	          trig_pres(i) = 1
	          if (tagc_pass(itrig).eq.3) trig_sat(i) = 1
	        endif
 89	      continue
	    else
	      DO 90 I = 1,3				! Check versus each of 3
	        RESULT = STR$CASE_BLIND_COMPARE (	! Compare 20 characters
	1		      TAGC_NAME(ITRIG)(1:20), 
	1		      TRIG_NAME(I)(1:20))
	        IF (RESULT .EQ. 0) THEN			! Found a match
	           TRIG_PRES(I) = 1			! Set flag it's in table
		   IF (TAGC_PASS(ITRIG).EQ.3) TRIG_SAT(I) = 1	! Pass bit: 
	        ENDIF
90	      CONTINUE
	  endif
100	  CONTINUE
C----------------------------------------------------------------------------
C	Now increment matrices
C----------------------------------------------------------------------------
	  DO I = 1,3
	     IF (TRIG_PRES(I).EQ.1) THEN		! Trigger was  in table
	        DO J = 1,3
	          IF (TRIG_PRES(J).EQ.1) THEN		! Trigger was  in table
	            MAT_PRES(I,J) = MAT_PRES(I,J) + 1	! Cumulative matrix 
	          ENDIF
	        ENDDO
	     ENDIF
	  ENDDO
C
	  DO I = 1,3
	     IF (TRIG_SAT(I).EQ.1) THEN			! Trigger was satisfied
	        DO J = 1,3
	          IF (TRIG_SAT(J).EQ.1) THEN		! Trigger was  satisfied
	            MAT_SAT(I,J) = MAT_SAT(I,J) + 1	! Cumulative matrix 
	          ENDIF
	       ENDDO
	     ENDIF
	  ENDDO
C----------------------------------------------------------------------------
C	Print out matrices
C----------------------------------------------------------------------------
	ELSEIF (ICALL.EQ.2) THEN		! Print matrices
	  PRINT 400, ((MAT_PRES(I,J),I=1,3),J=1,3)
400       FORMAT(' Triggers present in table'/
     & '  ELEC  MISSET1 MISSET2'/3(3I8/)/)
	  PRINT 500, ((MAT_SAT(I,J),I=1,3),J=1,3)
500       FORMAT(' Triggers satisfied '/
     & '  ELEC  MISSET1 MISSET2'/3(3I8/)/)
	ENDIF
	RETURN
	END
