C======================================================================
	SUBROUTINE TRGELE(JELE,IMATCH,JTL2EL,TL2ET,TL2ETA,TL2PHI,
     &               NTRGEL,point)
C======================================================================
C
C  Description:
C  ============
C  For a given ELES bank sees whether the trigger found that electron
C  Note- this is a poor man's way: the full trigger simulation starting with
C  the interceptor (TCSD) bank making a TL2D,3 bank is the right way.
C  Takes the first match- if there are 2 electrons in the same wedge, tough
C
C  Preconditions necessary before call:
C  ====================================
C   CDF Level 2  YBOS bank(TL2D) must be available (and of course the ELES bank)
C   Claudio's EWUNPK module has to be in the path upstream of this
C
C
C  Input argument
C  ==============
C	JELE		! ELES Bank number for electron to be matched
C	Point		! Pointer inside of Claudio's arrays
C
C  Output arguments
C  ==============
C	IMATCH		! =0 if no trigger electron matched, =1 if match found
C	JTL2EL		! Index of electron in Mercury El module in Cluster Mem	
C	TL2ET		! Et of electron in Mercury El module	(GeV)
C	TL2ETA		! Eta of electron in Mercury El module (-4.1 to 4.1)
C	TL2PHI		! Phi of electron in Mercury El module (0:2*pi)
C	NTRGEL		! Array of counters for successive cuts
C
C  Author:
C  =======
C  H.J. Frisch
C
C  Revision History:
C  =================
C  01-Mar-1989 - Original creation from TL2BTC
C
C======================================================================
$$IF VAX
	IMPLICIT NONE
$$ENDIF
C
$$INCLUDE 'C$INC:ERLEVL.INC'
$$INCLUDE 'C$INC:JOBSTA.INC'
$$INCLUDE 'C$INC:BCS   .INC'
$$INCLUDE 'C$INC:BTMASK.INC'
$$INCLUDE 'C$INC:PIVALU.INC'
$$INCLUDE 'YBOS$LIBRARY:ERRCOD.INC'
$$INCLUDE 'C$TRS:TSIN.INC'
$$INCLUDE 'C$TRS:TSOUT.INC'			! Level 2 arrays
$$INCLUDE 'C$INC:LOGUNI.INC'
$$INCLUDE 'W_Z$UC:ELESCM.INC'		! Claudio's commn blcks filled in UNPEVT


C External Declarations:
C =====================
	INTEGER BLOCAT

C Local Declarations:
C ==================
	INTEGER JELE	! ELES Bank number for electron to be matched
	INTEGER IMATCH	! =0 if no trigger electron matched, =1 if match found
	INTEGER JTL2EL	! Index of electron in Mercury El module in Cluster Mem	
	REAL TL2ET	! Et of electron in Mercury El module	(GeV)
	REAL TL2ETA	! Eta of electron in Mercury El module (0:41)
	REAL TL2PHI	! Phi of electron in Mercury El module (0:23)
	INTEGER I			! Misc index
	INTEGER NBANK			! Bank number for TL2D bank: 1= raw data
	INTEGER IERR			! Error flag = 0 normally
	INTEGER ICLUST			! Cluster pointer
	INTEGER INTTOT			! TOT energy in bits (1 cnt = .5)
	INTEGER INTEM 			! 1.25* EM energy in bits
	INTEGER IPT			! sign +3-bit PT
	INTEGER NTRGEL(8)		! Array of counters for successive cuts
	INTEGER TPHISEED	! Seed tower from ELES bank
	INTEGER POINT		! Pointer inside of Claudio's arrays for this
				! Eles bank
	
C Start of Code
C ===============
C----------------------------------------------------------------------------
C	Initialize
C----------------------------------------------------------------------------
	IMATCH = -1				! Number matched: -1 is error
	NBANK = 1				! Data (2=trigsim)
	DO 100 I =1,8
100	NTRGEL(I)=0
C----------------------------------------------------------------------------
C	Fill TSOUT common block with L2 quantities
C----------------------------------------------------------------------------
	CALL TL2BTC(NBANK,IERR)				! TL2D unpacking routine
	IF (IERR.NE.0) THEN				
	   CALL ERLOGR('TRGELE',ERWARN,0,0,
     1	   ' Problem with finding TL2D ')
	   IMATCH = -1
	   GO TO 2000					! Return
	ENDIF
C----------------------------------------------------------------------------
C	Now loop over the clusters looking for origin code=5 (2nd pass cluster
C	finding),and good EtEM, tot/em, and pt. See C$TRS:TSMERC.CDF for details
C----------------------------------------------------------------------------
	IF (L2NCL.EQ.0) THEN
	   IMATCH = 0			! No clusters at all found
	   GO TO 2000			! Return
	ENDIF	   
	DO 800 ICLUST = 1, L2NCL	 	! L2NCL is the total # of clusts
	   NTRGEL(1) = NTRGEL(1) +1		! Bump counter:1 = clusters	
	   if (nrun.lt.16299) then 
	     IF (L2ORIG(ICLUST).NE.1) GO TO 800	! Origin code:1=CF,2=L1,5=2nd ps
	   else
	     IF (L2ORIG(ICLUST).NE.5) GO TO 800	! Origin code:1=CF,2=L1,5=2nd ps
	   endif
	   NTRGEL(2) = NTRGEL(2) +1		! Bump counter:2 = 2nd pass cl	
	   IF (L2ET(1,ICLUST).LT.12.0) GO TO 800 ! ET of clust in GeV, 1=EM,2TOT
	   NTRGEL(3) = NTRGEL(3) +1		! Bump counter:3 = Et cut	
C
	   INTTOT = IFIX(2*L2ET(2,ICLUST))	! TOTenergy in bits (1 cnt = .5)
	   INTEM = IFIX(1.25*2*L2ET(1,ICLUST))	! 1.25* EM energy in bits
	   IF (INTTOT.GT.INTEM) GO TO 800	! TOT/EM > 1.25 (Mercury bd cut)
	   NTRGEL(4) = NTRGEL(4) +1		! Bump counter:4= tot/em cut
C
	   IF(L2STRK(ICLUST).NE.1) GO TO 800	! =1 if stiff track present
	   NTRGEL(5) = NTRGEL(5) +1		! Bump counter:5=stf trk cut
	   IPT = L2PT(ICLUST).AND.7		! sign +3-bit PT
	   IF (IPT.LT.3) GO TO 800		! 7 GeV bit cutoff
	   NTRGEL(6) = NTRGEL(6) +1		! Bump counter:6=pt cut
C----------------------------------------------------------------------------
C	At this point have a cluster which has satisfied the trigger-
C	now see if is in the same wedge as the ELES bank
C----------------------------------------------------------------------------
	   TPHISEED = TOWEAD(point) .AND. 127		! Phi of seed tower
	   IF (TPHISEED.NE.L2FSED(ICLUST)) GO TO 800	! phi wedge match
	   NTRGEL(7) = NTRGEL(7) +1		! Bump counter:7=same phi cut
	   IF( (L2YSED(ICLUST).GT.20.AND. ETADET(point).LT.0.0) .OR.
     &     (L2YSED(ICLUST).LT.21.AND. ETADET(point).GT.0.0)) GO TO 800
	   NTRGEL(8) = NTRGEL(8) +1		! Bump counter:8=same eta cut
C----------------------------------------------------------------------------
C	Have a match- an electron cluster which satisfies the trigger
C	conditions in the same wedge
C----------------------------------------------------------------------------
	IMATCH = 1
	TL2ET  = L2ET(1,ICLUST)			! em ET in GeV
	TL2ETA = L2YBAR(1,ICLUST)		! analog mean y in real units
	TL2PHI = L2FBAR(1,ICLUST)		! analog mean phi ditto (0:2pi)
	GO TO 2000				! bail out- 1st one takes it
800	CONTINUE
2000	RETURN
	END
